#include <C/Common/TRN_Exception.h>
#include <C/Common/TRN_ErrorCodes.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <map>
#include <sstream>
#include <string>

#include <C/Common/TRN_Exception.hpp>
#include <Common/Exception.h>
#include <Common/Threads.hpp>

using namespace std;

pdfnet_thread_space::mutex gl_cwrap_exception_lib_mutex;

map<string,Common::Exception> exception_map;

TRN_Exception GetPDFNetException(Common::Exception& ex)
{
	//boost::mutex::scoped_lock lock(gl_cwrap_exception_lib_mutex);
	pdfnet_thread_scope_lock(gl_cwrap_exception_lib_mutex);
	//gets the uniquely identifying line number
	//plus the file name
	stringstream s;
	s << ex.GetLineNumber() << ex.GetFileName() << ex.GetMessage();
	
	//looks in the global map for this exception
	std::map<string,Common::Exception>::iterator temp=exception_map.find(s.str());
	if(temp==exception_map.end())
	{
		//add this exception to the map
		string temp=s.str();
		exception_map[temp]=ex;
		Common::Exception* result=&(exception_map[temp]);
		return (TRN_Exception)result;
	}
	else
		return (TRN_Exception)&(temp->second);

}

TRN_Exception GetException(string& msg)
{
	//boost::mutex::scoped_lock lock(gl_cwrap_exception_lib_mutex);
	pdfnet_thread_scope_lock(gl_cwrap_exception_lib_mutex);
	//gets the uniquely identifying line number
	//plus the file name

	Common::Exception ex=Common::Exception("",0,"","",msg.c_str());

	//looks in the global map for this exception
	std::map<string,Common::Exception>::iterator temp=exception_map.find(msg);
	if(temp==exception_map.end())
	{
		//add this exception to the map	
		exception_map[msg]=ex;
		return (TRN_Exception)&(exception_map[msg]);
	}
	else
		return (TRN_Exception)&(temp->second);

}

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_CreateExceptionEx( const char* cond_expr, 
							const char* filename, TRN_Int32 linenumber, 
							const char* function, const char* message, TRN_UInt32 errorcode)
{
	Common::Exception temp(cond_expr,linenumber,
		filename,function,message,errorcode);
	return GetPDFNetException(temp);
}

TRN_API TRN_CreateException( const char* cond_expr, 
			const char* filename, TRN_Int32 linenumber, 
			const char* function, const char* message)
{
	Common::Exception temp(cond_expr,linenumber,
						filename,function,message);
	return GetPDFNetException(temp);
}

TRN_API_T(TRN_Int32) TRN_GetLineNum(TRN_Exception e)
{
	return ((Common::Exception*)e)->GetLineNumber();
}
TRN_API_T(const char*) TRN_GetCondExpr(TRN_Exception e)
{
	return ((Common::Exception*)e)->GetCondExpr();
}
TRN_API_T(const char*) TRN_GetFileName(TRN_Exception e)
{
	return ((Common::Exception*)e)->GetFileName();
}
TRN_API_T(const char*) TRN_GetFunction(TRN_Exception e)
{
	return ((Common::Exception*)e)->GetFunction();
}
TRN_API_T(const char*) TRN_GetMessage(TRN_Exception e)
{
	return ((Common::Exception*)e)->GetMessage();
}
TRN_API_T(TRN_UInt32) TRN_GetErrorCode(TRN_Exception e)
{
	return ((Common::Exception*)e)->GetErrorCode();
}

TRN_API_T(const char*) TRN_GetFullMessage(TRN_Exception e)
{
	return ((Common::Exception*)e)->what();
}

TRN_UString ExtractTRNUStringCodegen(const trn::UString& in_ustring)
{
	TRN_UString result;
	trn::UString res(in_ustring);
	result = (TRN_UString)res.mp_impl; res.mp_impl = 0;
	return result;
}


