#include <C/FDF/TRN_FDFDoc.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>
#include <FDF/FDFDoc.hpp>
#include <SDF/License.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace FDF;

TRN_API TRN_FDFDocCreate(TRN_FDFDoc* result)
{
	BEX;
	*result=(TRN_FDFDoc)IMPL_TRN_FDFDocCreateDefault();
	EEX;
}

TRN_API TRN_FDFDocCreateFromSDFDoc(TRN_SDFDoc sdfdoc, TRN_FDFDoc* result)
{
	BEX;
	((SDFDoc*)sdfdoc)->AddReference();
	*result=(TRN_FDFDoc)sdfdoc;
	EEX;
}

TRN_API TRN_FDFDocCreateFromFilePath (const char* filepath, TRN_FDFDoc* result)
{
	BEX;
	*result=(TRN_FDFDoc)IMPL_TRN_FDFDocCreateFromFilePath(filepath);
	EEX;
}

TRN_API TRN_FDFDocCreateFromUFilePath (const TRN_UString filepath, TRN_FDFDoc* result)
{
	BEX;
	UString temp(UStrCast(filepath));
	*result=(TRN_FDFDoc)IMPL_TRN_FDFDocCreateFromUFilePath(temp);
	EEX;
}

TRN_API TRN_FDFDocCreateFromStream (TRN_Filter stream, TRN_FDFDoc* result)
{
	BEX;
	AutoPtr<Filters::Filter> temp((Filters::Filter*)stream);
	*result=(TRN_FDFDoc)IMPL_TRN_FDFDocCreateFromStream(temp);
	EEX;
}

TRN_API TRN_FDFDocCreateFromMemoryBuffer (const char* buf, TRN_Size buf_size, TRN_FDFDoc* result)
{
	BEX;
	*result=(TRN_FDFDoc)IMPL_TRN_FDFDocCreateFromBuffer(buf,buf_size);
	EEX;
}

TRN_API TRN_FDFDocDestroy(TRN_FDFDoc doc)
{
	BEX;
	IMPL_TRN_FDFDocDestroy(FDocCast(doc));
	EEX;
}

TRN_API TRN_FDFDocIsModified(const TRN_FDFDoc doc, TRN_Bool* result) 
{
	BEX;
	*result=BToTB(IMPL_TRN_FDFDocIsModified(FDocCast(doc)));
	EEX;
}

TRN_API TRN_FDFDocSave(TRN_FDFDoc doc, const TRN_UString path)
{
	BEX;
	UString temp(UStrCast(path));
	IMPL_TRN_FDFDocSave(FDocCast(doc),temp);
	EEX;
}

TRN_API TRN_FDFDocSaveMemoryBuffer(TRN_FDFDoc doc, const char** out_buf, TRN_Size* out_buf_size)
{
	BEX;
	IMPL_TRN_FDFDocSave(FDocCast(doc),*out_buf,*out_buf_size);
	EEX;
}

TRN_API TRN_FDFDocGetTrailer (TRN_FDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_FDFDocGetTrailer(FDocCast(doc));
	EEX;
}

TRN_API TRN_FDFDocGetRoot(TRN_FDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_FDFDocGetRoot(FDocCast(doc));
	EEX;
}

TRN_API TRN_FDFDocGetFDF(TRN_FDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_FDFDocGetFDF(FDocCast(doc));
	EEX;
}

TRN_API TRN_FDFDocGetPDFFileName(TRN_FDFDoc doc, TRN_UString* result)
{
	BEX;
	RetUStr(IMPL_TRN_FDFDocGetPdfFileName(FDocCast(doc)));
	EEX;
}

TRN_API TRN_FDFDocSetPDFFileName(TRN_FDFDoc doc, const TRN_UString filepath)
{
	BEX;
	UString temp(UStrCast(filepath));
	IMPL_TRN_FDFDocSetPdfFileName(FDocCast(doc),temp);
	EEX;
}

TRN_API TRN_FDFDocGetID(TRN_FDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_FDFDocGetID(FDocCast(doc));
	EEX;
}

TRN_API TRN_FDFDocSetID(TRN_FDFDoc doc, TRN_Obj id)
{
	BEX;
	IMPL_TRN_FDFDocSetID(FDocCast(doc),ObjCast(id));
	EEX;
}

TRN_API TRN_FDFDocGetFieldIteratorBegin(TRN_FDFDoc doc, TRN_Iterator* result)
{
	BEX;
	FDFFieldIterator itr=IMPL_TRN_FDFDocFieldBegin(FDocCast(doc)), end=IMPL_TRN_FDFDocFieldEnd(FDocCast(doc));
	*result=(TRN_Iterator)new PolyFDFFieldIterator(itr,end);
	EEX;
}

TRN_API TRN_FDFDocGetFieldIterator(TRN_FDFDoc doc, const TRN_UString field_name, TRN_Iterator* result)
{
	BEX;
	UString temp(UStrCast(field_name));
	FDFFieldIterator itr=IMPL_TRN_FDFDocFieldFind(FDocCast(doc), temp), end=IMPL_TRN_FDFDocFieldEnd(FDocCast(doc));
	*result=(TRN_Iterator)new PolyFDFFieldIterator(itr,end);
	EEX;
}

TRN_API TRN_FDFDocGetField(TRN_FDFDoc doc, const TRN_UString field_name, TRN_FDFField* result)
{
	BEX;
	UString temp(UStrCast(field_name));
	FDFFieldIterator itr=IMPL_TRN_FDFDocFieldFind(FDocCast(doc),temp), end=IMPL_TRN_FDFDocFieldEnd(FDocCast(doc));
	*result=RetFField(*itr);
	EEX;
}

TRN_API TRN_FDFDocFieldCreate(TRN_FDFDoc doc, const TRN_UString field_name, enum TRN_FieldType type, TRN_Obj field_value, TRN_FDFField* result)
{
	BEX;
	UString temp(UStrCast(field_name));
    FDFField field = IMPL_TRN_FDFDocFieldCreate(FDocCast(doc),temp,(PDF::Field::Type)type,ObjCast(field_value));
	*result=RetFField(field);
    EEX;
}

TRN_API TRN_FDFDocFieldCreateFromString(TRN_FDFDoc doc, const TRN_UString field_name, enum TRN_FieldType type, TRN_UString field_value, TRN_FDFField* result)
{
	BEX;
	UString temp(UStrCast(field_name)), temp2(field_value);
    FDFField field = IMPL_TRN_FDFDocFieldCreate(FDocCast(doc),temp,(PDF::Field::Type)type,temp2);
	*result=RetFField(field);
	EEX;
}

TRN_API TRN_FDFDocGetSDFDoc (TRN_FDFDoc doc, TRN_SDFDoc* result)
{
	BEX;
	*result=(TRN_SDFDoc)doc;
	EEX;
}

TRN_API TRN_FDFDocCreateFromXFDF(const TRN_UString file_name, TRN_FDFDoc* result)
{
	BEX;
	UString temp(UStrCast(file_name));
	RefCountedPtr<SDFDoc> doc(IMPL_TRN_FDFDocCreateFromXFDF(temp));
	*result=(TRN_FDFDoc)doc.release();
	EEX;
}

TRN_API TRN_FDFDocSaveAsXFDF(TRN_FDFDoc doc, const TRN_UString file_name)
{
	BEX;
	UString temp(UStrCast(file_name));
	IMPL_TRN_FDFDocSaveAsXFDF(FDocCast(doc), temp);
	EEX;
}

TRN_API TRN_FDFDocSaveAsXFDFAsString(TRN_FDFDoc doc, TRN_UString* result)
{
	BEX;
	std::string utf8_xfdf = IMPL_TRN_FDFDocSaveAsXFDF(FDocCast(doc));
	UString temp(utf8_xfdf.c_str(), utf8_xfdf.size(), UString::e_utf8);
	RetUStr(temp);
	EEX;
}

TRN_API TRN_FDFDocMergeAnnots(TRN_FDFDoc doc, const TRN_UString command_file, const TRN_UString permitted_user)
{
	BEX;
	UString temp(UStrCast(command_file)), temp2(UStrCast(permitted_user));
	IMPL_TRN_FDFDocMergeAnnots(FDocCast(doc), temp, temp2);
	EEX;
}
