#include <C/Filters/TRN_FilterReader.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <Filters/Filter.h>
#include <Filters/FilterReader.h>
#include <Common/UString.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace Filters;

TRN_API TRN_FilterReaderCreate(TRN_Filter filter, TRN_FilterReader* result)
{
	BEX;
	if (filter) {
		*result=(TRN_FilterReader)new FilterReader(*((Filter*)filter));
	}
	else {
		*result=(TRN_FilterReader)new FilterReader();
	}

	EEX;
}

TRN_API TRN_FilterReaderDestroy (TRN_FilterReader reader)
{
	BEX;
	delete (FilterReader*)reader;
	EEX;
}

TRN_API TRN_FilterReaderAttachFilter(TRN_FilterReader reader, TRN_Filter filter)
{
	BEX;
	((FilterReader*)reader)->AttachFilter(*((Filter*)filter));
	EEX;
}

TRN_API TRN_FilterReaderGetAttachedFilter(TRN_FilterReader reader, TRN_Filter* result)
{
	BEX;
	*result=(TRN_Filter)((FilterReader*)reader)->GetAttachedFilter();
	EEX;
}

TRN_API TRN_FilterReaderSeek(TRN_FilterReader reader, TRN_Ptrdiff offset, enum TRN_FilterReferencePos origin)
{
	BEX;
	((FilterReader*)reader)->Seek(offset,(Filter::ReferencePos) origin);
	EEX;
}

TRN_API TRN_FilterReaderTell (TRN_FilterReader reader, TRN_Ptrdiff* result)
{
	BEX;
	UInt64 result_inner = ((FilterReader*)reader)->Tell();
	*result = (TRN_Size)Filter::VerifyMemSized(result_inner);
	EEX;
}

TRN_API TRN_FilterReaderCount (TRN_FilterReader reader, TRN_Size* result)
{
	BEX;
	UInt64 result_inner = ((FilterReader*)reader)->Count();
	*result = (TRN_Size)Filter::VerifyMemSized(result_inner);
	EEX;
}

TRN_API TRN_FilterReaderFlush (TRN_FilterReader reader)
{
	BEX;
	((FilterReader*)reader)->Flush();
	EEX;
}

TRN_API TRN_FilterReaderFlushAll (TRN_FilterReader reader)
{
	BEX;
	((FilterReader*)reader)->FlushAll();
	EEX;
}

TRN_API TRN_FilterReaderGet(TRN_FilterReader reader, int* result)
{
	BEX;
	*result=((FilterReader*)reader)->Get();
	EEX;
}

TRN_API TRN_FilterReaderPeek(TRN_FilterReader reader, int* result)
{
	BEX;
	*result=((FilterReader*)reader)->Peek();
	EEX;
}

TRN_API TRN_FilterReaderRead(TRN_FilterReader reader, TRN_UChar* buf, TRN_Size buf_size, TRN_Size* result)
{
	BEX;
	*result=((FilterReader*)reader)->Read(buf,buf_size);
	EEX;
}
