#include <C/PDF/TRN_Convert.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <Convert/Convert.h>
#include <PDF/Page.h>
#include <SDF/License.hpp>
#include <OOXML/Options/WordToPDFOptions.hpp>
#include <Filters/FilterReader.h>


#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_ConvertCreateBlackBoxContext(TRN_UString in_filename, TRN_BlackBoxContext *result)
{
	BEX;
	UString temp(in_filename);
	*result=(TRN_BlackBoxContext)(Convert::CreateBlackBoxContext(temp).release());
	EEX;
}
TRN_API TRN_ConvertFromXps(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename)
{
	BEX;
	// this is done inside of FromXps
	// License::Instance().VerifyPermission(License::e_perm_xps2pdf);

	UString temp(in_filename);
	Convert::FromXps(PDFDocCast(in_pdfdoc), temp,false);
	EEX;
}

TRN_API TRN_ConvertFromXpsMem (TRN_PDFDoc in_pdfdoc, const char* buf, size_t buf_sz )
{
	BEX;
	Convert::FromXps(PDFDocCast(in_pdfdoc),buf,buf_sz,false);
	EEX;
}

// EMF conversions are only supported on Windows
TRN_API TRN_ConvertFromEmf(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename)
{
	BEX;
	UString temp(in_filename);
	Convert::FromEmf(*((PDFDoc*)(&in_pdfdoc)), temp );		
	EEX;
}

// Text conversions
TRN_API TRN_ConvertFromText(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename, TRN_Obj options)
{
	BEX;
	UString temp(in_filename);
	Convert::FromText(*((PDFDoc*)(&in_pdfdoc)), temp, ObjCast(options), false);
	EEX;
}

TRN_API TRN_ConvertPageToEmf(TRN_Page in_page, TRN_UString in_filename)
{
	BEX;
	Page pg(PageCast(in_page));
	UString temp(in_filename);
	Convert::ToEmf(pg, temp );	
	EEX;
}

TRN_API TRN_ConvertDocToEmf(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename)
{
	BEX;
	UString temp(in_filename);
	Convert::ToEmf(*((PDFDoc*)(&in_pdfdoc)), temp );		
	EEX;
}

TRN_API TRN_ConvertPageToSvg(TRN_Page in_page, TRN_UString in_filename)
{
	BEX;
	//License::Instance().VerifyPermission(License::e_perm_pdf2svg);

	Page pg(PageCast(in_page));
	UString temp(in_filename);
	Convert::ToSvg(pg, temp);
	EEX;
}

TRN_API TRN_ConvertPageToSvgWithOptions(TRN_Page in_page, TRN_UString in_filename, TRN_Obj options)
{
	BEX;
	//License::Instance().VerifyPermission(License::e_perm_pdf2svg);

	Page pg(PageCast(in_page));
	UString temp(in_filename);
	Convert::ToSvg(pg, temp, ObjCast(options), false);	
	EEX;
}

TRN_API TRN_ConvertDocToSvg(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename)
{
	BEX;
	//License::Instance().VerifyPermission(License::e_perm_pdf2svg);

	UString temp(in_filename);
	Convert::ToSvg(*((PDFDoc*)(&in_pdfdoc)), temp);
	EEX;
}

TRN_API TRN_ConvertDocToSvgWithOptions(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename, TRN_Obj options)
{
	BEX;
	//License::Instance().VerifyPermission(License::e_perm_pdf2svg);

	UString temp(in_filename);
	Convert::ToSvg(*((PDFDoc*)(&in_pdfdoc)), temp, ObjCast(options), false);	
	EEX;
}

TRN_API TRN_ConvertToXps(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename, TRN_Obj options)
{
	BEX;
	// checked internally: License::Instance().VerifyPermission(License::e_perm_pdf2xps);

	UString temp(in_filename);
	Convert::ToXps(*((PDFDoc*)(&in_pdfdoc)), temp, ObjCast(options), false); // false = verify licenses
	EEX;
}

TRN_API TRN_ConvertFileToXps(TRN_UString in_inputFilename, TRN_UString in_outputFilename, TRN_Obj options)
{
	BEX;
	// checked internally: License::Instance().VerifyPermission(License::e_perm_pdf2xps);
	// checked internally: License::Instance().VerifyPermission(License::e_perm_printdrv);

	UString temp(in_inputFilename);
	UString temp2(in_outputFilename);
	Convert::ToXps(temp, temp2, ObjCast(options), false); // false = verify licenses
	EEX;
}

TRN_API TRN_ConvertFileToXod (TRN_UString in_inputFilename, TRN_UString in_outputFilename, TRN_Obj options)
{
	BEX;
	UString tmp(in_inputFilename);
	UString tmp2(in_outputFilename);
	Convert::ToSilverlight(tmp, tmp2, ObjCast(options), false);
	EEX;
}

TRN_API TRN_ConvertToXod (TRN_PDFDoc in_pdfdoc, TRN_UString in_outputFilename, TRN_Obj options)
{
	BEX;
	UString tmp(in_outputFilename);
	Convert::ToSilverlight(*((PDFDoc*)(&in_pdfdoc)), tmp, ObjCast(options), false);
	EEX;
}

TRN_API TRN_ConvertFileToXodStream (TRN_UString in_filename, TRN_Obj options, TRN_Filter* result)
{
	BEX;
	UString tmp(in_filename);
	*result=(TRN_Filter)(Convert::ToSilverlight(tmp, ObjCast(options), false).release());
	EEX;
}
TRN_API TRN_ConvertToXodStream (TRN_PDFDoc in_pdfdoc, TRN_Obj options, TRN_Filter* result)
{
	BEX;
	*result=(TRN_Filter)(Convert::ToSilverlight(*((PDFDoc*)(&in_pdfdoc)), ObjCast(options), false).release());
	EEX;
}

TRN_API TRN_ConvertToXodWithMonitor(TRN_PDFDoc in_pdfdoc, TRN_Obj options, TRN_ConversionMonitor* result)
{
	BEX;
	*result = (TRN_ConversionMonitor)(Convert::ToXodWithMonitor(*((PDFDoc*)(&in_pdfdoc)), ObjCast(options), false).release());
	EEX;
}

TRN_API TRN_ConversionMonitorNext(TRN_ConversionMonitor conversionMonitor, TRN_Bool* result) 
{
	BEX;
	*result = ((ConversionMonitor*)conversionMonitor)->Next();
	EEX;
}
TRN_API TRN_ConversionMonitorReady(TRN_ConversionMonitor conversionMonitor, TRN_Bool* result)
{
	BEX;
	*result = ((ConversionMonitor*)conversionMonitor)->Ready();
	EEX;
}
TRN_API TRN_ConversionMonitorProgress(TRN_ConversionMonitor conversionMonitor, TRN_UInt32* result)
{
	BEX;
	*result = ((ConversionMonitor*)conversionMonitor)->Progress();
	EEX;
}
TRN_API TRN_ConversionMonitorFilter(TRN_ConversionMonitor conversionMonitor, TRN_Filter* result)
{
	BEX;
	*result = (TRN_Filter)(((ConversionMonitor*)conversionMonitor)->Filter().release());
	EEX;
}
TRN_API TRN_ConversionMonitorDestroy(TRN_ConversionMonitor conversionMonitor)
{
	BEX;
	delete (ConversionMonitor*)conversionMonitor;
	EEX;
}

TRN_API TRN_ConvertWordToPdf(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename, TRN_Obj options)
{
	BEX;
	UString temp(in_filename);
	Convert::WordToPDF(*((PDFDoc*)(&in_pdfdoc)), temp, ObjCast(options));
	EEX;
}

TRN_API TRN_ConvertWordToPdfConversion(TRN_PDFDoc* in_pdfdoc, TRN_UString in_filename, TRN_Obj options, TRN_DocumentConversion* result)
{
	BEX;
	UString temp(in_filename);
	// in_pdfdoc is a pointer to a pointer to an sdf doc
	// a trn::PDFDoc is a pointer to an sdf doc, so dereference once to get a reference
	iDocumentConversion* ret = Convert::WordToPDFConversion((PDFDoc&)(*in_pdfdoc), temp, ObjCast(options)).release();
	*result = (TRN_DocumentConversion)ret;
	EEX;
}

TRN_API TRN_ConvertWordToPdfWithFilter(TRN_PDFDoc in_pdfdoc, TRN_Filter in_data, TRN_Obj options)
{
	BEX;
	Convert::WordToPDF(*((PDFDoc*)(&in_pdfdoc)), AutoPtr<Filters::Filter>((Filters::Filter*)in_data), ObjCast(options));
	EEX;
}

TRN_API TRN_ConvertWordToPdfConversionWithFilter(TRN_PDFDoc* in_pdfdoc, TRN_Filter in_data, TRN_Obj options, TRN_DocumentConversion* result)
{
	BEX;
	// in_pdfdoc is a pointer to a pointer to an sdf doc
	// a trn::PDFDoc is a pointer to an sdf doc, so dereference once to get a reference
	iDocumentConversion* ret = Convert::WordToPDFConversion((PDFDoc&)(*in_pdfdoc), AutoPtr<Filters::Filter>((Filters::Filter*)in_data), ObjCast(options)).release();
	*result = (TRN_DocumentConversion)ret;
	EEX;
}

TRN_API TRN_ConvertOfficeToPdfWithPath(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename, TRN_Obj options)
{
	BEX;
	UString temp(in_filename);
	Convert::OfficeToPDF(*((PDFDoc*)(&in_pdfdoc)), temp, ObjCast(options));
	EEX;
}

TRN_API TRN_ConvertOfficeToPdfWithFilter(TRN_PDFDoc in_pdfdoc, TRN_Filter in_data, TRN_Obj options)
{
	BEX;
	Convert::OfficeToPDF(*((PDFDoc*)(&in_pdfdoc)), AutoPtr<Filters::Filter>((Filters::Filter*)in_data), ObjCast(options));
	EEX;
}

TRN_API TRN_ConvertStreamingPdfConversionWithPdfAndPath(TRN_PDFDoc* in_pdfdoc, TRN_UString in_filename, TRN_Obj options, TRN_DocumentConversion* result)
{
	BEX;
	UString temp(in_filename);
	// in_pdfdoc is a pointer to a pointer to an sdf doc
	// a trn::PDFDoc is a pointer to an sdf doc, so dereference once to get a reference
	iDocumentConversion* ret = Convert::UniversalConversionWithDoc((PDFDoc&)(*in_pdfdoc), temp, ObjCast(options)).release();
	*result = (TRN_DocumentConversion)ret;
	EEX;
}

TRN_API TRN_ConvertStreamingPdfConversionWithPdfAndFilter(TRN_PDFDoc* in_pdfdoc, TRN_Filter in_data, TRN_Obj options, TRN_DocumentConversion* result)
{
	BEX;
	// in_pdfdoc is a pointer to a pointer to an sdf doc
	// a trn::PDFDoc is a pointer to an sdf doc, so dereference once to get a reference
	iDocumentConversion* ret = Convert::UniversalConversionWithDoc((PDFDoc&)(*in_pdfdoc), AutoPtr<Filters::Filter>((Filters::Filter*)in_data), ObjCast(options)).release();
	*result = (TRN_DocumentConversion)ret;
	EEX;
}

TRN_API TRN_ConvertStreamingPdfConversionWithPath(TRN_UString in_filename, TRN_Obj options, TRN_DocumentConversion* result)
{
	BEX;
	UString temp(in_filename);
	// in_pdfdoc is a pointer to a pointer to an sdf doc
	// a trn::PDFDoc is a pointer to an sdf doc, so dereference once to get a reference
	iDocumentConversion* ret = Convert::UniversalConversion(temp, ObjCast(options)).release();
	*result = (TRN_DocumentConversion)ret;
	EEX;
}

TRN_API TRN_ConvertStreamingPdfConversionWithFilter(TRN_Filter in_data, TRN_Obj options, TRN_DocumentConversion* result)
{
	BEX;
	// in_pdfdoc is a pointer to a pointer to an sdf doc
	// a trn::PDFDoc is a pointer to an sdf doc, so dereference once to get a reference
	iDocumentConversion* ret = Convert::UniversalConversion(AutoPtr<Filters::Filter>((Filters::Filter*)in_data), ObjCast(options)).release();
	*result = (TRN_DocumentConversion)ret;
	EEX;
}

TRN_API TRN_ConvertToPdf(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename)
{
	BEX;
	// checked internally: License::Instance().VerifyPermission(License::e_perm_printdrv);
	// checked internally: License::Instance().VerifyPermission(License::e_perm_xps2pdf);

	UString temp(in_filename);
	Convert::ToPdf(*((PDFDoc*)(&in_pdfdoc)), temp, false);	// false = verify licenses
	EEX;
}

TRN_API TRN_ConvertFromCAD(TRN_PDFDoc in_pdfdoc, TRN_UString in_filename, TRN_Obj options)
{
	BEX;
	UString temp(in_filename);
	Convert::FromCAD(*((PDFDoc*)(&in_pdfdoc)), temp, ObjCast(options), false);	// false = verify licenses
	EEX;
}

TRN_API TRN_ConvertFromTiff(TRN_PDFDoc in_pdfdoc, TRN_Filter in_data)
{
	BEX;
	Convert::FromTiff(*((PDFDoc*)(&in_pdfdoc)), AutoPtr<Filters::Filter>((Filters::Filter*)in_data), false);	// false = verify licenses
	EEX;
}

TRN_API TRN_ConvertRequiresPrinter(TRN_UString in_filename, TRN_Bool *result)
{
	BEX;
	UString temp(in_filename);
	*result = BToTB(Convert::RequiresPrinter(temp));	
	EEX;
}


TRN_API TRN_ConvertPrinterInstall(TRN_UString in_printerName)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_printdrv);

	UString temp(in_printerName);
	Convert::Printer::Install(temp);	
	EEX;
}

TRN_API TRN_ConvertPrinterUninstall()
{
	BEX;
	Convert::Printer::Uninstall();	
	EEX;
}

TRN_API TRN_ConvertPrinterGetPrinterName(TRN_UString *result)
{
	BEX;
	RetUStr(Convert::Printer::GetPrinterName());
	EEX;
}

TRN_API TRN_ConvertPrinterIsInstalled(TRN_UString in_printerName, TRN_Bool *result)
{
	BEX;
	UString temp(in_printerName);
	*result = BToTB(Convert::Printer::IsInstalled(temp));	
	EEX;
}


TRN_API TRN_ConvertPrinterSetPrinterName(TRN_UString in_printerName)
{
	BEX;
	UString temp(in_printerName);
	Convert::Printer::SetPrinterName(temp);	
	EEX;
}

TRN_API TRN_ConvertPrinterSetMode(enum TRN_ConvertPrinterMode mode)
{
	BEX;
	Convert::Printer::SetMode((Convert::Printer::Mode)mode);
	EEX;
}

TRN_API TRN_ConvertPrinterGetMode(enum TRN_ConvertPrinterMode *result)
{
	BEX;
	*result = (enum TRN_ConvertPrinterMode)Convert::Printer::GetMode();
	EEX;
}

TRN_API TRN_ConvertFileToHtml( TRN_UString in_inputFilename, TRN_UString in_outputFilename, TRN_Obj options )
{
	BEX;
	UString tmp(in_inputFilename);
	UString tmp2(in_outputFilename);
	Convert::ToHtml(tmp, tmp2, ObjCast(options), false);
	EEX;
}

TRN_API TRN_ConvertToHtml( TRN_PDFDoc in_pdfdoc, TRN_UString in_outputFilename, TRN_Obj options )
{
	BEX;
	UString tmp(in_outputFilename);
	Convert::ToHtml(*((PDFDoc*)(&in_pdfdoc)), tmp, ObjCast(options), false);
	EEX;
}

TRN_API TRN_ConvertFileToEpub( TRN_UString in_inputFilename, TRN_UString in_outputFilename, TRN_Obj html_options, TRN_Obj epub_options )
{
	BEX;
	UString tmp(in_inputFilename);
	UString tmp2(in_outputFilename);
	Convert::ToEpub(tmp, tmp2, ObjCast(html_options), ObjCast(epub_options), false);
	EEX;
}

TRN_API TRN_ConvertToEpub( TRN_PDFDoc in_pdfdoc, TRN_UString in_outputFilename, TRN_Obj html_options, TRN_Obj epub_options )
{
	BEX;
	UString tmp(in_outputFilename);
	Convert::ToEpub(*((PDFDoc*)(&in_pdfdoc)), tmp, ObjCast(html_options), ObjCast(epub_options), false);
	EEX;
}

TRN_API TRN_ConvertFileToTiff( TRN_UString in_inputFilename, TRN_UString in_outputFilename, TRN_Obj options )
{
	BEX;
	UString tmp(in_inputFilename);
	UString tmp2(in_outputFilename);
	Convert::ToTiff(tmp, tmp2, ObjCast(options), false);
	EEX;
}

TRN_API TRN_ConvertToTiff( TRN_PDFDoc in_pdfdoc, TRN_UString in_outputFilename, TRN_Obj options )
{
	BEX;
	UString tmp(in_outputFilename);
	Convert::ToTiff(*((PDFDoc*)(&in_pdfdoc)), tmp, ObjCast(options), false);
	EEX;
}
