#include <C/PDF/TRN_ElementBuilder.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/ElementBuilder.h>
#include <PDF/PDFDoc.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;


TRN_API TRN_ElementBuilderCreate(TRN_ElementBuilder* result)
{
	BEX;
	*result=(TRN_ElementBuilder)new ElementBuilder();
	EEX;
}


TRN_API TRN_ElementBuilderDestroy(TRN_ElementBuilder b)
{
	BEX;
	delete (ElementBuilder*)b;
	EEX;
}



TRN_API TRN_ElementBuilderReset(TRN_ElementBuilder b, TRN_GState gs)
{
	BEX;
	EBCast(b).Reset();
	EEX;
}


TRN_API TRN_ElementBuilderCreateImage(TRN_ElementBuilder b, TRN_Image img,TRN_Element* result)
{
	BEX;
	Image temp(ImgCast(img));
	*result=(TRN_Element)EBCast(b).CreateImage(temp);
	EEX;
}

TRN_API TRN_ElementBuilderCreateImageFromMatrix(TRN_ElementBuilder b, TRN_Image img, const TRN_Matrix2D* mtx, TRN_Element* result)
{
	BEX;
	Image temp(ImgCast(img));
	*result=(TRN_Element)EBCast(b).CreateImage(temp,M2DCast(mtx));
	EEX;
}

TRN_API TRN_ElementBuilderCreateImageScaled(TRN_ElementBuilder b, TRN_Image img, double x, double y, double hscale, double vscale, TRN_Element* result)
{
	BEX;
	Image temp(ImgCast(img));
	*result=(TRN_Element)EBCast(b).CreateImage(temp,x,y,hscale,vscale);
	EEX;
}

TRN_API TRN_ElementBuilderCreateGroupBegin(TRN_ElementBuilder b, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateGroupBegin();
	EEX;
}

TRN_API TRN_ElementBuilderCreateGroupEnd(TRN_ElementBuilder b, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateGroupEnd();
	EEX;
}

TRN_API TRN_ElementBuilderCreateShading(TRN_ElementBuilder b, TRN_Shading sh, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateShading(ShadeCast(sh));
	EEX;
}

TRN_API TRN_ElementBuilderCreateFormFromStream(TRN_ElementBuilder b, TRN_Obj form, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateForm(ObjCast(form));
	EEX;
}

TRN_API TRN_ElementBuilderCreateFormFromPage(TRN_ElementBuilder b, TRN_Page page, TRN_Element* result)
{
	BEX;
	Page temp(PageCast(page));
	*result=(TRN_Element)EBCast(b).CreateForm(temp);
	EEX;
}

TRN_API TRN_ElementBuilderCreateFormFromDoc(TRN_ElementBuilder b, TRN_Page page, TRN_PDFDoc doc, TRN_Element* result)
{
	BEX;
	PDFDoc* d=(PDFDoc*)&doc;
	*result=(TRN_Element)EBCast(b).CreateForm(Page(PageCast(page)),*d);
	EEX;
}

TRN_API TRN_ElementBuilderCreateTextBeginWithFont(TRN_ElementBuilder b, TRN_Font font, double font_sz, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateTextBegin(FontCast(font), font_sz);
	EEX;
}

TRN_API TRN_ElementBuilderCreateTextBegin(TRN_ElementBuilder b, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateTextBegin();
	EEX;
}

TRN_API TRN_ElementBuilderCreateTextEnd(TRN_ElementBuilder b, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateTextEnd();
	EEX;
}

TRN_API TRN_ElementBuilderCreateTextRun(TRN_ElementBuilder b, const char* text_data, TRN_Font font, double font_sz, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateTextRun(text_data,FontCast(font),font_sz);
	EEX;
}

TRN_API TRN_ElementBuilderCreateTextRunWithSize(TRN_ElementBuilder b, const char* text_data, TRN_UInt32 text_data_sz, TRN_Font font, double font_sz, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateTextRun(text_data,text_data_sz,FontCast(font),font_sz);
	EEX;
}

TRN_API TRN_ElementBuilderCreateTextRunUnsigned(TRN_ElementBuilder b, TRN_String text_data, TRN_Font font, double font_sz, TRN_Element* result)
{
	BEX;
	*result = (TRN_Element)EBCast(b).CreateTextRun((const char*)text_data.str, (unsigned int)text_data.len);
	EEX;
}

TRN_API TRN_ElementBuilderCreateNewTextRun(TRN_ElementBuilder b, const char* text_data, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateTextRun(text_data);
	EEX;
}

TRN_API TRN_ElementBuilderCreateNewTextRunWithSize(TRN_ElementBuilder b, const char* text_data, TRN_UInt32 text_data_sz, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateTextRun(text_data,text_data_sz);
	EEX;
}

TRN_API TRN_ElementBuilderCreateNewTextRunUnsigned(TRN_ElementBuilder b, TRN_String text_data, TRN_Element* result)
{
	BEX;
	*result = (TRN_Element)EBCast(b).CreateTextRun(text_data.str, (unsigned int)text_data.len);
	EEX;
}

TRN_API TRN_ElementBuilderCreateUnicodeTextRun(TRN_ElementBuilder b, const TRN_Unicode* text_data, TRN_UInt32 text_data_sz, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateUnicodeTextRun(text_data,text_data_sz);
	EEX;
}

TRN_API TRN_ElementBuilderCreateTextNewLineWithOffset(TRN_ElementBuilder b, double dx, double dy, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateTextNewLine(dx,dy);
	EEX;
}

TRN_API TRN_ElementBuilderCreateTextNewLine(TRN_ElementBuilder b, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateTextNewLine();
	EEX;
}

TRN_API TRN_ElementBuilderCreatePath(TRN_ElementBuilder b, const double* buf_points, int buf_points_size, const char* buf_seg_types, int buf_seg_types_size, TRN_Element* result)
{
	BEX;
	*result = (TRN_Element)EBCast(b).CreatePath(buf_points, buf_points_size, buf_seg_types, buf_seg_types_size);
	EEX;
}

TRN_API TRN_ElementBuilderCreateRect(TRN_ElementBuilder b, double x, double y, double width, double height, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateRect(x,y,width,height);
	EEX;
}

TRN_API TRN_ElementBuilderCreateEllipse(TRN_ElementBuilder b, double cx, double cy, double rx, double ry, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).CreateEllipse(cx,cy,rx,ry);
	EEX;
}

TRN_API TRN_ElementBuilderPathBegin(TRN_ElementBuilder b)
{
	BEX;
	EBCast(b).PathBegin();
	EEX;
}

TRN_API TRN_ElementBuilderPathEnd(TRN_ElementBuilder b, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)EBCast(b).PathEnd();
	EEX;
}

TRN_API TRN_ElementBuilderRect(TRN_ElementBuilder b, double x, double y, double width, double height)
{
	BEX;
	EBCast(b).Rect(x,y,width,height);
	EEX;
}

TRN_API TRN_ElementBuilderEllipse(TRN_ElementBuilder b, double cx, double cy, double rx, double ry)
{
	BEX;
	EBCast(b).Ellipse(cx, cy, rx, ry);
	EEX;
}

TRN_API TRN_ElementBuilderMoveTo(TRN_ElementBuilder b, double x, double y)
{
	BEX;
	EBCast(b).MoveTo(x,y);
	EEX;
}

TRN_API TRN_ElementBuilderLineTo(TRN_ElementBuilder b, double x, double y)
{
	BEX;
	EBCast(b).LineTo(x,y);
	EEX;
}

TRN_API TRN_ElementBuilderCurveTo(TRN_ElementBuilder b, double cx1, double cy1, double cx2, double cy2, double x2, double y2)
{
	BEX;
	EBCast(b).CurveTo(cx1,cy1,cx2,cy2,x2,y2);
	EEX;
}

TRN_API TRN_ElementBuilderArcTo(TRN_ElementBuilder b, double x, double y, double width, double height, double start, double extent)
{
	BEX;
	EBCast(b).ArcTo(x, y, width, height, start, extent);
	EEX;
}

TRN_API TRN_ElementBuilderArcTo2(TRN_ElementBuilder b, double xr, double yr, double rx, TRN_Bool isLargeArc, TRN_Bool sweep, double endX, double endY)
{
	BEX;
	EBCast(b).ArcTo(xr, yr, rx, TBToB(isLargeArc), TBToB(sweep), endX, endY);
	EEX;
}

TRN_API TRN_ElementBuilderClosePath(TRN_ElementBuilder b)
{
	BEX;
	EBCast(b).ClosePath();
	EEX;
}
