#include <C/PDF/TRN_ElementReader.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <PDF/ElementReader.h>
#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;


TRN_API TRN_ElementReaderCreate(TRN_ElementReader* result)
{
	BEX;
	*result=(TRN_ElementReader)new ElementReader( true );
	EEX;
}
TRN_API TRN_ElementReaderDestroy(TRN_ElementReader r)
{
	BEX;
	delete (ElementReader*)r;
	EEX;
}

TRN_API TRN_ElementReaderBeginOnPage(TRN_ElementReader r, TRN_Page page, TRN_OCGContext ctx)
{
	BEX;
	Page temp(PageCast(page));
	ERCast(r).Begin(temp, (OCG::Context*)ctx);
	EEX;
}

TRN_API TRN_ElementReaderBegin(TRN_ElementReader r,TRN_Obj content_stream, TRN_Obj resource_dict, TRN_OCGContext ctx)
{
	BEX;
	ERCast(r).Begin((Obj*)content_stream,(Obj*)resource_dict, (OCG::Context*)ctx);
	EEX;
}

TRN_API TRN_ElementReaderAppendResource(TRN_ElementReader r, TRN_Obj res)
{
	BEX;
	ERCast(r).AppendResource((Obj*)res);
	EEX;
}

TRN_API TRN_ElementReaderNext(TRN_ElementReader r, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)ERCast(r).Next();
	EEX;
}

TRN_API TRN_ElementReaderCurrent(TRN_ElementReader r, TRN_Element* result)
{
	BEX;
	*result=(TRN_Element)ERCast(r).Current();
	EEX;
}

TRN_API TRN_ElementReaderFormBegin(TRN_ElementReader r)
{
	BEX;
	ERCast(r).FormBegin();
	EEX;
}

TRN_API TRN_ElementReaderPatternBegin(TRN_ElementReader r, TRN_Bool fill_pattern, TRN_Bool reset_ctm_tfm)
{
	BEX;
	ERCast(r).PatternBegin(ERCast(r).Current()->GetGState(), TBToB(fill_pattern), TBToB(reset_ctm_tfm));
	EEX;
}

TRN_API TRN_ElementReaderType3FontBegin(TRN_ElementReader r, TRN_CharData* char_data, TRN_Obj resource_dict)
{
	BEX;
	trn::PDF::CharData cd(char_data->char_code, char_data->char_data, char_data->bytes, char_data->x, char_data->y);
	ERCast(r).Type3FontBegin(cd,(Obj*)resource_dict);
	EEX;
}

TRN_API TRN_ElementReaderEnd(TRN_ElementReader r, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ERCast(r).End());
	EEX;
}

TRN_API TRN_ElementReaderGetChangesIterator(TRN_ElementReader r, TRN_Iterator* result)
{
	BEX;
	GSChangesIterator itr=ERCast(r).ChangesBegin(), end=ERCast(r).ChangesEnd();
	*result=(TRN_Iterator)new PolyGSChangesIterator(itr,end);
	EEX;
}

TRN_API TRN_ElementReaderIsChanged(TRN_ElementReader r,enum TRN_GStateAttribute attrib, TRN_Bool* result) 
{
	BEX;
	*result=BToTB(ERCast(r).IsChanged((GState::GStateAttribute)attrib));
	EEX;
}

TRN_API TRN_ElementReaderClearChangeList(TRN_ElementReader r)
{
	BEX;
	ERCast(r).ClearChangeList();
	EEX;
}

TRN_API TRN_ElementReaderGetFont (TRN_ElementReader r,const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ERCast(r).GetFont(name);
	EEX;
}

TRN_API TRN_ElementReaderGetXObject (TRN_ElementReader r,const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ERCast(r).GetXObject(name);
	EEX;
}

TRN_API TRN_ElementReaderGetShading(TRN_ElementReader r,const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ERCast(r).GetShading(name);
	EEX;
}

TRN_API TRN_ElementReaderGetColorSpace (TRN_ElementReader r,const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ERCast(r).GetColorSpace(name);
	EEX;
}

TRN_API TRN_ElementReaderGetPattern (TRN_ElementReader r,const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ERCast(r).GetPattern(name);
	EEX;
}

TRN_API TRN_ElementReaderGetExtGState (TRN_ElementReader r,const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ERCast(r).GetExtGState(name);
	EEX;
}

