#include <C/PDF/TRN_ElementWriter.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>
#include <PDF/ElementWriter.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;



TRN_API TRN_ElementWriterCreate(TRN_ElementWriter* result)
{
	BEX;
	*result=(TRN_ElementWriter)new ElementWriter();
	EEX;
}

TRN_API TRN_ElementWriterDestroy(TRN_ElementWriter w)
{
	BEX;
	delete (ElementWriter*)w;
	EEX;
}

TRN_API TRN_ElementWriterBeginOnPage(TRN_ElementWriter w, TRN_Page page, enum TRN_ElementWriterWriteMode placement, TRN_Bool page_coord_sys, TRN_Bool compress, TRN_Obj res)
{
	BEX;
	Page temp(PageCast(page));
	EWCast(w).Begin(temp, static_cast<ElementWriter::WriteMode>(placement), TBToB(page_coord_sys), TBToB(compress), ObjCast(res));
	EEX;
}

TRN_API TRN_ElementWriterBegin(TRN_ElementWriter w, TRN_SDFDoc doc, TRN_Bool compress)
{
	BEX;
	EWCast(w).Begin(DocCast(doc),TBToB(compress));
	EEX;
}

TRN_API TRN_ElementWriterBeginOnObj( TRN_ElementWriter w, TRN_Obj obj, TRN_Bool compress, TRN_Obj res)
{
	BEX;
	EWCast(w).Begin( ObjCast(obj), TBToB(compress), ObjCast(res) );
	EEX;
}

TRN_API TRN_ElementWriterEnd(TRN_ElementWriter w, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)EWCast(w).End();
	EEX;
}

TRN_API TRN_ElementWriterWriteElement(TRN_ElementWriter w, TRN_Element element)
{
	BEX;
	EWCast(w).WriteElement(EleCast(element));
	EEX;
}

TRN_API TRN_ElementWriterWritePlacedElement(TRN_ElementWriter w, TRN_Element element)
{
	BEX;
	EWCast(w).WritePlacedElement(EleCast(element));
	EEX;
}

TRN_API TRN_ElementWriterWriteGStateChanges(TRN_ElementWriter w, TRN_Element element)
{
	BEX;
	EWCast(w).WriteGStateChanges(EleCast(element));
	EEX;
}

TRN_API TRN_ElementWriterFlush(TRN_ElementWriter w)
{
	BEX;
	EWCast(w).Flush();
	EEX;
}

TRN_API TRN_ElementWriterWriteBuffer(TRN_ElementWriter w, const char* data, int data_sz)
{
	BEX;
	EWCast(w).WriteBuffer(data,data_sz);
	EEX;
}

TRN_API TRN_ElementWriterWriteString(TRN_ElementWriter w, const char* str)
{
	BEX;
	EWCast(w).WriteString(str);
	EEX;
}

TRN_API TRN_ElementWriterSetDefaultGState(TRN_ElementWriter w, TRN_ElementReader r)
{
	BEX;
	EWCast(w).SetDefaultGState(ERCast(r));
	EEX;
}
