#include <C/PDF/TRN_Field.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <Common/UString.h>
#include <PDF/ElementBuilderImpl.hpp>
#include <PDF/Field.h>
#include <PDF/Page.h>
#include <PDF/Utility/ViewChangeCollection.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;

TRN_API TRN_FieldCreate (TRN_Obj field_dict, TRN_Field* result)
{
	BEX;
	result->leaf_node=field_dict;
	result->builder=0;
	EEX;
}

TRN_API TRN_FieldAssign (TRN_Field* left, const TRN_Field* right)
{
	BEX;
	FieldCast(left)=FieldCast(right);
	EEX;
}

TRN_API TRN_FieldDestroy(TRN_Field* field)
{
	BEX;
    delete (ElementBuilderImpl*)(field->builder);
    EEX;
}

TRN_API TRN_FieldIsValid(const TRN_Field* field, TRN_Bool* result) 
{
	BEX;
	*result=BToTB(FieldCast(field).IsValid());
	EEX;
}

TRN_API TRN_FieldGetType(const TRN_Field* field, enum TRN_FieldType* result)
{
	BEX;
	*result=(enum TRN_FieldType)FieldCast(field).GetType();
	EEX;
}

TRN_API TRN_FieldGetValue(const TRN_Field* field,TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)FieldCast(field).GetValue();
	EEX;
}

TRN_API TRN_FieldGetValueAsString(const TRN_Field* field, TRN_UString* result)
{
	BEX;
	RetUStr(FieldCast(field).GetValueAsString());
	EEX;
}

TRN_API TRN_FieldGetDefaultValueAsString(const TRN_Field* field, TRN_UString* result)
{
	BEX;
	RetUStr(FieldCast(field).GetDefaultValueAsString());
	EEX;
}

TRN_API TRN_FieldSetValueAsString(TRN_Field* field, TRN_UString value, TRN_ViewChangeCollection* result)
{
	BEX;
	UString temp(value);
	*result = (TRN_ViewChangeCollection) new ViewChangeCollection(FieldCast(field).SetValue(temp));
	EEX;
}

TRN_API TRN_FieldSetValue(TRN_Field* field, TRN_Obj value, TRN_ViewChangeCollection* result)
{
	BEX;
	*result = (TRN_ViewChangeCollection) new ViewChangeCollection (FieldCast(field).SetValue((Obj*)value));
	EEX;
}

TRN_API TRN_FieldSetValueAsBool(TRN_Field* field, TRN_Bool value, TRN_ViewChangeCollection* result)
{
	BEX;
	*result = (TRN_ViewChangeCollection) new ViewChangeCollection(FieldCast(field).SetValue(TBToB(value)));
	EEX;
}

TRN_API TRN_FieldGetTriggerAction(const TRN_Field* field, TRN_FieldEventType trigger, TRN_Obj* result)
{
	BEX;
	*result = (TRN_Obj)FieldCast(field).GetTriggerAction(ActionTriggerEvent(trigger));
	EEX;
}

TRN_API TRN_FieldGetValueAsBool(const TRN_Field* field, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FieldCast(field).GetValueAsBool());
	EEX;
}

TRN_API TRN_FieldSetMaxLen(TRN_Field* field, int max_len)
{
	BEX;
	FieldCast(field).SetMaxLen(max_len);
	EEX;
}

TRN_API TRN_FieldGetMaxLen(const TRN_Field* field, int* result)
{
	BEX;
	*result=FieldCast(field).GetMaxLen();
	EEX;
}

TRN_API TRN_FieldGetOptCount(const TRN_Field* field, int* result)
{
	BEX;
	*result=FieldCast(field).GetOptCount();
	EEX;
}

TRN_API TRN_FieldGetOpt(const TRN_Field* field, int index, TRN_UString* result)
{
	BEX;
	RetUStr(FieldCast(field).GetOpt(index));
	EEX;
}

TRN_API TRN_FieldGetDefaultAppearance(TRN_Field* field, TRN_GState* result)
{
	BEX;
	*result=(TRN_GState)FieldCast(field).GetDefaultAppearance();
	EEX;
}

TRN_API TRN_FieldRefreshAppearance(TRN_Field* field)
{
	BEX;
	FieldCast(field).RefreshAppearance();
	EEX;
}

TRN_API TRN_FieldEraseAppearance(TRN_Field* field)
{
	BEX;
	FieldCast(field).EraseAppearance();
	EEX;
}

TRN_API TRN_FieldGetDefaultValue(const TRN_Field* field, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)FieldCast(field).GetDefaultValue();	
	EEX;
}

TRN_API TRN_FieldGetName(const TRN_Field* field, TRN_UString* result)
{
	BEX;
	RetUStr(FieldCast(field).GetName());
	EEX;
}
TRN_API TRN_FieldGetPartialName(const TRN_Field* field, TRN_UString* result)
{
	BEX;
	RetUStr(FieldCast(field).GetPartialName());
	EEX;
}
TRN_API TRN_FieldRename(TRN_Field* field, const TRN_UString field_name)
{
	BEX;
	FieldCast(field).Rename((UString)field_name);
	EEX;
}

TRN_API TRN_FieldIsAnnot(const TRN_Field* field, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FieldCast(field).IsAnnot());
	EEX;
}

TRN_API TRN_FieldGetFlag(const TRN_Field* field, enum TRN_FieldFlag flag, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FieldCast(field).GetFlag((Field::Flag)flag));
	EEX;
}

TRN_API TRN_FieldSetFlag(TRN_Field* field, enum TRN_FieldFlag flag, TRN_Bool value)
{
	BEX;
	FieldCast(field).SetFlag((Field::Flag)flag,TBToB(value));
	EEX;
}

TRN_API TRN_FieldGetJustification(TRN_Field* field, enum TRN_FieldTextJustification* result)
{
	BEX;
	*result=(enum TRN_FieldTextJustification)FieldCast(field).GetJustification();	
	EEX;
}

TRN_API TRN_FieldSetJustification(TRN_Field* field, enum TRN_FieldTextJustification j)
{
	BEX;
	FieldCast(field).SetJustification((Field::TextJustification) j);	
	EEX;
}

TRN_API TRN_FieldGetUpdateRect(const TRN_Field* field, TRN_Rect* result)
{
    BEX;
    Rect r(FieldCast(field).GetUpdateRect());
    memcpy(result, &r, sizeof(r));
    EEX;
}

TRN_API TRN_FieldFlatten(TRN_Field* field, TRN_Page page)
{
	BEX;
	FieldCast(field).Flatten(Page(PageCast(page)));
	EEX;
}

TRN_API TRN_FieldFindInheritedAttribute (const TRN_Field* field, const char* attrib, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)FieldCast(field).FindInheritedAttribute(attrib);
	EEX;
}

TRN_API TRN_FieldGetSDFObj (const TRN_Field* field, TRN_Obj* result)
{
	BEX;
	*result=field->leaf_node;	
	EEX;
}

TRN_API TRN_FieldUseSignatureHandler(TRN_Field* field, const TRN_SignatureHandlerId signature_handler_id, TRN_Obj* result)
{
    BEX;
    *result = (TRN_Obj) FieldCast(field).UseSignatureHandler((const SignatureHandlerId) signature_handler_id);
    EEX;
}

TRN_API TRN_FieldIsLockedByDigitalSignature(const TRN_Field* field, TRN_Bool* result)
{
	BEX;
	*result = BToTB(FieldCast(field).IsLockedByDigitalSignature());
	EEX;
}
