#include <C/PDF/TRN_FileSpec.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/FileSpec.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_FileSpecCreate(TRN_SDFDoc doc, const TRN_UString path, TRN_Bool embed, TRN_FileSpec* result)
{
	BEX;
	UString temp(UStrCast(path));
	*result=(TRN_FileSpec)IMPL_TRN_FileSpecCreate(DocCast(doc),temp,TBToB(embed));
	EEX;
}

TRN_API TRN_FileSpecCreateURL(TRN_SDFDoc doc, const char* url, TRN_FileSpec* result)
{
	BEX;
	*result=(TRN_FileSpec)IMPL_TRN_FileSpecCreateURL(DocCast(doc),url);
	EEX;
}

TRN_API TRN_FileSpecCreateFromObj(TRN_Obj f, TRN_FileSpec* result)
{
	BEX;
	*result=(TRN_FileSpec)IMPL_TRN_FileSpecCreateFromObj((Obj*)f);
	EEX;
}

TRN_API TRN_FileSpecCopy (const TRN_FileSpec d, TRN_FileSpec* result)
{
	BEX;
	*result=(TRN_FileSpec)d;
	EEX;
}

TRN_API TRN_FileSpecCompare(TRN_FileSpec fs, const TRN_FileSpec d, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_FileSpecCompare(FSCast(fs),FSCast(d)));
	EEX;
}

TRN_API TRN_FileSpecIsValid(const TRN_FileSpec fs, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_FileSpecIsValid(FSCast(fs)));
	EEX;
}

TRN_API TRN_FileSpecExport(const TRN_FileSpec fs, const TRN_UString save_as, TRN_Bool* result)
{
	BEX;
	UString temp(UStrCast(save_as));
	*result=BToTB(IMPL_TRN_FileSpecExport(FSCast(fs),temp));
	EEX;
}

TRN_API TRN_FileSpecGetFileData(const TRN_FileSpec fs, TRN_Filter* result)
{
	BEX;
	*result=(TRN_Filter)IMPL_TRN_FileSpecGetFileData(FSCast(fs)).release();
	EEX;
}

TRN_API TRN_FileSpecGetFilePath(const TRN_FileSpec fs, TRN_UString* result)
{
	BEX;
	RetUStr(IMPL_TRN_FileSpecGetFilePath(FSCast(fs)));
	EEX;
}

TRN_API TRN_FileSpecSetDesc(const TRN_FileSpec fs, const TRN_UString desc)
{
	BEX;
	FileSpec spec(FSCast(fs));
	UString temp(UStrCast(desc));
	spec.SetDesc(temp);
	EEX;
}

TRN_API TRN_FileSpecGetSDFObj (const TRN_FileSpec fs, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)fs;
	EEX;
}

