#include <C/PDF/TRN_Image.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/Image.h>
#include <Filters/BufFilter.h>
#include <Filters/ExtBufInputFilter.hpp>
#include <Filters/ChainedFilter.hpp>
#include <Filters/MemoryFilter.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace Filters;
using namespace SDF;
using namespace PDF;


TRN_API TRN_ImageCreateFromFile(TRN_SDFDoc doc, const TRN_UString filename, TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	UString temp(UStrCast(filename));
	*result= (TRN_Image) Image::Create(DocCast(doc),temp,(Obj*)encoder_hints).GetSDFObj();
	EEX;
}

TRN_API TRN_ImageCreateFromMemory(TRN_SDFDoc doc,
				const unsigned char* buf,
				TRN_Size buf_size, 
				int width,
				int height,
				int bpc, 
				TRN_ColorSpace color_space, 
				TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image) Image::Create(DocCast(doc),
		(const char *)buf, buf_size, width, height, bpc,
		CSCast(color_space),ObjCast(encoder_hints)).GetSDFObj();
	EEX;
}

TRN_API TRN_ImageCreateFromStream(TRN_SDFDoc doc,
				TRN_FilterReader image_data, 				 
				int width,
				int height,
				int bpc, 
				TRN_ColorSpace color_space, 
				TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image) Image::Create(DocCast(doc),
		FRCast(image_data),width,height,bpc,
		CSCast(color_space),ObjCast(encoder_hints)).GetSDFObj();
	EEX;
}

TRN_API TRN_ImageCreateFromMemory2(TRN_SDFDoc doc,
	const unsigned char* buf,
	TRN_Size buf_size,
	TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	*result = (TRN_Image)Image::Create(DocCast(doc),
		(const char *)buf, buf_size, ObjCast(encoder_hints)).GetSDFObj();
	EEX;
}

TRN_API TRN_ImageCreateFromStream2(TRN_SDFDoc doc,
	TRN_Filter stream,
	TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	AutoPtr<Filters::Filter> wrap((Filters::Filter*)stream);
	*result = (TRN_Image)Image::Create(DocCast(doc), wrap, ObjCast(encoder_hints));
	EEX;
}

#if defined (_WIN32)
TRN_API TRN_ImageCreateFromBitmap(TRN_SDFDoc doc, TRN_GDIPlusBitmap bmp, TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image)Image::Create(DocCast(doc),
		(Gdiplus::Bitmap*)bmp,ObjCast(encoder_hints)).GetSDFObj();
	EEX;
}
#endif

TRN_API TRN_ImageCreateImageMask(TRN_SDFDoc doc,
				const char* buf, 
				TRN_Size buf_size, 
				int width,
				int height,
				TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image)Image::CreateImageMask(DocCast(doc),
		buf, buf_size, width,
		height,ObjCast(encoder_hints)).GetSDFObj();
	EEX;
}

TRN_API TRN_ImageCreateImageMaskFromStream(TRN_SDFDoc doc,
				TRN_FilterReader image_data, 				 
				int width,
				int height,
				TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image)Image::CreateImageMask(DocCast(doc),FRCast(image_data),width,height,ObjCast(encoder_hints)).GetSDFObj();;
	EEX;
}

TRN_API TRN_ImageCreateSoftMask(TRN_SDFDoc doc,
				const char* buf, 
				TRN_Size buf_size,
				int width,int height,int bpc, 
				TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image)Image::CreateSoftMask(DocCast(doc),
		buf, buf_size, width, height,
		bpc,ObjCast(encoder_hints)).GetSDFObj();;
	EEX;
}

TRN_API TRN_ImageCreateSoftMaskFromStream(TRN_SDFDoc doc,
				TRN_FilterReader image_data, 				 
				int width,
				int height,
				int bpc, 
				TRN_Obj encoder_hints, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image)Image::CreateSoftMask(DocCast(doc),
		FRCast(image_data),width,height,
		bpc,ObjCast(encoder_hints)).GetSDFObj();;
	EEX;
}

TRN_API TRN_ImageCreateDirectFromMemory(TRN_SDFDoc doc,
				const char* buf, 
				TRN_Size buf_size,
				int width, 
				int height, 
				int bpc, 
				TRN_ColorSpace color_space,
				enum TRN_ImageInputFilter input_format, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image)Image::Create(DocCast(doc),
		buf, buf_size, width, height, bpc,
		CSCast(color_space),(Image::InputFilter)input_format).GetSDFObj();;
	EEX;
}



TRN_API TRN_ImageCreateDirectFromStream(TRN_SDFDoc doc,
				TRN_FilterReader image_data, 				 
				int width, 
				int height, 
				int bpc, 
				TRN_ColorSpace color_space,
				enum TRN_ImageInputFilter input_format, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image) Image::Create(DocCast(doc),
		FRCast(image_data),width,height,bpc,
		CSCast(color_space),(Image::InputFilter)input_format).GetSDFObj();;
	EEX;
}

TRN_API TRN_ImageCreateFromObj(TRN_Obj image_xobject, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image)image_xobject;
	EEX;
}

TRN_API TRN_ImageCopy(const TRN_Image c, TRN_Image* result)
{
	BEX;
	*result=(TRN_Image)c;
	EEX;
}

TRN_API TRN_ImageGetSDFObj (TRN_Image img, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)img;
	EEX;
}

TRN_API TRN_ImageIsValid(const TRN_Image img, TRN_Bool* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=BToTB(g.IsValid());
	EEX;
}

TRN_API TRN_ImageGetImageData(TRN_Image img, TRN_Filter* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=(TRN_Filter)g.GetImageData().release();
	EEX;
}

TRN_API TRN_ImageGetImageDataSize(TRN_Image img, int* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=g.GetImageDataSize();
	EEX;
}

#if defined (_WIN32)
TRN_API TRN_ImageGetBitmap(TRN_Image img, TRN_GDIPlusBitmap* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=(TRN_GDIPlusBitmap)g.GetBitmap();
	EEX;
}

#endif

TRN_API TRN_ImageGetImageColorSpace(TRN_Image img, TRN_ColorSpace* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=(TRN_ColorSpace)new ColorSpace(g.GetImageColorSpace());
	EEX;
}

TRN_API TRN_ImageGetImageWidth(TRN_Image img, int* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=g.GetImageWidth();
	EEX;
}

TRN_API TRN_ImageGetImageHeight(TRN_Image img, int* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=g.GetImageHeight();
	EEX;
}

TRN_API TRN_ImageGetDecodeArray(TRN_Image img, TRN_Obj* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=(TRN_Obj)g.GetDecodeArray();
	EEX;
}

TRN_API TRN_ImageGetBitsPerComponent(TRN_Image img, int* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=g.GetBitsPerComponent();
	EEX;
}

TRN_API TRN_ImageGetComponentNum(TRN_Image img, int* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=g.GetComponentNum();
	EEX;
}

TRN_API TRN_ImageIsImageMask(TRN_Image img, TRN_Bool* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=BToTB(g.IsImageMask());
	EEX;
}

TRN_API TRN_ImageIsImageInterpolate(TRN_Image img, TRN_Bool* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=BToTB(g.IsImageInterpolate());
	EEX;
}

TRN_API TRN_ImageGetMask(TRN_Image img, TRN_Obj* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=(TRN_Obj)g.GetMask();
	EEX;
}

TRN_API TRN_ImageSetMask(TRN_Image img, TRN_Image image_mask)
{
	BEX;
	Image g(ImgCast(img));
	Image sg(ImgCast(image_mask));
	g.SetMask(sg);
	EEX;
}

TRN_API TRN_ImageSetMaskWithObj(TRN_Image img, TRN_Obj mask) 
{
	BEX;
	Image g(ImgCast(img));
	g.SetMask(ObjCast(mask));
	EEX;
}

TRN_API TRN_ImageGetSoftMask(TRN_Image img, TRN_Obj* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=(TRN_Obj)g.GetSoftMask();
	EEX;
}

TRN_API TRN_ImageSetSoftMask(TRN_Image img, TRN_Image soft_mask)
{
	BEX;
	Image g(ImgCast(img));
	Image sg(ImgCast(soft_mask));
	g.SetSoftMask(sg);
	EEX;
}

TRN_API TRN_ImageGetImageRenderingIntent(TRN_Image img, enum TRN_GStateRenderingIntent* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=(enum TRN_GStateRenderingIntent)g.GetImageRenderingIntent();
	EEX;
}

TRN_API TRN_ImageExport(TRN_Image img, const TRN_UString filename, int* result)
{
	BEX;
	Image g(ImgCast(img));
	UString temp(UStrCast(filename));
	*result=g.Export(temp);
	EEX;
}

TRN_API TRN_ImageExportFromStream(TRN_Image img, TRN_FilterWriter writer, int* result)
{
	BEX;
	Image g(ImgCast(img));
	*result=g.Export((FilterWriter*)writer, -1);
	EEX;
}

TRN_API TRN_ImageExportAsTiff(TRN_Image img, const TRN_UString filename)
{
	BEX;
	Image g(ImgCast(img));
	UString temp(UStrCast(filename));
	g.ExportAsTiff(temp);
	EEX;
}

TRN_API TRN_ImageExportAsTiffFromStream(TRN_Image img, TRN_FilterWriter writer)
{
	BEX;
	Image g(ImgCast(img));
	g.ExportAsTiff(FWCast(writer));
	EEX;
}

TRN_API TRN_ImageExportAsPng(TRN_Image img, const TRN_UString filename)
{
	BEX;
	Image g(ImgCast(img));
	UString temp(UStrCast(filename));
	g.ExportAsPng(temp);
	EEX;
}

TRN_API TRN_ImageExportAsPngFromStream(TRN_Image img, TRN_FilterWriter writer)
{
	BEX;
	Image g(ImgCast(img));
	g.ExportAsPng(FWCast(writer));
	EEX;
}
