#include <C/PDF/TRN_PDFDocInfo.h>
#include <C/PDF/TRN_Page.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>

#include <C/Common/TRN_Exception.hpp>
#include <PDF/PDFDocInfo.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_PDFDocInfoGetTitle(TRN_PDFDocInfo info, TRN_UString* result)
{
	BEX;
	RetUStr(IMPL_TRN_PDFDocInfoGetTitle(PDInfoCast(info)));
	EEX;
}

TRN_API TRN_PDFDocInfoGetTitleObj(TRN_PDFDocInfo info, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocInfoGetTitleObj(PDInfoCast(info));
	EEX;
}

TRN_API TRN_PDFDocInfoSetTitle(TRN_PDFDocInfo info, const TRN_UString title)
{
	BEX;
	UString temp(UStrCast(title));
	IMPL_TRN_PDFDocInfoSetTitle(PDInfoCast(info),temp);
	EEX;
}

TRN_API TRN_PDFDocInfoGetAuthor(TRN_PDFDocInfo info, TRN_UString* result)
{
	BEX;
	RetUStr(IMPL_TRN_PDFDocInfoGetAuthor(PDInfoCast(info)));
	EEX;
}

TRN_API TRN_PDFDocInfoGetAuthorObj(TRN_PDFDocInfo info, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocInfoGetAuthorObj(PDInfoCast(info));
	EEX;
}

TRN_API TRN_PDFDocInfoSetAuthor(TRN_PDFDocInfo info, const TRN_UString author)
{
	BEX;
	UString temp(UStrCast(author));
	IMPL_TRN_PDFDocInfoSetAuthor(PDInfoCast(info),temp);
	EEX;
}

TRN_API TRN_PDFDocInfoGetSubject(TRN_PDFDocInfo info, TRN_UString* result)
{
	BEX;
	RetUStr(IMPL_TRN_PDFDocInfoGetSubject(PDInfoCast(info)));
	EEX;
}

TRN_API TRN_PDFDocInfoGetSubjectObj(TRN_PDFDocInfo info, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocInfoGetSubjectObj(PDInfoCast(info));
	EEX;
}

TRN_API TRN_PDFDocInfoSetSubject(TRN_PDFDocInfo info, const TRN_UString subject)
{
	BEX;
	UString temp(UStrCast(subject));
	IMPL_TRN_PDFDocInfoSetSubject(PDInfoCast(info),temp);
	EEX;
}

TRN_API TRN_PDFDocInfoGetKeywords(TRN_PDFDocInfo info, TRN_UString* result)
{
	BEX;
	RetUStr(IMPL_TRN_PDFDocInfoGetKeywords(PDInfoCast(info)));
	EEX;
}

TRN_API TRN_PDFDocInfoGetKeywordsObj(TRN_PDFDocInfo info, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocInfoGetKeywordsObj(PDInfoCast(info));
	EEX;
}

TRN_API TRN_PDFDocInfoSetKeywords(TRN_PDFDocInfo info, const TRN_UString keywords)
{
	BEX;
	UString temp(UStrCast(keywords));
	IMPL_TRN_PDFDocInfoSetKeywords(PDInfoCast(info),temp);
	EEX;
}

TRN_API TRN_PDFDocInfoGetCreator(TRN_PDFDocInfo info, TRN_UString* result)
{
	BEX;
	RetUStr(IMPL_TRN_PDFDocInfoGetCreator(PDInfoCast(info)));
	EEX;
}

TRN_API TRN_PDFDocInfoGetCreatorObj(TRN_PDFDocInfo info, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocInfoGetCreatorObj(PDInfoCast(info));
	EEX;
}

TRN_API TRN_PDFDocInfoSetCreator(TRN_PDFDocInfo info, const TRN_UString creator)
{
	BEX;
	UString temp(UStrCast(creator));
	IMPL_TRN_PDFDocInfoSetCreator(PDInfoCast(info),temp);
	EEX;
}

TRN_API TRN_PDFDocInfoGetProducer(TRN_PDFDocInfo info, TRN_UString* result)
{
	BEX;
	RetUStr(IMPL_TRN_PDFDocInfoGetProducer(PDInfoCast(info)));
	EEX;
}

TRN_API TRN_PDFDocInfoGetProducerObj(TRN_PDFDocInfo info, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocInfoGetProducerObj(PDInfoCast(info));
	EEX;
}

TRN_API TRN_PDFDocInfoSetProducer(TRN_PDFDocInfo info, const TRN_UString producer)
{
	BEX;
	UString temp(UStrCast(producer));
	IMPL_TRN_PDFDocInfoSetProducer(PDInfoCast(info),temp);
	EEX;
}

TRN_API TRN_PDFDocInfoGetCreationDate(TRN_PDFDocInfo info, TRN_Date* result)
{
	BEX;
	Date d(IMPL_TRN_PDFDocInfoGetCreationDate(PDInfoCast(info)));
	assert(sizeof(TRN_Date)>=sizeof(d));
	memcpy(result, &d, sizeof(d));
	EEX;
}

TRN_API TRN_PDFDocInfoSetCreationDate(TRN_PDFDocInfo info, const TRN_Date* creation_date)
{
	BEX;
	IMPL_TRN_PDFDocInfoSetCreationDate(PDInfoCast(info), DateCast(creation_date));
	EEX;
}

TRN_API TRN_PDFDocInfoGetModDate(TRN_PDFDocInfo info, TRN_Date* result)
{
	BEX;
	Date d(IMPL_TRN_PDFDocInfoGetModDate(PDInfoCast(info)));
	assert(sizeof(TRN_Date)>=sizeof(d));
	memcpy(result, &d, sizeof(d));
	EEX;
}

TRN_API TRN_PDFDocInfoSetModDate(TRN_PDFDocInfo info, const TRN_Date* mod_date)
{
	BEX;
	IMPL_TRN_PDFDocInfoSetModDate(PDInfoCast(info),DateCast(mod_date));
	EEX;
}

TRN_API TRN_PDFDocInfoGetSDFObj(TRN_PDFDocInfo info, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)info;
	EEX;
}

TRN_API TRN_PDFDocInfoCreate (TRN_Obj tr, TRN_PDFDocInfo* result)
{
	BEX;
	*result=(TRN_PDFDocInfo)IMPL_TRN_PDFDocInfoCreate((Obj*)tr);
	EEX;
}

TRN_API TRN_PDFDocInfoCopy (const TRN_PDFDocInfo info, TRN_PDFDocInfo* result)
{
	BEX;
	*result=(TRN_PDFDocInfo)IMPL_TRN_PDFDocInfoCopy(PDInfoCast(info));
	EEX;
}
