#include <C/PDF/TRN_PDFViewCtrl.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <boost/static_assert.hpp>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/Annot.h>
#include <PDF/Point.h>
#include <PDFCtrlWX/PDFViewCtrl.h>
#include <SDF/License.hpp>
#include <PDF/ActionUtil.hpp>

using namespace PDF;
using namespace SDF;

BOOST_STATIC_ASSERT(sizeof(TRN_PDFViewCtrlEventHandlers)==sizeof(PDFViewCtrl::EventHandlers));
BOOST_STATIC_ASSERT(sizeof(TRN_PDFViewCtrlKeyEvent)==sizeof(PDFViewCtrl::KeyEvent));
BOOST_STATIC_ASSERT(sizeof(TRN_PDFViewCtrlMouseEvent)==sizeof(PDFViewCtrl::MouseEvent));
BOOST_STATIC_ASSERT(sizeof(TRN_PDFViewCtrlPaintEvent)==sizeof(PDFViewCtrl::PaintEvent));
BOOST_STATIC_ASSERT(sizeof(TRN_PDFViewCtrlSizeEvent)==sizeof(PDFViewCtrl::SizeEvent));
BOOST_STATIC_ASSERT(sizeof(TRN_PDFViewCtrlWindowEvent)==sizeof(PDFViewCtrl::WindowEvent));

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_PDFViewCtrlCreate(void* hwnd, void* hinstance, TRN_Bool compatibility, TRN_PDFViewCtrl* result)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_pdfview);
	*result=(TRN_PDFViewCtrl)new PDFViewCtrl(hwnd, hinstance, TBToB(compatibility));
	EEX;
}

TRN_API TRN_PDFViewCtrlDestroy(TRN_PDFViewCtrl view)
{
	BEX;
	delete (PDFViewCtrl*)view;
	EEX;
}

TRN_API TRN_PDFViewCtrlSetDoc(TRN_PDFViewCtrl view,  TRN_PDFDoc doc, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).SetPDoc(*((PDFDoc*)(&doc))));
	EEX;
}

TRN_API TRN_PDFViewCtrlCloseDoc(TRN_PDFViewCtrl view)
{
    BEX;
    ViewCCast(view).CloseDoc();
    EEX;
}

TRN_API TRN_PDFViewCtrlSetDocWithPassword(TRN_PDFViewCtrl view,  TRN_PDFDoc doc, const char* password, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).SetDoc(*((PDFDoc*)(&doc)),password));
	EEX;
}


TRN_API TRN_PDFViewCtrlGetDoc(TRN_PDFViewCtrl view, TRN_PDFDoc* result)
{
	BEX;
	PDFDoc* doc=ViewCCast(view).GetDoc();
	if(doc)
		*result=(TRN_PDFDoc)(doc->mp_sdf_doc);
	else
		*result=0;
	EEX;
}

TRN_API TRN_PDFViewCtrlDocLock(TRN_PDFViewCtrl view, TRN_Bool cancel)
{
	BEX;
	ViewCast(view).DocLock(TBToB(cancel));
	EEX;
}

TRN_API TRN_PDFViewCtrlDocUnlock(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCast(view).DocUnlock();
	EEX;
}

TRN_API TRN_PDFViewCtrlDocTryLock(TRN_PDFViewCtrl view, int milliseconds, TRN_Bool* result )
{
	BEX;
	*result = ViewCast(view).DocTryLock(milliseconds);
	EEX;
}

TRN_API TRN_PDFViewCtrlDocLockRead(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCast(view).DocLockRead();
	EEX;
}

TRN_API TRN_PDFViewCtrlDocUnlockRead(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCast(view).DocUnlockRead();
	EEX;
}

TRN_API TRN_PDFViewCtrlDocTryLockRead(TRN_PDFViewCtrl view, int milliseconds, TRN_Bool* result)
{
	BEX;
	*result = ViewCast(view).DocTryLockRead(milliseconds);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetPageViewMode(TRN_PDFViewCtrl view,  enum TRN_PDFViewCtrlPageViewMode mode)
{
	BEX;
	ViewCCast(view).SetPageViewMode((PDFView::PageViewMode)mode);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetPageViewMode( const TRN_PDFViewCtrl view, enum TRN_PDFViewCtrlPageViewMode* result)
{
	BEX;
	*result=(enum TRN_PDFViewCtrlPageViewMode)ViewCCast(view).GetPageViewMode();
	EEX;
}

TRN_API TRN_PDFViewCtrlSetPagePresentationMode(TRN_PDFViewCtrl view,  enum TRN_PDFViewCtrlPagePresentationMode mode)
{
	BEX;
	ViewCCast(view).SetPagePresentationMode((PDFView::PagePresentationMode)mode);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetPagePresentationMode( const TRN_PDFViewCtrl view,  enum TRN_PDFViewCtrlPagePresentationMode* result)
{
	BEX;
	*result=(enum TRN_PDFViewCtrlPagePresentationMode)ViewCCast(view).GetPagePresentationMode();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetCurrentPage( const TRN_PDFViewCtrl view,  int* result)
{
	BEX;
	*result=ViewCCast(view).GetCurrentPage();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetPageCount( const TRN_PDFViewCtrl view,  int* result)
{
	BEX;
	*result=ViewCCast(view).GetPageCount();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetVisiblePageCount( const TRN_PDFViewCtrl view,  int* result)
{
	BEX;
	*result=ViewCCast(view).GetVisiblePageCount();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetVisiblePage( const TRN_PDFViewCtrl view, int idx, int* result)
{
	BEX;
	*result=ViewCCast(view).GetVisiblePage(idx);
	EEX;
}

TRN_API TRN_PDFViewCtrlGotoFirstPage(TRN_PDFViewCtrl view,  TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).GotoFirstPage());
	EEX;
}

TRN_API TRN_PDFViewCtrlGotoLastPage(TRN_PDFViewCtrl view, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).GotoLastPage());
	EEX;
}

TRN_API TRN_PDFViewCtrlGotoNextPage(TRN_PDFViewCtrl view, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).GotoNextPage());
	EEX;
}

TRN_API TRN_PDFViewCtrlGotoPreviousPage(TRN_PDFViewCtrl view, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).GotoPreviousPage());
	EEX;
}

TRN_API TRN_PDFViewCtrlSetCurrentPage(TRN_PDFViewCtrl view,  int page_num, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).SetCurrentPage(page_num));
	EEX;
}

TRN_API TRN_PDFViewCtrlShowRect( TRN_PDFViewCtrl view, int page_num, const TRN_Rect* rect, TRN_Bool* result )
{
	BEX;
	*result=BToTB(ViewCCast(view).ShowRect(page_num, RectCast(rect)));
	EEX;
}

TRN_API TRN_PDFViewCtrlGetZoom( const TRN_PDFViewCtrl view, double* result)
{
	BEX;
	*result=ViewCCast(view).GetZoom();
	EEX;
}

TRN_API TRN_PDFViewCtrlSetZoom(TRN_PDFViewCtrl view,  double zoom, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).SetZoom(zoom));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetZoomOnPoint(TRN_PDFViewCtrl view,  int x, int y, double zoom, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).SetZoom(x,y,zoom));
	EEX;
}

TRN_API TRN_PDFViewCtrlSmartZoomOnPoint(TRN_PDFViewCtrl view,  int x, int y,  TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).SmartZoom(x,y));
	EEX;
}

TRN_API TRN_PDFViewCtrlRotateClockwise(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).RotateClockwise();
	EEX;
}

TRN_API TRN_PDFViewCtrlRotateCounterClockwise(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).RotateCounterClockwise();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetPageNumberFromScreenPt( const TRN_PDFViewCtrl view,  double x, double y, int* result)
{
	BEX;
	*result=ViewCCast(view).GetPageNumberFromScreenPt(x,y);
	EEX;
}

TRN_API TRN_PDFViewCtrlConvScreenPtToCanvasPt( const TRN_PDFViewCtrl view,  double* x, double* y)
{
	BEX;
	ViewCCast(view).ConvScreenPtToCanvasPt(*x,*y);
	EEX;
}

TRN_API TRN_PDFViewCtrlConvCanvasPtToScreenPt( const TRN_PDFViewCtrl view,  double* x, double* y)
{
	BEX;
	ViewCCast(view).ConvCanvasPtToScreenPt(*x,*y);
	EEX;
}

TRN_API TRN_PDFViewCtrlConvCanvasPtToPagePt( const TRN_PDFViewCtrl view,  double* x, double* y, int page_num)
{
	BEX;
	ViewCCast(view).ConvCanvasPtToPagePt(*x,*y,page_num);
	EEX;
}

TRN_API TRN_PDFViewCtrlConvPagePtToCanvasPt( const TRN_PDFViewCtrl view,  double* x, double* y, int page_num)
{
	BEX;
	ViewCCast(view).ConvPagePtToCanvasPt(*x,*y,page_num);
	EEX;
}

TRN_API TRN_PDFViewCtrlConvScreenPtToPagePt( const TRN_PDFViewCtrl view,  double* x, double* y, int page_num)
{
	BEX;
	ViewCCast(view).ConvScreenPtToPagePt(*x,*y,page_num);
	EEX;
}

TRN_API TRN_PDFViewCtrlConvPagePtToScreenPt( const TRN_PDFViewCtrl view,  double* x, double* y, int page_num)
{
	BEX;
	ViewCCast(view).ConvPagePtToScreenPt(*x,*y,page_num);
	EEX;
}

TRN_API TRN_PDFViewCtrlSnapToNearestInDoc(TRN_PDFViewCtrl view, double* screen_x, double* screen_y)
{
	BEX;
	PDF::Point in(*screen_x, *screen_y);
	PDF::Point out = ViewCCast(view).SnapToNearestInDoc(in);
	*screen_x = out.x;
	*screen_y = out.y;
	EEX;
}

TRN_API TRN_PDFViewCtrlSetSnappingMode(TRN_PDFViewCtrl view, unsigned int mode_flags)
{
	BEX;
	ViewCCast(view).SetSnappingMode(mode_flags);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetDeviceTransform( const TRN_PDFViewCtrl view,  int page_num, TRN_Matrix2D* result)
{
	BEX;
	*result=RetM2D(ViewCCast(view).GetDeviceTransform(page_num));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetErrorReportHandler(TRN_PDFViewCtrl view,  TRN_PDFViewCtrlErrorReportHandler error_proc, void* data)
{
	BEX;
	ViewCCast(view).SetErrorReportHandler(error_proc,data);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetCurrentPageHandler(TRN_PDFViewCtrl view,  TRN_PDFViewCtrlCurrentPageHandler curr_pagenum_proc, void* data)
{
	BEX;
	ViewCCast(view).SetCurrentPageHandler(curr_pagenum_proc,data);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetCurrentZoomHandler(TRN_PDFViewCtrl view,  TRN_PDFViewCtrlCurrentZoomHandler curr_zoom_proc, void* data)
{
	BEX;
	ViewCCast(view).SetCurrentZoomHandler(curr_zoom_proc,data);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetCanvasWidth( const TRN_PDFViewCtrl view, double* result)
{
	BEX;
	*result=ViewCCast(view).GetCanvasWidth();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetCanvasHeight( const TRN_PDFViewCtrl view, double* result)
{
	BEX;
	*result=ViewCCast(view).GetCanvasHeight();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetHScrollPos( const TRN_PDFViewCtrl view, double* result)
{
	BEX;
	*result=ViewCCast(view).GetHScrollPos();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetVScrollPos( const TRN_PDFViewCtrl view, double* result)
{
	BEX;
	*result=ViewCCast(view).GetVScrollPos();
	EEX;
}

TRN_API TRN_PDFViewCtrlOnScroll(TRN_PDFViewCtrl view,  int pix_dx, int pix_dy)
{
	BEX;
	ViewCCast(view).OnScroll(pix_dx,pix_dy);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetHScrollPos(TRN_PDFViewCtrl view,  double pos)
{
	BEX;
	ViewCCast(view).SetHScrollPos(pos);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetVScrollPos(TRN_PDFViewCtrl view,  double pos)
{
	BEX;
	ViewCCast(view).SetVScrollPos(pos);
	EEX;
}


TRN_API TRN_PDFViewCtrlIsFinishedRendering( const TRN_PDFViewCtrl view, TRN_Bool visible_region_only, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).IsFinishedRendering(TBToB(visible_region_only)));
	EEX;
}

TRN_API TRN_PDFViewCtrlCancelRendering(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).CancelRendering();
	EEX;
}

TRN_API TRN_PDFViewCtrlUpdate(TRN_PDFViewCtrl view, TRN_Bool all)
{
	BEX;
	ViewCCast(view).Update(TBToB(all));
	EEX;
}

TRN_API TRN_PDFViewCtrlUpdate2(TRN_PDFViewCtrl view, const TRN_Rect* rect)
{
	BEX;
	ViewCCast(view).Update(RectCast(rect));
	EEX;
}

TRN_API TRN_PDFViewCtrlUpdate3(TRN_PDFViewCtrl view, TRN_Annot annot, int num_page)
{
	BEX;
	ViewCCast(view).Update(Annot((Obj*)annot), num_page);
	EEX;
}

TRN_API TRN_PDFViewCtrlUpdateField(TRN_PDFViewCtrl view, const TRN_Field* field)
{
	BEX;
	ViewCCast(view).Update(FieldCast(field));
	EEX;
}

TRN_API TRN_PDFViewCtrlUpdatePageLayout(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).UpdatePageLayout();
	EEX;
}

TRN_API TRN_PDFViewCtrlHideAnnotation(TRN_PDFViewCtrl view, TRN_Annot annot)
{
	BEX;
	ViewCCast(view).HideAnnotation((Obj*)annot);
	EEX;
}

TRN_API TRN_PDFViewCtrlShowAnnotation(TRN_PDFViewCtrl view, TRN_Annot annot)
{
	BEX;
	ViewCCast(view).ShowAnnotation((Obj*)annot);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetDrawAnnotations(TRN_PDFViewCtrl view,  TRN_Bool render_annots)
{
	BEX;
	ViewCCast(view).SetDrawAnnotations(TBToB(render_annots));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetHighlightFields(TRN_PDFViewCtrl view,  TRN_Bool highlight)
{
	BEX;
	ViewCCast(view).SetHighlightFields(TBToB(highlight));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetRequiredFieldBorderColor(TRN_PDFView view, const TRN_ColorPt* col)
{
	BEX;
	ViewCCast(view).SetRequiredFieldBorderColor(CPCast(col));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetGamma(TRN_PDFViewCtrl view, double gamma)
{
	BEX;
	ViewCCast(view).SetGamma(gamma);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetOverprint(TRN_PDFViewCtrl view, enum TRN_PDFRasterizerOverprintPreviewMode op)
{
	BEX;
	ViewCCast(view).SetOverprint(PDFRasterizer::OverprintPreviewMode(op));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetAntiAliasing(TRN_PDFViewCtrl view,  TRN_Bool enable_aa)
{
	BEX;
	ViewCCast(view).SetAntiAliasing(TBToB(enable_aa));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetPathHinting(TRN_PDFViewCtrl view, TRN_Bool enable_hinting)
{
	BEX;
	ViewCCast(view).SetPathHinting(TBToB(enable_hinting));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetThinLineAdjustment(TRN_PDFViewCtrl view,  TRN_Bool grid_fit, TRN_Bool stroke_adjust)
{
	BEX;
	ViewCCast(view).SetThinLineAdjustment(TBToB(grid_fit), TBToB(stroke_adjust));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetImageSmoothing(TRN_PDFViewCtrl view,  TRN_Bool smoothing_enabled)
{
	BEX;
	ViewCCast(view).SetImageSmoothing(TBToB(smoothing_enabled));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetCaching(TRN_PDFViewCtrl view,  TRN_Bool enabled)
{
	BEX;
	ViewCCast(view).SetCaching(TBToB(enabled));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetRasterizerType (TRN_PDFViewCtrl view,  enum TRN_PDFRasterizerType type)
{
	BEX;
	ViewCCast(view).SetRasterizerType((PDFRasterizer::Type)type);
	EEX;
}


TRN_API TRN_PDFViewCtrlSetToolMode(TRN_PDFViewCtrl view,  enum TRN_PDFViewCtrlToolMode mode)
{
	BEX;
	ViewCCast(view).SetToolMode((PDFView::ToolMode)mode);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetToolMode( const TRN_PDFViewCtrl view, enum TRN_PDFViewCtrlToolMode* result)
{
	BEX;
	*result=(enum TRN_PDFViewCtrlToolMode)ViewCCast(view).GetToolMode();
	EEX;
}

TRN_API TRN_PDFViewCtrlSetTextSelectionMode(TRN_PDFViewCtrl view, enum TRN_PDFViewCtrlTextSelectionMode tm)
{
	BEX;
	ViewCCast(view).SetTextSelectionMode((PDFView::TextSelectionMode)tm);
	EEX;
}

TRN_API TRN_PDFViewCtrlSelect(TRN_PDFViewCtrl view,  double x1, double y1, double x2, double y2, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCCast(view).Select(x1, y1, x2, y2));
	EEX;
}

TRN_API TRN_PDFViewCtrlSelectWithSnapping(TRN_PDFViewCtrl view, double x1, double y1, double x2, double y2, 
	TRN_Bool snap_to_start, TRN_Bool snap_to_end, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCCast(view).SelectWithSnapping(x1, y1, x2, y2, TBToB(snap_to_start), TBToB(snap_to_end)));
	EEX;
}

TRN_API TRN_PDFViewCtrlSelectWithSmartSnapping(TRN_PDFViewCtrl view, double x1, double y1, double x2, double y2, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCCast(view).SelectWithSmartSnapping(x1, y1, x2, y2));
	EEX;
}

TRN_API TRN_PDFViewCtrlSelectByStruct(TRN_PDFViewCtrl view, double x1, double y1, int page1, double x2, double y2, int page2, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCCast(view).Select(x1, y1, page1, x2, y2, page2));
	EEX;
}

TRN_API TRN_PDFViewCtrlSelectByStructWithSnapping(TRN_PDFViewCtrl view, double x1, double y1, int page1, double x2, double y2, int page2,
	TRN_Bool snap_to_start, TRN_Bool snap_to_end, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCCast(view).SelectWithSnapping(x1, y1, page1, x2, y2, page2, TBToB(snap_to_start), TBToB(snap_to_end)));
	EEX;
}

TRN_API TRN_PDFViewCtrlSelectByStructWithSmartSnapping(TRN_PDFViewCtrl view, double x1, double y1, 
	int page1, double x2, double y2, int page2, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCCast(view).SelectWithSmartSnapping(x1, y1, page1, x2, y2, page2));
	EEX;
}

TRN_API TRN_PDFViewCtrlSelectByHighlights(TRN_PDFViewCtrl view,  TRN_Highlights h, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).Select(HLCast(h)));
	EEX;
}

TRN_API TRN_PDFViewCtrlSelectBySelection(TRN_PDFViewCtrl view, TRN_PDFViewSelection s, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).Select(VSCast(s)));
	EEX;
}

TRN_API TRN_PDFViewCtrlHasSelection(const TRN_PDFViewCtrl view, TRN_Bool* result)
{
	BEX;
	*result = ViewCCast(view).HasSelection();
	EEX;
}

TRN_API TRN_PDFViewCtrlClearSelection(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).ClearSelection();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetSelection(const TRN_PDFViewCtrl view, int pagenum, TRN_PDFViewSelection* result)
{
	BEX;
	PDFView::Selection* s = new PDFView::Selection(ViewCast(view).GetSelection(pagenum)); 
	*result = (TRN_PDFViewSelection) s;
	EEX;
}

TRN_API TRN_PDFViewCtrlGetSelectionBeginPage( const TRN_PDFViewCtrl view, int * result )
{
	BEX;
	*result = ViewCast(view).GetSelectionBeginPage();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetSelectionEndPage( const TRN_PDFViewCtrl view, int * result )
{
	BEX;
	*result = ViewCast(view).GetSelectionEndPage();
	EEX;
}

TRN_API TRN_PDFViewCtrlHasSelectionOnPage( const TRN_PDFViewCtrl view, int pagenum, TRN_Bool * result )
{
	BEX;
	*result = ViewCast(view).HasSelectionOnPage(pagenum);
	EEX;
}

TRN_API TRN_PDFViewCtrlPrepareWords(const TRN_PDFViewCtrl view, int page_num)
{
	BEX;
	ViewCast(view).PrepareWords(page_num);
	EEX;
}

TRN_API TRN_PDFViewCtrlWereWordsPrepared(const TRN_PDFViewCtrl view, int page_num, TRN_Bool * result)
{
	BEX;
	*result = ViewCast(view).WereWordsPrepared(page_num);
	EEX;
}

TRN_API TRN_PDFViewCtrlIsThereTextInRect(const TRN_PDFViewCtrl view, double x1, double y1, double x2, double y2, TRN_Bool * result)
{
	BEX;
	*result = ViewCast(view).IsThereTextInRect(x1, y1, x2, y2);
	EEX;
}

TRN_API TRN_PDFViewCtrlPrepareAnnotsForMouse(const TRN_PDFViewCtrl view, int page_num, double distance_threshold, double minimum_line_weight)
{
	BEX;
	ViewCast(view).PrepareAnnotsForMouse(page_num, distance_threshold, minimum_line_weight);
	EEX;
}

TRN_API TRN_PDFViewCtrlWereAnnotsForMousePrepared(const TRN_PDFViewCtrl view, int page_num, TRN_Bool * result)
{
	BEX;
	*result = ViewCast(view).WereAnnotsForMousePrepared(page_num);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetAnnotTypeUnder(const TRN_PDFViewCtrl view, double x, double y, enum TRN_AnnotType * result)
{
	BEX;
	*result = (TRN_AnnotType)ViewCast(view).GetAnnotTypeUnder(x, y);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetPageBorderVisibility(TRN_PDFViewCtrl view, TRN_Bool border_visible)
{
	BEX;
	ViewCCast(view).SetPageBorderVisibility(TBToB(border_visible));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetPageTransparencyGrid(TRN_PDFViewCtrl view, TRN_Bool trans_grid_visible)
{
	BEX;
	ViewCCast(view).SetPageTransparencyGrid(TBToB(trans_grid_visible));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetDefaultPageColor(TRN_PDFViewCtrl view, TRN_UInt8 r, TRN_UInt8 g, TRN_UInt8 b)
{
	BEX;
	ViewCCast(view).SetDefaultPageColor(r,g,b);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetBackgroundColor(TRN_PDFViewCtrl view, TRN_UInt8 r, TRN_UInt8 g, TRN_UInt8 b)
{
	BEX;
	ViewCCast(view).SetBackgroundColor(r,g,b);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetHorizontalAlign(TRN_PDFViewCtrl view, int align)
{
	BEX;
	ViewCCast(view).SetHorizontalAlign(align);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetVerticalAlign(TRN_PDFViewCtrl view, int align)
{
	BEX;
	ViewCCast(view).SetVerticalAlign(align);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetPageSpacing(TRN_PDFViewCtrl view, int horiz_col_space, int vert_col_space, int horiz_pad, int vert_pad)
{
	BEX;
	ViewCCast(view).SetPageSpacing(horiz_col_space, vert_col_space, horiz_pad, vert_pad);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetOCGContext(TRN_PDFViewCtrl view, TRN_OCGContext ctx) 
{
	BEX;
	ViewCCast(view).SetOCGContext((OCG::Context*)ctx);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetOCGContext(TRN_PDFViewCtrl view, TRN_OCGContext* result) 
{
	BEX;
	*result = (TRN_OCGContext) ViewCCast(view).GetOCGContext();
	EEX;
}



TRN_API TRN_PDFViewCtrlOnSize(TRN_PDFViewCtrl view, int width, int height)
{
	BEX;
	ViewCCast(view).OnSize(0,0,width,height);
	EEX;
}


TRN_API TRN_PDFViewCtrlOnChangeRect(TRN_PDFViewCtrl view,  int x, int y, int width, int height)
{
	BEX;
	ViewCCast(view).OnSize(x,y,width,height);
	EEX;
}




TRN_API TRN_PDFViewCtrlSetProgressiveRendering(TRN_PDFViewCtrl view, TRN_Bool progressive)
{
	BEX;
	ViewCCast(view).SetProgressiveRendering(TBToB(progressive));
	EEX;
}

TRN_API TRN_PDFViewCtrlShowNavPanel(TRN_PDFViewCtrl view, TRN_Bool show)
{
	BEX;
	ViewCCast(view).ShowNavPanel(TBToB(show));
	EEX;
}

TRN_API TRN_PDFViewCtrlIsNavPanelVisible(TRN_PDFViewCtrl view, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).IsNavPanelVisible());
	EEX;
}


TRN_API TRN_PDFViewCtrlCopy(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).Copy();
	EEX;
}

TRN_API TRN_PDFViewCtrlSelectAll(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).SelectAll();
	EEX;
}


TRN_API TRN_PDFViewCtrlEnableInteractiveForms(TRN_PDFViewCtrl view, TRN_Bool on)
{
	BEX;
	ViewCCast(view).EnableInteractiveForms(TBToB(on));
	EEX;
}

TRN_API TRN_PDFViewCtrlShowDialogs(TRN_PDFViewCtrl view, TRN_Bool show)
{
	BEX;
	ViewCCast(view).ShowDialogs(TBToB(show));
	EEX;
}

TRN_API TRN_PDFViewCtrlGetRotation( const TRN_PDFViewCtrl view, enum TRN_PageRotate* result)
{
	BEX;
	*result=(enum TRN_PageRotate)ViewCCast(view).GetRotation();
	EEX;
}

TRN_API TRN_PDFViewCtrlGetScreenRectForAnnot(const TRN_PDFViewCtrl view, TRN_Annot annot, int page_num, TRN_Rect* result)
{
	BEX;
	RectCast(result) = ViewCCast(view).GetScreenRectForAnnot(Annot((Obj*)annot), page_num);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetAnnotationAt(const TRN_PDFViewCtrl view, int x, int y, TRN_Annot* result)
{
    BEX;
    *result = (TRN_Annot)(ViewCCast(view).GetAnnotationAt(x, y).GetSDFObj());
    EEX;
}

TRN_API TRN_PDFViewCtrlSetSplitPosition(TRN_PDFViewCtrl view, int pos)
{
	BEX;
	ViewCCast(view).SetSplitPosition(pos);
	EEX;
}
TRN_API TRN_PDFViewCtrlGetSplitPosition( const TRN_PDFViewCtrl view, int* result)
{
	BEX;
	*result=ViewCCast(view).GetSplitPosition();
	EEX;
}



TRN_API TRN_PDFViewCtrlSetEnabledPanels(TRN_PDFViewCtrl view, TRN_UInt32 panels)
{
	BEX;
	ViewCCast(view).SetEnabledPanels(panels);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetEnabledPanels( const TRN_PDFViewCtrl view, TRN_UInt32* result)
{
	BEX;
	*result=ViewCCast(view).GetEnabledPanels();
	EEX;
}




TRN_API TRN_PDFViewCtrlSetSelectedPanel(TRN_PDFViewCtrl view, enum TRN_PDFViewCtrlPanelType panel)
{
	BEX;
	ViewCCast(view).SetSelectedPanel((PDFViewCtrl::PanelType) panel);
	EEX;
}


TRN_API TRN_PDFViewCtrlGetSelectedPanel( const TRN_PDFViewCtrl view, enum TRN_PDFViewCtrlPanelType* result)
{
	BEX;
	*result=(enum TRN_PDFViewCtrlPanelType)ViewCCast(view).GetSelectedPanel();
	EEX;
}


TRN_API TRN_PDFViewCtrlEnableScrollbar( TRN_PDFViewCtrl view, TRN_Bool show )
{
	BEX;
	ViewCCast(view).EnableScrollbar(show!=0);
	EEX;
}

TRN_API TRN_PDFViewCtrlShowToolbar( TRN_PDFViewCtrl view, TRN_Bool show )
{
	BEX;
	ViewCCast(view).ShowToolbar(show!=0);
	EEX;
}

TRN_API TRN_PDFViewCtrlShowNavToolbar( TRN_PDFViewCtrl view, TRN_Bool show )
{
	BEX;
	ViewCCast(view).ShowNavToolbar( show!=0 );
	EEX;
}

TRN_API TRN_PDFViewCtrlShowStatusBar( TRN_PDFViewCtrl view, TRN_Bool show ) 
{
	BEX;
	ViewCCast(view).ShowStatusBar( show!=0 );
	EEX;
}

TRN_API TRN_PDFViewCtrlShowMenuBar( TRN_PDFViewCtrl view, TRN_Bool show ) 
{
	BEX;
	ViewCCast(view).ShowMenuBar( show!=0 );
	EEX;
}


TRN_API TRN_PDFViewCtrlPrintDialog(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).Print();
	EEX;
}

TRN_API TRN_PDFViewCtrlPrint(TRN_PDFViewCtrl view, int first_page, 
							 int last_page, enum TRN_PageRotate rotation, 
							 int copies, const char* printer_name)
{
	BEX;
	ViewCCast(view).Print(first_page,last_page,(Page::Rotate)rotation,copies,printer_name);
	EEX;
}

TRN_API TRN_PDFViewCtrlPrintUStr(TRN_PDFViewCtrl view, int first_page, 
							 int last_page, enum TRN_PageRotate rotation, 
							 int copies, const TRN_UString printer_name)
{
	BEX;
	UString temp(UStrCast(printer_name));
	ViewCCast(view).Print(first_page,last_page,(Page::Rotate)rotation,copies,temp);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetCustomEventHandlers( TRN_PDFViewCtrl view, struct TRN_PDFViewCtrlEventHandlers* events)
{
	BEX;
	ViewCCast(view).SetCustomEventHandlers((PDFViewCtrl::EventHandlers*)events);
	EEX;
}


TRN_API TRN_PDFViewCtrlSetActionHandler( TRN_PDFViewCtrl view, TRN_PDFViewCtrlActionHandler action_callback, void* custom_data  )
{
    BEX;
	ViewCCast(view).SetActionHandler((PDF::PDFViewCtrl::ActionHandler)action_callback, custom_data );
    EEX;
}

TRN_API TRN_PDFViewCtrlSetWindowBackgroundColor( const TRN_PDFViewCtrl view, TRN_PDFViewCtrlWindowID winid, const TRN_ColorPt* color )
{
	BEX;
	ViewCCast(view).SetWindowBackgroundColor( (PDFViewCtrl::WindowID)winid, CPCast( &color ) );
	EEX;
}

#ifdef WIN32
TRN_API TRN_PDFViewCtrlSetWindowCursor( const TRN_PDFViewCtrl view, TRN_PDFViewCtrlWindowID winid, void* cursor )
{
	BEX;
	ViewCCast(view).SetWindowCursor( (PDFViewCtrl::WindowID)winid, cursor );
	EEX;
}
#endif

TRN_API TRN_PDFViewCtrlGetViewHeight( const TRN_PDFViewCtrl view, int* result)
{
	BEX;
	*result=ViewCCast(view).GetBufferHeight();
	EEX;
}


TRN_API TRN_PDFViewCtrlGetViewWidth( const TRN_PDFViewCtrl view, int* result)
{
	BEX;
	*result=ViewCCast(view).GetBufferWidth();
	EEX;
}

TRN_API TRN_PDFViewCtrlFind(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).Find();
	EEX;
}

TRN_API TRN_PDFViewCtrlRefresh(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).Refresh();
	EEX;
}


TRN_API TRN_PDFViewCtrlRequestRender(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).RequestRender();
	EEX;
}

TRN_API TRN_PDFViewCtrlCaptureMouse(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).CaptureMouse();
	EEX;
}

TRN_API TRN_PDFViewCtrlReleaseMouse(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCCast(view).ReleaseMouse();
	EEX;
}

TRN_API TRN_PDFViewCtrlHasCapture(TRN_PDFViewCtrl view, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCCast(view).HasCapture());
	EEX;
}

TRN_API TRN_PDFViewCtrlEnableLinkActivation(TRN_PDFViewCtrl view, TRN_Bool on)
{
	BEX;
	ViewCCast(view).EnableLinkActivation(TBToB(on));
	EEX;
}

TRN_API TRN_PDFViewCtrlSaveCurrentConfiguration( TRN_PDFViewCtrl view, const TRN_UString path, TRN_ProgressMonitor* progress)
{
	BEX;
	UString temp(UStrCast(path));
	ViewCCast(view).SaveCurrentConfiguration( temp, 0 );
	EEX;
}

TRN_API TRN_PDFViewCtrlLoadCurrentConfiguration( TRN_PDFViewCtrl view, const TRN_UString path ) 
{
	BEX;
	UString temp(UStrCast(path));
	ViewCCast(view).LoadCurrentConfiguration( temp );
	EEX;
}

TRN_API TRN_PDFViewCtrlGetDefaultAnnotation( TRN_PDFViewCtrl view, const char* type, TRN_Annot* result )
{
	BEX;
	*result = (TRN_Annot)(ViewCCast(view).GetDefaultAnnotation( type ).GetSDFObj());
	EEX;
}

TRN_API TRN_PDFViewCtrlSetFocus( TRN_PDFViewCtrl view )
{
	BEX;
	ViewCCast(view).SetFocus();
	EEX;
}

TRN_API TRN_PDFViewCtrlExecuteAction( TRN_PDFViewCtrl view, TRN_Action action ) {
	BEX;
	ActionParameter act((SDF::Obj*)action);
	ViewCCast(view).ExecuteAction( act );
	EEX;
}

TRN_API TRN_PDFViewCtrlDeletePages(TRN_PDFViewCtrl view) {
	BEX;	
	ViewCCast(view).DeletePages();
	EEX;
}

TRN_API TRN_PDFViewCtrlInsertBlankPages(TRN_PDFViewCtrl view) {
	BEX;	
	ViewCCast(view).InsertBlankPages();
	EEX;
}

TRN_API TRN_PDFViewCtrlInsertPages(TRN_PDFViewCtrl view, const TRN_UString src_name) {
	BEX;
	UString temp(UStrCast(src_name));
	ViewCCast(view).InsertPages(temp);
	EEX;
}


TRN_API TRN_PDFViewCtrlReplacePages(TRN_PDFViewCtrl view, const TRN_UString src_name) {
	BEX;
	UString temp(UStrCast(src_name));
	ViewCCast(view).ReplacePages(temp);
	EEX;
}

TRN_API TRN_PDFViewCtrlRotatePages(TRN_PDFViewCtrl view) {
	BEX;	
	ViewCCast(view).RotatePages();
	EEX;
}

TRN_API TRN_PDFViewCtrlExtractPages(TRN_PDFViewCtrl view) {
	BEX;	
	ViewCCast(view).ExtractPages();
	EEX;
}

TRN_API TRN_PDFViewCtrlCropPages(TRN_PDFViewCtrl view) {
	BEX;	
	ViewCCast(view).CropPages();
	EEX;
}

TRN_API TRN_PDFViewCtrlAddWatermark(TRN_PDFViewCtrl view) {
	BEX;	
	ViewCCast(view).AddWatermark();
	EEX;
}

TRN_API TRN_PDFViewCtrlDocProperties(TRN_PDFViewCtrl view) {
	BEX;	
	ViewCCast(view).DocProperties();
	EEX;
}


TRN_API TRN_PDFViewCtrlOpenURLAsync(TRN_PDFViewCtrl view, const char* url, TRN_UString cache_file, const char* password, TRN_Obj options)
{
	BEX;
	UString temp(UStrCast(cache_file));
	ViewCCast(view).OpenURL(url,password,temp,ObjCast(options));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetDownloadReportHandler( TRN_PDFViewCtrl view, TRN_PDFViewPartDownloadedProc proc, void* data )
{
	BEX;
	PDFView::PartDownloadedProc p = (PDFView::PartDownloadedProc)proc;
	ViewCCast(view).SetPartDownloadedProc(p, data);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetupThumbnails( TRN_PDFViewCtrl view, TRN_Bool use_embedded, TRN_Bool generate_at_runtime, TRN_Bool use_disk_cache, int thumb_max_side_length, size_t max_abs_cache_size, double max_perc_cache_size )
{
	BEX;
	ViewCCast(view).SetupThumbnails(TBToB(use_embedded), TBToB(generate_at_runtime), TBToB(use_disk_cache), thumb_max_side_length, max_abs_cache_size, max_perc_cache_size);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetThumbAsync( TRN_PDFViewCtrl view, int page_num, TRN_PDFViewThumbAsyncHandler proc, void* custom_data )
{
	BEX;
	PDFViewCtrl::ThumbAsyncHandler p = (PDFViewCtrl::ThumbAsyncHandler)proc;
	ViewCCast(view).GetThumbAsync(page_num, p, custom_data);
	EEX;
}

TRN_API TRN_PDFViewCtrlCancelAllThumbRequests( TRN_PDFViewCtrl view)
{
	BEX;
	ViewCast(view).CancelAllThumbRequests();	
	EEX;
}

TRN_API TRN_PDFViewCtrlClearThumbCache(TRN_PDFViewCtrl view)
{
	BEX;
	ViewCast(view).ClearThumbCache();
	EEX;
}

TRN_API TRN_PDFViewCtrlSetRequestRenderinWorkerThreadProc(TRN_PDFViewCtrl view, TRN_PDFViewRequestRenderInWorkerThreadProc proc, void * custom_data)
{
	BEX;
	ViewCast(view).SetRequestRenderInWorkerThreadProc(proc, custom_data);
	EEX;
}

TRN_API TRN_PDFViewCtrlSetUrlExtraction(const TRN_PDFViewCtrl view, TRN_Bool enabled)
{
	BEX;
	ViewCast(view).SetUrlExtraction(TBToB(enabled));
	EEX;
}

TRN_API TRN_PDFViewCtrlGetLinkAt(const TRN_PDFViewCtrl view, int x, int y, TRN_Rect* rect, TRN_UString* url)
{
	BEX;
	LinkifyPDF::LinkInfo link_info = ViewCast(view).GetLinkAt(x, y);
	RectCast(rect) = link_info.m_link_rect;
	UString temp(link_info.m_url);
	*url = (TRN_UString)temp.mp_impl;
	temp.mp_impl=0;
	EEX;
}

TRN_API TRN_PDFViewCtrlSetFindTextHandler(TRN_PDFViewCtrl view, TRN_PDFViewFindTextAsyncHandler proc, void * custom_data)
{
	BEX;
	PDFView::FindTextAsyncHandler p = (PDFView::FindTextAsyncHandler)proc;
	ViewCast(view).SetFindTextProc(p, custom_data);
	EEX;
}

TRN_API TRN_PDFViewCtrlFindTextAsync(TRN_PDFViewCtrl view, const TRN_UString search_str, TRN_Bool match_case, TRN_Bool match_whole_word,
									 TRN_Bool search_up, TRN_Bool reg_exp)
{
	BEX;	
	UString temp(UStrCast(search_str));
	ViewCast(view).FindTextAsync(temp, TBToB(match_case), TBToB(match_whole_word), TBToB(search_up), TBToB(reg_exp));
	EEX;
}

TRN_API TRN_PDFViewCtrlSetColorPostProcessMode( TRN_PDFViewCtrl view, enum TRN_PDFRasterizerColorPostProcessMode mode )
{
	BEX;
	ViewCCast(view).SetColorPostProcess((PDFRasterizer::ColorPostProcessMode)mode);
	EEX;
}

TRN_API TRN_PDFViewCtrlGetColorPostProcessMode( TRN_PDFViewCtrl view, enum TRN_PDFRasterizerColorPostProcessMode* result )
{
	BEX;
	*result=(enum TRN_PDFRasterizerColorPostProcessMode)ViewCCast(view).GetColorPostProcess();
	EEX;
}
