#include <C/PDF/TRN_Stamper.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/Stamper.h>
#include <SDF/License.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
	
using namespace SDF;
using namespace PDF;


//CONSTRUCTOR
TRN_API TRN_StamperCreate(enum TRN_StamperSizeType size_type, double a, double b, TRN_Stamper* result)
{
	BEX;
	*result = (TRN_Stamper)new Stamper((Stamper::SizeType&)size_type, a, b);
	EEX;
}


//DESTRUCTOR
TRN_API TRN_StamperDestroy(TRN_Stamper stamp)
{
	BEX;
	delete((Stamper*)stamp);
	EEX;
}

//STAMPS
TRN_API TRN_StamperStampImage(TRN_Stamper stamp, TRN_PDFDoc dest_doc, TRN_Image img, TRN_PageSet page_set)
{
	BEX;
	Image temp(ImgCast(img));
	PDFDoc* d=(PDFDoc*)&dest_doc;
	((Stamper*)stamp)->StampImage(*d, temp, *((PageSet*)page_set), 0);
	EEX;
}

TRN_API TRN_StamperStampPage(TRN_Stamper stamp, TRN_PDFDoc dest_doc, TRN_Page page, TRN_PageSet page_set)
{
	BEX;
	Page temp((SDF::Obj*)(page));
	PDFDoc* d=(PDFDoc*)&dest_doc;
	((Stamper*)stamp)->StampPage(*d, temp, *((PageSet*)page_set), 0);
	EEX;
}

TRN_API TRN_StamperStampText(TRN_Stamper stamp, TRN_PDFDoc dest_doc, TRN_UString txt, TRN_PageSet page_set)
{
	BEX;
	UString temp((SDF::Obj*)(txt));
	PDFDoc* d=(PDFDoc*)&dest_doc;
	((Stamper*)stamp)->StampText(*d, temp, *((PageSet*)page_set), 0);
	EEX;
}


//TEXT-ONLY METHODS
TRN_API TRN_StamperSetFont(TRN_Stamper stamp, TRN_Font font)
{
	BEX;
	((Stamper*)stamp)->SetFont(*(PDF::Font*)(font));
	EEX;
}

TRN_API TRN_StamperSetFontColor(TRN_Stamper stamp, const TRN_ColorPt* font_color)
{
	BEX;
	((Stamper*)stamp)->SetFontColor(*((ColorPt*)font_color));
	EEX;
}

TRN_API TRN_StamperSetTextAlignment(TRN_Stamper stamp, enum TRN_StamperTextAlignment text_alignment)
{
	BEX;
	((Stamper*)stamp)->SetTextAlignment((Stamper::TextAlignment&)text_alignment);
	EEX;
}


//APPEARANCE
TRN_API TRN_StamperSetOpacity(TRN_Stamper stamp, double opacity)
{
	BEX;
	((Stamper*)stamp)->SetOpacity(opacity);
	EEX;
}

TRN_API TRN_StamperSetRotation(TRN_Stamper stamp, double rotation)
{
	BEX;
	((Stamper*)stamp)->SetRotation(rotation);
	EEX;
}

TRN_API TRN_StamperSetSize(TRN_Stamper stamp, enum TRN_StamperSizeType size_type, double a, double b)
{
	BEX;
	((Stamper*)stamp)->SetSize((Stamper::SizeType&)size_type, a, b);
	EEX;
}

TRN_API TRN_StamperSetAsBackground(TRN_Stamper stamp, TRN_Bool background)
{
	BEX;
	((Stamper*)stamp)->SetAsBackground(TBToB(background));
	EEX;
}

TRN_API TRN_StamperSetAsAnnotation(TRN_Stamper stamp, TRN_Bool annotation)
{
	BEX;
	((Stamper*)stamp)->SetAsAnnotation(TBToB(annotation));
	EEX;
}

TRN_API TRN_StamperShowsOnScreen(TRN_Stamper stamp, TRN_Bool on_screen)
{
	BEX;
	((Stamper*)stamp)->ShowsOnScreen(TBToB(on_screen));
	EEX;
}

TRN_API TRN_StamperShowsOnPrint(TRN_Stamper stamp, TRN_Bool on_print)
{
	BEX;
	((Stamper*)stamp)->ShowsOnPrint(TBToB(on_print));
	EEX;
}

//POSITION
TRN_API TRN_StamperSetAlignment(TRN_Stamper stamp, enum TRN_StamperHorizontalAlignment horizontal_alignment, enum TRN_StamperVerticalAlignment vertical_alignment)
{
	BEX;
	((Stamper*)stamp)->SetAlignment((Stamper::HorizontalAlignment&)horizontal_alignment, (Stamper::VerticalAlignment&)vertical_alignment);
	EEX;
}

TRN_API TRN_StamperSetPosition(TRN_Stamper stamp, double x, double y, TRN_Bool use_percentage)
{
	BEX;
	((Stamper*)stamp)->SetPosition(x, y, TBToB(use_percentage));
	EEX;
}


//STATIC METHODS
TRN_API TRN_StamperDeleteStamps(TRN_PDFDoc doc, TRN_PageSet page_set)
{
	BEX;
	PDFDoc* d=(PDFDoc*)&doc;
	Stamper::DeleteStamps(*d, *((PageSet*)page_set), 0, false);
	EEX;
}

TRN_API TRN_StamperHasStamps(TRN_PDFDoc doc, TRN_PageSet page_set, TRN_Bool* result)
{
	BEX;
	PDFDoc* d=(PDFDoc*)&doc;
	*result = BToTB(Stamper::HasStamps(*d, *((PageSet*)page_set)));
	EEX;
}
