//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#include <C/PDF/TRN_VerificationOptions.h>
#include <C/Common/TRN_API_Config.hpp>
#include <C/Common/TRN_Exception.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <PDF/iVerificationOptions.hpp>
#include <PDF/VerificationOptions.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_VerificationOptionsCreate(enum TRN_VerificationOptions_SecurityLevel in_level, TRN_VerificationOptions* result)
{
	BEX;
	RefCountedPtr<trn::PDF::iVerificationOptions> tmp(new trn::PDF::VerificationOptions((trn::PDF::iVerificationOptions::SecurityLevel)in_level));
	*result = (TRN_VerificationOptions) tmp.release();
	EEX;
}

TRN_API TRN_VerificationOptionsAddTrustedCertificate(TRN_VerificationOptions const self, const TRN_UInt8* in_buf, const size_t in_buf_size)
{
	BEX;
	((trn::PDF::VerificationOptions*) self)->AddTrustedCertificate(in_buf, in_buf_size);
	EEX;
}


TRN_API TRN_VerificationOptionsRemoveTrustedCertificate(TRN_VerificationOptions const self, const TRN_UInt8* in_buf, const size_t in_buf_size)
{
	BEX;
	reinterpret_cast<trn::PDF::VerificationOptions*>(self)->RemoveTrustedCertificate(in_buf, in_buf_size);
	EEX;
}


TRN_API TRN_VerificationOptionsEnableModificationVerification(TRN_VerificationOptions self, TRN_Bool in_on_or_off)
{
	BEX;
	((trn::PDF::iVerificationOptions*)self )->EnableModificationVerification(in_on_or_off);
	EEX;
}


TRN_API TRN_VerificationOptionsEnableDigestVerification(TRN_VerificationOptions self, TRN_Bool in_on_or_off)
{
	BEX;
	((trn::PDF::iVerificationOptions*)self )->EnableDigestVerification(in_on_or_off);
	EEX;
}


TRN_API TRN_VerificationOptionsEnableTrustVerification(TRN_VerificationOptions self, TRN_Bool in_on_or_off)
{
	BEX;
	((trn::PDF::iVerificationOptions*)self )->EnableTrustVerification(in_on_or_off);
	EEX;
}

TRN_API TRN_VerificationOptionsDestroy(TRN_VerificationOptions self)
{
	BEX;
	if(self)
	{
		((trn::PDF::iVerificationOptions*)(self))->RemoveReference();
	}
	EEX;
}
