#include <C/SDF/TRN_NumberTree.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>
#include <SDF/NumberTree.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_NumberTreeCreate(TRN_Obj number_tree, TRN_NumberTree* result)
{
	BEX;
	*result=(TRN_NumberTree)IMPL_TRN_NumberTreeCreate((Obj*)number_tree);
	EEX;
}

TRN_API TRN_NumberTreeCopy (TRN_NumberTree const tree, TRN_NumberTree* result)
{
	BEX;
	*result=(TRN_NumberTree)IMPL_TRN_NumberTreeCopy((Obj*)tree);
	EEX;
}

TRN_API TRN_NumberTreeIsValid(TRN_NumberTree tree, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_NumberTreeIsValid((Obj*)tree));
	EEX;
}


TRN_API TRN_NumberTreeGetIterator(TRN_NumberTree tree, TRN_Int32 key, 
			 TRN_DictIterator* result)
{
	BEX;
	NumberTreeIterator temp=IMPL_TRN_NumberTreeFind((Obj*)tree,key);
	*result=(TRN_DictIterator)new PolyNumberTreeIterator(temp);
	EEX;
}

TRN_API TRN_NumberTreeGetValue(TRN_NumberTree tree, TRN_Int32 key, 
			 TRN_Obj* result)
{
	BEX;
	NumberTreeIterator end, temp=IMPL_TRN_NumberTreeFind((Obj*)tree,key);
	if(temp!=end)
	{
		*result=(TRN_Obj)(temp->Value());
	}
	else
		*result=0;
	EEX;
}

TRN_API TRN_NumberTreeGetIteratorBegin(TRN_NumberTree tree, 
								  TRN_DictIterator* result)
{
	BEX;
	NumberTreeIterator temp=IMPL_TRN_NumberTreeGetIterator((Obj*)tree);
	*result=(TRN_DictIterator)new PolyNumberTreeIterator(temp);
	EEX;
}

TRN_API TRN_NumberTreePut (TRN_NumberTree tree, TRN_Int32 key, TRN_Obj value)
{
	BEX;
	IMPL_TRN_NumberTreePut((Obj*)tree,key,(Obj*)value);
	EEX;
}

TRN_API TRN_NumberTreeEraseKey (TRN_NumberTree tree, TRN_Int32 key)
{
	BEX;
	IMPL_TRN_NumberTreeEraseKey((Obj*)tree,key);
	EEX;
}

TRN_API TRN_NumberTreeErase (TRN_NumberTree tree, TRN_DictIterator pos)
{
	BEX;
	IMPL_TRN_NumberTreeErase((Obj*)tree,((PolyNumberTreeIterator*)pos)->itr);
	EEX;
}

TRN_API TRN_NumberTreeGetSDFObj (TRN_NumberTree const tree, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_NumberTreeGetSDFObj((Obj*)tree);
	EEX;
}
