#include <C/SDF/TRN_SecurityHandler.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <boost/static_assert.hpp>
#include <C/Common/TRN_Exception.hpp>
#include <SDF/DefaultSecurityHandler.hpp>
#include <SDF/SDFDoc.h>
#include <SDF/SecurityHandler.h>

using namespace SDF;
using namespace std;

#define SHCast(handler) (((DefaultSecurityHandler*)handler))
#define VALID_SEC_HAND_ASSERT BASE_ASSERT( sh, "Operation on invalid object" )

// Make sure that flags match
BOOST_STATIC_ASSERT(
    e_SecurityHandler_owner == SecurityHandler::e_owner && 
    e_SecurityHandler_doc_open == SecurityHandler::e_doc_open &&
    e_SecurityHandler_doc_modify == SecurityHandler::e_doc_modify &&
    e_SecurityHandler_print == SecurityHandler::e_print &&
    e_SecurityHandler_print_high == SecurityHandler::e_print_high &&
    e_SecurityHandler_extract_content == SecurityHandler::e_extract_content &&
    e_SecurityHandler_mod_annot == SecurityHandler::e_mod_annot &&
    e_SecurityHandler_fill_forms == SecurityHandler::e_fill_forms &&
    e_SecurityHandler_access_support == SecurityHandler::e_access_support &&
    e_SecurityHandler_assemble_doc == SecurityHandler::e_assemble_doc
);

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_SecurityHandlerClone(TRN_SecurityHandler sh, TRN_SecurityHandler* result)
{
	BEX;
	if (sh) {
		*result = (TRN_SecurityHandler)new DefaultSecurityHandler(*SHCast(sh));
	}
	else {
		*result = 0; 
	}
	EEX;
}

TRN_API TRN_SecurityHandlerDestroy (TRN_SecurityHandler sh)
{
	BEX;
	delete (SecurityHandler*)sh;
	EEX;
}

TRN_API TRN_SecurityHandlerInitialize(TRN_SecurityHandler sh, TRN_SDFDoc doc, TRN_Obj encrypt_dict, void* custom_data)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->Initialize(DocCast(doc), ObjCast(encrypt_dict), custom_data);
	EEX;
}

TRN_API TRN_SecurityHandlerGetPermission(TRN_SecurityHandler sh, enum TRN_SecurityHandlerPermission p, TRN_Bool* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=BToTB(SHCast(sh)->GetPermission((SecurityHandler::Permission)p));
	EEX;
}

TRN_API TRN_SecurityHandlerAuthorize(TRN_SecurityHandler sh, enum TRN_SecurityHandlerPermission p, TRN_Bool* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=BToTB(SHCast(sh)->Authorize((SecurityHandler::Permission)p));
	EEX;
}

TRN_API TRN_SecurityHandlerAuthorizeFailed(TRN_SecurityHandler sh)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->AuthorizeFailed();
	EEX;
}

TRN_API TRN_SecurityHandlerGetAuthorizationData (TRN_SecurityHandler sh, enum TRN_SecurityHandlerPermission req_opr, TRN_Bool* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=BToTB(SHCast(sh)->GetAuthorizationData((SecurityHandler::Permission)req_opr));
	EEX;
}

TRN_API TRN_SecurityHandlerEditSecurityData(TRN_SecurityHandler sh, TRN_SDFDoc doc, TRN_Bool* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=BToTB(SHCast(sh)->EditSecurityData(DocCast(doc)));
	EEX;
}

TRN_API TRN_SecurityHandlerGetKeyLength(TRN_SecurityHandler sh, int* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=SHCast(sh)->GetKeyLength();
	EEX;
}

TRN_API TRN_SecurityHandlerGetEncryptionAlgorithmID(TRN_SecurityHandler sh, int* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=SHCast(sh)->GetEncryptionAlgorithmID();
	EEX;
}

TRN_API TRN_SecurityHandlerGetHandlerDocName(TRN_SecurityHandler sh, const char** result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=SHCast(sh)->GetHandlerDocName();
	EEX;
}

TRN_API TRN_SecurityHandlerIsModified (TRN_SecurityHandler sh, TRN_Bool* result)
{
	BEX;
	*result=BToTB(SHCast(sh)->IsModified());
	EEX;
}

TRN_API TRN_SecurityHandlerSetModified (TRN_SecurityHandler sh, TRN_Bool is_modified)
{
	BEX;
	SHCast(sh)->SetModified(TBToB(is_modified));
	EEX;
}

TRN_API TRN_SecurityHandlerCreate ( enum TRN_SecurityHandlerAlgorithmType crypt_type, TRN_SecurityHandler* result)	
{
	BEX;
	*result=(TRN_SecurityHandler(new DefaultSecurityHandler((StdSecurityHandler::AlgorithmType)crypt_type)));
	EEX;
}

TRN_API TRN_SecurityHandlerCreateFromEncCode (const char* name, int key_len, int enc_code, TRN_SecurityHandler* result)
{
	BEX;
	SDF::SecurityHandlerInfo info;
	info.m_enc_code = enc_code;
	info.m_key_length = key_len;
	*result=(TRN_SecurityHandler(new DefaultSecurityHandler(info, name)));
	EEX;
}

TRN_API TRN_SecurityHandlerCreateDefault (TRN_SecurityHandler* result)
{
	BEX;
	*result=(TRN_SecurityHandler)new DefaultSecurityHandler();
	EEX;
}

TRN_API TRN_SecurityHandlerChangeUserPassword(TRN_SecurityHandler sh,const char* password)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->ChangeUserPassword(password, strlen(password));
	EEX;
}

TRN_API TRN_SecurityHandlerChangeUserPasswordNonAscii(TRN_SecurityHandler sh,const char* password, size_t pwd_length )
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->ChangeUserPassword(password, pwd_length);
	EEX;
}

TRN_API TRN_SecurityHandlerGetUserPassword(TRN_SecurityHandler sh, const char** result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=SHCast(sh)->GetUserPassword();
	EEX;
}

TRN_API TRN_SecurityHandlerGetUserPasswordSize(TRN_SecurityHandler sh, TRN_UInt32* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=SHCast(sh)->GetUserPasswordSize();
	EEX;
}

TRN_API TRN_SecurityHandlerChangeMasterPassword(TRN_SecurityHandler sh, const char* password)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->ChangeMasterPassword(password, strlen(password));
	EEX;
}

TRN_API TRN_SecurityHandlerChangeMasterPasswordNonAscii(TRN_SecurityHandler sh,const char* password, size_t pwd_length )
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->ChangeMasterPassword(password,pwd_length);
	EEX;
}

TRN_API TRN_SecurityHandlerGetMasterPassword(TRN_SecurityHandler sh, const char** result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=SHCast(sh)->GetMasterPassword();
	EEX;
}

TRN_API TRN_SecurityHandlerGetMasterPasswordSize(TRN_SecurityHandler sh, TRN_UInt32* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=SHCast(sh)->GetMasterPasswordSize();
	EEX;
}

TRN_API TRN_SecurityHandlerSetPermission (TRN_SecurityHandler sh, enum TRN_SecurityHandlerPermission perm, TRN_Bool value)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->SetPermission((SecurityHandler::Permission)perm, TBToB(value));
	EEX;
}

TRN_API TRN_SecurityHandlerChangeRevisionNumber (TRN_SecurityHandler sh, int rev_num)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->ChangeRevisionNumber(rev_num);
	EEX;
}

TRN_API TRN_SecurityHandlerSetEncryptMetadata(TRN_SecurityHandler sh, TRN_Bool encrypt_metadata)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->SetEncryptMetadata(TBToB(encrypt_metadata));
	EEX;
}

TRN_API TRN_SecurityHandlerGetRevisionNumber (TRN_SecurityHandler sh, int* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=SHCast(sh)->GetRevisionNumber();
	EEX;
}

TRN_API TRN_SecurityHandlerIsUserPasswordRequired (TRN_SecurityHandler sh, TRN_Bool* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=BToTB(SHCast(sh)->IsUserPasswordRequired());
	EEX;
}

TRN_API TRN_SecurityHandlerIsMasterPasswordRequired (TRN_SecurityHandler sh, TRN_Bool* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=BToTB(SHCast(sh)->IsMasterPasswordRequired());
	EEX;
}

TRN_API TRN_SecurityHandlerIsAES(TRN_SecurityHandler sh, TRN_Bool* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=BToTB(SHCast(sh)->IsAES());
	EEX;
}

TRN_API TRN_SecurityHandlerIsAESObj(TRN_SecurityHandler sh, TRN_Obj stream, TRN_Bool* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=BToTB(SHCast(sh)->IsAES(ObjCast(stream)));
	EEX;
}

TRN_API TRN_SecurityHandlerIsRC4(TRN_SecurityHandler sh, TRN_Bool* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=BToTB(SHCast(sh)->IsRC4());
	EEX;
}

TRN_API TRN_SecurityHandlerInitPassword(TRN_SecurityHandler sh, const char* password)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->InitPassword(password, strlen(password));
	EEX;
}

TRN_API TRN_SecurityHandlerInitPasswordNonAscii(TRN_SecurityHandler sh, const char* password, size_t pwd_length)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->InitPassword(password, pwd_length);
	EEX;
}

TRN_API TRN_SecurityHandlerFillEncryptDict(TRN_SecurityHandler sh, TRN_SDFDoc doc, TRN_Obj* result)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	*result=(TRN_Obj) (SHCast(sh)->FillEncryptDictBase(DocCast(doc)));
	EEX;
}

TRN_API TRN_SecurityHandlerSetDerived(TRN_SecurityHandler sh, void* derived, 
	TRN_SecurityHandlerDerivedDestroyProc d,
	TRN_SecurityHandlerDerivedCloneProc c,
	TRN_SecurityHandlerAuthProc a,
	TRN_SecurityHandlerAuthFailedProc af,
	TRN_SecurityHandlerGetAuthDataProc ga,
	TRN_SecurityHandlerEditSecurDataProc es,
	TRN_SecurityHandlerFillEncryptDictProc fd)
{
	BEX;
	VALID_SEC_HAND_ASSERT;
	SHCast(sh)->InitProcs(derived, 
			(DefSecHdlrDerivedDestroyProc)d, 
			(DefSecHdlrDerivedCloneProc)c, 
			(DefSecHdlrAuthProc)a, 
			(DefSecHdlrAuthFailedProc)af, 
			(DefSecHdlrGetAuthDataProc)ga, 
			(DefSecHdlrEditSecurDataProc)es, 
			(DefSecHdlrFillEncryptDictProc)fd);
	EEX;
}

TRN_API TRN_SecurityHandlerGetDerived(TRN_SecurityHandler sh, void** derived,
									  TRN_SecurityHandlerDerivedDestroyProc* d,
									  TRN_SecurityHandlerDerivedCloneProc* c,
									  TRN_SecurityHandlerAuthProc* a,
									  TRN_SecurityHandlerAuthFailedProc* af,
									  TRN_SecurityHandlerGetAuthDataProc* ga,
									  TRN_SecurityHandlerEditSecurDataProc* es,
									  TRN_SecurityHandlerFillEncryptDictProc* fd)
{
	BEX;

	DefaultSecurityHandler* sec_hdlr = SHCast(sh);

	if (!sec_hdlr) {
		return 0;
	}

	if (derived){
		*derived = sec_hdlr->_derived_class;
	}

	if (d){
		*d = sec_hdlr->_destroy_derived;
	}

	if (c){
		*c = (TRN_SecurityHandlerDerivedCloneProc)sec_hdlr->_clone;
	}

	if (a){
		*a = (TRN_SecurityHandlerAuthProc)sec_hdlr->_a;
	}

	if (af){
		*af = sec_hdlr->_af;
	}

	if (ga){
		*ga = (TRN_SecurityHandlerGetAuthDataProc)sec_hdlr->_ga;
	}

	if (es){
		*es = (TRN_SecurityHandlerEditSecurDataProc)sec_hdlr->_es;
	}

	if (fd){
		*fd = (TRN_SecurityHandlerFillEncryptDictProc)sec_hdlr->_fd;
	}

	EEX;
}

TRN_API TRN_SecurityHandlerChangeUserPasswordUString(TRN_SecurityHandler sh, TRN_UString password)
{
    BEX;
    VALID_SEC_HAND_ASSERT;
    trn::UString tempUStr(UStrCast(password));
    SHCast(sh)->ChangeUserPassword(tempUStr);
    EEX;
}

TRN_API TRN_SecurityHandlerChangeUserPasswordBuffer(TRN_SecurityHandler sh, const TRN_UInt8* password_buf, size_t password_buf_size)
{
    BEX;
    VALID_SEC_HAND_ASSERT;
    BASE_ASSERT(password_buf != NULL, "Empty buffer.");
    std::vector<trn::UInt8> tempBuf(password_buf, password_buf + password_buf_size);
    SHCast(sh)->ChangeUserPassword(tempBuf);
    EEX;
}

TRN_API TRN_SecurityHandlerChangeMasterPasswordUString(TRN_SecurityHandler sh, TRN_UString password)
{
    BEX;
    VALID_SEC_HAND_ASSERT;
    trn::UString tempUStr(UStrCast(password));
    SHCast(sh)->ChangeMasterPassword(tempUStr);
    EEX;
}

TRN_API TRN_SecurityHandlerChangeMasterPasswordBuffer(TRN_SecurityHandler sh, const TRN_UInt8* password_buf, size_t password_buf_size)
{
    BEX;
    VALID_SEC_HAND_ASSERT;
    BASE_ASSERT(password_buf != NULL, "Empty buffer.");
    std::vector<trn::UInt8> tempBuf(password_buf, password_buf + password_buf_size);
    SHCast(sh)->ChangeMasterPassword(tempBuf);
    EEX;
}

TRN_API TRN_SecurityHandlerInitPasswordUString(TRN_SecurityHandler sh, TRN_UString password)
{
    BEX;
    VALID_SEC_HAND_ASSERT;
    trn::UString tempUStr(UStrCast(password));
    SHCast(sh)->InitPassword(tempUStr);
    EEX;
}

TRN_API TRN_SecurityHandlerInitPasswordBuffer(TRN_SecurityHandler sh, const TRN_UInt8* password_buf, size_t password_buf_size)
{
    BEX;
    VALID_SEC_HAND_ASSERT;
    BASE_ASSERT(password_buf != NULL, "Empty buffer.");
    std::vector<trn::UInt8> tempBuf(password_buf, password_buf + password_buf_size);
    SHCast(sh)->InitPassword(tempBuf);
    EEX;
}

