#include <C/Common/TRN_UString.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <string>

#include <C/Common/TRN_Exception.hpp>
#include <Common/Exception.h>
#include <Common/UStringImpl.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_UStringCreate(TRN_UString* result) 
{
	BEX;
	*result=(TRN_UString)IMPL_TRN_UStringCreate();
	EEX;
}

TRN_API TRN_UStringCopy(const TRN_UString str, TRN_UString* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=(TRN_UString)IMPL_TRN_UStringCopy(temp);
	EEX;
}

TRN_API TRN_UStringCreateFromCharacter(TRN_Unicode value, TRN_UString* result)
{
	BEX;
	*result=(TRN_UString)IMPL_TRN_UStringCreateFromCharacter(value);
	EEX;
}

TRN_API TRN_UStringCreateFromString(const TRN_Unicode* value, TRN_UString* result)
{
	BEX;
	*result=(TRN_UString)IMPL_TRN_UStringCreateFromString((Unicode*)value);
	EEX;
}

TRN_API TRN_UStringCreateFromUtf32String( const TRN_UInt32* str, int length, TRN_UString* result )
{
	BEX;
	*result=(TRN_UString)IMPL_TRN_UStringCreateFromUtf32( str, length );
	EEX;
}

TRN_API TRN_UStringCreateFromSubstring(const TRN_Unicode* value, int length, TRN_UString* result)
{
	BEX;
	*result=(TRN_UString)IMPL_TRN_UStringCreateFromSubstring((Unicode*)value,length);
	EEX;
}

TRN_API TRN_UStringCreateFromCharString(const char* buf, int buf_sz, TRN_UStringTextEncoding enc, TRN_UString* result)
{
	BEX;
	*result=(TRN_UString)IMPL_TRN_UStringCreateFromCharString(buf, buf_sz, (UString::TextEncoding)enc);
	EEX;
}

TRN_API TRN_UStringDestroy(TRN_UString to_destroy)
{
	BEX;
	UString temp(UStrCast(to_destroy));
	IMPL_TRN_UStringDestroy(temp);
	EEX;
}
TRN_API TRN_UStringAssignString(TRN_UString str, const TRN_Unicode* uni, int buf_sz)
{
	BEX;
	UString temp(UStrCast(str));
	IMPL_TRN_UStringAssignString(temp, (Unicode*)uni, buf_sz);
	EEX;
}
TRN_API TRN_UStringAssignUString(TRN_UString str, const TRN_UString uni)
{
	BEX;
	UString temp(UStrCast(str)), temp2(UStrCast(uni));
	IMPL_TRN_UStringAssignUString(temp, temp2);
	EEX;
}

TRN_API TRN_UStringAssignAscii(TRN_UString str, const char* sig_buf, int buf_sz, TRN_UStringTextEncoding enc)
{
	BEX;
	UString temp(UStrCast(str));
	IMPL_TRN_UStringAssignAscii(temp, sig_buf, buf_sz, (UString::TextEncoding)enc);
	EEX;
}

TRN_API TRN_UStringAssignConcat(TRN_UString str1, const TRN_UString str2)
{
	BEX;
	UString temp1(UStrCast(str1)), temp2(UStrCast(str2));
	IMPL_TRN_UStringAssignConcat(temp1, temp2);
	EEX;
}

TRN_API TRN_UStringConcat(const TRN_UString str1, const TRN_UString str2, TRN_UString result)
{
	BEX;
	UString temp1(UStrCast(str1)), temp2(UStrCast(str2)), temp_concat(UStrCast(result));
	IMPL_TRN_UStringConcat(temp1, temp2, temp_concat);
	EEX;
}

TRN_API TRN_UStringGetLength(const TRN_UString str, int* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=IMPL_TRN_UStringGetLength(temp);
	EEX;
}

TRN_API TRN_UStringIsEmpty(const TRN_UString str, TRN_Bool* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=BToTB(IMPL_TRN_UStringIsEmpty(temp));
	EEX;
}

TRN_API TRN_UStringGetBuffer(const TRN_UString str, const TRN_Unicode** result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=(const TRN_Unicode*)IMPL_TRN_UStringGetBuffer(temp);
	EEX;
}

TRN_API TRN_UStringCompare(const TRN_UString str1, const TRN_UString str2, int max_length, int* result)
{
	BEX;
	UString temp1(UStrCast(str1)),temp2(UStrCast(str2));
	*result=IMPL_TRN_UStringCompare(temp1,temp2,max_length);
	EEX;
}

TRN_API TRN_UStringSubStr(const TRN_UString str, int off, int count, TRN_UString* result)
{
	BEX;
	UString temp(UStrCast(str));
	RetUStr(IMPL_TRN_UStringSubStr(temp, off, count));
	EEX;
}

TRN_API TRN_UStringGetAt(const TRN_UString str, int idx, TRN_Unicode* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=IMPL_TRN_UStringGetAt(temp, idx);
	EEX;
}

TRN_API TRN_UStringPushBack(TRN_UString str, TRN_Unicode ch)
{
	BEX;
	UString temp(UStrCast(str));
	IMPL_TRN_UStringPushBack(temp, ch);
	EEX;
}

TRN_API TRN_UStringResize(TRN_UString str, int count, TRN_Unicode ch)
{
	BEX;
	UString temp(UStrCast(str));
	IMPL_TRN_UStringResize(temp, count, ch);
	EEX;
}

TRN_API TRN_UStringToLower(TRN_UString str)
{
	BEX;
	UString temp(UStrCast(str));
	IMPL_TRN_UStringToLower(temp);
	EEX;
}

TRN_API TRN_UStringToUpper(TRN_UString str)
{
	BEX;
	UString temp(UStrCast(str));
	IMPL_TRN_UStringToUpper(temp);
	EEX;
}

TRN_API TRN_UStringReserve(TRN_UString str, int count)
{
	BEX;
	UString temp(UStrCast(str));
	IMPL_TRN_UStringReserve(temp, count);
	EEX;
}

TRN_API TRN_UStringCStr(const TRN_UString str, const TRN_Unicode** result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=(TRN_Unicode*)IMPL_TRN_UStringCStr(temp);
	EEX;
}

TRN_API TRN_UStringIsInAscii(const TRN_UString str, TRN_Bool* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=BToTB(IMPL_TRN_UStringIsInAscii(temp));
	EEX;

}

TRN_API TRN_UStringConvertToAscii(const TRN_UString str, char* buf, int buf_sz, TRN_Bool null_term, int* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=IMPL_TRN_UStringConvertToAscii(temp, buf, buf_sz, TBToB(null_term));
	EEX;
}

TRN_API TRN_UStringConvertToUtf8(const TRN_UString str, char* buf, int buf_sz, TRN_Bool null_term, int* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=temp.ConvertToUtf8(buf, buf_sz, TBToB(null_term));
	EEX;
}

TRN_API TRN_UStringConvertToUtf32(const TRN_UString str, TRN_UInt32* in_out_buf, int buf_sz, TRN_Bool null_term, int* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=temp.ConvertToUtf32( in_out_buf, buf_sz, TBToB(null_term) );
	EEX;
}

TRN_API TRN_UStringConvertToPDFText(const TRN_UString str, char* sig_buf, int buf_sz,  TRN_Bool force_unicode, int* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=IMPL_TRN_UStringConvertToPDFText(temp, sig_buf, buf_sz, TBToB(force_unicode));
	EEX;
}
