#include <C/PDF/OCG/TRN_OCGConfig.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/OCG/Config.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;
using namespace OCG;

TRN_API TRN_OCGConfigCreateFromObj(TRN_Obj dict, TRN_OCGConfig* result)
{
	*result = (TRN_OCGConfig)dict;
	return 0;
}

TRN_API TRN_OCGConfigCreate(TRN_PDFDoc pdfdoc, TRN_Bool default_config, TRN_OCGConfig* result) 
{
	BEX;
	PDFDoc* doc=(PDFDoc*)&pdfdoc;
	Config cfg = Config::Create(*doc, TBToB(default_config));
	*result = (TRN_OCGConfig) cfg.GetSDFObj();
	EEX;
}

TRN_API TRN_OCGConfigCopy(const TRN_OCGConfig c, TRN_OCGConfig* result)
{
	*result = c;
	return 0;
}

TRN_API TRN_OCGConfigGetSDFObj (const TRN_OCGConfig c, TRN_Obj* result)
{
	*result = (TRN_Obj) c;
	return 0;
}

TRN_API TRN_OCGConfigGetOrder(const TRN_OCGConfig c, TRN_Obj* result)
{
	BEX;
	Config cfg((Obj*)c);
	*result = (TRN_Obj) cfg.GetOrder();
	EEX;
}

TRN_API TRN_OCGConfigSetOrder(const TRN_OCGConfig c, TRN_Obj value)
{
	BEX;
	Config cfg((Obj*)c);
	cfg.SetOrder((Obj*)value);
	EEX;
}

TRN_API TRN_OCGConfigGetName(const TRN_OCGConfig c, TRN_UString* result)
{
	BEX;
	Config cfg((Obj*)c);
	RetUStr(cfg.GetName());
	EEX;
}

TRN_API TRN_OCGConfigSetName(const TRN_OCGConfig c, TRN_UString value)
{
	BEX;
	Config cfg((Obj*)c);
	UString temp(value);
	cfg.SetName(temp);
	EEX;
}

TRN_API TRN_OCGConfigGetCreator(const TRN_OCGConfig c, TRN_UString* result)
{
	BEX;
	Config cfg((Obj*)c);
	RetUStr(cfg.GetCreator());
	EEX;
}

TRN_API TRN_OCGConfigSetCreator(const TRN_OCGConfig c, TRN_UString value)
{
	BEX;
	Config cfg((Obj*)c);
	UString temp(value);
	cfg.SetCreator(temp);
	EEX;
}

TRN_API TRN_OCGConfigGetInitBaseState(const TRN_OCGConfig c, const char** result)
{
	BEX;
	Config cfg((Obj*)c);
	*result = cfg.GetInitBaseState();
	EEX;
}

TRN_API TRN_OCGConfigSetInitBaseState(const TRN_OCGConfig c, const char* value)
{
	BEX;
	Config cfg((Obj*)c);
	cfg.SetInitBaseState(value);
	EEX;
}

TRN_API TRN_OCGConfigGetInitOnStates(const TRN_OCGConfig c, TRN_Obj* result)
{
	BEX;
	Config cfg((Obj*)c);
	*result = (TRN_Obj) cfg.GetInitOnStates();
	EEX;
}

TRN_API TRN_OCGConfigSetInitOnStates(const TRN_OCGConfig c, TRN_Obj value)
{
	BEX;
	Config cfg((Obj*)c);
	cfg.SetInitOnStates((Obj*)value);
	EEX;
}

TRN_API TRN_OCGConfigGetInitOffStates(const TRN_OCGConfig c, TRN_Obj* result)
{
	BEX;
	Config cfg((Obj*)c);
	*result = (TRN_Obj) cfg.GetInitOffStates();
	EEX;
}

TRN_API TRN_OCGConfigSetInitOffStates(const TRN_OCGConfig c, TRN_Obj value)
{
	BEX;
	Config cfg((Obj*)c);
	cfg.SetInitOffStates((Obj*)value);
	EEX;
}

TRN_API TRN_OCGConfigGetIntent(const TRN_OCGConfig c, TRN_Obj* result)
{
	BEX;
	Config cfg((Obj*)c);
	*result = (TRN_Obj) cfg.GetIntent();
	EEX;
}

TRN_API TRN_OCGConfigSetIntent(const TRN_OCGConfig c, TRN_Obj value){
	BEX;
	Config cfg((Obj*)c);
	cfg.SetIntent((Obj*)value);
	EEX;
}

TRN_API TRN_OCGConfigGetLockedOCGs(const TRN_OCGConfig c, TRN_Obj* result)
{
	BEX;
	Config cfg((Obj*)c);
	*result = (TRN_Obj) cfg.GetLockedOCGs();
	EEX;
}

TRN_API TRN_OCGConfigSetLockedOCGs(const TRN_OCGConfig c, TRN_Obj value)
{
	BEX;
	Config cfg((Obj*)c);
	cfg.SetLockedOCGs((Obj*)value);
	EEX;
}
