#include <C/PDF/OCG/TRN_OCGContext.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/OCG/Context.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;
using namespace OCG;

TRN_API TRN_OCGContextCreateFromConfig(TRN_OCGConfig cfg, TRN_OCGContext* result)
{
	BEX;
	Config c((Obj*)cfg);
	*result = (TRN_OCGContext) new Context(c);
	EEX;
}

TRN_API TRN_OCGContextDestroy(TRN_OCGContext ctx)
{
	BEX;
	delete (Context*) ctx;
	EEX;
}

TRN_API TRN_OCGContextCopy(const TRN_OCGContext c, TRN_OCGContext* result)
{
	BEX;
	*result = (TRN_OCGContext) new Context(*((Context*)c));
	EEX;
}

TRN_API TRN_OCGContextGetState(const TRN_OCGContext c, const TRN_OCG grp, TRN_Bool* result)
{
	BEX;
	Group g((Obj*)grp);
	*result=BToTB(((Context*)c)->GetState(g));
	EEX;
}

TRN_API TRN_OCGContextSetState(const TRN_OCGContext c, const TRN_OCG grp, TRN_Bool state)
{
	BEX;
	Group g((Obj*)grp);
	((Context*)c)->SetState(g, TBToB(state));
	EEX;
}

TRN_API TRN_OCGContextResetStates(const TRN_OCGContext c, TRN_Bool all_on)
{
	BEX;
	((Context*)c)->ResetStates(TBToB(all_on));
	EEX;
}

TRN_API TRN_OCGContextSetNonOCDrawing(const TRN_OCGContext c, TRN_Bool draw_non_OC)
{
	BEX;
	((Context*)c)->SetNonOCDrawing(TBToB(draw_non_OC));
	EEX;
}

TRN_API TRN_OCGContextGetNonOCDrawing(const TRN_OCGContext c, TRN_Bool* result)
{
	BEX;
	*result = BToTB( ((Context*)c)->GetNonOCDrawing() );
	EEX;
}

TRN_API TRN_OCGContextSetOCDrawMode(const TRN_OCGContext c, enum TRN_OCGContextOCDrawMode oc_draw_mode)
{
	BEX;
	((Context*)c)->SetOCDrawMode((OCG::Context::OCDrawMode)oc_draw_mode);
	EEX;
}

TRN_API TRN_OCGContextGetOCMode(const TRN_OCGContext c, enum TRN_OCGContextOCDrawMode* result)
{
	BEX;
	*result = (TRN_OCGContextOCDrawMode) ((Context*)c)->GetOCMode();
	EEX;
}
