#include <C/PDF/OCG/TRN_OCMD.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/OCG/OCMD.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;
using namespace OCG;

TRN_API TRN_OCMDCreate(TRN_PDFDoc pdfdoc, TRN_Obj ocgs, enum TRN_OCMDVisibilityPolicyType vis_policy, TRN_OCMD* result)
{
	BEX;
	PDFDoc* doc=(PDFDoc*)&pdfdoc;
	OCMD ocmd = OCMD::Create(*doc, (SDF::Obj*)ocgs, (OCMD::VisibilityPolicyType)vis_policy);
	*result = (TRN_OCMD) ocmd.GetSDFObj();
	EEX;
}

TRN_API TRN_OCMDCreateFromObj(TRN_Obj dict, TRN_OCMD* result)
{
	*result = (TRN_OCMD)dict;
	return 0;
}

TRN_API TRN_OCMDCopy(const TRN_OCMD c, TRN_OCMD* result)
{
	*result = c;
	return 0;
}

TRN_API TRN_OCMDGetSDFObj (const TRN_OCMD c, TRN_Obj* result)
{
	*result = (TRN_Obj) c;
	return 0;
}

TRN_API TRN_OCMDGetOCGs(const TRN_OCMD c, TRN_Obj* result)
{
	BEX;
	OCMD ocmd((Obj*)c);
	*result = (TRN_Obj) ocmd.GetOCGs();
	EEX;
}

TRN_API TRN_OCMDGetVisibilityExpression(const TRN_OCMD c, TRN_Obj* result)
{
	BEX;
	OCMD ocmd((Obj*)c);
	*result = (TRN_Obj) ocmd.GetVisibilityExpression();
	EEX;
}

TRN_API TRN_OCMDIsValid(const TRN_OCMD c, TRN_Bool* result)
{
	BEX;
	OCMD ocmd((Obj*)c);
	*result = BToTB(ocmd.IsValid());
	EEX;
}

TRN_API TRN_OCMDIsCurrentlyVisible(const TRN_OCMD c, const TRN_OCGContext ctx, TRN_Bool* result)
{
	BEX;
	OCMD ocmd((Obj*)c);
	*result = BToTB(ocmd.IsCurrentlyVisible(*(Context*)ctx));
	EEX;
}

TRN_API TRN_OCMDGetVisibilityPolicy(const TRN_OCMD c, enum TRN_OCMDVisibilityPolicyType* result)
{
	BEX;
	OCMD ocmd((Obj*)c);
	*result = (TRN_OCMDVisibilityPolicyType) ocmd.GetVisibilityPolicy();
	EEX;
}

TRN_API TRN_OCMDSetVisibilityPolicy(const TRN_OCMD c, enum TRN_OCMDVisibilityPolicyType vis_policy)
{
	BEX;
	OCMD ocmd((Obj*)c);
	ocmd.SetVisibilityPolicy((OCMD::VisibilityPolicyType)vis_policy);
	EEX;
}
