#include <C/PDF/PDFA/TRN_PDFACompliance.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/PDFA/PDFACompliance.h>
#include <SDF/License.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;
using namespace PDFA;
using namespace std;


TRN_API TRN_PDFAComplianceCreateFromFile(TRN_Bool convert, TRN_UString file_path, const char* password, enum TRN_PDFAComplianceConformance conf, enum TRN_PDFAComplianceErrorCode* exceptions, int exlength, int max_ref_objs, TRN_Bool first_stop, TRN_PDFACompliance* result)
{
	BEX;
	*result=(TRN_PDFACompliance)(new PDFACompliance(TBToB(convert), UString(UStrCast(file_path)), password, (PDF::PDFA::PDFACompliance::Conformance)conf, (PDF::PDFA::PDFACompliance::ErrorCode*)exceptions,  exlength, max_ref_objs, TBToB(first_stop)));
	EEX;
}


TRN_API TRN_PDFAComplianceCreateFromBuffer(TRN_Bool convert, const char* buf, int buf_size, const char* password, enum TRN_PDFAComplianceConformance conf, enum TRN_PDFAComplianceErrorCode* exceptions, int exlength, int max_ref_objs, TRN_Bool first_stop, TRN_PDFACompliance* result)
{
	BEX;
	*result=(TRN_PDFACompliance)(new PDFACompliance(TBToB(convert), buf, buf_size, password, (PDF::PDFA::PDFACompliance::Conformance)conf, (PDF::PDFA::PDFACompliance::ErrorCode*)exceptions, exlength, max_ref_objs, TBToB(first_stop)));
	EEX;
}

TRN_API TRN_PDFAComplianceDestroy(TRN_PDFACompliance pdfac)
{
	BEX;
	delete ((PDFACompliance*)pdfac);
	EEX;
}

 TRN_API TRN_PDFAComplianceGetErrorCount(TRN_PDFACompliance pdfac, int* result)
{
	BEX;
	*result=static_cast<int>(((PDFACompliance*)pdfac)->GetErrorCount());
	EEX;
}

 TRN_API TRN_PDFAComplianceGetError(TRN_PDFACompliance pdfac, int idx, enum TRN_PDFAComplianceErrorCode* result)
{
	BEX;
	*result = (TRN_PDFAComplianceErrorCode)(((PDFACompliance*)pdfac)->GetError(idx));
	EEX;
}

 TRN_API TRN_PDFAComplianceGetRefObjCount(TRN_PDFACompliance pdfac, enum TRN_PDFAComplianceErrorCode id, int* result)
{
	BEX;
	*result=static_cast<int>(((PDFACompliance*)pdfac)->GetRefObjCount((PDF::PDFA::PDFACompliance::ErrorCode)id));
	EEX;
}

 TRN_API TRN_PDFAComplianceGetRefObj(TRN_PDFACompliance pdfac, enum TRN_PDFAComplianceErrorCode  id, int err_idx, int* result)
{
	BEX;
	*result = static_cast<int>(((PDFACompliance*)pdfac)->GetRefObj((PDF::PDFA::PDFACompliance::ErrorCode)id, err_idx));
	EEX;
}

TRN_API TRN_PDFAComplianceGetPDFAErrorMessage(enum TRN_PDFAComplianceErrorCode  id, const char** result)
{
	BEX;
	*result=PDFACompliance::GetPDFAErrorMessage((PDF::PDFA::PDFACompliance::ErrorCode)id);
	EEX;
}

TRN_API TRN_PDFAComplianceSaveAsFromFileName(TRN_PDFACompliance pdfac, TRN_UString file_path, TRN_Bool linearized)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_pdfa);

	((PDFACompliance*)pdfac)->SaveAs(UString(UStrCast(file_path)), TBToB(linearized));
	EEX;
}

TRN_API TRN_PDFAComplianceSaveAsFromBuffer(TRN_PDFACompliance pdfac, const char** out_buf, TRN_Size* out_buf_size, TRN_Bool linearized)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_pdfa);

	((PDFACompliance*)pdfac)->SaveAs(*out_buf, *out_buf_size, TBToB(linearized));
	EEX;
}
