#include <C/PDF/Struct/TRN_SElement.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/Struct/ContentItem.h>
#include <PDF/Struct/SElement.h>
#include <PDF/Struct/SElement.hpp>
#include <PDF/Struct/STree.h>
#include <SDF/Name.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;

TRN_API TRN_SElementCreate(TRN_Obj dict, TRN_SElement* result)
{
	BEX;
	SDF::Obj* mp_obj=(SDF::Obj*)dict;
	result->obj=dict;
	if (mp_obj && mp_obj->IsDict()) {
		result->k = (TRN_Obj)(mp_obj->FindObj("K"));
	}
	EEX;
}
	
TRN_API TRN_SElementAssign (TRN_SElement* left, const TRN_SElement* right)
{
	BEX;
	SEleCast(left)=SEleCast(right);
	EEX;
}

TRN_API TRN_SElementCreateFromPDFDoc(TRN_PDFDoc doc, const char* struct_type, TRN_SElement* result)
{
	BEX;
    Struct::SElement selement = Struct::SElement::Create(PDocCast(doc), struct_type);
	*result = RetSEle(selement);	
	EEX;
}

TRN_API TRN_SElementInsert(TRN_SElement* e, TRN_SElement* kid, int insert_before)
{
	BEX;	
	SEleCast(e).Insert(SEleCast(kid), insert_before);	
	EEX;
}

TRN_API TRN_SElementCreateContentItem(TRN_SElement* e, TRN_PDFDoc doc, TRN_Page page, int insert_before, int* result)
{
	BEX;
	*result = SEleCast(e).CreateContentItem(PDocCast(doc), PageCast(page), insert_before);
	EEX;
}

TRN_API TRN_SElementIsValid(const TRN_SElement* e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(SEleCast(e).IsValid());
	EEX;
}

TRN_API TRN_SElementGetType(const TRN_SElement* e, const char** result)
{
	BEX;
	*result=SEleCast(e).GetType();
	EEX;
}

TRN_API TRN_SElementGetNumKids(const TRN_SElement* e, int* result)
{
	BEX;
	*result=SEleCast(e).GetNumKids();
	EEX;
}

TRN_API TRN_SElementIsContentItem(const TRN_SElement* e, int index, TRN_Bool* result)
{
	BEX;
	*result=BToTB(SEleCast(e).IsContentItem(index));
	EEX;
}

TRN_API TRN_SElementGetAsContentItem(const TRN_SElement* e, int index, TRN_ContentItem* result)
{
	BEX;
    Struct::ContentItem contentItem = SEleCast(e).GetAsContentItem(index);
	*result=RetCI(contentItem);
	EEX;
}

TRN_API TRN_SElementGetAsStructElem(const TRN_SElement* e, int index, TRN_SElement* result)
{
	BEX;
    Struct::SElement selement = SEleCast(e).GetAsStructElem(index);
	*result=RetSEle(selement);
	EEX;
}

TRN_API TRN_SElementGetParent(const TRN_SElement* e, TRN_SElement* result)
{
	BEX;
    Struct::SElement selement = SEleCast(e).GetParent();
	*result=RetSEle(selement);
	EEX;
}

TRN_API TRN_SElementGetStructTreeRoot(const TRN_SElement* e, TRN_STree* result)
{
	BEX;
	*result=(TRN_STree)(SEleCast(e).GetStructTreeRoot().GetSDFObj());
	EEX;
}

TRN_API TRN_SElementHasTitle(const TRN_SElement* e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(SEleCast(e).HasTitle());
	EEX;
}

TRN_API TRN_SElementGetTitle(const TRN_SElement* e, TRN_UString* result)
{
	BEX;
	RetUStr(SEleCast(e).GetTitle());
	EEX;
}

TRN_API TRN_SElementGetID(const TRN_SElement* e, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(SEleCast(e).GetID());
	EEX;
}

TRN_API TRN_SElementHasActualText(const TRN_SElement* e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(SEleCast(e).HasActualText());
	EEX;
}

TRN_API TRN_SElementGetActualText(const TRN_SElement* e, TRN_UString* result)
{
	BEX;
	RetUStr(SEleCast(e).GetActualText());
	EEX;
}

TRN_API TRN_SElementHasAlt(const TRN_SElement* e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(SEleCast(e).HasAlt());
	EEX;
}

TRN_API TRN_SElementGetAlt(const TRN_SElement* e, TRN_UString* result)
{
	BEX;
	RetUStr(SEleCast(e).GetAlt());
	EEX;
}

TRN_API TRN_SElementGetSDFObj(const TRN_SElement* e, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(SEleCast(e).GetSDFObj());
	EEX;
}
