#include <C/PDF/TRN_Action.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/Action.hpp>
#include <PDF/Destination.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_ActionCreateGoto(TRN_Destination dest, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)IMPL_TRN_ActionCreateGoto(DestCast(dest));
	EEX;
}

TRN_API TRN_ActionCreateGotoWithKey(TRN_String key, TRN_Destination dest, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)IMPL_TRN_ActionCreateGoto((const UChar*)key.str,(unsigned int)key.len,DestCast(dest));
	EEX;
}

TRN_API TRN_ActionCreateGotoRemote(const TRN_FileSpec file, int page_num, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)IMPL_TRN_ActionCreateGotoRemote(FSCast(file),page_num);
	EEX;
}

TRN_API TRN_ActionCreateGotoRemoteSetNewWindow(const TRN_FileSpec file, int page_num, TRN_Bool new_window, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)IMPL_TRN_ActionCreateGotoRemote(FSCast(file),page_num,TBToB(new_window));
	EEX;
}


TRN_API TRN_ActionCreateURI(TRN_SDFDoc sdfdoc, const char* uri, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)IMPL_TRN_ActionCreateURI(DocCast(sdfdoc), uri);
	EEX;
}


TRN_API TRN_ActionCreateURIWithUString(TRN_SDFDoc sdfdoc, TRN_UString uri, TRN_Action* result)
{
    BEX;
    UString temp(UStrCast(uri));
    *result = (TRN_Action) IMPL_TRN_ActionCreateURIWithUString(DocCast(sdfdoc), temp);
    EEX;
}

TRN_API TRN_ActionCreateSubmitForm(const TRN_FileSpec url, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)IMPL_TRN_ActionCreateSubmitForm(FSCast(url));
	EEX;
}

TRN_API TRN_ActionCreateLaunch(TRN_SDFDoc sdfdoc, const char* path, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)IMPL_TRN_ActionCreateLaunch(DocCast(sdfdoc), path);
	EEX;
}

TRN_API TRN_ActionCreateHideField(TRN_SDFDoc sdfdoc, int list_length, const char** field_list, TRN_Action* result)
{
	BEX;
	std::vector<std::string> buf;
	for(int i = 0; i < list_length; i++)
		buf.push_back(std::string(*(field_list+i)));
	*result=(TRN_Action)IMPL_TRN_ActionCreateHideField(DocCast(sdfdoc), buf);
	EEX;
}

TRN_API TRN_ActionCreateImportData(TRN_SDFDoc sdfdoc, const char* path, TRN_Action* result)
{
	BEX;
	*result = (TRN_Action)IMPL_TRN_ActionCreateImportData(DocCast(sdfdoc), path);
	EEX;
}

TRN_API TRN_ActionCreateResetForm(TRN_SDFDoc sdfdoc, TRN_Action* result)
{
	BEX;
	*result = (TRN_Action)IMPL_TRN_ActionCreateResetForm(DocCast(sdfdoc));
	EEX;
}

TRN_API TRN_ActionCreateJavaScript(TRN_SDFDoc sdfdoc, const char* script, TRN_Action* result)
{
	BEX;
	*result = (TRN_Action)IMPL_TRN_ActionCreateJavaScript(DocCast(sdfdoc), script);
	EEX;
}

TRN_API TRN_ActionCreate (TRN_Obj in_obj, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)in_obj;
	EEX;
}

TRN_API TRN_ActionCopy (const TRN_Action in_action, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)in_action;
	EEX;
}

TRN_API TRN_ActionCompare(TRN_Action action, const TRN_Action in_action, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_ActionCompare(ActCast(action),ActCast(in_action)));
	EEX;
}

TRN_API TRN_ActionIsValid(const TRN_Action action,TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_ActionIsValid(ActCast(action)));
	EEX;
}

TRN_API TRN_ActionGetType(const TRN_Action action, enum TRN_ActionType* result)
{
	BEX;
	*result=(enum TRN_ActionType)IMPL_TRN_ActionGetType(ActCast(action));
	EEX;
}

TRN_API TRN_ActionGetDest(TRN_Action action, TRN_Destination* result)
{
	BEX;
	*result=(TRN_Destination)IMPL_TRN_DestinationCreate(IMPL_TRN_ActionGetDest(ActCast(action)));
	EEX;
}

TRN_API TRN_ActionGetNext (const TRN_Action action,TRN_Obj* result)
{
	BEX;
	Action act((SDF::Obj*)action);
	*result=(TRN_Obj)act.GetNext();
	EEX;
}

TRN_API TRN_ActionGetSDFObj (const TRN_Action action,TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)action;
	EEX;
}

TRN_API TRN_Action_GetFormActionFlag(const TRN_Action action, enum TRN_ActionFormActionFlag flag, TRN_Bool* result)
{
	BEX;
	Action act((SDF::Obj*)action);
	*result = (TRN_Bool)(act.GetFormActionFlag(static_cast<Action::FormActionFlag>(flag)));
	EEX;
}


TRN_API TRN_Action_SetFormActionFlag(TRN_Action action, enum TRN_ActionFormActionFlag flag, TRN_Bool value)
{
	BEX;
	Action act((SDF::Obj*)action);
	act.SetFormActionFlag( static_cast<Action::FormActionFlag>(flag), (value!=0) );
	EEX;
}

TRN_API TRN_ActionNeedsWriteLock(const TRN_Action action, TRN_Bool* result)
{
	BEX;
	Action act((SDF::Obj*)action);
	*result = BToTB(act.NeedsWriteLock());
	EEX;
}

TRN_API TRN_Action_Execute( TRN_Action action )
{
	BEX;
	Action act((SDF::Obj*)action);
	act.Execute();
	EEX;
}

TRN_API TRN_ActionExecuteKeyStrokeAction(TRN_Action action, const TRN_KeyStrokeEventData data, TRN_KeyStrokeActionResult* result)
{
	BEX;
	Action act((SDF::Obj*)action);
	*result = (TRN_KeyStrokeActionResult)new Action::KeyStrokeActionResult(act.ExecuteKeyStrokeAction(*(Action::KeyStrokeEventData*)data));
	EEX;
}

TRN_API TRN_KeyStrokeActionResultIsValid(const TRN_KeyStrokeActionResult action_ret, TRN_Bool* result)
{
	BEX;
	*result = BToTB((*(Action::KeyStrokeActionResult*)action_ret).IsValid());
	EEX;
}

TRN_API TRN_KeyStrokeActionResultGetText(const TRN_KeyStrokeActionResult action_ret, TRN_UString* result)
{
	BEX;
	RetUStr((*(Action::KeyStrokeActionResult*)action_ret).GetAddedText());
	EEX;
}

TRN_API TRN_KeyStrokeActionResultDestroy(TRN_KeyStrokeActionResult action_ret)
{
	BEX;
	delete (Action::KeyStrokeActionResult*)action_ret;
	EEX;
}

TRN_API TRN_KeyStrokeActionResultCopy(TRN_KeyStrokeActionResult action_ret, TRN_KeyStrokeActionResult* result)
{
	BEX;
	*result = (TRN_KeyStrokeActionResult)new Action::KeyStrokeActionResult(*(Action::KeyStrokeActionResult*)action_ret);
	EEX;
}

TRN_API TRN_KeyStrokeActionResultAssign(TRN_KeyStrokeActionResult left, TRN_KeyStrokeActionResult right)
{
	BEX;
	*((Action::KeyStrokeActionResult*)left) = *((Action::KeyStrokeActionResult*)right);
	EEX;
}

TRN_API TRN_KeyStrokeEventDataCreate(const TRN_UString field_name, const TRN_UString current, const TRN_UString change, int selection_start, int selection_end, TRN_KeyStrokeEventData* result)
{
	BEX;
	UString temp_name(field_name);
	UString temp_current(current);
	UString temp_change(change);
	*result = (TRN_KeyStrokeEventData)new Action::KeyStrokeEventData(temp_name, temp_current, temp_change, selection_start, selection_end);
	EEX;
}

TRN_API TRN_KeyStrokeEventDataDestroy(TRN_KeyStrokeEventData data)
{
	BEX;
	delete (Action::KeyStrokeEventData*)data;
	EEX;
}

TRN_API TRN_KeyStrokeEventDataCopy(TRN_KeyStrokeEventData in_data, TRN_KeyStrokeEventData* result)
{
	BEX;
	*result = (TRN_KeyStrokeEventData)new Action::KeyStrokeEventData(*(Action::KeyStrokeEventData*)in_data);
	EEX;
}

TRN_API TRN_KeyStrokeEventDataAssign(TRN_KeyStrokeEventData left, TRN_KeyStrokeEventData right)
{
	BEX;
	*((Action::KeyStrokeEventData*)left) = *((Action::KeyStrokeEventData*)right);
	EEX;
}

