#include <C/PDF/TRN_Annot.h>
#include <C/PDF/TRN_ColorSpace.h>
#include <C/Common/TRN_API_Config.hpp>
#include <C/Common/TRN_PolymorphicVector.hpp>
#include <C/Common/TRN_Vector.h>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <string>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/Annots.h>
#include <PDF/Annots/SignatureWidget.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;
using namespace Annots;
using namespace std;


//1. General Annot		
//static Annot Create(SDF::SDFDoc& doc, Type type, const Rect& pos);
TRN_API TRN_AnnotCreate(TRN_SDFDoc doc, enum TRN_AnnotType type, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Annot::Create(DocCast(doc), static_cast<trn::PDF::Annot::Type>(type), *((Rect*)pos)).GetSDFObj());
	EEX;
}



//Annot (SDF::Obj* d = 0);
TRN_API TRN_AnnotCreateFromObj (TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Annot((Obj*)d).GetSDFObj());
	EEX;
}

//Annot (const Annot& d);
TRN_API TRN_AnnotCopy (const TRN_Annot d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)d;
	EEX;
}

////Annot& operator=(const Annot& d); 
//TRN_API TRN_AnnotAssign(TRN_Annot left, const TRN_Annot right, TRN_Annot* result)
//{
//	BEX;
//	Annot a((Obj*)left);
//	Annot b((Obj*)right);
//	//a.mp_obj=b.mp_obj;
//	//*result=(TRN_Annot)a.GetSDFOBj();
//	a.operator=(b);
//	//*result=(TRN_Annot)a.GetSDFObj();
//	EEX;
//}

//bool operator==(const Annot& d);
TRN_API TRN_AnnotCompare(const TRN_Annot annot, const TRN_Annot d, TRN_Bool* result)
{
	BEX;
	*result=BToTB(((Obj*)annot)->IsEqual((Obj*)d));
	EEX;
}


//bool IsValid() const;
TRN_API TRN_AnnotIsValid(TRN_Annot annot, TRN_Bool* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result=BToTB(a.IsValid());
	EEX;
}

//SDF::Obj* GetSDFObj () const;	
TRN_API TRN_AnnotGetSDFObj (TRN_Annot annot, TRN_Obj* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result=(TRN_Obj)a.GetSDFObj();
	EEX;
}

//Type GetType() const;
TRN_API TRN_AnnotGetType(TRN_Annot annot, enum TRN_AnnotType* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result=(TRN_AnnotType)a.GetType();
	EEX;
}

TRN_API TRN_AnnotIsMarkup( TRN_Annot annot, TRN_Bool* result )
{
	BEX;
	Annot a((Obj*)annot);
	*result=(TRN_AnnotType)a.IsMarkup();
	EEX;
}

TRN_API TRN_AnnotGetRect(TRN_Annot annot, TRN_Rect* result) 
{
	BEX;
	Annot a((Obj*)annot);
	Rect r(a.GetRect());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

TRN_API TRN_AnnotGetVisibleContentBox(TRN_Annot annot, TRN_Rect* result)
{
	BEX;
	Annot a((Obj*)annot);
	Rect r(a.GetVisibleContentBox());
	assert(sizeof(TRN_Rect) >= sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

//void SetRect(const Rect& p);
TRN_API TRN_AnnotSetRect(TRN_Annot annot, const TRN_Rect* pos)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetRect(RectCast(pos));
	EEX;
}



TRN_API TRN_AnnotSetContents(TRN_Annot annot, const TRN_UString cont ) {
	BEX;
	Annot a((Obj*)annot);
	UString temp(UStrCast(cont));
	a.SetContents(temp);
	EEX;
}

TRN_API TRN_AnnotGetContents(TRN_Annot annot, TRN_UString* result ) {
	BEX;
	Annot a((Obj*)annot);
	RetUStr(a.GetContents());
	EEX;
}

TRN_API TRN_AnnotGetTriggerAction(TRN_Annot annot, TRN_AnnotEventType trigger, TRN_Obj* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result = (TRN_Obj)a.GetTriggerAction(ActionTriggerEvent(trigger));
	EEX;
}

TRN_API TRN_AnnotGetPage(TRN_Annot annot, TRN_Page* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result=(TRN_Page)a.GetPage().GetSDFObj();
	EEX;
}

TRN_API TRN_AnnotSetPage(TRN_Annot annot, const TRN_Page pg)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetPage(PageCast(pg));
	EEX;
}

//SDF::Obj* GetUniqueID()
//*result=(TRN_Obj)IMPL_TRN_AnnotGetUniqueID(AnnCast(annot));
TRN_API TRN_AnnotGetUniqueID(TRN_Annot annot,TRN_Obj* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result=(TRN_Obj)a.GetUniqueID();
	EEX;
}

//void SetUniqueID(const char* id, int id_buf_sz = 0);
TRN_API TRN_AnnotSetUniqueID(TRN_Annot annot, const char* id, int id_buf_sz)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetUniqueID(id, id_buf_sz);
	EEX;
}

//Date GetDate() const;
TRN_API TRN_AnnotGetDate(TRN_Annot annot, TRN_Date* result)
{
	BEX;	
	Annot a((Obj*)annot);
	Date d(a.GetDate());
	assert(sizeof(TRN_Date)>=sizeof(d));
	memcpy(result, &d, sizeof(d));
	EEX;
}

//void SetDate(const Date& date)
TRN_API TRN_AnnotSetDate(TRN_Annot annot, const TRN_Date* date)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetDate(DateCast(date));
	EEX;
}

//bool GetFlag(Flag flag) const;
TRN_API TRN_AnnotGetFlag(TRN_Annot annot, enum TRN_AnnotFlag flag, TRN_Bool* result)
{
	//BEX;
	//*result=BToTB(IMPL_TRN_AnnotGetFlag(AnnCast(annot),(Annot::Flag)flag));
	//EEX;
	BEX;
	Annot a((Obj*)annot);
	*result=BToTB(a.GetFlag((Annot::Flag)flag));
	EEX;
}

//void SetFlag(Flag flag, bool value);
TRN_API TRN_AnnotSetFlag(TRN_Annot annot, enum TRN_AnnotFlag flag, TRN_Bool value)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetFlag((Annot::Flag)flag,TBToB(value));
	EEX;
}

//BorderStyleBorderStyle(Style s, int b_width, int b_hr, int b_vr);
TRN_API TRN_AnnotBorderStyleCreate(enum TRN_AnnotBorderStyleStyle s, double b_width, double b_hr, double b_vr, TRN_AnnotBorderStyle* result)
{
	BEX;
	*result=(TRN_AnnotBorderStyle)new Annot::BorderStyle((Annot::BorderStyle::Style)s,b_width,b_hr,b_vr);
	EEX;
}

//BorderStyleBorderStyle(Style s, int b_width, int b_hr, int b_vr, const std::vector<double>& b_dash)
TRN_API	TRN_AnnotBorderStyleCreateWithDashPattern(enum TRN_AnnotBorderStyleStyle s, double b_width, double b_hr, double b_vr, double* buffer, int buf_length, TRN_AnnotBorderStyle* result)
{
	BEX;
	vector<double> dash_ptrn(buf_length);
	memcpy(&(dash_ptrn[0]),buffer,buf_length*sizeof(double));
	*result=(TRN_AnnotBorderStyle)new Annot::BorderStyle((Annot::BorderStyle::Style)s,b_width,b_hr,b_vr,dash_ptrn);
	EEX;
}


//BorderStyleBorderStyle(const BorderStyle&);
TRN_API TRN_AnnotBorderStyleCopy(const TRN_AnnotBorderStyle bs,TRN_AnnotBorderStyle* result)
{
	BEX;
	*result=(TRN_AnnotBorderStyle)new Annot::BorderStyle(ABSCast(bs));
	EEX;
}

//BorderStyle GetBorderStyle() const;
TRN_API TRN_AnnotBorderStyleGetStyle(TRN_AnnotBorderStyle bs,enum TRN_AnnotBorderStyleStyle* result)
{
	BEX;
	*result=(enum TRN_AnnotBorderStyleStyle)ABSCast(bs).style;
	EEX;
}

//void SetBorderStyle(const BorderStyle& bs);
TRN_API TRN_AnnotBorderStyleSetStyle(TRN_AnnotBorderStyle bs,enum TRN_AnnotBorderStyleStyle style)
{
	BEX;
	ABSCast(bs).style=(Annot::BorderStyle::Style)style;
	EEX;
}

//To put in the destructor.
TRN_API TRN_AnnotBorderStyleDestroy(TRN_AnnotBorderStyle bs)
{
	BEX;
	delete (Annot::BorderStyle*)bs;
	EEX;
}


//SDF::Obj* GetAppearance(AnnotationState annot_state = e_normal, const char* app_state = 0);
TRN_API TRN_AnnotGetAppearance(TRN_Annot annot, enum TRN_AnnotState annot_state, const char* app_state, TRN_Obj* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result=(TRN_Obj)a.GetAppearance((Annot::AnnotationState)annot_state, app_state);
	EEX;
}


//void SetAppearance(SDF::Obj* app_stream, AnnotationState annot_state = e_normal, const char* app_state = 0);
TRN_API TRN_AnnotSetAppearance(TRN_Annot annot, TRN_Obj app_stream, enum TRN_AnnotState annot_state, const char* app_state)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetAppearance((Obj*)app_stream, (Annot::AnnotationState)annot_state, app_state);
	EEX;
}


//void RemoveAppearance(AnnotationState annot_state = e_normal, const char* app_state = 0);
TRN_API TRN_AnnotRemoveAppearance(TRN_Annot annot, enum TRN_AnnotState annot_state, const char* app_state)
{
	BEX;
	Annot a((Obj*)annot);
	a.RemoveAppearance((Annot::AnnotationState)annot_state, app_state);
	EEX;
}


//void Flatten(class Page page);
TRN_API TRN_AnnotFlatten(TRN_Annot annot, TRN_Page page)
{
	BEX;
	Annot a((Obj*)annot);
	a.Flatten(PageCast(page));
	EEX;
}

//const char* GetActiveAppearanceState() const;
TRN_API TRN_AnnotGetActiveAppearanceState(TRN_Annot annot, const char** result)
{
	BEX;
	Annot a((Obj*)annot);
	*result=a.GetActiveAppearanceState();
	EEX;
}

//void SetActiveAppearanceState(const char* astate);
TRN_API TRN_AnnotSetActiveAppearanceState(TRN_Annot annot, const char* astate)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetActiveAppearanceState(astate);
	EEX;
}


//In the future may need to do GetColorAsCYMK, or GetColorAsGray
TRN_API TRN_AnnotGetColor(TRN_Annot annot, TRN_ColorPt* result)
{
	BEX;
	Annot a((Obj*)annot);
	ColorPt c(a.GetColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}
//ColorPt GetColorAsRGB();
TRN_API TRN_AnnotGetColorAsRGB(TRN_Annot annot, TRN_ColorPt* result)
{
	BEX;
	Annot a((Obj*)annot);
	ColorPt c(a.GetColorAsRGB());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

TRN_API TRN_AnnotGetColorAsCMYK(TRN_Annot annot, TRN_ColorPt* result)
{
	BEX;
	Annot a((Obj*)annot);
	ColorPt c(a.GetColorAsCMYK());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

TRN_API TRN_AnnotGetColorAsGray(TRN_Annot annot, TRN_ColorPt* result)
{
	BEX;
	Annot a((Obj*)annot);
	ColorPt c(a.GetColorAsGray());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

//int GetAnnotColorCompNum() const;
TRN_API TRN_AnnotGetColorCompNum(TRN_Annot annot, int* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result=a.GetColorCompNum();
	EEX;
}

//void SetColor(const ColorPt& c, int comp_num=3); 
TRN_API TRN_AnnotSetColorDefault(TRN_Annot annot, const TRN_ColorPt* col)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetColor(CPCast(col), 3);
	EEX;
}

TRN_API TRN_AnnotSetColor(TRN_Annot annot, const TRN_ColorPt* col, int numcomp)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetColor(CPCast(col), numcomp);
	EEX;
}

//int GetStructParent() const;
TRN_API TRN_AnnotGetStructParent(TRN_Annot annot, int* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result=a.GetStructParent();
	EEX;
}

//void SetStructParent(const int keyval);//The value can be set, should be limited. 
TRN_API TRN_AnnotSetStructParent(TRN_Annot annot, int parkeyval)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetStructParent(parkeyval);
	EEX;
}

//SDF::Obj* GetOptionalContent() const;
TRN_API TRN_AnnotGetOptionalContent(TRN_Annot annot, TRN_Obj* result)
{
//	TRN_API TRN_DestinationGetExplicitDestObj (TRN_Destination dest, TRN_Obj* result)
//{
//	BEX;
//	*result=(TRN_Obj)IMPL_TRN_DestinationGetExplicitDestObj(DestCast(dest));
//	EEX;
//}
	BEX;
	Annot a((Obj*)annot);
	*result=(TRN_Obj)a.GetOptionalContent();
	EEX;
}

//void SetOptionalContent(SDF::Obj* oc);
TRN_API TRN_AnnotSetOptionalContent(TRN_Annot annot, const TRN_Obj content)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetOptionalContent(ObjCast(content));
	EEX;
}

TRN_API TRN_AnnotResize(TRN_Annot annot, const TRN_Rect* rect) {
	BEX;
	Annot a((Obj*)annot);
	a.Resize( RectCast(rect) );
	EEX;
}

TRN_API TRN_AnnotGetRotation(const TRN_Annot annot, int* result)
{
	BEX;
	Annot a((Obj*)annot);
	*result = a.GetRotation();
	EEX;
}

TRN_API TRN_AnnotSetRotation(TRN_Annot annot, int rotation)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetRotation(rotation);
	EEX;
}

TRN_API TRN_AnnotRefreshAppearance(TRN_Annot annot)
{
	BEX;
	Annot a((Obj*)annot);
	a.RefreshAppearance();
	EEX;
}

TRN_API TRN_AnnotGetCustomData(TRN_Annot annot, TRN_UString key, TRN_UString* result)
{
	BEX;
	Annot a((Obj*)annot);
	RetUStr(a.GetCustomData(UString(key)));
	EEX;
}

TRN_API TRN_AnnotSetCustomData(TRN_Annot annot, TRN_UString key, TRN_UString value)
{
	BEX;
	Annot a((Obj*)annot);
	a.SetCustomData(UString(key), UString(value));
	EEX;
}
TRN_API TRN_AnnotDeleteCustomData(TRN_Annot annot, TRN_UString key)
{
	BEX;
	Annot a((Obj*)annot);
	a.DeleteCustomData(UString(key));
	EEX;
}

//the border effect functions
TRN_API TRN_AnnotBorderStyleGetWidth(TRN_AnnotBorderStyle bs, double* result)
{
	BEX;
	*result=ABSCast(bs).width;
	EEX;
}

TRN_API TRN_AnnotBorderStyleSetWidth(TRN_AnnotBorderStyle bs, double width)
{
	BEX;
	ABSCast(bs).width=width;
	EEX;
}

TRN_API TRN_AnnotBorderStyleGetHR(TRN_AnnotBorderStyle bs, double* result)
{
	BEX;
	*result=ABSCast(bs).hr;
	EEX;
}

TRN_API TRN_AnnotBorderStyleSetHR(TRN_AnnotBorderStyle bs,double horizontal_radius)
{
	BEX;
	ABSCast(bs).hr=horizontal_radius;
	EEX;
}

TRN_API TRN_AnnotBorderStyleGetVR(TRN_AnnotBorderStyle bs, double* result)
{
	BEX;
	*result=ABSCast(bs).vr;
	EEX;
}

TRN_API TRN_AnnotBorderStyleSetVR(TRN_AnnotBorderStyle bs, double vertical_radius)
{
	BEX;
	ABSCast(bs).vr=vertical_radius;
	EEX;
}

TRN_API TRN_AnnotBorderStyleGetDashPattern(TRN_AnnotBorderStyle bs, double** out_buf, int* out_buf_length)
{
	BEX;
	*out_buf=&(ABSCast(bs).dash[0]);
	*out_buf_length = int(ABSCast(bs).dash.size());
	EEX;
}

TRN_API TRN_AnnotBorderStyleSetDashPattern(TRN_AnnotBorderStyle bs, const double* const buffer, int buf_length)
{
	BEX;
	vector<double> dash_ptrn(buf_length);
	memcpy(&(dash_ptrn[0]),buffer,buf_length*sizeof(double));
	ABSCast(bs).dash=dash_ptrn;
	EEX;
}

TRN_API TRN_AnnotGetBorderStyle(TRN_Annot annot,TRN_AnnotBorderStyle* result)
{
	BEX;
	*result=(TRN_AnnotBorderStyle)new Annot::BorderStyle(Annot((Obj*)annot).GetBorderStyle());
	EEX;
}

TRN_API TRN_AnnotSetBorderStyle(TRN_Annot annot, const TRN_AnnotBorderStyle bs, TRN_Bool oldStyleOnly )
{
	BEX;
	Annot((Obj*)annot).SetBorderStyle(ABSCast(bs), (oldStyleOnly!=0) );
	EEX;
}

TRN_API TRN_AnnotGetBorderStyleStyle( const TRN_AnnotBorderStyle bs, TRN_AnnotBorderStyleStyle* result)
{
	BEX;
	*result = static_cast<TRN_AnnotBorderStyleStyle>(ABSCast(bs).style);
	EEX;
}

TRN_API TRN_AnnotSetBorderStyleStyle( TRN_AnnotBorderStyle bs, const TRN_AnnotBorderStyleStyle bst )
{
	BEX;
	ABSCast(bs).style = (static_cast<trn::PDF::Annot::BorderStyle::Style>(bst));
	EEX;
}

TRN_API TRN_AnnotBorderStyleAssign( const TRN_AnnotBorderStyle from, TRN_AnnotBorderStyle to )
{
	BEX;
	ABSCast(to).operator= ( ABSCast(from) );
	EEX;
}

TRN_API TRN_AnnotBorderStyleCompare( const TRN_AnnotBorderStyle a, const TRN_AnnotBorderStyle b, TRN_Bool *result ) 
{
	BEX;
	*result = (ABSCast(a) == ABSCast(b));
	EEX;
}


//Caret
TRN_API TRN_CaretAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Caret((Obj*)d).GetSDFObj());
	EEX;
}

//Caret(const Annot& ann) : Markup(ann.GetSDFObj()) {} 
TRN_API TRN_CaretAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Caret((Obj*)annot).GetSDFObj());
	EEX;
}


//static Caret Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_CaretAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Caret::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}



//const char* GetContent() const;
TRN_API TRN_CaretAnnotGetSymbol(TRN_Annot caret, const char** result)
{
	BEX;
	Caret c((Obj*)caret);
	*result=c.GetSymbol();
	EEX;
}

//void SetContent(const char* contt);
TRN_API TRN_CaretAnnotSetSymbol(TRN_Annot caret, const char* symbol)
{
	BEX;
	Caret c((Obj*)caret);
	c.SetSymbol(symbol);
	EEX;
}


//3. Circle
		
//Circle(SDF::Obj* d = 0);
TRN_API TRN_CircleAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Circle((Obj*)d).GetSDFObj());
	EEX;
}


//Circle(const Annot& ann) : Markup(ann.GetSDFObj()) {} 
TRN_API TRN_CircleAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Circle((Obj*)annot).GetSDFObj());
	EEX;
}


//static Circle Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_CircleAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Circle::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}



//ColorPt GetInteriorColor() const;
TRN_API TRN_CircleAnnotGetInteriorColor(TRN_Annot circle, TRN_ColorPt* result)
{
	BEX;
	Circle ci((Obj*)circle);
	ColorPt c(ci.GetInteriorColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

//int GetInteriorColorCompNum() const;
TRN_API TRN_CircleAnnotGetInteriorColorCompNum(TRN_Annot circle, int* result)
{
	BEX;
	Circle ci((Obj*)circle);
	*result=ci.GetInteriorColorCompNum();
	EEX;
}

//void SetInteriorColor(const ColorPt& c, int CompNum);
TRN_API TRN_CircleAnnotSetInteriorColorDefault(TRN_Annot circle, const TRN_ColorPt* col)
{
	BEX;
	Circle c((Obj*)circle);
	c.SetInteriorColor(CPCast(col), 3);
	EEX;
}

TRN_API TRN_CircleAnnotSetInteriorColor(TRN_Annot circle, const TRN_ColorPt* col, int numcomp)
{
	BEX;
	Circle c((Obj*)circle);
	c.SetInteriorColor(CPCast(col), numcomp);
	EEX;
}

//Rect GetContentRect() const;
TRN_API TRN_CircleAnnotGetContentRect(TRN_Annot circle, TRN_Rect* result)
{
	BEX;
	Circle c((Obj*)circle);
	Rect r(c.GetContentRect());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

//void SetContentRect(const Rect& cr);
TRN_API TRN_CircleAnnotSetContentRect(TRN_Annot circle, const TRN_Rect* cr)
{
	BEX;
	Circle c((Obj*)circle);
	c.SetContentRect(RectCast(cr));
	EEX;
}


//Rect GetPadding() const;
TRN_API TRN_CircleAnnotGetPadding(TRN_Annot circle, TRN_Rect* result)
{
	BEX;
	Circle c((Obj*)circle);
	Rect r(c.GetPadding());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

//void SetPadding(const Rect& rd);
TRN_API TRN_CircleAnnotSetPadding(TRN_Annot circle, const TRN_Rect* cr)
{
	BEX;
	Circle c((Obj*)circle);
	c.SetPadding(RectCast(cr));
	EEX;
}



//4. FileAttachment

TRN_API TRN_FileAttachmentAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(FileAttachment((Obj*)d).GetSDFObj());
	EEX;
}

TRN_API TRN_FileAttachmentAnnotExport(TRN_Annot fileatt, const TRN_UString save_as, TRN_Bool* result)
{
	BEX;
	UString temp(UStrCast(save_as));
	FileAttachment fa((Obj*)fileatt);
	*result=BToTB(fa.Export(temp));
	EEX;
}

TRN_API TRN_FileAttachmentAnnotCreateFromAnnot(TRN_Annot fileatt, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(FileAttachment((Obj*)fileatt).GetSDFObj());
	EEX;
}

TRN_API TRN_FileAttachmentAnnotCreateWithFileSpec(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_FileSpec fspec, enum TRN_FileAttachmentAnnotIcon icon_name, TRN_Annot* result)
{
	BEX;
	FileSpec fs((Obj*)fspec);
	FileAttachment ft = FileAttachment::Create(DocCast(doc), *((Rect*)pos), fs, static_cast<FileAttachment::Icon>(icon_name));
	*result=(TRN_Annot)(ft.GetSDFObj());
	EEX;
}

TRN_API TRN_FileAttachmentAnnotCreateWithIcon(TRN_SDFDoc doc, const TRN_Rect* pos, const TRN_UString path, enum TRN_FileAttachmentAnnotIcon icon_name, TRN_Annot* result)
{
	BEX;
	UString temp(UStrCast(path));
	FileAttachment ft = FileAttachment::Create(DocCast(doc), *((Rect*)pos), temp, static_cast<FileAttachment::Icon>(icon_name));
	*result=(TRN_Annot)(ft.GetSDFObj());
	EEX;
}

TRN_API TRN_FileAttachmentAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, const TRN_UString path, const char* iconname, TRN_Annot* result)
{
	BEX;
	UString temp(UStrCast(path));
	FileAttachment ft=FileAttachment::Create( DocCast(doc), *((Rect*)pos), temp, iconname );
	*result=(TRN_Annot)(ft.GetSDFObj());
	EEX;
}

TRN_API TRN_FileAttachmentAnnotCreateDefault(TRN_SDFDoc doc, const TRN_Rect* pos, const TRN_UString path, TRN_Annot* result)
{
	BEX;
	UString temp(UStrCast(path));
	FileAttachment ft = FileAttachment::Create( DocCast(doc), *((Rect*)pos), temp, FileAttachment::e_PushPin);
	*result=(TRN_Annot)(ft.GetSDFObj());
	EEX;
}

TRN_API TRN_FileAttachmentAnnotGetFileSpec(TRN_Annot fileatt, TRN_FileSpec* result)
{
	BEX;
	FileAttachment ft((Obj*)fileatt);
	*result=(TRN_FileSpec)(ft.GetFileSpec().GetSDFObj());
	EEX;
}

TRN_API TRN_FileAttachmentAnnotSetFileSpec(TRN_Annot fileatt, TRN_FileSpec fspec)
{
	BEX;
	FileAttachment ft((Obj*)fileatt);
	FileSpec fs((Obj*)fspec);
	ft.SetFileSpec(fs);
	EEX;
}

TRN_API TRN_FileAttachmentAnnotGetIcon(TRN_Annot fileatt, enum TRN_FileAttachmentAnnotIcon* result)
{
	BEX;
	FileAttachment ft((Obj*)fileatt);
	*result=(TRN_FileAttachmentAnnotIcon)ft.GetIcon();
	EEX;
}

TRN_API TRN_FileAttachmentAnnotSetIcon(TRN_Annot fileatt, enum TRN_FileAttachmentAnnotIcon icon)
{
	BEX;
	FileAttachment ft((Obj*)fileatt);
	ft.SetIcon((Annots::FileAttachment::Icon)icon);
	EEX;
}

TRN_API TRN_FileAttachmentAnnotGetIconName(TRN_Annot fileatt, const char** result)
{
	BEX;
	FileAttachment ft((Obj*)fileatt);
	*result=ft.GetIconName();
	EEX;
}

TRN_API TRN_FileAttachmentAnnotSetIconName(TRN_Annot fileatt, const char* iname)
{
	BEX;
	FileAttachment ft((Obj*)fileatt);
	ft.SetIconName(iname);
	EEX;
}


//5. FreeText

//FreeText(SDF::Obj* d = 0);	
TRN_API TRN_FreeTextAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(FreeText((Obj*)d).GetSDFObj());
	EEX;
}

//FreeText(const Annot& ann) : Markup(ann.GetSDFObj()) {}
TRN_API TRN_FreeTextAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(FreeText((Obj*)annot).GetSDFObj());
	EEX;
}

//static FreeText Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_FreeTextAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(FreeText::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}

//const char* GetDefaultAppearance() const;
TRN_API TRN_FreeTextAnnotGetDefaultAppearance(TRN_Annot ft, TRN_UString* result )
{
	BEX;
	FreeText f((Obj*)ft);
	RetUStr(f.GetDefaultAppearance());
	EEX;
}

//void SetDefaultAppearance(const char* app_str)
TRN_API TRN_FreeTextAnnotSetDefaultAppearance(TRN_Annot ft, const char* defApp)
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetDefaultAppearance(defApp);
	EEX;
}


//int GetQuaddingFormat() const;
TRN_API TRN_FreeTextAnnotGetQuaddingFormat(TRN_Annot ft, int* result)
{
	BEX;
	FreeText f((Obj*)ft);
	*result=f.GetQuaddingFormat();
	EEX;
}

//void SetQuaddingFormat(const int app_qform)
TRN_API TRN_FreeTextAnnotSetQuaddingFormat(TRN_Annot ft, int format)
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetQuaddingFormat(format);
	EEX;
}

//void GetCalloutLinePoints(Point& p1, Point& p2, Point& p3) const; 
TRN_API TRN_FreeTextAnnotGetCalloutLinePoints(TRN_Annot ft, const TRN_Point* p1, const TRN_Point* p2, const TRN_Point* p3)
{
	BEX;
	FreeText f((Obj*)ft);
	f.GetCalloutLinePoints((PointCast(p1)), (PointCast(p2)), (PointCast(p3)));
	EEX;
}

//void SetCalloutLinePoints(const Point& p1, const Point& p2, const Point& p3); 
TRN_API TRN_FreeTextAnnotSetCalloutLinePoints(TRN_Annot ft, const TRN_Point* p1, const TRN_Point* p2, const TRN_Point* p3)
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetCalloutLinePoints((PointCast(p1)), (PointCast(p2)), (PointCast(p3)));
	EEX;
}

//void SetCalloutLinePoints(const Point& p1, const Point& p2);
TRN_API TRN_FreeTextAnnotSetCalloutLinePointsTwo(TRN_Annot ft, const TRN_Point* p1, const TRN_Point* p2)
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetCalloutLinePoints((PointCast(p1)), (PointCast(p2)));
	EEX;
}

//IntentName GetIntentName() const;
TRN_API TRN_FreeTextAnnotGetIntentName(TRN_Annot ft, enum TRN_FreeTextAnnotIntentName* result)
{
	BEX;
	FreeText f((Obj*)ft);
	*result=(TRN_FreeTextAnnotIntentName)f.GetIntentName();
	EEX;
}


//void SetIntentName(IntentName mode=e_FreeText);
TRN_API TRN_FreeTextAnnotSetIntentName(TRN_Annot ft, enum TRN_FreeTextAnnotIntentName mode)
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetIntentName((Annots::FreeText::IntentName)mode);
	EEX;
}

TRN_API TRN_FreeTextAnnotSetIntentNameDefault(TRN_Annot ft)
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetIntentName(Annots::FreeText::e_FreeText);
	EEX;
}


//EndingStyle GetEndingStyle() const;
TRN_API TRN_FreeTextAnnotGetEndingStyle(TRN_Annot ft, enum TRN_LineAnnotEndingStyle* result)
{
	BEX;
	FreeText f((Obj*)ft);
	*result= static_cast<TRN_LineAnnotEndingStyle>(f.GetEndingStyle());
	EEX;
}

//void SetEndingStyle(EndingStyle est);
TRN_API TRN_FreeTextAnnotSetEndingStyle(TRN_Annot ft, enum TRN_LineAnnotEndingStyle style)
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetEndingStyle( static_cast<Line::EndingStyle>(style) );
	EEX;
}

//void SetEndingStyle(const char* est);
TRN_API TRN_FreeTextAnnotSetEndingStyleName(TRN_Annot ft, const char* est)
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetEndingStyle(est);
	EEX;
}


TRN_API TRN_FreeTextAnnotSetTextColor( TRN_Annot ft, const TRN_ColorPt* color, int col_components )
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetTextColor( CPCast(color), col_components );
	EEX;
}

TRN_API TRN_FreeTextAnnotGetTextColor( TRN_Annot ft, TRN_ColorPt* out_color, int* out_col_components )
{
	BEX;
	ColorPt col;
	FreeText f((Obj*)ft);
	f.GetTextColor( col, *out_col_components );
	TRN_ColorPtAssign( out_color, (const TRN_ColorPt*)&col);
	EEX;
}

TRN_API TRN_FreeTextAnnotSetLineColor( TRN_Annot ft, const TRN_ColorPt* color, int col_components )
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetLineColor( CPCast(color), col_components );
	EEX;
}

TRN_API TRN_FreeTextAnnotGetLineColor( TRN_Annot ft, TRN_ColorPt* out_color, int* out_col_components )
{
	BEX;
	ColorPt col;
	FreeText f((Obj*)ft);
	f.GetLineColor( col, *out_col_components );
	TRN_ColorPtAssign( out_color, (const TRN_ColorPt*)&col);
	EEX;
}

TRN_API TRN_FreeTextAnnotSetFontSize( TRN_Annot ft, double font_size )
{
	BEX;
	FreeText f((Obj*)ft);
	f.SetFontSize(font_size);
	EEX;
}

TRN_API TRN_FreeTextAnnotGetFontSize( TRN_Annot ft, double* result )
{
	BEX;
	FreeText f((Obj*)ft);
	*result = f.GetFontSize();
	EEX;
}


//6. Highlight

//Highlight(SDF::Obj* d);	
TRN_API TRN_HighlightAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Highlight((Obj*)d).GetSDFObj());
	EEX;
}


//Highlight(const Annot& ann) : TextMarkup(ann.GetSDFObj()) {} 
TRN_API TRN_HighlightAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Highlight((Obj*)annot).GetSDFObj());
	EEX;
}


//static Highlight Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_HighlightAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Highlight::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}


//7. Ink
//Ink(SDF::Obj* d = 0);
TRN_API TRN_InkAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Ink((Obj*)d).GetSDFObj());
	EEX;
}

//Ink(const Annot& ann) : Markup(ann.GetSDFObj()) {} 
TRN_API TRN_InkAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Ink((Obj*)annot).GetSDFObj());
	EEX;
}

//static Ink Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_InkAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Ink::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}

//int GetPathCount()const;
TRN_API TRN_InkAnnotGetPathCount(TRN_Annot ink, int* result)
{
	BEX;
	Ink i((Obj*)ink);
	*result=i.GetPathCount();
	EEX;
}

//int GetPointCount(unsigned int pathindex)const;
TRN_API TRN_InkAnnotGetPointCount(TRN_Annot ink, int pathindex, int* result)
{
	BEX;
	Ink i((Obj*)ink);
	*result=i.GetPointCount(pathindex);
	EEX;
}

//Point GetPoint(unsigned int pathindex, unsigned int pointindex)const;
TRN_API TRN_InkAnnotGetPoint(TRN_Annot ink, unsigned int pathindex, unsigned int pointindex, TRN_Point* result)
{
	//BEX;
	//Ink i((Obj*)ink)
	//*result=(TRN_Point)i.GetPoint(pathindex, pointindex);
	//EEX;
	BEX;	
	Ink i((Obj*)ink);
	Point p(i.GetPoint(pathindex, pointindex));
	assert(sizeof(TRN_Point)>=sizeof(p));
	memcpy(result, &p, sizeof(p));
	EEX;
}

//void SetPoint(unsigned int pathindex, unsigned int pointindex, const Point& pt);
TRN_API TRN_InkAnnotSetPoint(TRN_Annot ink, unsigned int pathindex, unsigned int pointindex, const TRN_Point* point)
{
	BEX;
	Ink i((Obj*)ink);
	i.SetPoint(pathindex, pointindex, *((Point*)point));
	EEX;
}

//bool Erase(const Point& pt1, const Point& pt2, double width);
TRN_API TRN_InkAnnotErase(TRN_Annot ink, const TRN_Point* pt1, const TRN_Point* pt2, double width, TRN_Bool* result) 
{
	BEX;
	Ink i((Obj*)ink);
	*result=BToTB(i.Erase(*((Point*)pt1), *((Point*)pt2), width));
	EEX;
}

// bool GetHighlightIntent();
TRN_API TRN_InkAnnotGetHighlightIntent(TRN_InkAnnot ink, TRN_Bool* result)
{

	BEX;
	Ink i((Obj*)ink);
	*result = BToTB(i.GetHighlightIntent());
	EEX;
}

//	void SetHighlightIntent(bool highlight);
TRN_API TRN_InkAnnotSetHighlightIntent(TRN_InkAnnot ink, TRN_Bool highlight)
{
	BEX;
	Ink i((Obj*)ink);
	i.SetHighlightIntent(TBToB(highlight));
	EEX;
}

//8. Line

//Line::Line(SDF::Obj* d=0);
TRN_API TRN_LineAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Line((Obj*)d).GetSDFObj());
	EEX;
}

//Line(const Annot& ann) : Markup(ann.GetSDFObj()) {}	
TRN_API TRN_LineAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Line((Obj*)annot).GetSDFObj());
	EEX;
}

//static Line Create(SDF::SDFDoc& doc, const Rect& pos);		
TRN_API TRN_LineAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Line::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}

//Point GetStartPoint() const;
TRN_API TRN_LineAnnotGetStartPoint(TRN_Annot line, TRN_Point* result)
{
	//BEX;
	//Line l((Obj*)line);
	//*result=(TRN_Point)l.GetStartPoint();
	//EEX;
	BEX;	
	Line l((Obj*)line);
	Point p(l.GetStartPoint());
	assert(sizeof(TRN_Point)>=sizeof(p));
	memcpy(result, &p, sizeof(p));
	EEX;
}

//void SetStartPoint(const Point& sp);
TRN_API TRN_LineAnnotSetStartPoint(TRN_Annot line, const TRN_Point* stp)
{
	BEX;
	Line l((Obj*)line);
	l.SetStartPoint(PointCast(stp));
	EEX;
}

//Point GetEndPoint () const;
TRN_API TRN_LineAnnotGetEndPoint(TRN_Annot line, TRN_Point* result)
{
	//BEX;
	//Line l((Obj*)line);
	//*result=(TRN_Point)l.GetEndPoint();
	//EEX;
	BEX;	
	Line l((Obj*)line);
	Point p(l.GetEndPoint());
	assert(sizeof(TRN_Point)>=sizeof(p));
	memcpy(result, &p, sizeof(p));
	EEX;
}

//void SetEndPoint(const Point& ep);
TRN_API TRN_LineAnnotSetEndPoint(TRN_Annot line, const TRN_Point* etp)
{
	BEX;
	Line l((Obj*)line);
	l.SetEndPoint(PointCast(etp));
	EEX;
}

//EndingStyle GetStartStyle() const;
TRN_API TRN_LineAnnotGetStartStyle(TRN_Annot line, enum TRN_LineAnnotEndingStyle* result)
{
	BEX;
	Line l((Obj*)line);
	*result=(TRN_LineAnnotEndingStyle)l.GetStartStyle();
	EEX;
}

//void SetStartStyle(EndingStyle sst);
TRN_API TRN_LineAnnotSetStartStyle(TRN_Annot line, enum TRN_LineAnnotEndingStyle ss)
{
	BEX;
	Line l((Obj*)line);
	l.SetStartStyle((Annots::Line::EndingStyle)ss);
	EEX;
}

//EndingStyle GetEndStyle() const;
TRN_API TRN_LineAnnotGetEndStyle(TRN_Annot line, enum TRN_LineAnnotEndingStyle* result)
{
	BEX;
	Line l((Obj*)line);
	*result=(TRN_LineAnnotEndingStyle)l.GetEndStyle();
	EEX;
}

//void SetEndStyle(EndingStyle est);
TRN_API TRN_LineAnnotSetEndStyle(TRN_Annot line, enum TRN_LineAnnotEndingStyle es)
{
	BEX;
	Line l((Obj*)line);
	l.SetEndStyle((Annots::Line::EndingStyle)es);
	EEX;
}


//double GetLeaderLineLength() const;
TRN_API TRN_LineAnnotGetLeaderLineLength(TRN_Annot line, double* result)
{
	BEX;
	Line l((Obj*)line);
	*result=l.GetLeaderLineLength();
	EEX;
}

//void SetLeaderLineLength(const double& ll);
TRN_API TRN_LineAnnotSetLeaderLineLength(TRN_Annot line, double length)
{
	BEX;
	Line l((Obj*)line);
	l.SetLeaderLineLength(length);
	EEX;
}

//double GetLeaderLineExtensionLength() const;
TRN_API TRN_LineAnnotGetLeaderLineExtensionLength(TRN_Annot line, double* result)
{
	BEX;
	Line l((Obj*)line);
	*result=l.GetLeaderLineExtensionLength();
	EEX;
}

//void SetLeaderLineExtensionLength(const double& ll);
TRN_API TRN_LineAnnotSetLeaderLineExtensionLength(TRN_Annot line, double length)
{
	BEX;
	Line l((Obj*)line);
	l.SetLeaderLineExtensionLength(length);
	EEX;
}

//bool GetShowCaption() const;
TRN_API TRN_LineAnnotGetShowCaption(TRN_Annot line, TRN_Bool* result)
{
	BEX;
	Line l((Obj*)line);
	*result=BToTB(l.GetShowCaption());
	EEX;
}


//void SetShowCaption(bool showCaption); 
TRN_API TRN_LineAnnotSetShowCaption(TRN_Annot line, TRN_Bool sorn)
{
	BEX;
	Line l((Obj*)line);
	l.SetShowCaption(TBToB(sorn));
	EEX;
}


//IntentType GetIntentType() const;
TRN_API TRN_LineAnnotGetIntentType(TRN_Annot line, enum TRN_LineAnnotIntentType* result)
{
	BEX;
	Line l((Obj*)line);
	*result=(TRN_LineAnnotIntentType)l.GetIntentType();
	EEX;
}

//void SetIntentType(IntentType style);
TRN_API TRN_LineAnnotSetIntentType(TRN_Annot line, enum TRN_LineAnnotIntentType it)
{
	BEX;
	Line l((Obj*)line);
	l.SetIntentType((Annots::Line::IntentType)it);
	EEX;
}

//CapPos GetCaptionPosition() const;
TRN_API TRN_LineAnnotGetCapPos(TRN_Annot line, enum TRN_LineAnnotCapPos* result)
{
	BEX;
	Line l((Obj*)line);
	*result=(TRN_LineAnnotCapPos)l.GetCaptionPosition();
	EEX;
}

//void SetCapPos(const CapPos& style);
TRN_API TRN_LineAnnotSetCapPos(TRN_Annot line, enum TRN_LineAnnotCapPos it)
{
	BEX;
	Line l((Obj*)line);
	l.SetCaptionPosition((Annots::Line::CapPos)it);
	EEX;
}

//double GetLeaderLineOffset() const;
TRN_API TRN_LineAnnotGetLeaderLineOffset(TRN_Annot line, double* result)
{
	BEX;
	Line l((Obj*)line);
	*result=l.GetLeaderLineOffset();
	EEX;
}

//void SetLeaderLineOffset(double ll);
TRN_API TRN_LineAnnotSetLeaderLineOffset(TRN_Annot line, double length)
{
	BEX;
	Line l((Obj*)line);
	l.SetLeaderLineOffset(length);
	EEX;
}


//double GetTextHOffset() const;
TRN_API TRN_LineAnnotGetTextHOffset(TRN_Annot line, double* result)
{
	BEX;
	Line l((Obj*)line);
	*result=l.GetTextHOffset();
	EEX;
}

//void SetTextHOffset(double offset);
TRN_API TRN_LineAnnotSetTextHOffset(TRN_Annot line, double offset)
{
	BEX;
	Line l((Obj*)line);
	l.SetTextHOffset(offset);
	EEX;
}

//double GetTextVOffset() const;
TRN_API TRN_LineAnnotGetTextVOffset(TRN_Annot line, double* result)
{
	BEX;
	Line l((Obj*)line);
	*result=l.GetTextVOffset();
	EEX;
}

//void SetTextVOffset(double offset);
TRN_API TRN_LineAnnotSetTextVOffset(TRN_Annot line, double offset)
{
	BEX;
	Line l((Obj*)line);
	l.SetTextVOffset(offset);
	EEX;
}



//9. Link

//TRN_API TRN_AnnotGetLinkAction(TRN_Annot annot, TRN_Action* result)
//{
//	BEX;
//	*result=(TRN_Action)IMPL_TRN_AnnotGetLinkAction(AnnCast(annot));
//	EEX;
//}
//
//TRN_API TRN_AnnotSetLinkAction(TRN_Annot annot, const TRN_Action action)
//{
//	BEX;
//	IMPL_TRN_AnnotSetLinkAction(AnnCast(annot),ActCast(action));
//	EEX;
//}

//Link(SDF::Obj* d = 0);
TRN_API TRN_LinkAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Link((Obj*)d).GetSDFObj());
	EEX;
}

//Link(const Annot& ann) : Annot(ann.GetSDFObj()) {} 
TRN_API TRN_LinkAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Link((Obj*)annot).GetSDFObj());
	EEX;
}

//static Link Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_LinkAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Link::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}


////static Link Create(SDF::SDFDoc& doc, const Rect& pos, const Action& action);
//TRN_API TRN_LinkAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Action action, TRN_Annot* result)
//{
//	BEX;
//	Link link=Link::Create(DocCast(doc), *((Rect*)pos));
//	link.SetAction(ActCast(action));
//	*result=(TRN_Annot)(link.GetSDFObj());
//	EEX;
//}

//void RemoveAction();
TRN_API TRN_LinkAnnotRemoveAction(TRN_Annot link/*, TRN_Action action*/)
{
	BEX;
	Link l((Obj*)link);
	l.RemoveAction(/*ActCast(action*//*)*/);
	EEX;
}

//Action GetAction() const;
TRN_API TRN_LinkAnnotGetAction(TRN_Annot link, TRN_Action* result)
{
	BEX;
	Link l((Obj*)link);
	*result=(TRN_Action)l.GetAction().GetSDFObj();
	EEX;
}

//void SetAction(const Action& action);
TRN_API TRN_LinkAnnotSetAction(TRN_Annot link, TRN_Action action)
{
	BEX;
	Link l((Obj*)link);
	l.SetAction(ActCast(action));
	EEX;
}

//TRN_LinkAnnotHighlightingMode
//HighlightingMode GetHighlightingMode() const;
TRN_API TRN_LinkAnnotGetHighlightingMode(TRN_Annot link, enum TRN_LinkAnnotHighlightingMode* result)
{
	BEX;
	Link l((Obj*)link);
	*result=(TRN_LinkAnnotHighlightingMode)l.GetHighlightingMode();
	EEX;
}

//void SetHighlightingMode(HighlightingMode mode);
TRN_API TRN_LinkAnnotSetHighlightingMode(TRN_Annot link, enum TRN_LinkAnnotHighlightingMode value)
{
	BEX;
	Link l((Obj*)link);
	l.SetHighlightingMode((Annots::Link::HighlightingMode)value);
	EEX;
}

//int GetQuadPointCount() const;
TRN_API TRN_LinkAnnotGetQuadPointCount(TRN_Annot link, int* result)
{
	BEX;
	Link l((Obj*)link);
	*result=l.GetQuadPointCount();
	EEX;
}

//QuadPoint GetQuadPoint(int idx) const;
TRN_API TRN_LinkAnnotGetQuadPoint(TRN_Annot link, int idx, TRN_QuadPoint* result)
{
	//BEX;
	//Link l((Obj*)link);
	//*result=(TRN_QuadPoint)l.GetQuadPoint(idx);
	//EEX;
	BEX;	
	Link l((Obj*)link);
	QuadPoint qp(l.GetQuadPoint(idx));
	assert(sizeof(TRN_QuadPoint)>=sizeof(qp));
	memcpy(result, &qp, sizeof(qp));
	EEX;
}

//void SetQuadPoint(int idx, const QuadPoint& qp); 
TRN_API TRN_LinkAnnotSetQuadPoint(TRN_Annot link, int idx, const TRN_QuadPoint* qp)
{
	BEX;
	Link l((Obj*)link);
	l.SetQuadPoint(idx, QPCast(qp));
	EEX;
}

TRN_API TRN_GetNormalizedUrl(const TRN_UString url, TRN_UString* result)
{
    BEX;
    UString temp_in(UStrCast(url));
	std::wstring temp_out = Link::GetNormalizedUrl(temp_in.ConvertToNativeWString());
	UString out(temp_out);
	RetUStr(out);
    EEX;
}

//10. Markup

//Markup(SDF::Obj* d = 0);
TRN_API TRN_MarkupAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Markup((Obj*)d)).GetSDFObj();
	EEX;
}


//Markup(const Annot& ann) : Annot(ann.GetSDFObj()) {} 
TRN_API TRN_MarkupAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Markup((Obj*)annot).GetSDFObj());
	EEX;
}

//const char* GetTitle();
TRN_API TRN_MarkupAnnotGetTitle(TRN_Annot markup, TRN_UString* result )
{
	BEX;
	Markup mu((Obj*)markup);	
	RetUStr(mu.GetTitle());
	EEX;
}

//void SetTitle(const char* title)
TRN_API TRN_MarkupAnnotSetTitle(TRN_Annot markup, const char* title)
{
	BEX;
	Markup mu((Obj*)markup);
	mu.SetTitle(title);
	EEX;
}


TRN_API TRN_MarkupAnnotSetTitleUString(TRN_Annot markup, TRN_UString title)
{
	BEX;
	Markup mu((Obj*)markup);	
	UString temp(UStrCast(title));
	mu.SetTitle(temp);
	EEX;
}

//Popup GetPopup() const; //Note, popup is a kind of annotation. 
TRN_API TRN_MarkupAnnotGetPopup(TRN_Annot markup, TRN_Annot* result)
{
	BEX;
	Markup mu((Obj*)markup);	
	*result=(TRN_Annot)mu.GetPopup().GetSDFObj();
	EEX;
}

//void SetPopup(const Popup& bs);
TRN_API TRN_MarkupAnnotSetPopup(TRN_Annot markup, TRN_Annot ppup)
{
	BEX;
	Markup mu((Obj*)markup);	
	mu.SetPopup(AnnCast(ppup));
	EEX;
}

//double GetOpacity() const;
TRN_API TRN_MarkupAnnotGetOpacity(TRN_Annot markup, double* result)
{
	BEX;
	Markup mu((Obj*)markup);	
	*result=mu.GetOpacity();
	EEX;
}

//void SetOpacity(double op);
TRN_API TRN_MarkupAnnotSetOpacity(TRN_Annot markup, double op)
{
	BEX;
	Markup mu((Obj*)markup);	
	mu.SetOpacity(op);
	EEX;
}

//bool Export(const UString& save_as = "") const;
//TRN_API TRN_AnnotGetTextContents(TRN_Annot annot, TRN_Obj* result)
//{
//	BEX;
//	*result=(TRN_Obj)IMPL_TRN_AnnotGetTextContents(AnnCast(annot));
//	EEX;
//}

//TRN_API TRN_AnnotSetTextContents(TRN_Annot annot, const TRN_UString contents)
//{
//	BEX;
//	UString temp(UStrCast(contents));
//	IMPL_TRN_AnnotSetTextContents(AnnCast(annot),temp);
//	EEX;
//}
//UString GetSubject() const;

//TRN_API TRN_SElementGetActualText(const TRN_SElement* e, TRN_UString* result)
//{
//	BEX;
//	RetUStr(SEleCast(e).GetActualText());
//	EEX;
//}
TRN_API TRN_MarkupAnnotGetSubject(TRN_Annot markup, TRN_UString* result)
{
	BEX;
	Markup mu((Obj*)markup);	
	RetUStr(mu.GetSubject());
	EEX;
}

//void SetSubject(const UString& subj)
TRN_API TRN_MarkupAnnotSetSubject(TRN_Annot markup, TRN_UString contents)
{
	BEX;
	Markup mu((Obj*)markup);	
	UString temp(UStrCast(contents));
	mu.SetSubject(temp);
	EEX;
}

//TRN_API TRN_AnnotGetDate(TRN_Annot annot, TRN_Date* result)
//{
//	BEX;
//	Date d(IMPL_TRN_AnnotGetDate(AnnCast(annot)));
//	assert(sizeof(TRN_Date)>=sizeof(d));
//	memcpy(result, &d, sizeof(d));
//	EEX;
//}
//TRN_API TRN_AnnotSetDate(TRN_Annot annot, const TRN_Date* date)
//{
//	BEX;
//	IMPL_TRN_AnnotSetDate(AnnCast(annot),DateCast(date));
//	EEX;
//}
//Date GetCreationDates() const;
TRN_API TRN_MarkupAnnotGetCreationDates(TRN_Annot markup, TRN_Date* result)
{
	BEX;	
	Markup mu((Obj*)markup);
	Date d(mu.GetCreationDates());
	assert(sizeof(TRN_Date)>=sizeof(d));
	memcpy(result, &d, sizeof(d));
	EEX;
}

//void SetCreationDates(const Date& dt);
TRN_API TRN_MarkupAnnotSetCreationDates(TRN_Annot markup, const TRN_Date* date)
{
	BEX;
	Markup mu((Obj*)markup);
	mu.SetCreationDates(DateCast(date));
	EEX
}

//BorderEffect GetBorderEffect() const;
TRN_API TRN_MarkupAnnotGetBorderEffect(TRN_Annot markup, enum TRN_MarkupAnnotBorderEffect* result)
{
	BEX;
	Markup mu((Obj*)markup);
	*result=(TRN_MarkupAnnotBorderEffect)mu.GetBorderEffect();
	EEX;
}

//void SetBorderEffect(BorderEffect effect);
TRN_API TRN_MarkupAnnotSetBorderEffect(TRN_Annot markup, enum TRN_MarkupAnnotBorderEffect effect)
{
	BEX;
	Markup mu((Obj*)markup);
	mu.SetBorderEffect((Annots::Markup::BorderEffect)effect);
	EEX;
}

//double GetBorderEffectIntensity() const;
TRN_API TRN_MarkupAnnotGetBorderEffectIntensity(TRN_Annot markup, double* result)
{
	BEX;
	Markup mu((Obj*)markup);
	*result=mu.GetBorderEffectIntensity();
	EEX;
}

//void SetBorderEffectIntentsity(double intensity);
TRN_API TRN_MarkupAnnotSetBorderEffectIntensity(TRN_Annot markup, double intensity)
{
	BEX;
	Markup mu((Obj*)markup);
	mu.SetBorderEffectIntensity(intensity);
	EEX;
}

//11. Movie

//Movie(SDF::Obj* d = 0);	
TRN_API TRN_MovieAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Movie((Obj*)d).GetSDFObj());
	EEX;
}

//Movie(const Annot& ann) : Annot(ann.GetSDFObj()) {} 
TRN_API TRN_MovieAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Movie((Obj*)annot).GetSDFObj());
	EEX;
}

//static Movie Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_MovieAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Movie::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}


//UString GetTitle() const;
TRN_API TRN_MovieAnnotGetTitle(TRN_Annot movie, TRN_UString* result)
{
	BEX;
	Movie mu((Obj*)movie);	
	RetUStr(mu.GetTitle());
	EEX;
}

//void SetTitle(const UString& title);
TRN_API TRN_MovieAnnotSetTitle(TRN_Annot movie, TRN_UString contents)
{
	BEX;
	Movie mu((Obj*)movie);	
	UString temp(UStrCast(contents));
	mu.SetTitle(temp);
	EEX;
}


//bool IsToBePlayed() const;
TRN_API TRN_MovieAnnotIsToBePlayed(TRN_Annot movie, TRN_Bool* result)
{
	BEX;
	Movie mu((Obj*)movie);	
	*result=BToTB(mu.IsToBePlayed());
	EEX;
}

//void SetToBePlayed(bool isplay=true);
TRN_API TRN_MovieAnnotSetToBePlayed(TRN_Annot movie, TRN_Bool playono)
{
	BEX;
	Movie mu((Obj*)movie);	
	mu.SetToBePlayed(TBToB(playono));
	EEX;
}


//12. PolyLine
//PolyLine(SDF::Obj* d = 0);	
TRN_API TRN_PolyLineAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(PolyLine((Obj*)d).GetSDFObj());
	EEX;
}

//PolyLine(const Annot& ann) : Markup(ann.GetSDFObj()) {} 
TRN_API TRN_PolyLineAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(PolyLine((Obj*)annot).GetSDFObj());
	EEX;
}

//static PolyLine Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_PolyLineAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(PolyLine::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}

//int GetVertexCount() const;
TRN_API TRN_PolyLineAnnotGetVertexCount(TRN_Annot polyline, int* result)
{
	BEX;
	PolyLine pl((Obj*)polyline);	
	*result=pl.GetVertexCount();
	EEX;
}

//Point GetVertex(int idx) const; //Note the default should be just a single rect that is equal to Rect entry. 
TRN_API TRN_PolyLineAnnotGetVertex(TRN_Annot polyline, int idx, TRN_Point* result)
{
	//BEX;
	//PolyLine l((Obj*)polyline);
	//*result=(TRN_Point)l.GetVertex(idx);
	//EEX;
	BEX;	
	PolyLine l((Obj*)polyline);
	Point p(l.GetVertex(idx));
	assert(sizeof(TRN_Point)>=sizeof(p));
	memcpy(result, &p, sizeof(p));
	EEX;
}

//void SetVertex(int idx, const Point& pt);
TRN_API TRN_PolyLineAnnotSetVertex(TRN_Annot polyline, int idx, const TRN_Point* pt)
{
	BEX;
	PolyLine l((Obj*)polyline);
	l.SetVertex(idx, PointCast(pt));
	EEX;
}

//EndingStyle GetStartStyle() const;
TRN_API TRN_PolyLineAnnotGetStartStyle(TRN_Annot polyline, enum TRN_LineAnnotEndingStyle* result)
{
	BEX;
	PolyLine l((Obj*)polyline);
	*result=(TRN_LineAnnotEndingStyle)l.GetStartStyle();
	EEX;
}

//void SetStartStyle(EndingStyle sst);
TRN_API TRN_PolyLineAnnotSetStartStyle(TRN_Annot polyline, enum TRN_LineAnnotEndingStyle style)
{
	BEX;
	PolyLine l((Obj*)polyline);
	l.SetStartStyle((Annots::PolyLine::EndingStyle)style);
	EEX;
}

//EndingStyle GetEndStyle() const;
TRN_API TRN_PolyLineAnnotGetEndStyle(TRN_Annot polyline, enum TRN_LineAnnotEndingStyle* result)
{
	BEX;
	PolyLine l((Obj*)polyline);
	*result=(TRN_LineAnnotEndingStyle)l.GetEndStyle();
	EEX;
}

//void SetEndStyle(EndingStyle est);
TRN_API TRN_PolyLineAnnotSetEndStyle(TRN_Annot polyline, enum TRN_LineAnnotEndingStyle style)
{
	BEX;
	PolyLine l((Obj*)polyline);
	l.SetEndStyle((Annots::PolyLine::EndingStyle)style);
	EEX;
}

//IntentType GetIntentName() const;
TRN_API TRN_PolyLineAnnotGetIntentName(TRN_Annot polyline, enum TRN_PolyLineAnnotIntentType* result)
{
	BEX;
	PolyLine l((Obj*)polyline);
	*result=(TRN_PolyLineAnnotIntentType)l.GetIntentName();
	EEX;
}

//void SetIntentName(IntentType mode);
TRN_API TRN_PolyLineAnnotSetIntentName(TRN_Annot polyline, enum TRN_PolyLineAnnotIntentType style)
{
	BEX;
	PolyLine l((Obj*)polyline);
	l.SetIntentName((Annots::PolyLine::IntentType)style);
	EEX;
}



//Polygon(SDF::Obj* d = 0);	
TRN_API TRN_PolygonAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Polygon((Obj*)d).GetSDFObj());
	EEX;
}

//Polygon(const Annot& ann) : Markup(ann.GetSDFObj()) {} 
TRN_API TRN_PolygonAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Polygon((Obj*)annot).GetSDFObj());
	EEX;
}

//static Polygon Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_PolygonAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Polygon::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}



//13. Popup

//Popup(SDF::Obj* d = 0);	
TRN_API TRN_PopupAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Popup((Obj*)d).GetSDFObj());
	EEX;
}

//Popup(const Annot& ann) : Annot(ann.GetSDFObj()) {}
TRN_API TRN_PopupAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Popup((Obj*)annot).GetSDFObj());
	EEX;
}

//static Popup Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_PopupAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Popup::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}


//Annot GetParent() const;
TRN_API TRN_PopupAnnotGetParent(TRN_Annot popup, TRN_Annot* result)
{
	BEX;
	Popup pp((Obj*)popup);	
	*result=(TRN_Annot)(pp.GetParent()).GetSDFObj();
	EEX;
}

//void SetParent(const Annot& parent);
TRN_API TRN_PopupAnnotSetParent(TRN_Annot popup, TRN_Annot parent)
{
	BEX;
	Popup pp((Obj*)popup);		
	pp.SetParent(AnnCast(parent));
	EEX;
}

//bool IsOpen() const;
TRN_API TRN_PopupAnnotIsOpen(TRN_Annot popup, TRN_Bool* result)
{
	BEX;
	Popup p((Obj*)popup);	
	*result=BToTB(p.IsOpen());
	EEX;
}

//void SetOpen(bool is_open);
TRN_API TRN_PopupAnnotSetOpen(TRN_Annot popup, TRN_Bool closeono)
{
	BEX;
	Popup p((Obj*)popup);	
	p.SetOpen(TBToB(closeono));
	EEX;
}



//14. Redaction

//Redaction(SDF::Obj* d = 0);	
TRN_API TRN_RedactionAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Redaction((Obj*)d).GetSDFObj());
	EEX;
}

//Redaction(const Annot& ann) : Markup(ann.GetSDFObj()) {} 
TRN_API TRN_RedactionAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Redaction((Obj*)annot).GetSDFObj());
	EEX;
}

//static Redaction Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_RedactionAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Redaction::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}





//int GetQuadPointCount();
TRN_API TRN_RedactionAnnotGetQuadPointCount(TRN_Annot redaction, int* result)
{
	BEX;
	Redaction r((Obj*)redaction);
	*result=r.GetQuadPointCount();
	EEX;
}


//QuadPoint GetQuadPoint(int idx) const;
TRN_API TRN_RedactionAnnotGetQuadPoint(TRN_Annot redaction, int idx, TRN_QuadPoint* result)
{
	//BEX;
	//Redaction r((Obj*)redaction);
	//*result=(TRN_QuadPoint)r.GetQuadPoint(idx);
	//EEX;
	BEX;	
	Redaction r((Obj*)redaction);
	QuadPoint qp(r.GetQuadPoint(idx));
	assert(sizeof(TRN_QuadPoint)>=sizeof(qp));
	memcpy(result, &qp, sizeof(qp));
	EEX;
}

//void SetQuadPoint(int idx, const QuadPoint& pt); 
TRN_API TRN_RedactionAnnotSetQuadPoint(TRN_Annot redaction, int idx, const TRN_QuadPoint* qp)
{
	BEX;
	Redaction r((Obj*)redaction);
	r.SetQuadPoint(idx, QPCast(qp));
	EEX;
}

//ColorPt GetInteriorColor() const;
TRN_API TRN_MarkupAnnotGetInteriorColor(TRN_Annot markup, TRN_ColorPt* result)
{
	BEX;
	Markup ci((Obj*)markup);
	ColorPt c(ci.GetInteriorColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

//int GetInteriorColorCompNum() const;
TRN_API TRN_MarkupAnnotGetInteriorColorCompNum(TRN_Annot markup, int* result)
{
	BEX;
	Markup ci((Obj*)markup);
	*result=ci.GetInteriorColorCompNum();
	EEX;
}

//void SetInteriorColor(const ColorPt& c, int cs=3);
TRN_API TRN_MarkupAnnotSetInteriorColorRGB(TRN_Annot markup, const TRN_ColorPt* col)
{
	BEX;
	Markup c((Obj*)markup);
	c.SetInteriorColor(CPCast(col), 3);
	EEX;
}

TRN_API TRN_MarkupAnnotSetInteriorColor(TRN_Annot markup, const TRN_ColorPt* col, int numcomp)
{
	BEX;
	Markup c((Obj*)markup);
	c.SetInteriorColor(CPCast(col), numcomp);
	EEX;
}



//Rect GetContentRect() const;
TRN_API TRN_MarkupAnnotGetContentRect(TRN_Annot markup, TRN_Rect* result)
{
	BEX;
	Markup c((Obj*)markup);
	Rect r(c.GetContentRect());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}


//void SetContentRect(const Rect& cr);
TRN_API TRN_MarkupAnnotSetContentRect(TRN_Annot markup, const TRN_Rect* cr)
{
	BEX;
	Markup c((Obj*)markup);
	c.SetContentRect(RectCast(cr));
	EEX;
}



//Rect GetPadding() const;
TRN_API TRN_MarkupAnnotGetPadding(TRN_Annot markup, TRN_Rect* result)
{
	BEX;
	Markup c((Obj*)markup);
	Rect r(c.GetPadding());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}


//void SetPadding(const Rect& rd);
TRN_API TRN_MarkupAnnotSetPadding(TRN_Annot markup, const TRN_Rect* cr)
{
	BEX;
	Markup c((Obj*)markup);
	c.SetPadding(RectCast(cr));
	EEX;
}

//void RotateAppearance(double angle);
TRN_API TRN_MarkupAnnotRotateAppearance(TRN_MarkupAnnot markup, double angle)
{
	BEX;
	Markup c((Obj*)markup);
	c.RotateAppearance(angle);
	EEX;
}

//SDF::Obj* GetAppFormXO()const;
TRN_API TRN_RedactionAnnotGetAppFormXO(TRN_Annot redaction, TRN_Obj* result)
{
	BEX;
	Redaction r((Obj*)redaction);
	*result=(TRN_Obj)r.GetAppFormXO();
	EEX;
}

//void SetAppFormXO(/*const*/ SDF::Obj* formxo);
TRN_API TRN_RedactionAnnotSetAppFormXO(TRN_Annot redaction, TRN_Obj formxo)
{
	BEX;
	Redaction r((Obj*)redaction);
	r.SetAppFormXO((Obj*) formxo);
	EEX;
}

//UString GetOverlayText() const;
TRN_API TRN_RedactionAnnotGetOverlayText(TRN_Annot redaction, TRN_UString* result)
{
	BEX;
	Redaction mu((Obj*)redaction);	
	RetUStr(mu.GetOverlayText());
	EEX;
}

//void SetOverlayText(const UString& title);
TRN_API TRN_RedactionAnnotSetOverlayText(TRN_Annot redaction, TRN_UString contents)
{
	BEX;
	Redaction mu((Obj*)redaction);	
	UString temp(UStrCast(contents));
	mu.SetOverlayText(temp);
	EEX;
}

//bool GetUseRepeat() const;
TRN_API TRN_RedactionAnnotGetUseRepeat(TRN_Annot redaction, TRN_Bool* result)
{
	BEX;
	Redaction p((Obj*)redaction);	
	*result=BToTB(p.GetUseRepeat());
	EEX;
}

//void SetUseRepeat(bool repeat=false);
TRN_API TRN_RedactionAnnotSetUseRepeat(TRN_Annot redaction, TRN_Bool closeono)
{
	BEX;
	Redaction p((Obj*)redaction);	
	p.SetUseRepeat(TBToB(closeono));
	EEX;
}

//UString GetOverlayTextAppearance() const;
TRN_API TRN_RedactionAnnotGetOverlayTextAppearance(TRN_Annot redaction, TRN_UString* result)
{
	BEX;
	Redaction mu((Obj*)redaction);	
	RetUStr(mu.GetOverlayTextAppearance());
	EEX;
}

//void SetOverlayTextAppearance(const UString& app);
TRN_API TRN_RedactionAnnotSetOverlayTextAppearance(TRN_Annot redaction, TRN_UString contents)
{
	BEX;
	Redaction mu((Obj*)redaction);	
	UString temp(UStrCast(contents));
	mu.SetOverlayTextAppearance(temp);
	EEX;
}


//QuadForm GetQuadForm() const;
TRN_API TRN_RedactionAnnotGetQuadForm(TRN_Annot redaction, enum TRN_RedactionAnnotQuadForm* result)
{
	BEX;
	Redaction l((Obj*)redaction);
	*result=(TRN_RedactionAnnotQuadForm)l.GetQuadForm();
	EEX;
}

//void SetQuadForm(QuadForm form=e_LeftJustified);
TRN_API TRN_RedactionAnnotSetQuadForm(TRN_Annot redaction, enum TRN_RedactionAnnotQuadForm style)
{
	BEX;
	Redaction l((Obj*)redaction);
	l.SetQuadForm((Annots::Redaction::QuadForm)style);
	EEX;
}


//15. RubberStamp

//RubberStamp(SDF::Obj* d = 0);
TRN_API TRN_RubberStampAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(RubberStamp((Obj*)d).GetSDFObj());
	EEX;
}


//RubberStamp(const Annot& ann) : Markup(ann.GetSDFObj()) {} 
TRN_API TRN_RubberStampAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(RubberStamp((Obj*)annot).GetSDFObj());
	EEX;
}


//static RubberStamp Create(SDF::SDFDoc& doc, const Rect& pos, Icon icon= e_Draft);
//TRN_API TRN_RubberStampAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, enum TRN_RubberStamp stp, TRN_Annot* result)
//{
//	BEX;
//	RubberStamp stamp=RubberStamp::Create(DocCast(doc), *((Rect*)pos));
//	stamp.SetIcon((Annots::RubberStamp::Icon)stp);
//	*result=(TRN_Annot)(stamp.GetSDFObj());
//	EEX;
//}

//Create, default
TRN_API TRN_RubberStampAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	RubberStamp stamp=RubberStamp::Create(DocCast(doc), *((Rect*)pos));
	stamp.SetIcon(Annots::RubberStamp::e_Draft);
	*result=(TRN_Annot)(stamp.GetSDFObj());
	EEX;
}

//Icon GetIcon() const;
TRN_API TRN_RubberStampAnnotGetIcon(TRN_Annot stamp, enum TRN_RubberStampAnnotIcon* result)
{
	BEX;
	RubberStamp l((Obj*)stamp);
	*result=(TRN_RubberStampAnnotIcon)(l.GetIcon());
	EEX;
}

//TRN_API TRN_RedactionAnnotGetQuadForm(TRN_Annot redaction, enum TRN_RedactionAnnotQuadForm* result)
//{
//	BEX;
//	Redaction l((Obj*)redaction);
//	*result=(TRN_RedactionAnnotQuadForm)l.GetQuadForm();
//	EEX;
//}

//void SetIcon(Icon type=e_Draft);
TRN_API TRN_RubberStampAnnotSetIcon(TRN_Annot stamp, enum TRN_RubberStampAnnotIcon style)
{
	BEX;
	RubberStamp l((Obj*)stamp);
	l.SetIcon((Annots::RubberStamp::Icon)style);
	EEX;
}


//SetIconDefault. 
TRN_API TRN_RubberStampAnnotSetIconDefault(TRN_Annot stamp)
{
	BEX;
	RubberStamp l((Obj*)stamp);
	l.SetIcon(RubberStamp::e_Draft);
	EEX;
}

//const char* GetIconName() const; 
TRN_API TRN_RubberStampAnnotGetIconName(TRN_Annot stamp, const char** result)
{
	BEX;
	RubberStamp l((Obj*)stamp);
	*result=l.GetIconName();
	EEX;
}

//void SetIcon(const char* icon);
TRN_API TRN_RubberStampAnnotSetIconName(TRN_Annot stamp, const char* style)
{
	BEX;
	RubberStamp l((Obj*)stamp);
	l.SetIcon(style);
	EEX;
}




//16. Screen

//Screen(SDF::Obj* d = 0);
TRN_API TRN_ScreenAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Screen((Obj*)d).GetSDFObj());
	EEX;
}

//Screen(const Annot& ann) : Annot(ann.GetSDFObj()) {} 
TRN_API TRN_ScreenAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Screen((Obj*)annot).GetSDFObj());
	EEX;
}

//static Screen Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_ScreenAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Screen::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}


//UString GetTitle() const;
TRN_API TRN_ScreenAnnotGetTitle(TRN_Annot s, TRN_UString* result)
{
	BEX;
	Screen mu((Obj*)s);	
	RetUStr(mu.GetTitle());
	EEX;
}

//void SetTitle(const UString& title);
TRN_API TRN_ScreenAnnotSetTitle(TRN_Annot s, TRN_UString contents)
{
	BEX;
	Screen mu((Obj*)s);	
	UString temp(UStrCast(contents));
	mu.SetTitle(temp);
	EEX;
}


//Action GetAction() const;
TRN_API TRN_ScreenAnnotGetAction(TRN_Annot screen, TRN_Action* result)
{
	BEX;
	Screen l((Obj*)screen);
	*result=(TRN_Action)l.GetAction().GetSDFObj();
	EEX;
}

//void SetAction(const Action& action);
TRN_API TRN_ScreenAnnotSetAction(TRN_Annot screen, TRN_Action action)
{
	BEX;
	Screen l((Obj*)screen);
	l.SetAction(ActCast(action));
	EEX;
}

//ColorPt GetBorderColor() const;
TRN_API TRN_ScreenAnnotGetBorderColor(TRN_Annot screen, TRN_ColorPt* result)
{
	BEX;
	Screen ci((Obj*)screen);
	ColorPt c(ci.GetBorderColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

//void SetBorderColor(const ColorPt& c, int CompNum); 	
TRN_API TRN_ScreenAnnotSetBorderColor(TRN_Annot screen, const TRN_ColorPt* col, int numcomp)
{
	BEX;
	Screen c((Obj*)screen);
	c.SetBorderColor(CPCast(col), numcomp);
	EEX;
}

//int GetBorderColorCompNum() const;
TRN_API TRN_ScreenAnnotGetBorderColorCompNum(TRN_Annot screen, int* result)
{
	BEX;
	Screen ci((Obj*)screen);
	*result=ci.GetBorderColorCompNum();
	EEX;
}

//ColorPt GetBackgroundColor() const;
TRN_API TRN_ScreenAnnotGetBackgroundColor(TRN_Annot screen, TRN_ColorPt* result)
{
	BEX;
	Screen ci((Obj*)screen);
	ColorPt c(ci.GetBackgroundColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

//void SetBackgroundColor(const ColorPt& c, int CompNum); 
TRN_API TRN_ScreenAnnotSetBackgroundColor(TRN_Annot screen, const TRN_ColorPt* col, int numcomp)
{
	BEX;
	Screen c((Obj*)screen);
	c.SetBackgroundColor(CPCast(col), numcomp);
	EEX;
}

//ColorPt GetBackgroundColorCompNum() const;
TRN_API TRN_ScreenAnnotGetBackgroundColorCompNum(TRN_Annot screen, int* result)
{
	BEX;
	Screen ci((Obj*)screen);
	*result=ci.GetBackgroundColorCompNum();
	EEX;
}


//void SetStaticCaptionText(const UString& ct);
TRN_API TRN_ScreenAnnotSetStaticCaptionText(TRN_Annot screen, TRN_UString contents)
{
	BEX;
	Screen mu((Obj*)screen);	
	UString temp(UStrCast(contents));
	mu.SetStaticCaptionText(temp);
	EEX;
}

TRN_API TRN_ScreenAnnotGetStaticCaptionText(TRN_Annot screen, TRN_UString* result)
{
	BEX;
	Screen mu((Obj*)screen);	
	RetUStr(mu.GetStaticCaptionText());
	EEX;
}

//UString GetRolloverCaptionText() const;
TRN_API TRN_ScreenAnnotGetRolloverCaptionText(TRN_Annot screen, TRN_UString* result)
{
	BEX;
	Screen mu((Obj*)screen);	
	RetUStr(mu.GetRolloverCaptionText());
	EEX;
}

//void SetRolloverCaptionText(const UString& ct);
TRN_API TRN_ScreenAnnotSetRolloverCaptionText(TRN_Annot screen, TRN_UString contents)
{
	BEX;
	Screen mu((Obj*)screen);	
	UString temp(UStrCast(contents));
	mu.SetRolloverCaptionText(temp);
	EEX;
}

//UString GetMouseDownCaptionText() const;
TRN_API TRN_ScreenAnnotGetMouseDownCaptionText(TRN_Annot screen, TRN_UString* result)
{
	BEX;
	Screen mu((Obj*)screen);	
	RetUStr(mu.GetMouseDownCaptionText());
	EEX;
}

//void SetMouseDownCaptionText(const UString& ct);
TRN_API TRN_ScreenAnnotSetMouseDownCaptionText(TRN_Annot screen, TRN_UString contents)
{
	BEX;
	Screen mu((Obj*)screen);	
	UString temp(UStrCast(contents));
	mu.SetMouseDownCaptionText(temp);
	EEX;
}



//void SetStaticIcon(SDF::Obj* ic);
TRN_API TRN_ScreenAnnotSetStaticIcon(TRN_Annot screen, TRN_Obj icon)
{
	BEX;
	Screen scr((Obj*)screen);
	scr.SetStaticIcon((Obj*)icon);
	EEX;
}
//SDF::Obj* GetStaticIcon() const;
TRN_API TRN_ScreenAnnotGetStaticIcon(TRN_Annot screen,TRN_Obj* result)
{
	BEX;
	Screen scr((Obj*)screen);
	*result=(TRN_Obj)scr.GetStaticIcon();
	EEX;
}

//SDF::Obj* GetRolloverIcon() const;
TRN_API TRN_ScreenAnnotSetRolloverIcon(TRN_Annot screen, TRN_Obj icon)
{
	BEX;
	Screen scr((Obj*)screen);
	scr.SetRolloverIcon((Obj*)icon);
	EEX;
}

//void SetRolloverIcon(SDF::Obj* ic);
TRN_API TRN_ScreenAnnotGetRolloverIcon(TRN_Annot screen,TRN_Obj* result)
{
	BEX;
	Screen scr((Obj*)screen);
	*result=(TRN_Obj)scr.GetRolloverIcon();
	EEX;
}

//SDF::Obj* GetMouseDownIcon() const;
TRN_API TRN_ScreenAnnotSetMouseDownIcon(TRN_Annot screen, TRN_Obj icon)
{
	BEX;
	Screen scr((Obj*)screen);
	scr.SetMouseDownIcon((Obj*)icon);
	EEX;
}

//void SetMouseDownIcon(SDF::Obj* ic);
TRN_API TRN_ScreenAnnotGetMouseDownIcon(TRN_Annot screen,TRN_Obj* result)
{
	BEX;
	Screen scr((Obj*)screen);
	*result=(TRN_Obj)scr.GetMouseDownIcon();
	EEX;
}


//ScaleType GetScaleType() const;
TRN_API TRN_ScreenAnnotGetScaleType(TRN_Annot screen, enum TRN_ScreenAnnotScaleType* result)
{
	BEX;
	Screen l((Obj*)screen);
	*result=(TRN_ScreenAnnotScaleType)l.GetScaleType();
	EEX;
}


//void SetScaleType(ScaleType st);
TRN_API TRN_ScreenAnnotSetScaleType(TRN_Annot screen, enum TRN_ScreenAnnotScaleType style)
{
	BEX;
	Screen l((Obj*)screen);
	l.SetScaleType((Annots::Screen::ScaleType)style);
	EEX;
}


//ScaleCondition GetScaleCondition() const;
TRN_API TRN_ScreenAnnotGetScaleCondition(TRN_Annot screen, enum TRN_ScreenAnnotScaleCondition* result)
{
	BEX;
	Screen l((Obj*)screen);
	*result=(TRN_ScreenAnnotScaleCondition)l.GetScaleCondition();
	EEX;
}

//void SetScaleCondition(ScaleCondition sd);
TRN_API TRN_ScreenAnnotSetScaleCondition(TRN_Annot screen, enum TRN_ScreenAnnotScaleCondition style)
{
	BEX;
	Screen l((Obj*)screen);
	l.SetScaleCondition((Annots::Screen::ScaleCondition)style);
	EEX;
}


//IconCaptionRelation GetIconCaptionRelation() const;
TRN_API TRN_ScreenAnnotGetIconCaptionRelation(TRN_Annot screen, enum TRN_ScreenAnnotIconCaptionRelation* result)
{
	BEX;
	Screen l((Obj*)screen);
	*result=(TRN_ScreenAnnotIconCaptionRelation)l.GetIconCaptionRelation();
	EEX;
}

//void SetIconCaptionRelation(IconCaptionRelation icr);
TRN_API TRN_ScreenAnnotSetIconCaptionRelation(TRN_Annot screen, enum TRN_ScreenAnnotIconCaptionRelation style)
{
	BEX;
	Screen l((Obj*)screen);
	l.SetIconCaptionRelation((Annots::Screen::IconCaptionRelation)style);
	EEX;
}

//double GetHIconLeftOver() const;
TRN_API TRN_ScreenAnnotGetHIconLeftOver(TRN_Annot screen, double* result)
{
	BEX;
	Screen l((Obj*)screen);
	*result=l.GetHIconLeftOver();
	EEX;
}

//void SetHIconLeftOver(double hl);
TRN_API TRN_ScreenAnnotSetHIconLeftOver(TRN_Annot screen, double hl)
{
	BEX;
	Screen l((Obj*)screen);
	l.SetHIconLeftOver(hl);
	EEX;
}

//double GetVIconLeftOver() const;
TRN_API TRN_ScreenAnnotGetVIconLeftOver(TRN_Annot screen, double* result)
{
	BEX;
	Screen l((Obj*)screen);
	*result=l.GetVIconLeftOver();
	EEX;
}

//void SetVIconLeftOver(double vl);
TRN_API TRN_ScreenAnnotSetVIconLeftOver(TRN_Annot screen, double vl)
{
	BEX;
	Screen l((Obj*)screen);
	l.SetVIconLeftOver(vl);
	EEX;
}

//bool GetFitFull() const;
TRN_API TRN_ScreenAnnotGetFitFull(const TRN_Annot screen, TRN_Bool* result)
{
	BEX;
	Screen l((Obj*)screen);
	*result=BToTB(l.GetFitFull());
	EEX;
}

//void SetFitFull(bool ff);
TRN_API TRN_ScreenAnnotSetFitFull(TRN_Annot screen, TRN_Bool ff)
{
	BEX;
	Screen a((Obj*)screen);
	a.SetFitFull(TBToB(ff));
	EEX;
}


//17. Sound

//Sound(SDF::Obj* d = 0);
TRN_API TRN_SoundAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Sound((Obj*)d).GetSDFObj());
	EEX;
}

//Sound(const Annot& ann) : Markup(ann.GetSDFObj()) {}
TRN_API TRN_SoundAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Sound((Obj*)annot).GetSDFObj());
	EEX;
}

//static Sound Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_SoundAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Sound::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}

TRN_API TRN_SoundAnnotCreateWithData(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Filter no_own_stream, int sample_bits, int sample_freq, int num_channels, TRN_SoundAnnot* result)
{
	BEX;
	AutoPtr<Filters::Filter> filt_auto_ptr((Filters::Filter*)no_own_stream);
	*result=(TRN_Annot)(Sound::CreateWithData(DocCast(doc), *((Rect*)pos), filt_auto_ptr, sample_bits, sample_freq, num_channels).GetSDFObj());
	EEX;
}

TRN_API TRN_SoundAnnotCreateAtPoint(TRN_SDFDoc doc, const TRN_Point* pos, TRN_Annot* result) {
	BEX;
	*result=(TRN_Annot)(Sound::Create(DocCast(doc), *((Point*)pos)).GetSDFObj());
	EEX;
}

//SDF::Obj* GetSoundStream() const;
TRN_API TRN_SoundAnnotGetSoundStream(TRN_Annot sound,TRN_Obj* result)
{
	BEX;
	Sound scr((Obj*)sound);
	*result=(TRN_Obj)scr.GetSoundStream();
	EEX;
}

//void SetSoundStream(SDF::Obj* sound_stream);
TRN_API TRN_SoundAnnotSetSoundStream(TRN_Annot sound, TRN_Obj icon)
{
	BEX;
	Sound scr((Obj*)sound);
	scr.SetSoundStream((Obj*)icon);
	EEX;
}

//Icon GetIcon() const;
TRN_API TRN_SoundAnnotGetIcon(TRN_Annot sound, enum TRN_SoundAnnotIcon* result)
{
	BEX;
	Sound l((Obj*)sound);
	*result=(TRN_SoundAnnotIcon)l.GetIcon();
	EEX;
}


//void SetIcon(Icon type=e_Speaker);
TRN_API TRN_SoundAnnotSetIcon(TRN_Annot sound, enum TRN_SoundAnnotIcon type)
{
	BEX;
	Sound l((Obj*)sound);
	l.SetIcon((Annots::Sound::Icon)type);
	EEX;
}

//const char* GetIconName() const;
TRN_API TRN_SoundAnnotGetIconName(TRN_Annot sound, const char** result)
{
	BEX;
	Sound l((Obj*)sound);
	*result=l.GetIconName();
	EEX;
}

//void SetIcon(const char* type);
TRN_API TRN_SoundAnnotSetIconName(TRN_Annot sound, const char* type)
{
	BEX;
	Sound l((Obj*)sound);
	l.SetIcon(type);
	EEX;
}

//18. Square

//Square(SDF::Obj* d = 0);
TRN_API TRN_SquareAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Square((Obj*)d).GetSDFObj());
	EEX;
}

//Square(const Annot& ann) : Markup(ann.GetSDFObj()) {} 
TRN_API TRN_SquareAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Square((Obj*)annot).GetSDFObj());
	EEX;
}

//static Square Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_SquareAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Square::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}


//ColorPt GetInteriorColor() const;
TRN_API TRN_SquareAnnotGetInteriorColor(TRN_Annot square, TRN_ColorPt* result)
{
	BEX;
	Square ci((Obj*)square);
	ColorPt c(ci.GetInteriorColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}


//int GetInteriorColorCompNum() const;
TRN_API TRN_SquareAnnotGetInteriorColorCompNum(TRN_Annot square, int* result)
{
	BEX;
	Square ci((Obj*)square);
	*result=ci.GetInteriorColorCompNum();
	EEX;
}


//void SetInteriorColor(const ColorPt& c, int CompNum);
TRN_API TRN_SquareAnnotSetInteriorColorDefault(TRN_Annot square, const TRN_ColorPt* col)
{
	BEX;
	Square c((Obj*)square);
	c.SetInteriorColor(CPCast(col), 3);
	EEX;
}

TRN_API TRN_SquareAnnotSetInteriorColor(TRN_Annot square, const TRN_ColorPt* col, int numcomp)
{
	BEX;
	Square c((Obj*)square);
	c.SetInteriorColor(CPCast(col), numcomp);
	EEX;
}


//Rect GetContentRect() const;
TRN_API TRN_SquareAnnotGetContentRect(TRN_Annot square, TRN_Rect* result)
{
	BEX;
	Square c((Obj*)square);
	Rect r(c.GetContentRect());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}


//void SetContentRect(const Rect& cr);
TRN_API TRN_SquareAnnotSetContentRect(TRN_Annot square, const TRN_Rect* cr)
{
	BEX;
	Square c((Obj*)square);
	c.SetContentRect(RectCast(cr));
	EEX;
}



//Rect GetPadding() const;
TRN_API TRN_SquareAnnotGetPadding(TRN_Annot square, TRN_Rect* result)
{
	BEX;
	Square c((Obj*)square);
	Rect r(c.GetPadding());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}


//void SetPadding(const Rect& rd);
TRN_API TRN_SquareAnnotSetPadding(TRN_Annot square, const TRN_Rect* cr)
{
	BEX;
	Square c((Obj*)square);
	c.SetPadding(RectCast(cr));
	EEX;
}




//19. Squiggly

//Squiggly(SDF::Obj* d);
TRN_API TRN_SquigglyAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Squiggly((Obj*)d).GetSDFObj());
	EEX;
}


//Squiggly(const Annot& ann) : TextMarkup(ann.GetSDFObj()) {}
TRN_API TRN_SquigglyAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Squiggly((Obj*)annot).GetSDFObj());
	EEX;
}


//static Squiggly Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_SquigglyAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Squiggly::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}





//20. StrikeOut

//StrikeOut(SDF::Obj* d);
TRN_API TRN_StrikeOutAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(StrikeOut((Obj*)d).GetSDFObj());
	EEX;
}


//StrikeOut(const Annot& ann) : TextMarkup(ann.GetSDFObj()) {} 
TRN_API TRN_StrikeOutAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(StrikeOut((Obj*)annot).GetSDFObj());
	EEX;
}


//static StrikeOut Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_StrikeOutAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(StrikeOut::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}




//21. Text

//Text(SDF::Obj* d = 0);
TRN_API TRN_TextAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=((TRN_Annot)Text((Obj*)d).GetSDFObj());
	EEX;
}


//Text(const Annot& mku) : Markup(mku.GetSDFObj()) {}
TRN_API TRN_TextAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Text((Obj*)annot).GetSDFObj());
	EEX;
}


//static Text Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_TextAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Text::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}


TRN_API TRN_TextAnnotCreateAtPoint(TRN_SDFDoc doc, const TRN_Point* pos, TRN_Annot* result) {
	BEX;
	*result=(TRN_Annot)(Text::Create(DocCast(doc), *((Point*)pos)).GetSDFObj());
	EEX;
}


//static Text Create(SDF::SDFDoc& doc, const Rect& pos, const UString& contents);	

//bool IsOpen() const;
TRN_API TRN_TextAnnotIsOpen(TRN_Annot text, TRN_Bool* result)
{
	BEX;
	Text p((Obj*)text);	
	*result=BToTB(p.IsOpen());
	EEX;
}

//void Text::SetOpen(bool isopen);
TRN_API TRN_TextAnnotSetOpen(TRN_Annot text, TRN_Bool isopen)
{
	BEX;
	Text p((Obj*)text);	
	p.SetOpen(TBToB(isopen));
	EEX;
}

//Icon GetIcon() const; 
TRN_API TRN_TextAnnotGetIcon(TRN_Annot text, enum TRN_TextAnnotIcon* result)
{
	BEX;
	Text ft((Obj*)text);
	*result=(TRN_TextAnnotIcon)ft.GetIcon();
	EEX;
}

//void Text::SetIcon(Icon type);
TRN_API TRN_TextAnnotSetIcon(TRN_Annot text, enum TRN_TextAnnotIcon icon)
{
	BEX;
	Text ft((Obj*)text);
	ft.SetIcon((Annots::Text::Icon)icon);
	EEX;
}

TRN_API TRN_TextAnnotSetIconDefault(TRN_Annot text)
{
	BEX;
	Text ft((Obj*)text);
	ft.SetIcon(Annots::Text::e_Note);
	EEX;
}

//const char* GetIconName() const; 
TRN_API TRN_TextAnnotGetIconName(TRN_Annot text, const char** result)
{
	BEX;
	Text l((Obj*)text);
	*result=l.GetIconName();
	EEX;
}

//void SetIcon(Icon icon=e_Note);
TRN_API TRN_TextAnnotSetIconName(TRN_Annot text, const char* icon)
{
	BEX;
	Text l((Obj*)text);
	l.SetIcon(icon);
	EEX;
}

//UString GetState() const;
TRN_API TRN_TextAnnotGetState(TRN_Annot text, TRN_UString* result)
{
	BEX;
	Text mu((Obj*)text);	
	RetUStr(mu.GetState());
	EEX;
}

//void SetState(const UString& state="");
TRN_API TRN_TextAnnotSetState(TRN_Annot text, TRN_UString contents)
{
	BEX;
	Text mu((Obj*)text);	
	UString temp(UStrCast(contents));
	mu.SetState(temp);
	EEX;
}


//UString GetStateModel() const;
TRN_API TRN_TextAnnotGetStateModel(TRN_Annot text, TRN_UString* result)
{
	BEX;
	Text mu((Obj*)text);	
	RetUStr(mu.GetStateModel());
	EEX;
}

//void SetStateModel(const UString& statemodule);
TRN_API TRN_TextAnnotSetStateModel(TRN_Annot text, TRN_UString contents)
{
	BEX;
	Text mu((Obj*)text);	
	UString temp(UStrCast(contents));
	mu.SetStateModel(temp);
	EEX;
}


//22. TextMarkup

//TextMarkup::TextMarkup(SDF::Obj* d);
TRN_API TRN_TextMarkupAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(TextMarkup((Obj*)d).GetSDFObj());
	EEX;
}


//TextMarkup(const Annot& ann) : Markup(ann.GetSDFObj()) {} 
TRN_API TRN_TextMarkupAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(TextMarkup((Obj*)annot).GetSDFObj());
	EEX;
}


//int GetQuadPointCount();
TRN_API TRN_TextMarkupAnnotGetQuadPointCount(TRN_Annot textmarkup, int* result)
{
	BEX;
	TextMarkup l((Obj*)textmarkup);
	*result=l.GetQuadPointCount();
	EEX;
}

//QuadPoint GetQuadPoint(int idx) const;
TRN_API TRN_TextMarkupAnnotGetQuadPoint(TRN_Annot textmarkup, int idx, TRN_QuadPoint* result)
{
	//BEX;
	//TextMarkup l((Obj*)textmarkup);
	//*result=(TRN_QuadPoint)l.GetQuadPoint(idx);
	//EEX;
	BEX;	
	TextMarkup l((Obj*)textmarkup);
	QuadPoint qp(l.GetQuadPoint(idx));
	assert(sizeof(TRN_QuadPoint)>=sizeof(qp));
	memcpy(result, &qp, sizeof(qp));
	EEX;
}



//void SetQuadPoint(int idx, const QuadPoint& qp); 
TRN_API TRN_TextMarkupAnnotSetQuadPoint(TRN_Annot textmarkup, int idx, const TRN_QuadPoint* qp)
{
	BEX;
	TextMarkup l((Obj*)textmarkup);
	l.SetQuadPoint(idx, QPCast(qp));
	EEX;
}







//23. Underline

//Underline(SDF::Obj* d);
TRN_API TRN_UnderlineAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Underline((Obj*)d).GetSDFObj());
	EEX;
}


//Underline(const Annot& ann) : TextMarkup(ann.GetSDFObj()) {} 
TRN_API TRN_UnderlineAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Underline((Obj*)annot).GetSDFObj());
	EEX;
}


//static Underline Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_UnderlineAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Underline::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}




//24. Watermark

//Watermark(SDF::Obj* d = 0);
TRN_API TRN_WatermarkAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Watermark((Obj*)d).GetSDFObj());
	EEX;
}


//Watermark(const Annot& ann) : Annot(ann.GetSDFObj()) {}
TRN_API TRN_WatermarkAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Watermark((Obj*)annot).GetSDFObj());
	EEX;
}

//static Watermark Create(SDF::SDFDoc& doc, const Rect& pos);
TRN_API TRN_WatermarkAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Watermark::Create(DocCast(doc), *((Rect*)pos)).GetSDFObj());
	EEX;
}




//25. Widget

//Widget(SDF::Obj* d = 0);	
TRN_API TRN_WidgetAnnotCreateFromObj(TRN_Obj d, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Widget((Obj*)d).GetSDFObj());
	EEX;
}

//Widget(const Annot& ann) : Annot(ann.GetSDFObj()) {} 
TRN_API TRN_WidgetAnnotCreateFromAnnot(TRN_Annot annot, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Widget((Obj*)annot).GetSDFObj());
	EEX;
}

//static Widget Create(SDF::SDFDoc& doc, const Rect& pos, Field field);
TRN_API TRN_WidgetAnnotCreate(TRN_SDFDoc doc, const TRN_Rect* pos, TRN_Field* fd, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)(Widget::Create(DocCast(doc), *((Rect*)pos), FieldCast(fd)).GetSDFObj());
	EEX;
}


//Field GetField() const;	
TRN_API TRN_WidgetAnnotGetField(TRN_Annot widget, TRN_Field* result)
{
	BEX;
	Widget l((Obj*)widget);
    Field field = l.GetField();
	*result=RetField(field);
	EEX;
}

//HighlightingMode GetHighlightingMode() const;
TRN_API TRN_WidgetAnnotGetHighlightingMode(TRN_Annot widget, enum TRN_WidgetAnnotHighlightingMode* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result=(TRN_WidgetAnnotHighlightingMode)l.GetHighlightingMode();
	EEX;
}

//void SetHighlightingMode(HighlightingMode mode=e_invert);
TRN_API TRN_WidgetAnnotSetHighlightingMode(TRN_Annot widget, enum TRN_WidgetAnnotHighlightingMode value)
{
	BEX;
	Widget l((Obj*)widget);
	l.SetHighlightingMode((Annots::Widget::HighlightingMode)value);
	EEX;
}

//Action GetAction() const;
TRN_API TRN_WidgetAnnotGetAction(TRN_Annot widget, TRN_Action* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result=(TRN_Action)l.GetAction().GetSDFObj();
	EEX;
}

//void SetAction(const Action& action);
TRN_API TRN_WidgetAnnotSetAction(TRN_Annot widget, TRN_Action action)
{
	BEX;
	Widget l((Obj*)widget);
	l.SetAction(ActCast(action));
	EEX;
}

//int GetBorderColor() const;
TRN_API TRN_WidgetAnnotGetBorderColor(TRN_Annot widget, TRN_ColorPt* result)
{
	BEX;
	Widget ci((Obj*)widget);
	ColorPt c(ci.GetBorderColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

//ColorPt SetBorderColor(const ColorPt& c, int CompNum) const;
TRN_API TRN_WidgetAnnotSetBorderColor(TRN_Annot widget, const TRN_ColorPt* col, int numcomp)
{
	BEX;
	Widget c((Obj*)widget);
	c.SetBorderColor(CPCast(col), numcomp);
	EEX;
}

//void GetBorderColorCompNum(); 					
TRN_API TRN_WidgetAnnotGetBorderColorCompNum(TRN_Annot widget, int* result)
{
	BEX;
	Widget ci((Obj*)widget);
	*result=ci.GetBorderColorCompNum();
	EEX;
}

//int GetBackgroundColorCompNum() const;
TRN_API TRN_WidgetAnnotGetBackgroundColorCompNum(TRN_Annot widget, int* result)
{
	BEX;
	Widget ci((Obj*)widget);
	*result=ci.GetBackgroundColorCompNum();
	EEX;
}


//ColorPt GetBackgroundColor() const;
TRN_API TRN_WidgetAnnotGetBackgroundColor(TRN_Annot widget, TRN_ColorPt* result)
{
	BEX;
	Widget ci((Obj*)widget);
	ColorPt c(ci.GetBackgroundColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

//void SetBackgroundColor(const ColorPt& c, int CompNum); 
TRN_API TRN_WidgetAnnotSetBackgroundColor(TRN_Annot widget, const TRN_ColorPt* col, int numcomp)
{
	BEX;
	Widget c((Obj*)widget);
	c.SetBackgroundColor(CPCast(col), numcomp);
	EEX;
}

//UString GetStaticCaptionText() const;
TRN_API TRN_WidgetAnnotGetStaticCaptionText(TRN_Annot widget, TRN_UString* result)
{
	BEX;
	Widget mu((Obj*)widget);	
	RetUStr(mu.GetStaticCaptionText());
	EEX;
}


//void SetStaticCaptionText(const UString& ct);
TRN_API TRN_WidgetAnnotSetStaticCaptionText(TRN_Annot widget, TRN_UString contents)
{
	BEX;
	Widget mu((Obj*)widget);	
	UString temp(UStrCast(contents));
	mu.SetStaticCaptionText(temp);
	EEX;
}

//UString GetRolloverCaptionText() const;
TRN_API TRN_WidgetAnnotGetRolloverCaptionText(TRN_Annot widget, TRN_UString* result)
{
	BEX;
	Widget mu((Obj*)widget);	
	RetUStr(mu.GetRolloverCaptionText());
	EEX;
}

//void SetRolloverCaptionText(const UString& ct);
TRN_API TRN_WidgetAnnotSetRolloverCaptionText(TRN_Annot widget, TRN_UString contents)
{
	BEX;
	Widget mu((Obj*)widget);	
	UString temp(UStrCast(contents));
	mu.SetRolloverCaptionText(temp);
	EEX;
}

//UString GetMouseDownCaptionText() const;
TRN_API TRN_WidgetAnnotGetMouseDownCaptionText(TRN_Annot widget, TRN_UString* result)
{
	BEX;
	Widget mu((Obj*)widget);	
	RetUStr(mu.GetMouseDownCaptionText());
	EEX;
}

//void SetMouseDownCaptionText(const UString& ct);
TRN_API TRN_WidgetAnnotSetMouseDownCaptionText(TRN_Annot widget, TRN_UString contents)
{
	BEX;
	Widget mu((Obj*)widget);	
	UString temp(UStrCast(contents));
	mu.SetMouseDownCaptionText(temp);
	EEX;
}

//SDF::Obj* GetStaticIcon() const;
TRN_API TRN_WidgetAnnotGetStaticIcon(TRN_Annot widget,TRN_Obj* result)
{
	BEX;
	Widget scr((Obj*)widget);
	*result=(TRN_Obj)scr.GetStaticIcon();
	EEX;
}


//void SetStaticIcon(SDF::Obj* ic);
TRN_API TRN_WidgetAnnotSetStaticIcon(TRN_Annot widget, TRN_Obj icon)
{
	BEX;
	Widget scr((Obj*)widget);
	scr.SetStaticIcon((Obj*)icon);
	EEX;
}

//SDF::Obj* GetRolloverIcon() const;
TRN_API TRN_WidgetAnnotGetRolloverIcon(TRN_Annot widget,TRN_Obj* result)
{
	BEX;
	Widget scr((Obj*)widget);
	*result=(TRN_Obj)scr.GetRolloverIcon();
	EEX;
}


//void SetRolloverIcon(SDF::Obj* ic);
TRN_API TRN_WidgetAnnotSetRolloverIcon(TRN_Annot widget, TRN_Obj icon)
{
	BEX;
	Widget scr((Obj*)widget);
	scr.SetRolloverIcon((Obj*)icon);
	EEX;
}

//SDF::Obj* GetMouseDownIcon() const;
TRN_API TRN_WidgetAnnotGetMouseDownIcon(TRN_Annot widget,TRN_Obj* result)
{
	BEX;
	Widget scr((Obj*)widget);
	*result=(TRN_Obj)scr.GetMouseDownIcon();
	EEX;
}

//void SetMouseDownIcon(SDF::Obj* ic);
TRN_API TRN_WidgetAnnotSetMouseDownIcon(TRN_Annot widget, TRN_Obj icon)
{
	BEX;
	Widget scr((Obj*)widget);
	scr.SetMouseDownIcon((Obj*)icon);
	EEX;
}

//ScaleType GetScaleType() const;
TRN_API TRN_WidgetAnnotGetScaleType(TRN_Annot widget, enum TRN_WidgetAnnotScaleType* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result=(TRN_WidgetAnnotScaleType)l.GetScaleType();
	EEX;
}


//void SetScaleType(ScaleType st);
TRN_API TRN_WidgetAnnotSetScaleType(TRN_Annot widget, enum TRN_WidgetAnnotScaleType style)
{
	BEX;
	Widget l((Obj*)widget);
	l.SetScaleType((Annots::Widget::ScaleType)style);
	EEX;
}

//IconCaptionRelation GetIconCaptionRelation() const;
TRN_API TRN_WidgetAnnotGetIconCaptionRelation(TRN_Annot widget, enum TRN_WidgetAnnotIconCaptionRelation* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result=(TRN_WidgetAnnotIconCaptionRelation)l.GetIconCaptionRelation();
	EEX;
}

//void SetIconCaptionRelation(IconCaptionRelation icr);
TRN_API TRN_WidgetAnnotSetIconCaptionRelation(TRN_Annot widget, enum TRN_WidgetAnnotIconCaptionRelation style)
{
	BEX;
	Widget l((Obj*)widget);
	l.SetIconCaptionRelation((Annots::Widget::IconCaptionRelation)style);
	EEX;
}


//ScaleCondition GetScaleCondition() const;
TRN_API TRN_WidgetAnnotGetScaleCondition(TRN_Annot widget, enum TRN_WidgetAnnotScaleCondition* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result=(TRN_WidgetAnnotScaleCondition)l.GetScaleCondition();
	EEX;
}

//void SetScaleCondition(ScaleCondition sd);
TRN_API TRN_WidgetAnnotSetScaleCondition(TRN_Annot widget, enum TRN_WidgetAnnotScaleCondition style)
{
	BEX;
	Widget l((Obj*)widget);
	l.SetScaleCondition((Annots::Widget::ScaleCondition)style);
	EEX;
}

//double GetHIconLeftOver() const;
TRN_API TRN_WidgetAnnotGetHIconLeftOver(TRN_Annot widget, double* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result=l.GetHIconLeftOver();
	EEX;
}

//void SetHIconLeftOver(double hl);
TRN_API TRN_WidgetAnnotSetHIconLeftOver(TRN_Annot widget, double hl)
{
	BEX;
	Widget l((Obj*)widget);
	l.SetHIconLeftOver(hl);
	EEX;
}

//double GetVIconLeftOver() const;
TRN_API TRN_WidgetAnnotGetVIconLeftOver(TRN_Annot widget, double* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result=l.GetVIconLeftOver();
	EEX;
}

//void SetVIconLeftOver(double vl);
TRN_API TRN_WidgetAnnotSetVIconLeftOver(TRN_Annot widget, double vl)
{
	BEX;
	Widget l((Obj*)widget);
	l.SetVIconLeftOver(vl);
	EEX;
}

//bool GetFitFull() const;
TRN_API TRN_WidgetAnnotGetFitFull(const TRN_Annot widget, TRN_Bool* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result=BToTB(l.GetFitFull());
	EEX;
}

//void SetFitFull(bool ff);
TRN_API TRN_WidgetAnnotSetFitFull(TRN_Annot widget, TRN_Bool ff)
{
	BEX;
	Widget a((Obj*)widget);
	a.SetFitFull(TBToB(ff));
	EEX;
}

TRN_API TRN_WidgetAnnotSetFontSize(TRN_Annot widget, double font_size)
{
	BEX;
	Widget a((Obj*)widget);
	a.SetFontSize(font_size);
	EEX;
}

TRN_API TRN_WidgetAnnotSetTextColor(TRN_WidgetAnnot widget, const TRN_ColorPt* col, int col_comp)
{
	BEX;
	Widget a((Obj*)widget);
	a.SetTextColor(CPCast(col), col_comp);
	EEX;
}
TRN_API TRN_SignatureWidgetCreate(TRN_PDFDoc doc, const TRN_Rect* pos, TRN_UString field_name, TRN_SignatureWidget* result)
{
	BEX;
	*result = (TRN_SignatureWidget)SignatureWidget::Create(*PDocCast(doc), *((const Rect*)pos), UString(UStrCast(field_name))).GetSDFObj();
	EEX;
}
TRN_API TRN_SignatureWidgetCreateWithField(TRN_PDFDoc doc, const TRN_Rect* pos, const TRN_Field* field, TRN_SignatureWidget* result)
{
	BEX;
	*result = (TRN_SignatureWidget)SignatureWidget::Create(*PDocCast(doc), *((const Rect*)pos), FieldCast(field)).GetSDFObj();
	EEX;
}
TRN_API TRN_SignatureWidgetCreateWithDigitalSignatureField(TRN_PDFDoc doc, const TRN_Rect* pos, const TRN_DigitalSignatureField* field, TRN_SignatureWidget* result)
{
	BEX;
	*result = (TRN_SignatureWidget)SignatureWidget::Create(*PDocCast(doc), *((const Rect*)pos), DigSigFieldCast(field)).GetSDFObj();
	EEX;
}
TRN_API TRN_SignatureWidgetCreateFromObj(TRN_Obj obj, TRN_SignatureWidget* result)
{
	BEX;
	*result = (TRN_SignatureWidget)obj;
	EEX;
}
TRN_API TRN_SignatureWidgetCreateFromAnnot(TRN_Annot annot, TRN_SignatureWidget* result)
{
	BEX;
	*result = (TRN_SignatureWidget)annot;
	EEX;
}

TRN_API TRN_WidgetAnnotSetFont(TRN_WidgetAnnot widget, const TRN_Font font)
{
	BEX;
	Widget a((Obj*)widget);
	a.SetFont(FontCast(font));
	EEX;
}
TRN_API TRN_SignatureWidgetCreateSignatureAppearance(TRN_SignatureWidget self, TRN_Image img)
{
	BEX;
	Image temp(ImgCast(img));
	SignatureWidget(ObjCast(self)).CreateSignatureAppearance(temp);
	EEX;
}

TRN_API TRN_WidgetAnnotGetFontSize(TRN_WidgetAnnot widget, double* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result = l.GetFontSize();
	EEX;
}
TRN_API TRN_SignatureWidgetGetDigitalSignatureField(TRN_SignatureWidget self, TRN_DigitalSignatureField* result)
{
	BEX;
	result->mp_field_dict_obj = (TRN_Obj) self;
	EEX;
}
TRN_API TRN_ComboBoxWidgetCreate(TRN_PDFDoc doc, const TRN_Rect* pos, TRN_UString field_name, TRN_ComboBoxWidget* result)
{
	BEX;
	*result = (TRN_ComboBoxWidget)ComboBoxWidget::Create(*PDocCast(doc), *((Rect*)pos), UString(UStrCast(field_name))).GetSDFObj();
	EEX;
}
TRN_API TRN_ComboBoxWidgetCreateWithField(TRN_PDFDoc doc, const TRN_Rect* pos, const TRN_Field* field, TRN_ComboBoxWidget* result)
{
	BEX;
	*result = (TRN_ComboBoxWidget)ComboBoxWidget::Create(*PDocCast(doc), *((Rect*)pos), FieldCast(field)).GetSDFObj();
	EEX;
}
TRN_API TRN_ComboBoxWidgetCreateFromObj(TRN_Obj obj, TRN_ComboBoxWidget* result)
{
	BEX;
	*result = (TRN_ComboBoxWidget)obj;
	EEX;
}

TRN_API TRN_WidgetAnnotGetTextColor(TRN_WidgetAnnot widget, TRN_ColorPt* out_color, int* out_col_components)
{
	BEX;
	ColorPt col;
	Widget l((Obj*)widget);
	l.GetTextColor(col, *out_col_components);
	TRN_ColorPtAssign(out_color, (const TRN_ColorPt*)&col);
	EEX;
}

TRN_API TRN_WidgetAnnotGetFont(TRN_WidgetAnnot widget, TRN_Font* result)
{
	BEX;
	Widget l((Obj*)widget);
	*result = (TRN_Font)new Font(l.GetFont());
	EEX;
}
TRN_API TRN_ComboBoxWidgetCreateFromAnnot(TRN_Annot annot, TRN_ComboBoxWidget* result)
{
	BEX;
	*result = (TRN_ComboBoxWidget)annot;
	EEX;
}

TRN_API TRN_ComboBoxWidgetAddOption(TRN_ComboBoxWidget combobox, TRN_UString value)
{
	BEX;
	ComboBoxWidget annot(ObjCast(combobox));
	annot.AddOption(UString(UStrCast(value)));
	EEX;
}

TRN_API TRN_ComboBoxWidgetAddOptions(TRN_ComboBoxWidget combobox, TRN_UString* options, TRN_UInt32 options_size)
{
	BEX;
	ComboBoxWidget annot(ObjCast(combobox));
	FlexVector<UString> input;
	input.reserve(options_size);
	for (UInt32 i = 0; i < options_size; ++i)
	{
		input.push_back(UString(options[i]));
	}
	annot.AddOptions(input);
	EEX;
}

TRN_API TRN_ComboBoxWidgetSetSelectedOption(TRN_ComboBoxWidget combobox, TRN_UString value)
{
	BEX;
	ComboBoxWidget annot(ObjCast(combobox));
	annot.SetSelectedOption(UString(value));
	EEX;
}

TRN_API TRN_ComboBoxWidgetGetSelectedOption(TRN_ComboBoxWidget combobox, TRN_UString* result)
{
	BEX;
	ComboBoxWidget annot(ObjCast(combobox));
	RetUStr(annot.GetSelectedOption());
	EEX;
}

TRN_API TRN_ComboBoxWidgetGetOptions(TRN_ComboBoxWidget combobox, TRN_Vector* result)
{
	BEX;
	ComboBoxWidget annot(ObjCast(combobox));
	FlexVector<UString> tmp = annot.GetOptions();

	PolyVectorBase<TRN_UString>* tmp2 = new PolyVectorBase<TRN_UString>();
	*result = (TRN_Vector)tmp2;
	try
	{
		for (FlexVector<trn::UString>::iterator it = tmp.begin(); it != tmp.end(); ++it)
		{
			TRN_UString result;
			TRN_UStringCopy((TRN_UString)it->mp_impl, &result);
			tmp2->m_vector.push_back(result);
		}
	}
	catch (...)
	{
		TRN_VectorDestroy(*result);
		throw;
	}
	EEX;
}

TRN_API TRN_ComboBoxWidgetReplaceOptions(TRN_ComboBoxWidget combobox, TRN_UString* options, TRN_UInt32 options_size)
{
	BEX;
	ComboBoxWidget annot(ObjCast(combobox));
	FlexVector<UString> input;
	input.reserve(options_size);
	for (UInt32 i = 0; i < options_size; ++i)
	{
		input.push_back(UString(options[i]));
	}
	annot.ReplaceOptions(input);
	EEX;
}

TRN_API TRN_ComboBoxWidgetRemoveOption(TRN_ComboBoxWidget combobox, TRN_UString value)
{
	BEX;
	ComboBoxWidget annot(ObjCast(combobox));
	annot.RemoveOption(UString(UStrCast(value)));
	EEX;
}


TRN_API TRN_ListBoxWidgetCreate(TRN_PDFDoc doc, const TRN_Rect* pos, TRN_UString field_name, TRN_ListBoxWidget* result)
{
	BEX;
	*result = (TRN_ListBoxWidget)ListBoxWidget::Create(*PDocCast(doc), *((Rect*)pos), UString(UStrCast(field_name))).GetSDFObj();
	EEX;
}
TRN_API TRN_ListBoxWidgetCreateWithField(TRN_PDFDoc doc, const TRN_Rect* pos, const TRN_Field* field, TRN_ListBoxWidget* result)
{
	BEX;
	*result = (TRN_ListBoxWidget)ListBoxWidget::Create(*PDocCast(doc), *((Rect*)pos), FieldCast(field)).GetSDFObj();
	EEX;
}

TRN_API TRN_ListBoxWidgetCreateFromObj(TRN_Obj obj, TRN_ListBoxWidget* result)
{
	BEX;
	*result = (TRN_ListBoxWidget)obj;
	EEX;
}
TRN_API TRN_ListBoxWidgetCreateFromAnnot(TRN_Annot annot, TRN_ListBoxWidget* result)
{
	BEX;
	*result = (TRN_ListBoxWidget)annot;
	EEX;
}
TRN_API TRN_ListBoxWidgetAddOption(TRN_ListBoxWidget listbox, TRN_UString value)
{
	BEX;
	ListBoxWidget annot(ObjCast(listbox));
	annot.AddOption(UString(UStrCast(value)));
	EEX;
}

TRN_API TRN_ListBoxWidgetAddOptions(TRN_ListBoxWidget listbox, TRN_UString* options, TRN_UInt32 options_size)
{
	BEX;
	ListBoxWidget annot(ObjCast(listbox));
	FlexVector<UString> input;
	input.reserve(options_size);
	for (UInt32 i = 0; i < options_size; ++i)
	{
		input.push_back(UString(options[i]));
	}
	annot.AddOptions(input);
	EEX;
}

TRN_API TRN_ListBoxWidgetSetSelectedOptions(TRN_ListBoxWidget listbox, TRN_UString* options, TRN_UInt32 options_size)
{
	BEX;
	ListBoxWidget annot(ObjCast(listbox));
	FlexVector<UString> input;
	input.reserve(options_size);
	for (UInt32 i = 0; i < options_size; ++i)
	{
		input.push_back(UString(options[i]));
	}
	annot.SetSelectedOptions(input);
	EEX;
}

TRN_API TRN_ListBoxWidgetGetSelectedOptions(TRN_ListBoxWidget listbox, TRN_Vector* result)
{
	BEX;
	ListBoxWidget annot(ObjCast(listbox));
	FlexVector<UString> tmp = annot.GetSelectedOptions();

	PolyVectorBase<TRN_UString>* tmp2 = new PolyVectorBase<TRN_UString>();
	*result = (TRN_Vector)tmp2;
	try
	{
		for (FlexVector<trn::UString>::iterator it = tmp.begin(); it != tmp.end(); ++it)
		{
			TRN_UString result;
			TRN_UStringCopy((TRN_UString)it->mp_impl, &result);
			tmp2->m_vector.push_back(result);
		}
	}
	catch (...)
	{
		TRN_VectorDestroy(*result);
		throw;
	}
	EEX;
}

TRN_API TRN_ListBoxWidgetGetOptions(TRN_ListBoxWidget listbox, TRN_Vector* result)
{
	BEX;
	ListBoxWidget annot(ObjCast(listbox));
	FlexVector<UString> tmp = annot.GetOptions();

	PolyVectorBase<TRN_UString>* tmp2 = new PolyVectorBase<TRN_UString>();
	*result = (TRN_Vector)tmp2;
	try
	{
		for (FlexVector<trn::UString>::iterator it = tmp.begin(); it != tmp.end(); ++it)
		{
			TRN_UString result;
			TRN_UStringCopy((TRN_UString)it->mp_impl, &result);
			tmp2->m_vector.push_back(result);
		}
	}
	catch (...)
	{
		TRN_VectorDestroy(*result);
		throw;
	}
	EEX;
}

TRN_API TRN_ListBoxWidgetReplaceOptions(TRN_ListBoxWidget listbox, TRN_UString* options, TRN_UInt32 options_size)
{
	BEX;
	ListBoxWidget annot(ObjCast(listbox));
	FlexVector<UString> input;
	input.reserve(options_size);
	for (UInt32 i = 0; i < options_size; ++i)
	{
		input.push_back(UString(options[i]));
	}
	annot.ReplaceOptions(input);
	EEX;
}

TRN_API TRN_ListBoxWidgetRemoveOption(TRN_ListBoxWidget listbox, TRN_UString value)
{
	BEX;
	ListBoxWidget annot(ObjCast(listbox));
	annot.RemoveOption(UString(UStrCast(value)));
	EEX;
}

TRN_API TRN_TextWidgetCreate(TRN_PDFDoc doc, const TRN_Rect* pos, TRN_UString field_name, TRN_TextWidget* result)
{
	BEX;
	*result = (TRN_TextWidget)TextWidget::Create(*PDocCast(doc), *((Rect*)pos), UString(UStrCast(field_name))).GetSDFObj();
	EEX;
}
TRN_API TRN_TextWidgetCreateWithField(TRN_PDFDoc doc, const TRN_Rect* pos, const TRN_Field* field, TRN_TextWidget* result)
{
	BEX;
	*result = (TRN_TextWidget)TextWidget::Create(*PDocCast(doc), *((Rect*)pos), FieldCast(field)).GetSDFObj();
	EEX;
}

TRN_API TRN_TextWidgetCreateFromObj(TRN_Obj obj, TRN_TextWidget* result)
{
	BEX;
	*result = (TRN_TextWidget)obj;
	EEX;
}
TRN_API TRN_TextWidgetCreateFromAnnot(TRN_Annot annot, TRN_TextWidget* result)
{
	BEX;
	*result = (TRN_TextWidget)annot;
	EEX;
}

TRN_API TRN_TextWidgetSetText(TRN_TextWidget widget, TRN_UString text)
{
	BEX;
	TextWidget(ObjCast(widget)).SetText(UString(text));
	EEX;
}

TRN_API TRN_TextWidgetGetText(TRN_TextWidget widget, TRN_UString* result)
{
	BEX;
	RetUStr(TextWidget(ObjCast(widget)).GetText());
	EEX;
}

TRN_API TRN_CheckBoxWidgetCreate(TRN_PDFDoc doc, const TRN_Rect* pos, TRN_UString field_name, TRN_CheckBoxWidget* result)
{
	BEX;
	*result = (TRN_CheckBoxWidget)CheckBoxWidget::Create(*PDocCast(doc), *((Rect*)pos), UString(UStrCast(field_name))).GetSDFObj();
	EEX;
}

TRN_API TRN_CheckBoxWidgetCreateWithField(TRN_PDFDoc doc, const TRN_Rect* pos, const TRN_Field* field, TRN_CheckBoxWidget* result)
{
	BEX;
	*result = (TRN_CheckBoxWidget)CheckBoxWidget::Create(*PDocCast(doc), *((Rect*)pos), FieldCast(field)).GetSDFObj();
	EEX;
}

TRN_API TRN_CheckBoxWidgetCreateFromObj(TRN_Obj obj, TRN_CheckBoxWidget* result)
{
	BEX;
	*result = (TRN_CheckBoxWidget)obj;
	EEX;
}

TRN_API TRN_CheckBoxWidgetCreateFromAnnot(TRN_Annot annot, TRN_CheckBoxWidget* result)
{
	BEX;
	*result = (TRN_CheckBoxWidget)annot;
	EEX;
}

TRN_API TRN_CheckBoxWidgetIsChecked(TRN_CheckBoxWidget button, TRN_Bool* result)
{
	BEX;
	CheckBoxWidget annot(ObjCast(button));
	*result = BToTB(annot.IsChecked());
	EEX;
}

TRN_API TRN_CheckBoxWidgetSetChecked(TRN_CheckBoxWidget button, TRN_Bool result)
{
	BEX;
	CheckBoxWidget annot(ObjCast(button));
	annot.SetChecked(TBToB(result));
	EEX;
}

TRN_API TRN_RadioButtonWidgetCreateFromObj(TRN_Obj obj, TRN_RadioButtonWidget* result)
{
	BEX;
	*result = (TRN_RadioButtonWidget)obj;
	EEX;
}
TRN_API TRN_RadioButtonWidgetCreateFromAnnot(TRN_Annot annot, TRN_RadioButtonWidget* result)
{
	BEX;
	*result = (TRN_RadioButtonWidget)annot;
	EEX;
}
TRN_API TRN_RadioButtonWidgetIsEnabled(TRN_RadioButtonWidget button, TRN_Bool* result)
{
	BEX;
	RadioButtonWidget annot(ObjCast(button));
	*result = BToTB(annot.IsEnabled());
	EEX;
}

TRN_API TRN_RadioButtonWidgetEnableButton(TRN_RadioButtonWidget button)
{
	BEX;
	RadioButtonWidget annot(ObjCast(button));
	annot.EnableButton();
	EEX;
}

TRN_API TRN_RadioButtonWidgetGetGroup(TRN_RadioButtonWidget button, TRN_RadioButtonGroup* result)
{
	BEX;
	RadioButtonWidget annot(ObjCast(button));
	*result = (TRN_RadioButtonGroup) new RadioButtonGroup(annot.GetGroup());
	EEX;
}

TRN_API TRN_PushButtonWidgetCreate(TRN_PDFDoc doc, const TRN_Rect* pos, TRN_UString field_name, TRN_PushButtonWidget* result)
{
	BEX;
	*result = (TRN_PushButtonWidget)PushButtonWidget::Create(*PDocCast(doc), *((Rect*)pos), UString(UStrCast(field_name))).GetSDFObj();
	EEX;
}

TRN_API TRN_PushButtonWidgetCreateWithField(TRN_PDFDoc doc, const TRN_Rect* pos, const TRN_Field* field, TRN_PushButtonWidget* result)
{
	BEX;
	*result = (TRN_PushButtonWidget)PushButtonWidget::Create(*PDocCast(doc), *((Rect*)pos), FieldCast(field)).GetSDFObj();
	EEX;
}

TRN_API TRN_PushButtonWidgetCreateFromObj(TRN_Obj obj, TRN_PushButtonWidget* result)
{
	BEX;
	*result = (TRN_PushButtonWidget)obj;
	EEX;
}
TRN_API TRN_PushButtonWidgetCreateFromAnnot(TRN_Annot annot, TRN_PushButtonWidget* result)
{
	BEX;
	*result = (TRN_PushButtonWidget)annot;
	EEX;
}




