#include <C/PDF/TRN_Bookmark.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/Bookmark.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_BookmarkCreate(TRN_PDFDoc in_doc, TRN_UString in_title, TRN_Bookmark* result)
{
	BEX;
	UString temp(UStrCast(in_title));
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkCreate(PDocCast(in_doc),temp);
	EEX;
}


TRN_API TRN_BookmarkCreateFromObj(TRN_Obj in_bookmark_dict, TRN_Bookmark* result)
{
	BEX;
	*result=(TRN_Bookmark)in_bookmark_dict;
	EEX;
}

TRN_API TRN_BookmarkCopy(const TRN_Bookmark in_bookmark, TRN_Bookmark* result)
{
	BEX;
	*result=(TRN_Bookmark)in_bookmark;
	EEX;
}

TRN_API TRN_BookmarkCompare(TRN_Bookmark bm, const TRN_Bookmark in_bookmark, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_BookmarkCompare(BMCast(bm),BMCast(in_bookmark)));
	EEX;
}

TRN_API TRN_BookmarkIsValid(const TRN_Bookmark bm, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_BookmarkIsValid(BMCast(bm)));
	EEX;
}

TRN_API TRN_BookmarkHasChildren(TRN_Bookmark bm, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_BookmarkHasChildren(BMCast(bm)));
	EEX;
}

TRN_API TRN_BookmarkGetNext(TRN_Bookmark bm, TRN_Bookmark* result)
{
	BEX;
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkGetNext(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkGetPrev(TRN_Bookmark bm, TRN_Bookmark* result)
{
	BEX;
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkGetPrev(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkGetFirstChild(TRN_Bookmark bm, TRN_Bookmark* result)
{
	BEX;
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkGetFirstChild(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkGetLastChild(TRN_Bookmark bm, TRN_Bookmark* result)
{
	BEX;
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkGetLastChild(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkGetParent(TRN_Bookmark bm, TRN_Bookmark* result)
{
	BEX;
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkGetParent(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkFind(TRN_Bookmark bm, const TRN_UString in_title, TRN_Bookmark* result)
{
	BEX;
	UString temp(UStrCast(in_title));
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkFind(BMCast(bm),temp);
	EEX;
}

TRN_API TRN_BookmarkAddNewChild(TRN_Bookmark bm, const TRN_UString in_title, TRN_Bookmark* result)
{
	BEX;
	UString temp(UStrCast(in_title));
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkAddChild(BMCast(bm),temp);
	EEX;
}

TRN_API TRN_BookmarkAddChild(TRN_Bookmark bm, TRN_Bookmark in_bookmark)
{
	BEX;
	IMPL_TRN_BookmarkAddChild(BMCast(bm),BMCast(in_bookmark));
	EEX;
}

TRN_API TRN_BookmarkAddNewNext(TRN_Bookmark bm, const TRN_UString in_title, TRN_Bookmark* result)
{
	BEX;
	UString temp(UStrCast(in_title));
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkAddNext(BMCast(bm),temp);
	EEX;
}

TRN_API TRN_BookmarkAddNext(TRN_Bookmark bm, TRN_Bookmark in_bookmark)
{
	BEX;
	IMPL_TRN_BookmarkAddNext(BMCast(bm),BMCast(in_bookmark));
	EEX;
}

TRN_API TRN_BookmarkAddNewPrev(TRN_Bookmark bm, const TRN_UString in_title, TRN_Bookmark* result)
{
	BEX;
	UString temp(UStrCast(in_title));
	*result=(TRN_Bookmark)IMPL_TRN_BookmarkAddPrev(BMCast(bm),temp);
	EEX;
}

TRN_API TRN_BookmarkAddPrev(TRN_Bookmark bm,TRN_Bookmark in_bookmark)
{
	BEX;
	IMPL_TRN_BookmarkAddPrev(BMCast(bm),BMCast(in_bookmark));
	EEX;
}

TRN_API TRN_BookmarkDelete(TRN_Bookmark bm)
{
	BEX;
	IMPL_TRN_BookmarkDelete(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkUnlink(TRN_Bookmark bm)
{
	BEX;
	IMPL_TRN_BookmarkUnlink(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkGetIndent(TRN_Bookmark bm, int* result)
{
	BEX;
	*result=IMPL_TRN_BookmarkGetIndent(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkIsOpen(TRN_Bookmark bm, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_BookmarkIsOpen(BMCast(bm)));
	EEX;
}

TRN_API TRN_BookmarkSetOpen(TRN_Bookmark bm, TRN_Bool in_open)
{
	BEX;
	IMPL_TRN_BookmarkSetOpen(BMCast(bm),TBToB(in_open));
	EEX;
}

TRN_API TRN_BookmarkGetOpenCount(TRN_Bookmark bm, int* result)
{
	BEX;
	*result=IMPL_TRN_BookmarkGetOpenCount(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkGetTitle(TRN_Bookmark bm, TRN_UString* result)
{
	BEX;
	RetUStr(IMPL_TRN_BookmarkGetTitle(BMCast(bm)));
	EEX;
}

TRN_API TRN_BookmarkGetTitleObj(TRN_Bookmark bm, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_BookmarkGetTitleObj(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkSetTitle(TRN_Bookmark bm, const TRN_UString title)
{
	BEX;
	UString temp(UStrCast(title));
	IMPL_TRN_BookmarkSetTitle(BMCast(bm),temp);
	EEX;
}

TRN_API TRN_BookmarkGetAction(TRN_Bookmark bm, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)IMPL_TRN_BookmarkGetAction(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkSetAction(TRN_Bookmark bm, TRN_Action in_action)
{
	BEX;
	IMPL_TRN_BookmarkSetAction(BMCast(bm),ActCast(in_action));
	EEX;
}

TRN_API TRN_BookmarkRemoveAction(TRN_Bookmark bm)
{
	BEX;
	IMPL_TRN_BookmarkRemoveAction(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkGetFlags(TRN_Bookmark bm, int* result)
{
	BEX;
	*result=IMPL_TRN_BookmarkGetFlags(BMCast(bm));
	EEX;
}

TRN_API TRN_BookmarkSetFlags(TRN_Bookmark bm, int in_flags)
{
	BEX;
	IMPL_TRN_BookmarkSetFlags(BMCast(bm),in_flags);
	EEX;
}

TRN_API TRN_BookmarkGetColor(TRN_Bookmark bm, double* out_r, double* out_g, double* out_b)
{
	BEX;
	IMPL_TRN_BookmarkGetColor(BMCast(bm),*out_r,*out_g,*out_b);
	EEX;
}

TRN_API TRN_BookmarkSetColor(TRN_Bookmark bm, double in_r, double in_g, double in_b)
{
	BEX;
	IMPL_TRN_BookmarkSetColor(BMCast(bm),in_r,in_g,in_b);
	EEX;
}

TRN_API TRN_BookmarkGetSDFObj(const TRN_Bookmark bm, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)bm;
	EEX;
}
