#include <C/PDF/TRN_ColorSpace.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/ColorSpace.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;

TRN_API TRN_ColorPtCreate(TRN_ColorPt** result)
{
	BEX;
	*result = new TRN_ColorPt;
	memset(*result, 0, sizeof(TRN_ColorPt));
	ColorPt* c=(ColorPt*)(*result);
	c->c=c->_GetCAddress();
	EEX;
}

TRN_API TRN_ColorPtInit(double x, double y, double z, double w, TRN_ColorPt* result)
{
	BEX;
	assert(sizeof(TRN_ColorPt)>=sizeof(ColorPt));
	ColorPt* c=(ColorPt*)result;
	c->c=c->_GetCAddress();
	c->Set(x,y,z,w);
	EEX;
}


TRN_API TRN_ColorPtAssign(TRN_ColorPt* left, const TRN_ColorPt* right)
{
	BEX;
	((ColorPt*)left)->operator=(*((ColorPt*)right));
	EEX;
}

TRN_API TRN_ColorPtCompare(const TRN_ColorPt* left, const TRN_ColorPt* right, TRN_Bool* result)
{
	BEX;
	*result=BToTB(CPCast(left)==CPCast(right));
	EEX;
}


TRN_API TRN_ColorPtDestroy(TRN_ColorPt* cp)
{
	BEX;
	ColorPt* c=(ColorPt*)cp;
	if(c->c!=c->_GetCAddress())
		delete [] c->c;
	EEX;
}

TRN_API TRN_ColorPtDestroy2(TRN_ColorPt* cp)
{
	BEX;
	TRN_ColorPtDestroy(cp);
	delete cp;
	EEX;
}

TRN_API TRN_ColorPtSet(TRN_ColorPt* cp, double x, double y, double z, double w)
{
	BEX;
	((ColorPt*)cp)->Set(x,y,z,w);
	EEX;
}


TRN_API TRN_ColorPtSetByIndex(TRN_ColorPt* cp, int colorant_index, double colorant_value)
{
	BEX;
	((ColorPt*)cp)->Set(colorant_index,colorant_value);
	EEX;
}
TRN_API TRN_ColorPtGet(const TRN_ColorPt* cp, int colorant_index, double* result)
{
	BEX;
	*result=((const ColorPt*)cp)->Get(colorant_index);
	EEX;
}
TRN_API TRN_ColorPtSetColorantNum(TRN_ColorPt* cp, int num)
{
	BEX;
	((ColorPt*)cp)->SetColorantNum(num);
	EEX;
}


























TRN_API TRN_ColorSpaceCreateDeviceGray(TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(ColorSpace::CreateDeviceGray());
	EEX;
}

TRN_API TRN_ColorSpaceCreateDeviceRGB(TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(ColorSpace::CreateDeviceRGB());
	EEX;
}

TRN_API TRN_ColorSpaceCreateDeviceCMYK(TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(ColorSpace::CreateDeviceCMYK());
	EEX;
}

TRN_API TRN_ColorSpaceCreateICCFromFile( TRN_SDFDoc doc, const TRN_UString filepath, TRN_ColorSpace* result )
{
	BEX;
	*result = (TRN_ColorSpace)(new ColorSpace( ColorSpace::CreateICC( *(SDFDoc*)(doc), *((UString*)filepath) ) ));
	EEX;
}

TRN_API TRN_ColorSpaceCreateICCFromFilter( TRN_SDFDoc doc, TRN_Filter filter, TRN_ColorSpace* result )
{
	BEX;
	AutoPtr<trn::Filters::Filter> ptr((trn::Filters::Filter*)filter);
	*result = (TRN_ColorSpace)(new ColorSpace( ColorSpace::CreateICC( *(SDFDoc*)(doc), ptr ) ));
	EEX;
}

TRN_API TRN_ColorSpaceCreateICCFromBuffer( TRN_SDFDoc doc, const TRN_UChar* buf, const TRN_UInt32 buf_sz, TRN_ColorSpace* result )
{
	BEX;
	*result = (TRN_ColorSpace)(new ColorSpace( ColorSpace::CreateICC( *(SDFDoc*)(doc), buf, buf_sz) ));
	EEX;
}



TRN_API TRN_ColorSpaceCreatePattern(TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(ColorSpace::CreatePattern());
	EEX;
}

TRN_API TRN_ColorSpaceCreate (TRN_Obj color_space, TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace((Obj*)color_space);
	EEX;
}


TRN_API TRN_ColorSpaceAssign(TRN_ColorSpace left, TRN_ColorSpace right)
{
	BEX;
	*((ColorSpace*)left)=*((ColorSpace*)right);
	EEX;
}


TRN_API TRN_ColorSpaceDestroy(TRN_ColorSpace cs)
{
	BEX;
	delete (ColorSpace*)cs;
	EEX;
}

TRN_API TRN_ColorSpaceGetComponentNumFromObj(enum TRN_ColorSpaceType cs_type, TRN_Obj cs_obj, int* result)
{
	BEX;
	*result=ColorSpace::GetComponentNum((ColorSpace::Type)cs_type,(Obj*)cs_obj);
	EEX;
}

TRN_API TRN_ColorSpaceGetTypeFromObj(TRN_Obj cs_obj, enum TRN_ColorSpaceType* result)
{
	BEX;
	*result=(enum TRN_ColorSpaceType)ColorSpace::GetType((Obj*)cs_obj);
	EEX;
}

TRN_API TRN_ColorSpaceGetType(const TRN_ColorSpace cs, enum TRN_ColorSpaceType* result)
{
	BEX;
	*result=(enum TRN_ColorSpaceType)CSCast(cs).GetType();
	EEX;
}

TRN_API TRN_ColorSpaceGetSDFObj (TRN_ColorSpace cs, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)CSCast(cs).GetSDFObj();
	EEX;
}

TRN_API TRN_ColorSpaceGetComponentNum(TRN_ColorSpace cs, int* result)
{
	BEX;
	*result=CSCast(cs).GetComponentNum();
	EEX;
}

TRN_API TRN_ColorSpaceInitColor(TRN_ColorSpace cs, TRN_ColorPt* out_colorants)
{
	BEX;
	CSCast(cs).InitColor(CPCast(out_colorants));
	EEX;
}

TRN_API TRN_ColorSpaceInitComponentRanges(TRN_ColorSpace cs, 
										  double* out_decode_low, double* out_decode_range,
										 int num_comps)
{
	BEX;
	std::vector<double> low,range;
	CSCast(cs).InitComponentRanges(low,range);
	BASE_ASSERT(low.size()==num_comps && range.size()==num_comps, "Component Range Sizes Incorrect");
	memcpy(out_decode_low,&(low[0]),num_comps*sizeof(double));
	memcpy(out_decode_range,&(range[0]),num_comps*sizeof(double));
	EEX;
}

TRN_API TRN_ColorSpaceConvert2Gray(TRN_ColorSpace cs, const TRN_ColorPt* in_color, TRN_ColorPt* out_color)
{
	BEX;
	CSCast(cs).Convert2Gray(CPCast(in_color),CPCast(out_color));
	EEX;
}

TRN_API TRN_ColorSpaceConvert2RGB(TRN_ColorSpace cs, const TRN_ColorPt* in_color, TRN_ColorPt* out_color)
{
	BEX;
	CSCast(cs).Convert2RGB(CPCast(in_color),CPCast(out_color));
	EEX;
}

TRN_API TRN_ColorSpaceConvert2CMYK(TRN_ColorSpace cs, const TRN_ColorPt* in_color, TRN_ColorPt* out_color)
{
	BEX;
	CSCast(cs).Convert2CMYK(CPCast(in_color),CPCast(out_color));
	EEX;
}

TRN_API TRN_ColorSpaceGetAlternateColorSpace(TRN_ColorSpace cs, TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(CSCast(cs).GetAlternateColorSpace());
	EEX;
}

TRN_API TRN_ColorSpaceGetBaseColorSpace(TRN_ColorSpace cs, TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(CSCast(cs).GetBaseColorSpace());
	EEX;
}

TRN_API TRN_ColorSpaceGetHighVal(TRN_ColorSpace cs, int* result)
{
	BEX;
	*result=CSCast(cs).GetHighVal();
	EEX;
}

TRN_API TRN_ColorSpaceGetLookupTable(TRN_ColorSpace cs, const TRN_UChar** result)
{
	BEX;
	*result=(const TRN_UChar*)CSCast(cs).GetLookupTable();
	EEX;
}

TRN_API TRN_ColorSpaceGetBaseColor(TRN_ColorSpace cs, TRN_UChar color_idx, TRN_ColorPt* out_color)
{
	BEX;
	CSCast(cs).GetBaseColor((UChar)color_idx,CPCast(out_color));
	EEX;
}

TRN_API TRN_ColorSpaceGetTintFunction(TRN_ColorSpace cs, TRN_Function* result)
{
	BEX;
	*result=(TRN_Function)new Function(CSCast(cs).GetTintFunction());
	EEX;
}

TRN_API TRN_ColorSpaceIsAll(TRN_ColorSpace cs, TRN_Bool* result)
{
	BEX;
	*result=BToTB(CSCast(cs).IsAll());
	EEX;
}

TRN_API TRN_ColorSpaceIsNone(TRN_ColorSpace cs, TRN_Bool* result)
{
	BEX;
	*result=BToTB(CSCast(cs).IsNone());
	EEX;
}


