#include <C/PDF/TRN_Date.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/Date.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;

TRN_API TRN_DateInit (TRN_UInt16 year, char month, char day, char hour, 
					  char minute, char second, TRN_Date* result)
{
	BEX;
	Date d(year, month, day, hour, minute, second);
	assert(sizeof(TRN_Date)>=sizeof(d));
	memcpy(result, &d, sizeof(d));
	EEX;
}

TRN_API TRN_DateAssign (TRN_Date* left, const TRN_Date* right)
{
	BEX;
	((Date*)left)->operator =(*((Date*)right));
	EEX;
}

TRN_API TRN_DateIsValid(const TRN_Date* date, TRN_Bool* result)
{
	BEX;
	*result=BToTB(((Date*)date)->IsValid());
	EEX;
}

TRN_API TRN_DateAttach(TRN_Date* date, TRN_Obj d)
{
	BEX;
	((Date*)date)->Attach((Obj*)d);
	EEX;
}

TRN_API TRN_DateUpdate(TRN_Date* date, TRN_Obj d, TRN_Bool* result)
{
	BEX;
	*result=BToTB(((Date*)date)->Update((Obj*)d));
	EEX;
}

TRN_API_T(void) TRN_DateSetCurrentTime(TRN_Date* date)
{
	((Date*)date)->SetCurrentTime();
}

TRN_API_T(TRN_UInt16) TRN_DateGetYear(TRN_Date* date) {
	return ((Date*)date)->year;
}

TRN_API_T(TRN_UChar) TRN_DateGetMonth(TRN_Date* date) {
	return ((Date*)date)->month;
}

TRN_API_T(TRN_UChar) TRN_DateGetDay(TRN_Date* date) {
	return ((Date*)date)->day;
}

TRN_API_T(TRN_UChar) TRN_DateGetHour(TRN_Date* date) {
	return ((Date*)date)->hour;
}

TRN_API_T(TRN_UChar) TRN_DateGetMinute(TRN_Date* date) {
	return ((Date*)date)->minute;
}

TRN_API_T(TRN_UChar) TRN_DateGetSecond(TRN_Date* date) {
	return ((Date*)date)->second;
}

TRN_API_T(TRN_UChar) TRN_DateGetUT(TRN_Date* date) {
	return ((Date*)date)->UT;
}

TRN_API_T(TRN_UChar) TRN_DateGetUTHour(TRN_Date* date) {
	return ((Date*)date)->UT_hour;
}

TRN_API_T(TRN_UChar) TRN_DateGetUTMin(TRN_Date* date) {
	return ((Date*)date)->UT_minutes;
}
