//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#include <C/PDF/TRN_DigitalSignatureField.h>
#include <C/Common/TRN_API_Config.hpp>
#include <C/Common/TRN_Exception.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <PDF/DigitalSignatureField.h>
#include <C/Common/TRN_PolymorphicVector.hpp>
#include <SDF/SignatureHandler.h>
#include <Common/UStringImpl.hpp>
#include <C/Common/TRN_Vector.h>
#include <PDF/VerificationResult.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_DigitalSignatureFieldHasCryptographicSignature(const TRN_DigitalSignatureField* self, TRN_Bool* result)
{
	BEX;
	*result = ((trn::PDF::DigitalSignatureField*)self )->HasCryptographicSignature();
	EEX;
}


TRN_API TRN_DigitalSignatureFieldGetSubFilter(const TRN_DigitalSignatureField* self, enum TRN_DigitalSignatureField_SubFilterType* result)
{
	BEX;
	*(trn::UInt32*)result = (TRN_DigitalSignatureField_SubFilterType) ((trn::PDF::DigitalSignatureField*)self )->GetSubFilter();
	EEX;
}


TRN_API TRN_DigitalSignatureFieldGetSignatureName(const TRN_DigitalSignatureField* self, TRN_UString* result)
{
	BEX;
	RetUStr(((trn::PDF::DigitalSignatureField*)self )->GetSignatureName());
	EEX;
}


TRN_API TRN_DigitalSignatureFieldGetLocation(const TRN_DigitalSignatureField* self, TRN_UString* result)
{
	BEX;
	RetUStr(((trn::PDF::DigitalSignatureField*)self )->GetLocation());
	EEX;
}


TRN_API TRN_DigitalSignatureFieldGetReason(const TRN_DigitalSignatureField* self, TRN_UString* result)
{
	BEX;
	RetUStr(((trn::PDF::DigitalSignatureField*)self )->GetReason());
	EEX;
}


TRN_API TRN_DigitalSignatureFieldGetContactInfo(const TRN_DigitalSignatureField* self, TRN_UString* result)
{
	BEX;
	RetUStr(((trn::PDF::DigitalSignatureField*)self )->GetContactInfo());
	EEX;
}


TRN_API TRN_DigitalSignatureFieldGetCertCount(const TRN_DigitalSignatureField* self, TRN_UInt32* result)
{
	BEX;
	*result = ((trn::PDF::DigitalSignatureField*)self )->GetCertCount();
	EEX;
}


TRN_API TRN_DigitalSignatureFieldHasVisibleAppearance(const TRN_DigitalSignatureField* self, TRN_Bool* result)
{
	BEX;
	*result = ((trn::PDF::DigitalSignatureField*)self )->HasVisibleAppearance();
	EEX;
}


TRN_API TRN_DigitalSignatureFieldSetContactInfo(TRN_DigitalSignatureField* self, const TRN_UString in_contact_info)
{
	BEX;
	((trn::PDF::DigitalSignatureField*)self )->SetContactInfo(UString(in_contact_info));
	EEX;
}


TRN_API TRN_DigitalSignatureFieldSetLocation(TRN_DigitalSignatureField* self, const TRN_UString in_location)
{
	BEX;
	((trn::PDF::DigitalSignatureField*)self )->SetLocation(UString(in_location));
	EEX;
}


TRN_API TRN_DigitalSignatureFieldSetReason(TRN_DigitalSignatureField* self, const TRN_UString in_reason)
{
	BEX;
	((trn::PDF::DigitalSignatureField*)self )->SetReason(UString(in_reason));
	EEX;
}


TRN_API TRN_DigitalSignatureFieldSetDocumentPermissions(TRN_DigitalSignatureField* self, enum TRN_DigitalSignatureField_DocumentPermissions in_perms)
{
	BEX;
	((trn::PDF::DigitalSignatureField*)self )->SetDocumentPermissions((trn::PDF::DigitalSignatureField::DocumentPermissions)in_perms);
	EEX;
}


TRN_API TRN_DigitalSignatureFieldSignOnNextSave(TRN_DigitalSignatureField* self, const TRN_UString in_pkcs12_keyfile_path, const TRN_UString in_password)
{
	BEX;
	((trn::PDF::DigitalSignatureField*)self )->SignOnNextSave(UString(in_pkcs12_keyfile_path), UString(in_password));
	EEX;
}


TRN_API TRN_DigitalSignatureFieldCertifyOnNextSave(TRN_DigitalSignatureField* self, const TRN_UString in_pkcs12_keyfile_path, const TRN_UString in_password)
{
	BEX;
	((trn::PDF::DigitalSignatureField*)self )->CertifyOnNextSave(UString(in_pkcs12_keyfile_path), UString(in_password));
	EEX;
}


TRN_API TRN_DigitalSignatureFieldIsLockedByDigitalSignature(const TRN_DigitalSignatureField* self, TRN_Bool* result)
{
	BEX;
	*result = ((trn::PDF::DigitalSignatureField*)self )->IsLockedByDigitalSignature();
	EEX;
}


TRN_API TRN_DigitalSignatureFieldGetDocumentPermissions(const TRN_DigitalSignatureField* self, enum TRN_DigitalSignatureField_DocumentPermissions* result)
{
	BEX;
	*(trn::UInt32*)result = ((trn::PDF::DigitalSignatureField*)self )->GetDocumentPermissions();
	EEX;
}


TRN_API TRN_DigitalSignatureFieldClearSignature(TRN_DigitalSignatureField* self)
{
	BEX;
	((trn::PDF::DigitalSignatureField*)self )->ClearSignature();
	EEX;
}

// Not codegened originally:

TRN_API TRN_DigitalSignatureFieldCreateFromField(const TRN_Field* d, TRN_DigitalSignatureField* result)
{
	BEX;
	BASE_ASSERT(FieldCast(d).IsValid() && FieldCast(d).GetType() == trn::PDF::Field::e_signature,
	"Field argument to DigitalSignatureField constructor is not valid or not a Signature field");
	
	trn::PDF::Field actual_field = FieldCast(d).GetActualField();
	trn::SDF::Obj* actual_field_obj = actual_field.GetSDFObj();

	BASE_ASSERT(actual_field_obj && actual_field_obj->IsDict(), "Field dictionary is invalid");

	result->mp_field_dict_obj = (TRN_Obj) actual_field_obj;
	EEX;
}

TRN_API TRN_DigitalSignatureFieldGetSigningTime(const TRN_DigitalSignatureField* self, TRN_Date* result)
{
	BEX;
	trn::PDF::Date date(((trn::PDF::DigitalSignatureField*)self)->GetSigningTime());
	assert(sizeof(TRN_Date) >= sizeof(date));
	memcpy(result, &date, sizeof(date));
	EEX;
}

TRN_API TRN_DigitalSignatureFieldGetCert(const TRN_DigitalSignatureField* self, TRN_UInt32 in_index, TRN_Vector* result)
{
	BEX;
	*result = (TRN_Vector)(new PolyVectorByte(((const trn::PDF::DigitalSignatureField*)self)->GetCert(in_index)));
	EEX;
}

TRN_API TRN_DigitalSignatureFieldSetFieldPermissions(TRN_DigitalSignatureField* self, const enum TRN_DigitalSignatureField_FieldPermissions in_action, const TRN_UString* in_field_names_list, TRN_UInt32 in_field_names_size)
{
	BEX;
	// TODO: improve (do without copying)
	std::vector<UString> tmp;
	for (UInt32 i = 0; i < in_field_names_size; i++)
	{
		tmp.push_back(UString(in_field_names_list[i])); // special UString constructor from impl ptr
	}
	DigSigFieldCast(self).SetFieldPermissions((trn::PDF::DigitalSignatureField::FieldPermissions) ((UInt32) in_action), tmp);
	EEX;
}

TRN_API TRN_DigitalSignatureFieldSignOnNextSaveFromBuffer(TRN_DigitalSignatureField* self, const TRN_UInt8* in_pkcs12_buffer, const TRN_UInt32 in_pkcs12_buf_size, const TRN_UString in_password)
{
	BEX;
	// special UString constructor from impl ptr
	DigSigFieldCast(self).SignOnNextSave((const UInt8*) in_pkcs12_buffer, (UInt32) in_pkcs12_buf_size, UString(in_password));
	EEX;
}

TRN_API TRN_DigitalSignatureFieldSignOnNextSaveWithCustomHandler(TRN_DigitalSignatureField* self, const TRN_SignatureHandlerId in_signature_handler_id)
{
	BEX;
	DigSigFieldCast(self).SignOnNextSaveWithCustomHandler((const trn::SDF::SignatureHandlerId) in_signature_handler_id);
	EEX;
}

TRN_API TRN_DigitalSignatureFieldCertifyOnNextSaveFromBuffer(TRN_DigitalSignatureField* self, const TRN_UInt8* in_pkcs12_buffer, const TRN_UInt32 in_pkcs12_buf_size, const TRN_UString in_password)
{
	BEX;
	// special UString constructor from impl ptr
	DigSigFieldCast(self).CertifyOnNextSave((const UInt8*)in_pkcs12_buffer, (UInt32)in_pkcs12_buf_size, UString(in_password));
	EEX;
}

TRN_API TRN_DigitalSignatureFieldCertifyOnNextSaveWithCustomHandler(TRN_DigitalSignatureField* self, const TRN_SignatureHandlerId in_signature_handler_id)
{
	BEX;
	DigSigFieldCast(self).CertifyOnNextSaveWithCustomHandler((const trn::SDF::SignatureHandlerId) in_signature_handler_id);
	EEX;
}

TRN_API TRN_DigitalSignatureFieldGetSDFObj(const TRN_DigitalSignatureField* self, TRN_Obj* result)
{
	BEX;
	*result = self->mp_field_dict_obj;
	EEX;
}

TRN_API TRN_DigitalSignatureFieldGetLockedFields(const TRN_DigitalSignatureField* self, TRN_Vector* result)
{
	// Broadly similar to TRN_PDFDrawGetSeparationBitmaps implementation
	BEX;
	FlexVector<trn::UString> tmp(DigSigFieldCast(self).GetLockedFields());

	PolyVectorBase<TRN_UString>* tmp2 = new PolyVectorBase<TRN_UString>();
	*result = (TRN_Vector) tmp2;
	try
	{
		for (FlexVector<trn::UString>::iterator it = tmp.begin(); it != tmp.end(); ++it)
		{
			TRN_UString result;
			TRN_UStringCopy((TRN_UString)it->mp_impl, &result);
			tmp2->m_vector.push_back(result);
		}
	}
	catch (...)
	{
		TRN_VectorDestroy(*result);
		throw;
	}
	EEX;
}

TRN_API TRN_DigitalSignatureFieldVerify(const TRN_DigitalSignatureField* self, const TRN_VerificationOptions in_opts, TRN_VerificationResult* result)
{
	BEX;
	*result = (TRN_VerificationResult)(DigSigFieldCast(self).Verify(*(trn::PDF::VerificationOptions*) in_opts).release());
	EEX;
}
