#include <C/PDF/TRN_Element.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <PDF/Element.h>
#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_ElementGetType(TRN_Element e, enum TRN_ElementType* result)
{
	BEX;
	*result=(enum TRN_ElementType)EleCast(e)->GetType();
	EEX;
}

TRN_API TRN_ElementGetGState(TRN_Element e, TRN_GState* result)
{
	BEX;
	*result=(TRN_GState)EleCast(e)->GetGState();
	EEX;
}

TRN_API TRN_ElementGetCTM(TRN_Element e, TRN_Matrix2D* result)
{
	BEX;
	*result=RetM2D(EleCast(e)->GetCTM());
	EEX;
}

TRN_API TRN_ElementGetBBox(TRN_Element e, TRN_Rect* out_bbox, TRN_Bool* result)
{
	BEX;
	*result=BToTB(EleCast(e)->GetBBox(RectCast(out_bbox)));
	EEX;
}

TRN_API TRN_ElementGetParentStructElement(TRN_Element e, TRN_SElement* result)
{
	BEX;
    Struct::SElement selement = EleCast(e)->GetParentStructElement();
	*result=RetSEle(selement);
	EEX;
}

TRN_API TRN_ElementGetStructMCID(TRN_Element e, int* result)
{
	BEX;
	*result=EleCast(e)->GetStructMCID();
	EEX;
}

TRN_API TRN_ElementIsOCVisible(TRN_Element e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(EleCast(e)->IsOCVisible());
	EEX;
}

TRN_API TRN_ElementIsClippingPath(TRN_Element e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(EleCast(e)->IsClippingPath());
	EEX;
}

TRN_API TRN_ElementIsStroked(TRN_Element e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(EleCast(e)->IsStroked());
	EEX;
}

TRN_API TRN_ElementIsFilled(TRN_Element e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(EleCast(e)->IsFilled());
	EEX;
}

TRN_API TRN_ElementIsWindingFill(TRN_Element e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(EleCast(e)->IsWindingFill());
	EEX;
}

TRN_API TRN_ElementIsClipWindingFill(TRN_Element e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(EleCast(e)->IsClipWindingFill());
	EEX;
}

TRN_API TRN_ElementGetPathTypes(TRN_Element e, const char** result)
{
	BEX;
	*result=EleCast(e)->GetPathTypes();
	EEX;
}

TRN_API TRN_ElementGetPathTypesCount(TRN_Element e, int* result)
{
	BEX;
	*result=EleCast(e)->GetPathTypesCount();
	EEX;
}

TRN_API TRN_ElementGetPathPoints(TRN_Element e, const double** result)
{
	BEX;
	*result=EleCast(e)->GetPathPoints();
	EEX;
}

TRN_API TRN_ElementGetPathPointCount(TRN_Element e, int* result)
{
	BEX;
	*result=EleCast(e)->GetPointCount();
	EEX;
}

TRN_API TRN_ElementSetPathClip(TRN_Element e, TRN_Bool clip)
{
	BEX;
	EleCast(e)->SetPathClip(TBToB(clip));
	EEX;
}

TRN_API TRN_ElementSetPathStroke(TRN_Element e, TRN_Bool stroke)
{
	BEX;
	EleCast(e)->SetPathStroke(TBToB(stroke));
	EEX;
}

TRN_API TRN_ElementSetPathFill(TRN_Element e, TRN_Bool fill)
{
	BEX;
	EleCast(e)->SetPathFill(TBToB(fill));
	EEX;
}

TRN_API TRN_ElementSetWindingFill(TRN_Element e, TRN_Bool winding_rule)
{
	BEX;
	EleCast(e)->SetWindingFill(TBToB(winding_rule));
	EEX;
}

TRN_API TRN_ElementSetClipWindingFill(TRN_Element e, TRN_Bool winding_rule)
{
	BEX;
	EleCast(e)->SetClipWindingFill(TBToB(winding_rule));
	EEX;
}

TRN_API TRN_ElementSetPathPoints(TRN_Element e, const double* buf_in_points, int buf_count)
{
	BEX;
	EleCast(e)->SetPathPoints(buf_in_points, buf_count);
	EEX;
}

TRN_API TRN_ElementSetPathTypes(TRN_Element e, const char* in_seg_types, int count)
{
	BEX;
	EleCast(e)->SetPathTypes(in_seg_types,count);
	EEX;
}

TRN_API TRN_ElementGetXObject(TRN_Element e, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)EleCast(e)->GetXObject();
	EEX;
}

TRN_API TRN_ElementGetImageData(TRN_Element e, TRN_Filter* result)
{
	BEX;
	*result=(TRN_Filter)(EleCast(e)->GetImageData().release());
	EEX;
}

TRN_API TRN_ElementGetImageDataSize(TRN_Element e, int* result)
{
	BEX;
	*result=EleCast(e)->GetImageDataSize();
	EEX;
}

#if defined (_WIN32)
TRN_API TRN_ElementGetBitmap(TRN_Element e, TRN_GDIPlusBitmap* result)
{
	BEX;
	*result=(TRN_GDIPlusBitmap)EleCast(e)->GetBitmap();
	EEX;
}
#endif

TRN_API TRN_ElementGetImageColorSpace(TRN_Element e, TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(EleCast(e)->GetImageColorSpace());
	EEX;
}

TRN_API TRN_ElementGetImageWidth(TRN_Element e, int* result)
{
	BEX;
	*result=EleCast(e)->GetImageWidth();
	EEX;
}

TRN_API TRN_ElementGetImageHeight(TRN_Element e, int* result)
{
	BEX;
	*result=EleCast(e)->GetImageHeight();
	EEX;
}

TRN_API TRN_ElementGetDecodeArray(TRN_Element e, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)EleCast(e)->GetDecodeArray();
	EEX;
}

TRN_API TRN_ElementGetBitsPerComponent(TRN_Element e, int* result)
{
	BEX;
	*result=EleCast(e)->GetBitsPerComponent();
	EEX;
}

TRN_API TRN_ElementGetComponentNum(TRN_Element e, int* result)
{
	BEX;
	*result=EleCast(e)->GetComponentNum();
	EEX;
}

TRN_API TRN_ElementIsImageMask(TRN_Element e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(EleCast(e)->IsImageMask());
	EEX;
}

TRN_API TRN_ElementIsImageInterpolate(TRN_Element e, TRN_Bool* result) 
{
	BEX;
	*result=BToTB(EleCast(e)->IsImageInterpolate());
	EEX;
}

TRN_API TRN_ElementGetMask(TRN_Element e, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)EleCast(e)->GetMask();
	EEX;
}

TRN_API TRN_ElementGetImageRenderingIntent(TRN_Element e, enum TRN_GStateRenderingIntent* result)
{
	BEX;
	*result=(enum TRN_GStateRenderingIntent)EleCast(e)->GetImageRenderingIntent();
	EEX;
}

TRN_API TRN_ElementGetTextString(TRN_Element e, TRN_UString* result)
{
	BEX;
	RetUStr(EleCast(e)->GetTextString());
	EEX;
}

TRN_API TRN_ElementGetTextData(TRN_Element e,  const TRN_UChar** result)
{
	BEX;
	*result=(const TRN_UChar*)EleCast(e)->GetTextData();
	EEX;
}

TRN_API TRN_ElementGetTextDataSize(TRN_Element e, TRN_UInt32* result)
{
	BEX;
	*result=EleCast(e)->GetTextDataSize();
	EEX;
}

TRN_API TRN_ElementGetTextMatrix(TRN_Element e, TRN_Matrix2D* result)
{
	BEX;
	*result=RetM2D(EleCast(e)->GetTextMatrix());
	EEX;
}

TRN_API TRN_ElementGetCharIterator(TRN_Element e, TRN_Iterator* result)
{
	BEX;
	CharIterator itr=EleCast(e)->CharBegin(),end=EleCast(e)->CharEnd();
	*result=(TRN_Iterator)new PolyCharIterator(itr,end);
	EEX;
}

TRN_API TRN_ElementGetTextLength(TRN_Element e, double* result)
{
	BEX;
	*result=EleCast(e)->GetTextLength();
	EEX;
}

TRN_API TRN_ElementGetPosAdjustment(TRN_Element e, double* result)
{
	BEX;
	*result=EleCast(e)->GetPosAdjustment();
	EEX;
}

TRN_API TRN_ElementGetNewTextLineOffset(TRN_Element e, double* out_x, double* out_y)
{
	BEX;
	EleCast(e)->GetNewTextLineOffset(*out_x,*out_y);
	EEX;
}

TRN_API TRN_ElementHasTextMatrix(TRN_Element e, TRN_Bool* result)
{
	BEX;
	*result=BToTB(EleCast(e)->HasTextMatrix());
	EEX;
}

TRN_API TRN_ElementSetTextData(TRN_Element e, const TRN_UChar* buf_text_data, int buf_text_data_size)
{
	BEX;
	EleCast(e)->SetTextData((const UChar*)buf_text_data, buf_text_data_size);
	EEX;
}

TRN_API TRN_ElementSetTextMatrix(TRN_Element e, const TRN_Matrix2D* mtx)
{
	BEX;
	EleCast(e)->SetTextMatrix(M2DCast(mtx));
	EEX;
}

TRN_API TRN_ElementSetTextMatrixEntries(TRN_Element e, double a, double b, double c, double d, double h, double v)
{
	BEX;
	EleCast(e)->SetTextMatrix(a,b,c,d,h,v);
	EEX;
}

TRN_API TRN_ElementSetPosAdjustment(TRN_Element e, double adjust)
{
	BEX;
	EleCast(e)->SetPosAdjustment(adjust);
	EEX;
}

TRN_API TRN_ElementUpdateTextMetrics(TRN_Element e)
{
	BEX;
	EleCast(e)->UpdateTextMetrics();
	EEX;
}

TRN_API TRN_ElementSetNewTextLineOffset(TRN_Element e, double dx, double dy)
{
	BEX;
	EleCast(e)->SetNewTextLineOffset(dx,dy);
	EEX;
}

TRN_API TRN_ElementGetShading(TRN_Element e, TRN_Shading* result)
{
	BEX;
	*result=(TRN_Shading)new Shading(EleCast(e)->GetShading());
	EEX;
}

TRN_API TRN_ElementGetMCPropertyDict(TRN_Element e, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)EleCast(e)->GetMCPropertyDict();
	EEX;
}

TRN_API TRN_ElementGetMCTag(TRN_Element e, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)EleCast(e)->GetMCTag();
	EEX;
}

