#include <C/PDF/TRN_Flattener.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <Convert/Flattener.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;

#define FlattenerCast(flattener) (*((Flattener*)flattener))

TRN_API TRN_FlattenerCreate (TRN_Flattener* result)
{
	BEX;
	*result=(TRN_Flattener)new Flattener();
	EEX;
}

TRN_API TRN_FlattenerDestroy(TRN_Flattener flattener)
{
	BEX;
	delete((Flattener*)flattener);
	EEX;
}

TRN_API TRN_FlattenerSetDPI(TRN_Flattener flattener, TRN_UInt32 dpi)
{
	BEX;
	FlattenerCast(flattener).SetDPI(static_cast<UInt32>(dpi));
	EEX;
}

TRN_API TRN_FlattenerSetThreshold(TRN_Flattener flattener, enum TRN_FlattenerThreshold threshold)
{
	BEX;
    FlattenerCast(flattener).SetThreshold(static_cast<FlattenPDF::FlattenThreshold>(threshold));
	EEX;
}

TRN_API TRN_FlattenerSetMaximumImagePixels(TRN_Flattener flattener, TRN_UInt32 max_pixels)
{
	BEX;
	FlattenerCast(flattener).SetMaximumImagePixels(static_cast<UInt32>(max_pixels));
	EEX;
}

TRN_API TRN_FlattenerSetPreferJPG(TRN_Flattener flattener, TRN_Bool jpg)
{
	BEX;
	FlattenerCast(flattener).SetPreferJpg(TBToB(jpg));
	EEX;
}

TRN_API TRN_FlattenerSetJPGQuality(TRN_Flattener flattener, TRN_UInt32 quality)
{
	BEX;
	FlattenerCast(flattener).SetJpgQuality(quality);
	EEX;
}

TRN_API TRN_FlattenerSetPathHinting(TRN_Flattener flattener, TRN_Bool hinting)
{
	BEX;
	FlattenerCast(flattener).SetPathHinting(TBToB(hinting));
	EEX;
}

TRN_API TRN_FlattenerProcess(TRN_Flattener flattener, TRN_PDFDoc doc, enum TRN_FlattenerMode mode)
{
	BEX;
	PDFDoc* d=(PDFDoc*)&doc;
	FlattenerCast(flattener).Process(*d, static_cast<Flattener::Mode>(mode), false);
	EEX;
}

TRN_API TRN_FlattenerProcessPage(TRN_Flattener flattener, TRN_Page page, enum TRN_FlattenerMode mode)
{
	BEX;
	Page p(PageCast(page));
	FlattenerCast(flattener).Process(p, static_cast<Flattener::Mode>(mode), false);
	EEX;
};
