#include <C/PDF/TRN_Font.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <string>
#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>
#include <PDF/Font.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace std;
using namespace SDF;
using namespace Common;
using namespace PDF;

TRN_API TRN_FontDestroy(TRN_Font font)
{
	BEX;
	delete (Font*) font;
	EEX;
}


TRN_API TRN_FontCreateFromObj(TRN_Obj font_dict,TRN_Font* result)
{
	BEX;
	*result=(TRN_Font)new Font((Obj*)font_dict);
    EEX;
}


TRN_API TRN_FontCreate(TRN_SDFDoc doc , enum TRN_FontStandardType1Font type, TRN_Font* result)
{
	BEX;
	*result=(TRN_Font)new Font(Font::Create(DocCast(doc),(Font::StandardType1Font)type, false ));
	EEX;
}

TRN_API TRN_FontCreateAndEmbed(TRN_SDFDoc doc , enum TRN_FontStandardType1Font type, TRN_Font* result)
{
	BEX;
	*result=(TRN_Font)new Font(Font::Create(DocCast(doc),(Font::StandardType1Font)type, true ));
	EEX;
}


TRN_API TRN_FontCreateTrueTypeFont(TRN_SDFDoc doc , TRN_UString font_path,
								   TRN_Bool embed, TRN_Bool subset, TRN_Font* result)
{
	BEX;
	UString temp(UStrCast(font_path));
	*result=(TRN_Font)new Font(Font::CreateTrueTypeFont(DocCast(doc),temp,TBToB(embed),TBToB(subset)));
    EEX;
}

#ifdef _WIN32
TRN_API TRN_FontCreateTrueTypeFont2(TRN_SDFDoc doc , const void* type,
									TRN_Bool embed, TRN_Bool subset,TRN_Font* result)
{
	BEX;
	*result=(TRN_Font)new Font(Font::CreateTrueTypeFont2(DocCast(doc),type,TBToB(embed),TBToB(subset)));
    EEX;
}
#endif


TRN_API TRN_FontCreateCIDTrueTypeFont(TRN_SDFDoc doc , TRN_UString type,
									  TRN_Bool embed, TRN_Bool subset,enum TRN_FontEncoding encoding,
									  TRN_UInt32 ttc_font_index, TRN_Font* result)
{
	BEX;
	UString temp(UStrCast(type));
	*result=(TRN_Font)new Font(Font::CreateCIDTrueTypeFont(DocCast(doc),temp,TBToB(embed),TBToB(subset),(Font::Encoding)encoding,ttc_font_index));
    EEX;
}

#ifdef _WIN32
TRN_API TRN_FontCreateCIDTrueTypeFont2(TRN_SDFDoc doc , const void* logfont,
									   TRN_Bool embed, TRN_Bool subset, enum TRN_FontEncoding encoding, TRN_Font* result)
{
	BEX;
	*result=(TRN_Font)new Font(Font::CreateCIDTrueTypeFont2(DocCast(doc),logfont,TBToB(embed),TBToB(subset),(Font::Encoding)encoding));
    EEX;
}

TRN_API TRN_FontCreateCIDTrueTypeFont2Unicode(TRN_SDFDoc doc, const void* logfont,
	TRN_Bool embed, TRN_Bool subset, enum TRN_FontEncoding encoding, TRN_Font* result)
{
	BEX;
	*result = (TRN_Font)new Font(Font::CreateCIDTrueTypeFont2Unicode(DocCast(doc), logfont, TBToB(embed), TBToB(subset), (Font::Encoding)encoding));
	EEX;
}
#endif

TRN_API TRN_FontCreateFromFontDescriptor(TRN_SDFDoc doc , TRN_Font from, const TRN_UString char_set, TRN_Font* result)
{
	BEX;
	UString temp(UStrCast(char_set));
	*result=(TRN_Font)new Font(Font::Create(DocCast(doc), FontCast(from), &temp));
	EEX;
}

TRN_API TRN_FontCreateFromName(TRN_SDFDoc doc, const char* name, const TRN_UString char_set, TRN_Font* result)
{
	BEX;
	UString temp(UStrCast(char_set));
	*result=(TRN_Font)new Font(Font::Create(DocCast(doc), name, &temp));
	EEX;
}

TRN_API TRN_FontCreateType1Font(TRN_SDFDoc doc , TRN_UString type,
								TRN_Bool embed, TRN_Font* result)
{
	BEX;
	UString temp(UStrCast(type));
	*result=(TRN_Font)new Font(Font::CreateType1Font(DocCast(doc),temp,TBToB(embed)));
    EEX;
}

TRN_API TRN_FontAssign(TRN_Font left, TRN_Font right)
{
	BEX;
	FontCast(left)=FontCast(right);
	EEX;
}

TRN_API TRN_FontGetType(TRN_Font font,enum TRN_FontType* result)
{
	BEX;
	*result=(enum TRN_FontType)FontCast(font).GetType();
    EEX;
}

TRN_API TRN_FontIsSimple(TRN_Font font, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsSimple());
    EEX;
}

TRN_API TRN_FontGetTypeFromObj(TRN_Obj font_dict, enum TRN_FontType* result)
{
	BEX;
	*result=(enum TRN_FontType)Font::GetType((Obj*)font_dict);
    EEX;
}

TRN_API TRN_FontGetSDFObj(TRN_Font font, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)FontCast(font).GetSDFObj();
    EEX;
}

TRN_API TRN_FontGetDescriptor(TRN_Font font, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)FontCast(font).GetDescriptor();
    EEX;
}

TRN_API TRN_FontGetName(TRN_Font font, const char**  result)
{
	BEX;
	*result=FontCast(font).GetName();
    EEX;
}

TRN_API TRN_FontGetFamilyName(TRN_Font font,const char** result)
{
	BEX;
	*result=FontCast(font).GetFamilyName();
    EEX;
}

TRN_API TRN_FontIsFixedWidth(TRN_Font font, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsFixedWidth());
    EEX;
}

TRN_API TRN_FontIsSerif(TRN_Font font,TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsSerif());
    EEX;
}

TRN_API TRN_FontIsSymbolic(TRN_Font font,TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsSymbolic());
    EEX;
}

TRN_API TRN_FontIsItalic(TRN_Font font, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsItalic());
    EEX;
}

TRN_API TRN_FontIsAllCap(TRN_Font font, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsAllCap());
    EEX;
}

TRN_API TRN_FontIsForceBold(TRN_Font font, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsForceBold());
    EEX;
}

TRN_API TRN_FontIsHorizontalMode(TRN_Font font, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsHorizontalMode());
    EEX;
}

TRN_API TRN_FontGetWidth(TRN_Font font, TRN_UInt32 char_code, double* result)
{
	BEX;
	*result = FontCast(font).GetWidth(char_code);
    EEX;
}

TRN_API TRN_FontGetMaxWidth(TRN_Font font, double* result)
{
	BEX;
	*result=FontCast(font).GetMaxWidth();
    EEX;
}

TRN_API TRN_FontGetMissingWidth(TRN_Font font, double* result)
{
	BEX;
	*result=FontCast(font).GetMissingWidth();
    EEX;
}

TRN_API TRN_FontGetCharCodeIterator(TRN_Font font,TRN_Iterator* result)
{
	BEX;
	FontCharCodeIterator itr=FontCast(font).CharCodeBegin(), end=FontCast(font).CharCodeEnd();
	*result=(TRN_Iterator)new PolyFontCharCodeIterator(itr,end);
    EEX;
}

TRN_API TRN_FontGetGlyphPath(TRN_Font font, TRN_UInt32 char_code,
							 TRN_UChar* buf_oprs, int* out_buf_oprs_sz, 
							 double* buf_data, int* out_buf_data_sz,
							 int* out_glyph_index,
							 TRN_Bool conics2cubics, 
							 TRN_Matrix2D* transform, TRN_Bool* result)
{
	BEX;
	vector<UChar>  get_oprs;
	vector<double>  get_data;
	*result=BToTB(FontCast(font).GetGlyphPath(char_code,get_oprs,get_data,
		TBToB(conics2cubics), (Matrix2D*)transform, false, out_glyph_index));
	if (buf_data && buf_oprs)
	{
		memcpy(buf_oprs, &(get_oprs[0]), get_oprs.size()*sizeof(UChar));
		memcpy(buf_data, &(get_data[0]), get_data.size()*sizeof(double));
	}
	*out_buf_oprs_sz = int(get_oprs.size());
	*out_buf_data_sz = int(get_data.size());
    EEX;
}


TRN_API TRN_FontMapToUnicode(TRN_Font font, TRN_UInt32 char_code, 
							 TRN_Unicode* out_uni_arr, const int in_uni_sz, 
							 int* out_chars, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).MapToUnicode(char_code,(Unicode*)out_uni_arr,in_uni_sz, *out_chars));
    EEX;
}

TRN_API TRN_FontGetEncoding(TRN_Font font, const char*** result)
{
	BEX;
	*result=FontCast(font).GetEncoding();
    EEX;
}

TRN_API TRN_FontIsEmbedded(TRN_Font font, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsEmbedded());
    EEX;
}

TRN_API TRN_FontGetEmbeddedFontName(TRN_Font font, const char** result)
{
	BEX;
	*result=FontCast(font).GetEmbeddedFontName();
    EEX;
}

TRN_API TRN_FontGetEmbeddedFont(TRN_Font font, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)FontCast(font).GetEmbeddedFont();
    EEX;
}

TRN_API TRN_FontGetEmbeddedFontBufSize(TRN_Font font,int* result)
{
	BEX;
	*result=FontCast(font).GetEmbeddedFontBufSize();
    EEX;
}

TRN_API TRN_FontGetUnitsPerEm(TRN_Font font, TRN_UInt16* result)
{
	BEX;
	*result=FontCast(font).GetUnitsPerEm();
    EEX;
}

TRN_API TRN_FontGetBBox(TRN_Font font, TRN_Rect* result)
{
	BEX;
	Rect r(FontCast(font).GetBBox());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
    EEX;
}

TRN_API TRN_FontGetAscent(TRN_Font font, double* result)
{
	BEX;
	*result=FontCast(font).GetAscent();
    EEX;
}

TRN_API TRN_FontGetDescent(TRN_Font font, double* result)
{
	BEX;
	*result=FontCast(font).GetDescent();
    EEX;
}

TRN_API TRN_FontGetStandardType1FontType(TRN_Font font, int* result)
{
	BEX;
	*result=FontCast(font).GetStandardType1FontType();
    EEX;
}

TRN_API TRN_FontIsCFF(TRN_Font font, TRN_Bool* result)
{
	BEX;
	*result=BToTB(FontCast(font).IsCFF());
    EEX;
}

TRN_API TRN_FontGetType3FontMatrix(TRN_Font font, TRN_Matrix2D* result)
{
	BEX;
    Common::Matrix2D mtx = FontCast(font).GetType3FontMatrix();
	*result=RetM2D(mtx);
    EEX;
}

TRN_API TRN_FontGetType3GlyphStream(TRN_Font font, TRN_UInt32 char_code, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)FontCast(font).GetType3GlyphStream(char_code);
    EEX;
}

TRN_API TRN_FontGetVerticalAdvance(TRN_Font font,TRN_UInt32 char_code, double* out_pos_vect_x, double* out_pos_vect_y, double* result)
{
	BEX;
	*result=FontCast(font).GetVerticalAdvance(char_code,*out_pos_vect_x,*out_pos_vect_y);
    EEX;
}

TRN_API TRN_FontGetDescendant(TRN_Font font, TRN_Font* result)
{
	BEX;
	*result=(TRN_Font)new Font(FontCast(font).GetDescendant());
    EEX;
}

TRN_API TRN_FontMapToCID(TRN_Font font, TRN_UInt32 char_code, TRN_UInt32* result)
{
	BEX;
	*result=FontCast(font).MapToCID(char_code);
    EEX;
}

TRN_API TRN_FontMapToCID2(TRN_Font font, const TRN_UChar* char_data, int char_data_avail, TRN_UInt32* out_charcode, TRN_UInt32* out_cid, int* result)
{
	BEX;
	*result=FontCast(font).MapToCID((const UChar*)char_data,char_data_avail,*out_charcode, *out_cid);
    EEX;
}

