#include <C/PDF/TRN_Optimizer.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/Optimizer.hpp>
#include <SDF/License.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

//IMAGE SETTINGS
TRN_API TRN_OptimizerImageSettingsInit(TRN_OptimizerImageSettings* settings)
{
	BEX;
	trn::PDF::Optimizer::ImageSettings image_settings;
	memcpy(settings, &image_settings, sizeof(image_settings));
	EEX;
}

//MONO IMAGE SETTINGS
TRN_API TRN_OptimizerMonoImageSettingsInit(TRN_OptimizerMonoImageSettings* settings)
{
	BEX;
	trn::PDF::Optimizer::MonoImageSettings mono_image_settings;
	memcpy(settings, &mono_image_settings, sizeof(mono_image_settings));
	EEX;
}

TRN_API TRN_OptimizerTextSettingsInit(TRN_OptimizerTextSettings* settings)
{
	BEX;
	trn::PDF::Optimizer::TextSettings text_settings;
	memcpy(settings,&text_settings,sizeof(text_settings));
	EEX;
}

//OPTIMIZER
TRN_API TRN_OptimizerOptimize(TRN_PDFDoc doc, const TRN_OptimizerImageSettings* color_image_settings, 
							  const TRN_OptimizerImageSettings* grayscale_image_settings, 
							  const TRN_OptimizerMonoImageSettings* mono_image_settings, 
							  const TRN_OptimizerTextSettings* text_settings,
							  TRN_Bool remove_custom)
{
	BEX;
	Optimizer optimizer;
	PDFDoc* d=(PDFDoc*)&doc;
	
	//License::Instance().VerifyPermission(License::e_perm_optimizer);
	
	Optimizer::ImageSettings* c = (Optimizer::ImageSettings*)color_image_settings;
	Optimizer::ImageSettings* g = (Optimizer::ImageSettings*)grayscale_image_settings;
	Optimizer::MonoImageSettings* m = (Optimizer::MonoImageSettings*)mono_image_settings;
	Optimizer::TextSettings* t = (Optimizer::TextSettings*)text_settings;
	optimizer.RemoveCustomEntries(TBToB(remove_custom));
	optimizer.Optimize(*d, *c, *g, *m, *t, false);
	EEX;
}
