#include <C/PDF/TRN_PDFDoc.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>
#include <C/PDF/TRN_ColorSpace.h>
#include <PDF/PDFDoc.hpp>
#include <SDF/License.hpp>
#include <SDF/SignatureHandler.h>
#include <PDF/Download/LinearDownloader.hpp>
#include <SDF/SDFDocImpl.hpp>
#include <Silverlight/Converter/SilverConverter.h>
#include <SDF/UndoManager.hpp>
#include <SDF/SDFOptionsBase.hpp>
#include <Common/Options/JSONOptionsBase.hpp>
#include <PDF/DiffOptions.hpp>
#include <C/Common/TRN_PolymorphicVector.hpp>
#include <PDF/ViewerOptimizedOptions.h>

#ifdef JDM
// Implementation files located at PDFTron/Trunk/DeveloperSandbox/Ivan/DOM
#include <PDF/DOM/DomElementReader.hpp>
#include <PDF/DOM/DomElementWriter.hpp>
#include <PDF/DOM/DomDisplayList.hpp>
#endif // JDM

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;
using namespace std;

TRN_API TRN_PDFDocCreate (TRN_PDFDoc* result)
{
	BEX;
	*result=(TRN_PDFDoc)IMPL_TRN_PDFDocCreateEmpty();
	EEX;
}

TRN_API TRN_PDFDocCreateFromSDFDoc (TRN_SDFDoc sdfdoc, TRN_PDFDoc* result)
{
	BEX;
	((SDFDoc*)sdfdoc)->AddReference();
	*result=(TRN_PDFDoc)sdfdoc;	
	EEX;
}

TRN_API TRN_PDFDocCreateFromUFilePath (const TRN_UString filepath, TRN_PDFDoc* result)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_read);
	UString temp(UStrCast(filepath));
	*result=(TRN_PDFDoc)IMPL_TRN_PDFDocCreateFromUFileName(temp);
	EEX;
}

TRN_API TRN_PDFDocCreateFromFilePath (const char* filepath, TRN_PDFDoc* result)
{
	BEX;
	UString temp(UStrCast(filepath));
	License::Instance().VerifyPermission(License::e_perm_read);
	*result=(TRN_PDFDoc)IMPL_TRN_PDFDocCreateFromFileName(filepath);
	EEX;
}

TRN_API TRN_PDFDocCreateFromFilter (TRN_Filter stream, TRN_PDFDoc* result)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_read);
	AutoPtr<Filters::Filter> temp((Filters::Filter*)stream);
	*result=(TRN_PDFDoc)IMPL_TRN_PDFDocCreateFromFilter(temp);
	EEX;
}

TRN_API TRN_PDFDocCreateFromBuffer (const TRN_UChar* buf, TRN_Size buf_size, TRN_PDFDoc* result)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_read);
	*result=(TRN_PDFDoc)IMPL_TRN_PDFDocCreateFromBuffer((const char*)buf,buf_size);
	EEX;
}

TRN_API TRN_PDFDocCreateFromLayoutEls (const char* buf, TRN_Size buf_size, TRN_PDFDoc* result)
{
	BEX;
	*result=(TRN_PDFDoc)IMPL_TRN_PDFDocCreateFromLayoutEls(buf,buf_size);
	EEX;
}

TRN_API TRN_PDFDocCreateShallowCopy(TRN_PDFDoc source, TRN_PDFDoc* result)
{
	BEX;
	SDF::SDFDoc* core_sdfdoc = (SDF::SDFDoc*)source;
	if (core_sdfdoc)
	{
		core_sdfdoc->AddReference();
	}
	*result = source;
	EEX;
}

TRN_API TRN_PDFDocDestroy (TRN_PDFDoc doc)
{
	BEX;
	IMPL_TRN_PDFDocDestroy(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocIsEncrypted(TRN_PDFDoc doc, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_PDFDocIsEncrypted(PDocCast(doc)));
	EEX;
}

TRN_API TRN_PDFDocInitSecurityHandler (TRN_PDFDoc doc, void* custom_data, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_PDFDocInitSecurityHandler(PDocCast(doc),custom_data));
	EEX;
}

TRN_API TRN_PDFDocInitStdSecurityHandler (TRN_PDFDoc doc, const char* password, int password_sz, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_PDFDocInitStdSecurityHandler(PDocCast(doc),password,password_sz));
	EEX;
}

TRN_API TRN_PDFDocInitStdSecurityHandlerUString(TRN_PDFDoc doc, TRN_UString password, TRN_Bool* result)
{
    BEX;
    trn::UString tempUStr(UStrCast(password));
    *result = BToTB(IMPL_TRN_PDFDocInitStdSecurityHandlerUString(PDocCast(doc), tempUStr));
    EEX;
}

TRN_API TRN_PDFDocInitStdSecurityHandlerBuffer(TRN_PDFDoc doc, const TRN_UInt8* password_buf, size_t password_buf_size, TRN_Bool* result)
{
    BEX;
    std::vector<trn::UInt8> tempBuf(password_buf, password_buf + password_buf_size);
    *result = BToTB(IMPL_TRN_PDFDocInitStdSecurityHandlerBuffer(PDocCast(doc), tempBuf));
    EEX;
}

TRN_API TRN_PDFDocGetSecurityHandler (TRN_PDFDoc doc, TRN_SecurityHandler* result)
{
	BEX;
	*result=(TRN_SecurityHandler)IMPL_TRN_PDFDocGetSecurityHandler(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocSetSecurityHandler (TRN_PDFDoc doc, TRN_SecurityHandler handler)
{
	BEX;
	AutoPtr<SecurityHandler> temp((SecurityHandler*)handler);
	IMPL_TRN_PDFDocSetSecurityHandler(PDocCast(doc),temp);
	EEX;
}

TRN_API TRN_PDFDocRemoveSecurity(TRN_PDFDoc doc)
{
	BEX;
	IMPL_TRN_PDFDocRemoveSecurity(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocGetDocInfo(TRN_PDFDoc doc, TRN_PDFDocInfo* result )
{
	BEX;
	*result=(TRN_PDFDocInfo)IMPL_TRN_PDFDocGetDocInfo(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocGetViewPrefs(TRN_PDFDoc doc, TRN_PDFDocViewPrefs* result)
{
	BEX;
	*result=(TRN_PDFDocViewPrefs)IMPL_TRN_PDFDocGetViewPrefs(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocIsModified (const TRN_PDFDoc doc, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_PDFDocIsModified(PDocCast(doc)));
	EEX;
}

TRN_API TRN_PDFDocHasRepairedXRef(const TRN_PDFDoc doc, TRN_Bool* result)
{
	BEX;
	*result = BToTB(IMPL_TRN_PDFDocHasRepairedXRef(PDocCast(doc)));
	EEX;
}

TRN_API TRN_PDFDocIsLinearized(const TRN_PDFDoc doc, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_PDFDocIsLinearized(PDocCast(doc)));
	EEX;
}

TRN_API TRN_PDFDocSave(TRN_PDFDoc doc, const TRN_UString path, TRN_UInt32 flags)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_write);
#ifndef TRN_HAS_FILESYSTEM
	// we disable demo bombing when we don't have a filesystem
	// so we should apply demo stamps
	License::Instance().IfDemoApplyStamp(PDocCast(doc));
#endif
	//trn::Common::ProgressMonitor* pm = progress != NULL ? (trn::Common::ProgressMonitor*) *progress : NULL;
	UString temp(UStrCast(path));
	IMPL_TRN_PDFDocSave(PDocCast(doc), temp, flags, 0);
	//IMPL_TRN_PDFDocSave(PDocCast(doc), temp, flags, pm);
	EEX;
}

TRN_API TRN_PDFDocSaveMemoryBuffer(TRN_PDFDoc doc, TRN_UInt32 flags, const char** out_buf, TRN_Size* out_buf_size)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_write);
#ifndef TRN_HAS_FILESYSTEM
	// we disable demo bombing when we don't have a filesystem
	// so we should apply demo stamps
	License::Instance().IfDemoApplyStamp(PDocCast(doc));
#endif
	IMPL_TRN_PDFDocSaveMemoryBuffer(PDocCast(doc), *out_buf, *out_buf_size, flags, 0);
	EEX;
}

TRN_API TRN_PDFDocSaveStream(TRN_PDFDoc doc, TRN_Filter stream, TRN_UInt32 flags) 
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_write);
#ifndef TRN_HAS_FILESYSTEM
	// we disable demo bombing when we don't have a filesystem
	// so we should apply demo stamps
	License::Instance().IfDemoApplyStamp(PDocCast(doc));
#endif
	Filters::Filter* flt = (Filters::Filter*)stream;
	IMPL_TRN_PDFDocSaveStream(PDocCast(doc), *flt, flags, 0);
	EEX;
}

TRN_API TRN_PDFDocGetPageIterator(TRN_PDFDoc doc, TRN_UInt32 page_number, TRN_Iterator* result )
{
	BEX;
	PageIterator itr=IMPL_TRN_PDFDocPageFind(PDocCast(doc), page_number), end=IMPL_TRN_PDFDocPageEnd(PDocCast(doc));
	*result=(TRN_Iterator)new PolyPageIterator(itr,end);
	EEX;
}


TRN_API TRN_PDFDocGetPage(TRN_PDFDoc doc, TRN_UInt32 page_number, TRN_Page* result)
{
	BEX;
	PageIterator itr=IMPL_TRN_PDFDocPageFind(PDocCast(doc),page_number), end=IMPL_TRN_PDFDocPageEnd(PDocCast(doc));
	if(itr!=end)
		*result=(TRN_Page)(*itr).mp_leaf_node;
	else
		*result=0;
	EEX;
}

TRN_API TRN_PDFDocPageRemove(TRN_PDFDoc doc, const TRN_Iterator page_itr)
{
	BEX;
	PolyPageIterator* temp=dynamic_cast<PolyPageIterator*>((PolyIterator*)page_itr);
	BASE_ASSERT(temp!=0,"Incorrect Iterator Type.");
	IMPL_TRN_PDFDocPageRemove(temp->m_itr);
	EEX;
}

TRN_API TRN_PDFDocPageInsert(TRN_PDFDoc doc, TRN_Iterator where, TRN_Page page)
{
	BEX;
	PolyPageIterator* temp=dynamic_cast<PolyPageIterator*>((PolyIterator*)where);
	BASE_ASSERT(temp!=0,"Incorrect Iterator Type.");
	IMPL_TRN_PDFDocPageInsert(PDocCast(doc),temp->m_itr,PageCast(page));
	EEX;
}

TRN_API TRN_PDFDocInsertPages(TRN_PDFDoc dest_doc, TRN_UInt32 insert_before_page_number, TRN_PDFDoc src_doc, TRN_UInt32 start_page, TRN_UInt32 end_page, enum TRN_PDFDocInsertFlag flag)
{
	BEX;
	trn::PDF::PageSet source_pageset(start_page, end_page);
	IMPL_TRN_PDFDocInsertPages(PDocCast(dest_doc), insert_before_page_number, PDocCast(src_doc), source_pageset, (PDF::PDFDoc::InsertFlag) flag, (trn::Common::ProgressMonitor*) 0);
	EEX;
}

TRN_API TRN_PDFDocInsertPageSet(TRN_PDFDoc dest_doc, TRN_UInt32 insert_before_page_number, TRN_PDFDoc src_doc, TRN_PageSet source_page_set, enum TRN_PDFDocInsertFlag flag)
{
	BEX;
	IMPL_TRN_PDFDocInsertPages(PDocCast(dest_doc), insert_before_page_number, PDocCast(src_doc), *((PDF::PageSet*)source_page_set), (PDF::PDFDoc::InsertFlag) flag, (trn::Common::ProgressMonitor*) 0);
	EEX;
}

TRN_API TRN_PDFDocMovePages(TRN_PDFDoc dest_doc, TRN_UInt32 move_before_page_number, TRN_PDFDoc src_doc, TRN_UInt32 start_page, TRN_UInt32 end_page, enum TRN_PDFDocInsertFlag flag)
{
	BEX;
	trn::PDF::PageSet source_pageset(start_page, end_page);
	IMPL_TRN_PDFDocMovePages(PDocCast(dest_doc), move_before_page_number, PDocCast(src_doc), source_pageset, (trn::PDF::PDFDoc::InsertFlag)flag, (trn::Common::ProgressMonitor*)0);
	EEX;
}

TRN_API TRN_PDFDocMovePageSet(TRN_PDFDoc dest_doc, TRN_UInt32 move_before_page_number, TRN_PDFDoc src_doc, TRN_PageSet source_page_set, enum TRN_PDFDocInsertFlag flag)
{
	BEX;
	IMPL_TRN_PDFDocMovePages(PDocCast(dest_doc), move_before_page_number, PDocCast(src_doc), *((trn::PDF::PageSet*)source_page_set), (trn::PDF::PDFDoc::InsertFlag)flag, (trn::Common::ProgressMonitor*)0);
	EEX;
}

TRN_API TRN_PDFDocPagePushFront(TRN_PDFDoc doc, TRN_Page page)
{
	BEX;
	IMPL_TRN_PDFDocPagePushFront(PDocCast(doc),PageCast(page));
	EEX;
}

TRN_API TRN_PDFDocPagePushBack(TRN_PDFDoc doc, TRN_Page page)
{
	BEX;
	IMPL_TRN_PDFDocPagePushBack(PDocCast(doc),PageCast(page));
	EEX;
}

TRN_API TRN_PDFDocImportPages(TRN_PDFDoc doc, const TRN_Page* page_buf, int buf_size, TRN_Bool import_bookmarks, TRN_Page* buf_result)
{
	BEX;
	list<Page> in_list;
	int i;
	for(i=0; i<buf_size; i++)
	{
		Page temp(PageCast(page_buf[i]));
		in_list.push_back(temp);
	}
	list<Page> res=IMPL_TRN_PDFDocImportPages(PDocCast(doc),in_list,TBToB(import_bookmarks));
	list<Page>::iterator itr, end=res.end();
	for(i=0, itr=res.begin(); itr!=end; ++itr, ++i)
		buf_result[i] = (TRN_Page)((*itr).mp_leaf_node);
	EEX;
}

TRN_API TRN_PDFDocPageCreate(TRN_PDFDoc doc, const TRN_Rect* media_box, TRN_Page* result)
{
	BEX;
	*result=(TRN_Page)IMPL_TRN_PDFDocPageCreate(PDocCast(doc),*((const Rect*)media_box));
	EEX;
}

TRN_API TRN_PDFDocGetFirstBookmark(TRN_PDFDoc doc, TRN_Bookmark* result )
{
	BEX;
	*result=(TRN_Bookmark)IMPL_TRN_PDFDocGetFirstBookmark(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocAddRootBookmark(TRN_PDFDoc doc, TRN_Bookmark root_bookmark)
{
	BEX;
	IMPL_TRN_PDFDocAddRootBookmark(PDocCast(doc),BMCast(root_bookmark));
	EEX;
}

TRN_API TRN_PDFDocGetTrailer (TRN_PDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocGetTrailer(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocGetRoot(TRN_PDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocGetRoot(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocJSContextInitialize(TRN_PDFDoc doc)
{
	BEX;
	IMPL_TRN_PDFDocJSContextInitialize(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocGetPages(TRN_PDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocGetPages(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocGetPageCount (TRN_PDFDoc doc, int* result)
{
	BEX;
	*result=IMPL_TRN_PDFDocGetPageCount(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFGetDownloadedByteCount (TRN_PDFDoc doc, int* result)
{
    
    
    BEX;
    *result = PDocCast(doc)->GetDownloadedByteCount();
    EEX;
}

TRN_API TRN_PDFGetTotalRemoteByteCount (TRN_PDFDoc doc, int* result)
{
    BEX;
    *result = PDocCast(doc)->GetTotalRemoteByteCount();
    EEX;
}

TRN_API TRN_PDFDocGetFieldIteratorBegin(TRN_PDFDoc doc, TRN_Iterator* result)
{
	BEX;
	FieldIterator itr=IMPL_TRN_PDFDocFieldBegin(PDocCast(doc)), end=IMPL_TRN_PDFDocFieldEnd(PDocCast(doc));
	*result=(TRN_Iterator)new PolyFieldIterator(itr,end);
	EEX;
}

TRN_API TRN_PDFDocGetFieldIterator(TRN_PDFDoc doc, const TRN_UString field_name, TRN_Iterator* result)
{
	BEX;
	UString temp(UStrCast(field_name));
	FieldIterator itr=IMPL_TRN_PDFDocFieldFind(PDocCast(doc), temp), end=IMPL_TRN_PDFDocFieldEnd(PDocCast(doc));
	*result=(TRN_Iterator)new PolyFieldIterator(itr,end);
	EEX;
}

TRN_API TRN_PDFDocGetField(TRN_PDFDoc doc, const TRN_UString field_name, TRN_Field* result)
{
	BEX;
	UString temp(UStrCast(field_name));
	FieldIterator itr=IMPL_TRN_PDFDocFieldFind(PDocCast(doc),temp), end=IMPL_TRN_PDFDocFieldEnd(PDocCast(doc));
	if(itr!=end)
		*result=RetField(*itr);
	else
	{
		result->leaf_node=0;
		result->builder=0;
	}
	EEX;
}
TRN_API TRN_PDFDocFieldCreateFromStrings(TRN_PDFDoc doc, const TRN_UString field_name, 
							  enum TRN_FieldType type, TRN_UString field_value , TRN_UString def_field_value, TRN_Field* result)
{
	BEX;
	UString t1(UStrCast(field_name)), t2(UStrCast(field_value)), t3(UStrCast(def_field_value));
    trn::PDF::Field field = IMPL_TRN_PDFDocFieldCreate(PDocCast(doc), t1, (Field::Type)type, t2, t3);
	*result=RetField(field);
    EEX;
}

TRN_API TRN_PDFDocFieldCreate(TRN_PDFDoc doc, const TRN_UString field_name, 
							  enum TRN_FieldType type, TRN_Obj field_value , TRN_Obj def_field_value, TRN_Field* result)
{
	BEX;
	UString temp(UStrCast(field_name));
    trn::PDF::Field field = IMPL_TRN_PDFDocFieldCreate(PDocCast(doc),temp,(Field::Type)type,(Obj*)field_value,(Obj*)def_field_value);
	*result=RetField(field);
	EEX;
}

TRN_API TRN_PDFDocRefreshFieldAppearances(TRN_PDFDoc doc)
{
	BEX;
	IMPL_TRN_PDFDocRefreshFieldAppearances(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocFlattenAnnotations(TRN_PDFDoc doc, TRN_Bool forms_only)
{
	BEX;
	IMPL_TRN_PDFDocFlattenAnnotations(PDocCast(doc), TBToB(forms_only));
	EEX;
}

TRN_API TRN_PDFDocFlattenAnnotationsAdvanced(TRN_PDFDoc doc, TRN_UInt32 flags)
{
	BEX;
	IMPL_TRN_PDFDocFlattenAnnotationsAdvanced(PDocCast(doc), flags);
	EEX;
}

TRN_API TRN_PDFDocGetAcroForm(TRN_PDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocGetAcroForm(PDocCast(doc));
	EEX;
}
TRN_API TRN_PDFDocFDFExtract(TRN_PDFDoc doc, enum TRN_PDFDocExtractFlag flag, TRN_FDFDoc* result)
{
	BEX;
	*result = (TRN_FDFDoc)IMPL_TRN_PDFDocFDFExtract(PDocCast(doc), (PDF::PDFDoc::ExtractFlag)flag).release();
	EEX;
}

TRN_API TRN_PDFDocFDFExtractPageSet(TRN_PDFDoc doc, TRN_PageSet pages_to_extract, enum TRN_PDFDocExtractFlag flag, TRN_FDFDoc* result)
{
	BEX;
	*result = (TRN_FDFDoc)IMPL_TRN_PDFDocFDFExtractPageSet(PDocCast(doc), (PDF::PDFDoc::ExtractFlag)flag, *(PDF::PageSet*)pages_to_extract).release();
	EEX;
}

TRN_API TRN_PDFDocFDFExtractAnnots(TRN_PDFDoc doc, const TRN_Annot* annot_buf, int buf_size, TRN_FDFDoc* result)
{
	BEX;
	std::vector<SDF::Obj*> annots;
	for (int i = 0; i < buf_size; ++i)
	{
		annots.push_back(ObjCast(annot_buf[i]));
	}
	*result = (TRN_FDFDoc)IMPL_TRN_PDFDocFDFExtractAnnots(PDocCast(doc), annots).release();
	EEX;
}

TRN_API TRN_PDFDocFDFExtractCommand(TRN_PDFDoc doc, const TRN_Annot* annot_added_buf, int annot_added_buf_size, const TRN_Annot* annot_modified_buf, int annot_modified_buf_size, const TRN_Annot* annot_deleted_buf, int annot_deleted_buf_size, TRN_FDFDoc* result)
{
	BEX;
	std::vector<SDF::Obj*> annots_added, annots_modified, annots_deleted;
	for (int i = 0; i < annot_added_buf_size; ++i)
	{
		annots_added.push_back(ObjCast(annot_added_buf[i]));
	}
	for (int i = 0; i < annot_modified_buf_size; ++i)
	{
		annots_modified.push_back(ObjCast(annot_modified_buf[i]));
	}
	for (int i = 0; i < annot_deleted_buf_size; ++i)
	{
		annots_deleted.push_back(ObjCast(annot_deleted_buf[i]));
	}
	*result = (TRN_FDFDoc)IMPL_TRN_PDFDocFDFExtractAnnotsCommand(PDocCast(doc), annots_added, annots_modified, annots_deleted).release();
	EEX;
}

TRN_API TRN_PDFDocFDFMerge(TRN_PDFDoc doc, TRN_FDFDoc fdf_doc)
{
	BEX;
	IMPL_TRN_PDFDocFDFMerge(PDocCast(doc),FDocCast(fdf_doc));
	EEX;
}

TRN_API TRN_PDFDocFDFUpdate(TRN_PDFDoc doc, TRN_FDFDoc fdf_doc)
{
	BEX;
	IMPL_TRN_PDFDocFDFUpdate(PDocCast(doc), FDocCast(fdf_doc));
	EEX;
}

TRN_API TRN_PDFDocFDFUpdateAppearanceDocs(TRN_PDFDoc doc, TRN_FDFDoc fdf_doc, TRN_AppearanceDocument* doc_list, int doc_list_size)
{
	BEX;
	std::map<UString, SDF::SDFDoc*> appearance_doc_map;
	std::map<UString, SDF::SDFDoc*>* appearance_map_input = 0;
	if (doc_list && doc_list_size)
	{
		appearance_map_input = &appearance_doc_map;
		for (int i = 0; i < doc_list_size; ++i)
		{
			TRN_AppearanceDocument current = doc_list[i];
			appearance_doc_map[UString(UStrCast(current.id))] = PDocCast(current.doc);
		}
	}
	IMPL_TRN_PDFDocFDFUpdate(PDocCast(doc), FDocCast(fdf_doc), appearance_map_input);
	EEX;
}

TRN_API TRN_PDFDocGetOpenAction(TRN_PDFDoc doc, TRN_Action* result)
{
	BEX;
	*result=(TRN_Action)IMPL_TRN_PDFDocGetOpenAction(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocSetOpenAction(TRN_PDFDoc doc, const TRN_Action action)
{
	BEX;
	IMPL_TRN_PDFDocSetOpenAction(PDocCast(doc),ActCast(action));
	EEX;
}

TRN_API TRN_PDFDocAddFileAttachment(TRN_PDFDoc doc, const TRN_UString file_key, TRN_FileSpec embedded_file)
{
	BEX;
	UString temp(UStrCast(file_key));
	IMPL_TRN_PDFDocAddFileAttachment(PDocCast(doc), temp, FSCast(embedded_file));
	EEX;
}
TRN_API TRN_PDFDocGetPageLabel(TRN_PDFDoc doc, int page_num, TRN_PageLabel* result)
{
	BEX;
    trn::PDF::PageLabel label = IMPL_TRN_PDFDocGetPageLabel(PDocCast(doc),page_num);
	*result=RetPLbl(label);
	EEX;
}

TRN_API TRN_PDFDocSetPageLabel(TRN_PDFDoc doc, int page_num, const TRN_PageLabel* label)
{
	BEX;
	IMPL_TRN_PDFDocSetPageLabel(PDocCast(doc),page_num,PLblCast(label));
	EEX;
}

TRN_API TRN_PDFDocRemovePageLabel(TRN_PDFDoc doc, int page_num)
{
	BEX;
	IMPL_TRN_PDFDocRemovePageLabel(PDocCast(doc),page_num);
	EEX;
}

TRN_API TRN_PDFDocGetStructTree(TRN_PDFDoc doc, TRN_STree* result)
{
	BEX;
	*result=(TRN_STree)IMPL_TRN_PDFDocGetStructTree(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocHasOC(TRN_PDFDoc doc, TRN_Bool* result) {
	BEX;
	*result=BToTB(IMPL_TRN_PDFDocHasOC(PDocCast(doc)));
	EEX;
}

TRN_API TRN_PDFDocGetOCGs(TRN_PDFDoc doc, TRN_Obj* result) {
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocGetOCGs(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocGetOCGConfig(TRN_PDFDoc doc, TRN_OCGConfig* result) {
	BEX;
	*result=(TRN_OCGConfig)IMPL_TRN_PDFDocGetOCGConfig(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectName(TRN_PDFDoc doc, const char* name, TRN_Obj* result) {
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocCreateIndirectName(PDocCast(doc),name);
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectArray(TRN_PDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocCreateIndirectArray(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectBool(TRN_PDFDoc doc, TRN_Bool value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocCreateIndirectBool(PDocCast(doc),TBToB(value));
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectDict(TRN_PDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocCreateIndirectDict(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectNull(TRN_PDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocCreateIndirectNull(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectNumber(TRN_PDFDoc doc, double value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PDFDocCreateIndirectNumber(PDocCast(doc),value);
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectString(TRN_PDFDoc doc, const TRN_UChar* buf_value, TRN_UInt32 buf_size, TRN_Obj* result)
{
	BEX;
	*result = (TRN_Obj)IMPL_TRN_PDFDocCreateIndirectString(PDocCast(doc), buf_value, buf_size);
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectStringFromUString(TRN_PDFDoc doc, const TRN_UString str, TRN_Obj* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=(TRN_Obj)IMPL_TRN_PDFDocCreateIndirectString(PDocCast(doc),temp);
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectStreamFromFilter(TRN_PDFDoc doc, TRN_FilterReader data, TRN_Filter filter_chain, TRN_Obj* result)
{
	BEX;
	AutoPtr<Filters::Filter> temp((Filters::Filter*)filter_chain);
	*result=(TRN_Obj)IMPL_TRN_PDFDocCreateIndirectStream(PDocCast(doc),*((Filters::FilterReader*)data),temp);
	EEX;
}

TRN_API TRN_PDFDocCreateIndirectStream(TRN_PDFDoc doc, const char* data, const TRN_Size data_size, TRN_Filter filter_chain, TRN_Obj* result)
{
	BEX;
	AutoPtr<Filters::Filter> temp((Filters::Filter*)filter_chain);
	*result=(TRN_Obj)IMPL_TRN_PDFDocCreateIndirectStream(PDocCast(doc),data,data_size,temp);
	EEX;
}

TRN_API TRN_PDFDocGetSDFDoc (TRN_PDFDoc doc, TRN_SDFDoc* result)
{
	BEX;
	*result=(TRN_SDFDoc)doc;
	EEX;
}

TRN_API TRN_PDFDocLock(TRN_PDFDoc doc)
{
	BEX;
	IMPL_TRN_PDFDocLock(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocUnlock(TRN_PDFDoc doc)
{
	BEX;
	IMPL_TRN_PDFDocUnlock(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocLockRead(TRN_PDFDoc doc)
{
	BEX;
	IMPL_TRN_PDFDocLockRead(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFDocUnlockRead(TRN_PDFDoc doc)
{
	BEX;
	IMPL_TRN_PDFDocUnlockRead(PDocCast(doc));
	EEX;
}



TRN_API TRN_PDFDocTryLock( TRN_PDFDoc doc, TRN_Bool* result )
{
	BEX;
	*result=BToTB(PDocCast(doc)->TryLock(0));
	EEX;
}

TRN_API TRN_PDFDocTimedLock( TRN_PDFDoc doc, int milliseconds, TRN_Bool* result )
{
	BEX;
	*result=BToTB(PDocCast(doc)->TryLock(milliseconds));
	EEX;
}

TRN_API TRN_PDFDocTryLockRead( TRN_PDFDoc doc, TRN_Bool* result )
{
	BEX;
	*result=BToTB(PDocCast(doc)->TryLockRead(0));
	EEX;
}

TRN_API TRN_PDFDocTimedLockRead( TRN_PDFDoc doc, int milliseconds, TRN_Bool* result )
{
	BEX;
	*result=BToTB(PDocCast(doc)->TryLockRead(milliseconds));
	EEX;
}


TRN_API TRN_PDFDocAddHighlights(TRN_PDFDoc doc, const TRN_UString hilite)
{
	BEX;
	UString temp(UStrCast(hilite));
	PDFDoc& pdfdoc = *((PDFDoc*)(&doc));
	pdfdoc.AddHighlights(temp);
	EEX;
}

TRN_API TRN_PDFDocIsTagged(TRN_PDFDoc doc, TRN_Bool* result)
{
	BEX;
	PDFDoc& pdfdoc = *((PDFDoc*)(&doc));
	*result=BToTB(pdfdoc.IsTagged());	
	EEX;
}

TRN_API TRN_PDFDocHasSignatures(TRN_PDFDoc doc, TRN_Bool* result)
{
	BEX;
	PDFDoc& pdfdoc = *((PDFDoc*) (&doc));
    bool r = pdfdoc.HasSignatures();
    if (result != NULL)
        *result = BToTB(r);
    EEX;
}

TRN_API TRN_PDFDocHasDownloader(TRN_PDFDoc doc, TRN_Bool* result)
{
    BEX;
    *result = PDocCast(doc)->HasDownloader();
    EEX;
}

TRN_API TRN_PDFDocAddSignatureHandler(TRN_PDFDoc doc, TRN_SignatureHandler signature_handler, TRN_SignatureHandlerId* result)
{
	BEX;
	PDFDoc& pdfdoc = *((PDFDoc*) (&doc));
    SignatureHandler* sigHandler = (SignatureHandler*) signature_handler;
    SignatureHandlerId r = pdfdoc.AddSignatureHandler(AutoPtr<SignatureHandler>(sigHandler));
    if (result != NULL)
        *result = r;
    EEX;
}

TRN_API TRN_PDFDocAddStdSignatureHandlerFromFile(TRN_PDFDoc doc, const TRN_UString pkcs12_file, const TRN_UString pkcs12_pass, TRN_SignatureHandlerId* result)
{
	BEX;
	PDFDoc& pdfdoc = *((PDFDoc*) (&doc));
	UString p12file(UStrCast(pkcs12_file));
	UString p12pass(UStrCast(pkcs12_pass));
    SignatureHandlerId r = pdfdoc.AddStdSignatureHandler(p12file, p12pass);
    if (result != NULL)
        *result = r;
    EEX;
}

TRN_API TRN_PDFDocAddStdSignatureHandlerFromBuffer(TRN_PDFDoc doc, const TRN_UInt8* pkcs12_buffer, const TRN_Size pkcs12_buffsize, const TRN_UString pkcs12_pass, TRN_SignatureHandlerId* result)
{
    BEX;
	PDFDoc& pdfdoc = *((PDFDoc*) (&doc));
	UString p12pass(UStrCast(pkcs12_pass));
    SignatureHandlerId r = pdfdoc.AddStdSignatureHandler(pkcs12_buffer, pkcs12_buffsize, p12pass);
    if (result != NULL)
        *result = r;
    EEX;
}

TRN_API TRN_PDFDocRemoveSignatureHandler(TRN_PDFDoc doc, const TRN_SignatureHandlerId signature_handler_id)
{
	BEX;
	PDFDoc& pdfdoc = *((PDFDoc*) (&doc));
    pdfdoc.RemoveSignatureHandler((const SignatureHandlerId) signature_handler_id, true);
    EEX;
}

TRN_API TRN_PDFDocGetSignatureHandler(TRN_PDFDoc doc, const TRN_SignatureHandlerId signature_handler_id, TRN_SignatureHandler* result)
{
	BEX;
	PDFDoc& pdfdoc = *((PDFDoc*) (&doc));
    SignatureHandler* sigHandler = pdfdoc.GetSignatureHandler((const SignatureHandlerId) signature_handler_id);
    if (result != NULL)
        *result = (TRN_SignatureHandler) sigHandler;
    else
        *result = NULL;
    EEX;
}

TRN_API TRN_PDFDocGenerateThumbnails(TRN_PDFDoc doc, TRN_UInt32 size)
{
    BEX;
	PDFDoc& pdfdoc = *((PDFDoc*) (&doc));
    pdfdoc.GenerateThumbs((trn::UInt32) size);
    EEX;
}

TRN_API TRN_PDFDocAppendVisualDiffWithOptsObj(TRN_PDFDoc doc, TRN_Page p1, TRN_Page p2, TRN_Obj opts_dict)
{
	BEX;
	TRN_optionbase opt_in;
	opt_in.type = e_option_type_sdf;
	opt_in.impl = opts_dict;
	TRN_PDFDocAppendVisualDiff(doc, p1, p2, &opt_in);
	EEX;
}

TRN_API TRN_PDFDocAppendVisualDiff(TRN_PDFDoc doc, TRN_Page p1, TRN_Page p2, const TRN_OptionBase* opts)
{
	BEX;
	UniquePtr<OptionsBase> opts_base;
	if(opts)
	{
		if (opts->type == e_option_type_sdf)
		{
			opts_base.reset(new SDF::SDFOptionsBase(static_cast<SDF::Obj*>(opts->impl)));
		}
		else if (opts->type == e_option_type_json)
		{
			opts_base.reset(new JSONOptionsBase(StringRef(static_cast<const char*>(opts->impl))));
		}
	}
	PDFDoc& pdfdoc = *((PDFDoc*)(&doc));
	DiffOptions diff_options(opts_base.get());
	pdfdoc.AppendVisualDiff(PageCast(p1), PageCast(p2), diff_options);
	EEX;
}

TRN_API TRN_PDFDocSaveViewerOptimized(TRN_PDFDoc doc, const TRN_UString path, const TRN_Obj opts)
{
	BEX;
	PDFDoc& pdfdoc = *((PDFDoc*)(&doc));
	UString upath(path);
	ViewerOptimizedOptions options(ObjCast(opts));
	pdfdoc.SaveViewerOptimized(upath, options);
	EEX;
}

TRN_API TRN_DownloaderCreate(struct TRN_DownloadCallbacks* callbacks, size_t file_size, TRN_UString trn_cache_file, TRN_Downloader* result)
{
	BEX;
	UString cache_file(trn_cache_file);
	AutoPtr<Download::LinearDownloader> downloader(new Download::LinearDownloader(*(Download::DownloadCallbacks*)callbacks,file_size,cache_file,0));
	downloader->Init();
	*result = (TRN_Downloader)downloader.release();
	EEX;
}

TRN_API TRN_DownloaderStop(TRN_Downloader downloader)
{
	BEX;
	((Download::LinearDownloader*) downloader)->Delete();
	EEX;
}

TRN_API TRN_DownloaderIsLinearizationValid(TRN_Downloader downloader, TRN_Bool* result)
{
	BEX;
	*result = BToTB(((Download::LinearDownloader*) downloader)->IsLinearizationValid());
	EEX;
}

TRN_API TRN_PDFDocDownloaderInitialize(TRN_PDFDoc doc, TRN_Downloader downloader)
{
	BEX;
	IMPL_TRN_PDFDocDownloaderInitialize(PDocCast(doc),(Download::LinearDownloader*)downloader);
	EEX;
}

TRN_API TRN_PDFDocDownloadPages(TRN_PDFDoc doc, TRN_UInt32* pages_arr, TRN_Size pages_arr_size, TRN_Bool cancel_others, TRN_Bool download_thumbs)
{
	BEX;
	PDocCast(doc)->DownloadPages(pages_arr,pages_arr_size,TBToB(cancel_others),TBToB(download_thumbs));
	EEX;
}

TRN_API TRN_PDFDocDownloadThumb(TRN_PDFDoc doc, TRN_UInt32 page_num, TRN_Obj* result)
{
	BEX;
	*result = (TRN_Obj)PDocCast(doc)->DownloadThumb(page_num);
	EEX;
}


TRN_API TRN_DownloadComplete(const char* data, size_t start_byte, size_t size, TRN_DownloadContainer cont)
{
	BEX;
#ifdef TRN_HAS_DOWNLOADER
	Download::ChunkDownloaded(data,start_byte,size,(Download::DownloadContainer*) cont);
#endif
	EEX;
}

TRN_API TRN_PDFDocGetTriggerAction(TRN_PDFDoc doc, TRN_PDFDocEventType trigger, TRN_Obj* result)
{
	BEX;
	*result = (TRN_Obj)IMPL_TRN_PDFDocGetTriggerAction(PDocCast(doc), (ActionTriggerEvent)trigger);
	EEX;
}

TRN_API TRN_DownloaderGetRequiredChunks(TRN_Downloader downloader, TRN_UInt32 page_num, TRN_Size* buf, TRN_Size size)
{
	BEX;
	std::set<UInt64> set;
	((Download::LinearDownloader*) downloader)->GetRequiredChunkSet(page_num, set);
	BASE_ASSERT(size == set.size(), "Size for GetRequiredChunks is incorrect. Please call GetRequiredChunksSize first to obtain this value.");
	std::set<UInt64>::iterator itr = set.begin(), end = set.end();
	for(UInt64 i=0; itr!=end; ++itr, ++i) {
		buf[i] = sizeT(*itr);
	}
	EEX;
}

TRN_API TRN_DownloaderGetRequiredChunksSize(TRN_Downloader downloader, TRN_UInt32 page_num, TRN_Size* result)
{
	BEX;
	std::set<UInt64> set;
	((Download::LinearDownloader*) downloader)->GetRequiredChunkSet(page_num, set);
	*result = set.size();
	EEX;
	return 0;
}

TRN_API TRN_GetDownloadChunkSize(TRN_Size* result)
{
	BEX;
	*result = DOWN_CHUNK_SIZE;
	EEX;
}

TRN_API TRN_PDFDocGetFinalObjRef(TRN_PDFDoc doc, TRN_UInt32 original_obj_num, TRN_UInt16 original_gen_num, TRN_UInt32* out_obj_num, TRN_UInt16* out_gen_num)
{
	BEX;
	std::map<UInt32, pair<UInt32, UInt16> >& objnum_map = PDocCast(doc)->mp_doc->m_original_objnum_map;
	std::map<UInt32, pair<UInt32, UInt16> >::iterator itr = objnum_map.find(original_obj_num);
	if (itr == objnum_map.end()) 
	{
		*out_obj_num = original_obj_num;
		*out_gen_num = original_gen_num;
	}
	else
	{
		*out_obj_num = itr->second.first;
		*out_gen_num = itr->second.second;
	}
	EEX;
}


TRN_API TRN_PDFDocAppearancesToXodBuffer(TRN_PDFDoc in_pdfdoc, TRN_AppearanceReferenceList reference_list, TRN_Vector* result)
{
	BEX;
#ifdef TRN_HAS_XOD
	vector<AppearanceReference>& references = *(vector<AppearanceReference>*)reference_list;
	PageAppearanceMapping mapping;
	mapping[1] = references;

	silverlight::SilverOptions soptions(false);
	soptions.m_dpi = 300;
	AutoPtr<Filters::Filter> filter = silverlight::WriteAppearancesAsXod(PDFDocCast(in_pdfdoc), mapping, soptions, 1, 2);
	Filters::FilterReader reader(*filter);
	AutoPtr<FlexVector<UChar> > vec(new FlexVector<UChar>());
	reader.Read(*vec.get());

	*result = (TRN_Vector)(new PolyVectorByte(vec));
#else
	BASE_ASSERT(false, "AppearancesToXodBuffer is not available in this build");
#endif
	EEX;
}

TRN_API TRN_PDFDocGetGeometryCollectionForPage(TRN_PDFDoc in_pdfdoc, int page_num, TRN_GeometryCollection* result)
{
	BEX;
	*result = (TRN_GeometryCollection)(IMPL_TRN_PPDFDocGetGeometryCollectionForPage(PDFDocCast(in_pdfdoc), page_num).release());
	EEX;
}

TRN_API TRN_PDFDocGetUndoManager(TRN_PDFDoc in_pdfdoc, TRN_UndoManager* result)
{
	BEX;
	*result = (TRN_UndoManager)(PDFDocCast(in_pdfdoc).GetUndoManager().release());
	EEX;
}

TRN_API TRN_PDFDocCreateDigitalSignatureField(TRN_PDFDoc doc, const TRN_UString in_sig_field_name, TRN_DigitalSignatureField* result)
{
	BEX;
	UString temp(UStrCast(in_sig_field_name));
	result->mp_field_dict_obj = (TRN_Obj) PDFDocCast(doc).CreateDigitalSignatureField(temp).GetSDFObj();
	EEX;
}

TRN_API TRN_PDFDocGetDigitalSignatureField(TRN_PDFDoc doc, const TRN_UString in_sig_field_name, TRN_DigitalSignatureField* result)
{
	BEX;
	UString temp(UStrCast(in_sig_field_name));
	DigitalSignatureFieldIterator itr=IMPL_TRN_PDFDocFieldFind(PDocCast(doc),temp), end=IMPL_TRN_PDFDocFieldEnd(PDocCast(doc));
	if(itr!=end)
		result->mp_field_dict_obj = (TRN_Obj) itr->GetSDFObj();
	else
	{
		result->mp_field_dict_obj= NULL;
	}
	EEX;
}

TRN_API TRN_PDFDocGetDigitalSignatureFieldIteratorBegin(TRN_PDFDoc doc, TRN_Iterator* result)
{
	BEX;
	DigitalSignatureFieldIterator itr = IMPL_TRN_PDFDocDigitalSignatureFieldBegin(PDocCast(doc));
	DigitalSignatureFieldIterator end = IMPL_TRN_PDFDocDigitalSignatureFieldEnd();
	*result = (TRN_Iterator)new trn::PolyDigitalSignatureFieldIterator(itr, end);
	EEX;
}

TRN_API TRN_PDFDocGetDigitalSignaturePermissions(TRN_PDFDoc doc, enum TRN_DigitalSignatureField_DocumentPermissions* result)
{
	BEX;
	*(trn::UInt32*)result = IMPL_TRN_PDFDocGetDigitalSignaturePermissions(PDocCast(doc));
	EEX;
}

#ifdef JDM

TRN_API TRN_DispListCreate(TRN_Page page, TRN_DispList* result)
{
	BEX;
	DomDisplayList* disp = new DomDisplayList();
	disp->Load(PageCast(page), 0);
	*result = (TRN_DispList) disp;
	EEX;
}

TRN_API TRN_DispListTag(TRN_DispList lst, double* rects, TRN_Size rect_num, const char* tag, TRN_Obj prop_dict, TRN_Bool intersect_mode, TRN_Bool reshuffle, TRN_Bool* result)
{
	BEX;
	DomDisplayList* dl = (DomDisplayList*) lst;
	vector<Rect> r; 
	r.reserve(rect_num);
	for (TRN_Size i=0; i<rect_num; ++i) 
	{
		r.push_back(Rect(rects[i*4], rects[i*4+1], rects[i*4+2], rects[i*4+3]));
	}
	*result = BToTB(dl->Tag(r, tag, (Obj*)prop_dict, TBToB(intersect_mode), TBToB(reshuffle) ));
	EEX;
}

TRN_API TRN_DispListClearTags(TRN_DispList lst)
{
	BEX;
	DomDisplayList* dl = (DomDisplayList*) lst;
	dl->TagClear();
	EEX;
}

TRN_API TRN_DispListSave(TRN_DispList lst, TRN_Page page)
{
	BEX;
	Page pg(PageCast(page));
	DomElementWriter writer;
	DomDisplayList* dl = (DomDisplayList*) lst;
	writer.Write(&pg, dl);
	EEX;
}

TRN_API TRN_DispListDestroy(TRN_DispList lst)
{
	BEX;
	DomDisplayList* dl = (DomDisplayList*) lst;
	delete dl;
	EEX;
}

#endif
