#include <C/PDF/TRN_PDFView.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicVector.hpp>
#include <Common/BoostNetwork.hpp>
#include <Common/PixelTypes.hpp>
#include <PDF/PDFView.h>
#include <PDF/Annot.h>
#include <PDF/Point.h>
#include <PDF/CoreDocumentConversion.hpp>
#include <SDF/License.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#ifndef trn
using namespace trn;
#endif
using namespace PDF;
using namespace SDF;

TRN_API TRN_PDFViewCreate(TRN_PDFView* result)
{
	BEX;
	*result=(TRN_PDFView)new PDFView();
	EEX;
}

TRN_API TRN_PDFViewDestroy(TRN_PDFView view)
{
	BEX;
	delete (PDFView*)view;
	EEX;
}

TRN_API TRN_PDFViewSetDoc(TRN_PDFView view,  TRN_PDFDoc doc)
{
	BEX;
    License::Instance().VerifyPermission(License::e_perm_pdfview);
	ViewCast(view).SetDoc(*((PDFDoc*)(&doc)));
	EEX;
}

TRN_API TRN_OpenUniversalDoc(TRN_PDFView view, TRN_DocumentConversion conversion)
{
	BEX;
    License::Instance().VerifyPermission(License::e_perm_pdfview);
	// TRN_DocumentConversion is a bare pointer to a trn::iDocumentConversion
	ViewCast(view).OpenUniversalDoc(
		RefCountedConversion((trn::PDF::iDocumentConversion*)conversion));
	EEX;
}

TRN_API TRN_PDFViewCloseDoc(TRN_PDFView view)
{
    BEX;
    ViewCast(view).CloseDoc();
    EEX;
}

TRN_API TRN_PDFViewGetDoc(TRN_PDFView view, TRN_PDFDoc* result)
{
	BEX;
	PDFDoc* doc=ViewCast(view).GetDoc();
	if(doc)
		*result=(TRN_PDFDoc)(doc->mp_sdf_doc);
	else
		*result=0;
	EEX;
}

TRN_API TRN_PDFViewDocLock(TRN_PDFView view, TRN_Bool cancel)
{
	BEX;
	ViewCast(view).DocLock(TBToB(cancel));
	EEX;
}

TRN_API TRN_PDFViewDocUnlock(TRN_PDFView view)
{
	BEX;
	ViewCast(view).DocUnlock();
	EEX;
}

TRN_API TRN_PDFViewDocTryLock(TRN_PDFView view, int milliseconds, TRN_Bool* result)
{
	BEX;
	*result = ViewCast(view).DocTryLock(milliseconds);
	EEX;
}

TRN_API TRN_PDFViewDocLockRead(TRN_PDFView view)
{
	BEX;
	ViewCast(view).DocLockRead();
	EEX;
}

TRN_API TRN_PDFViewDocUnlockRead(TRN_PDFView view)
{
	BEX;
	ViewCast(view).DocUnlockRead();
	EEX;
}

TRN_API TRN_PDFViewDocTryLockRead(TRN_PDFView view, int milliseconds, TRN_Bool* result)
{
	BEX;
	*result = ViewCast(view).DocTryLockRead(milliseconds);
	EEX;
}

TRN_API TRN_PDFViewRefreshAndUpdate(TRN_PDFView view, const TRN_ViewChangeCollection view_change)
{
	BEX;
	ViewCast(view).RefreshAndUpdate(*(ViewChangeCollection*)view_change);
	EEX;
}

TRN_API TRN_PDFViewSetPageRefViewMode(TRN_PDFView view, enum TRN_PDFViewPageViewMode mode)
{
    BEX;
    ViewCast(view).SetPageRefViewMode((PDFView::PageViewMode)mode);
    EEX;
}

TRN_API TRN_PDFViewGetPageRefViewMode( const TRN_PDFView view, enum TRN_PDFViewPageViewMode* result)
{
	BEX;
	*result=(enum TRN_PDFViewPageViewMode)ViewCast(view).GetPageRefViewMode();
	EEX;
}

TRN_API TRN_PDFViewSetPageViewMode(TRN_PDFView view,  enum TRN_PDFViewPageViewMode mode)
{
	BEX;
	ViewCast(view).SetPageViewMode((PDFView::PageViewMode)mode);
	EEX;
}


TRN_API TRN_PDFViewGetPageViewMode( const TRN_PDFView view, enum TRN_PDFViewPageViewMode* result)
{
	BEX;
	*result=(enum TRN_PDFViewPageViewMode)ViewCast(view).GetPageViewMode();
	EEX;
}

TRN_API TRN_PDFViewSetPagePresentationMode(TRN_PDFView view,  enum TRN_PDFViewPagePresentationMode mode)
{
	BEX;
	ViewCast(view).SetPagePresentationMode((PDFView::PagePresentationMode)mode);
	EEX;
}

TRN_API TRN_PDFViewGetPagePresentationMode( const TRN_PDFView view,  enum TRN_PDFViewPagePresentationMode* result)
{
	BEX;
	*result=(enum TRN_PDFViewPagePresentationMode)ViewCast(view).GetPagePresentationMode();
	EEX;
}

TRN_API TRN_PDFViewGetCurrentPage( const TRN_PDFView view,  int* result)
{
	BEX;
	*result=ViewCast(view).GetCurrentPage();
	EEX;
}

TRN_API TRN_PDFViewGetPageCount( const TRN_PDFView view,  int* result)
{
	BEX;
	*result=ViewCast(view).GetPageCount();
	EEX;
}

TRN_API TRN_PDFViewGetVisiblePageCount( const TRN_PDFView view,  int* result)
{
	BEX;
	*result=ViewCast(view).GetVisiblePageCount();
	EEX;
}

TRN_API TRN_PDFViewGetVisiblePage( const TRN_PDFView view, int idx, int* result)
{
	BEX;
	*result=ViewCast(view).GetVisiblePage(idx);
	EEX;
}

TRN_API TRN_PDFViewSetUrlExtraction(const TRN_PDFView view, TRN_Bool enabled)
{
	BEX;
	ViewCast(view).SetUrlExtraction(TBToB(enabled));
	EEX;
}

TRN_API TRN_PDFViewGetLinkAt(const TRN_PDFView view, int x, int y, TRN_Rect* rect, TRN_UString* url)
{
	BEX;
	LinkifyPDF::LinkInfo link_info = ViewCast(view).GetLinkAt(x, y);
	RectCast(rect) = link_info.m_link_rect;
	UString* temp = (UString*)url;
	*temp = link_info.m_url;
	EEX;
}

TRN_API TRN_PDFViewGotoFirstPage(TRN_PDFView view,  TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).GotoFirstPage());
	EEX;
}

TRN_API TRN_PDFViewGotoLastPage(TRN_PDFView view, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).GotoLastPage());
	EEX;
}

TRN_API TRN_PDFViewGotoNextPage(TRN_PDFView view, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).GotoNextPage());
	EEX;
}

TRN_API TRN_PDFViewGotoPreviousPage(TRN_PDFView view, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).GotoPreviousPage());
	EEX;
}

TRN_API TRN_PDFViewSetCurrentPage(TRN_PDFView view,  int page_num, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).SetCurrentPage(page_num));
	EEX;
}

TRN_API TRN_PDFViewShowRect( TRN_PDFView view, int page_num, const TRN_Rect* rect, TRN_Bool* result )
{
	BEX;
	*result=BToTB(ViewCast(view).ShowRect(page_num, RectCast(rect)));
	EEX;
}

TRN_API TRN_PDFViewGetZoom( const TRN_PDFView view, double* result)
{
	BEX;
	*result=ViewCast(view).GetZoom();
	EEX;
}

TRN_API TRN_PDFViewSetZoom(TRN_PDFView view,  double zoom, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).SetZoom(zoom, true));
	EEX;
}

TRN_API TRN_PDFViewSetZoomOnPoint(TRN_PDFView view,  int x, int y, double zoom, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).SetZoom(x,y,zoom, true));
	EEX;
}

TRN_API TRN_PDFViewSmartZoomOnPoint(TRN_PDFView view,  int x, int y, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).SmartZoom(x,y));
	EEX;
}

TRN_API TRN_PDFViewRotateClockwise(TRN_PDFView view)
{
	BEX;
	ViewCast(view).RotateClockwise();
	EEX;
}

TRN_API TRN_PDFViewRotateCounterClockwise(TRN_PDFView view)
{
	BEX;
	ViewCast(view).RotateCounterClockwise();
	EEX;
}

TRN_API TRN_PDFViewGetPageNumberFromScreenPt( const TRN_PDFView view,  double x, double y, int* result)
{
	BEX;
	*result=ViewCast(view).GetPageNumberFromScreenPt(x,y);
	EEX;
}

TRN_API TRN_PDFViewConvScreenPtToCanvasPt( const TRN_PDFView view,  double* x, double* y)
{
	BEX;
	ViewCast(view).ConvScreenPtToCanvasPt(*x,*y);
	EEX;
}

TRN_API TRN_PDFViewConvCanvasPtToScreenPt( const TRN_PDFView view,  double* x, double* y)
{
	BEX;
	ViewCast(view).ConvCanvasPtToScreenPt(*x,*y);
	EEX;
}

TRN_API TRN_PDFViewConvCanvasPtToPagePt( const TRN_PDFView view,  double* x, double* y, int page_num)
{
	BEX;
	ViewCast(view).ConvCanvasPtToPagePt(*x,*y,page_num);
	EEX;
}

TRN_API TRN_PDFViewConvPagePtToCanvasPt( const TRN_PDFView view,  double* x, double* y, int page_num)
{
	BEX;
	ViewCast(view).ConvPagePtToCanvasPt(*x,*y,page_num);
	EEX;
}

TRN_API TRN_PDFViewConvScreenPtToPagePt( const TRN_PDFView view,  double* x, double* y, int page_num)
{
	BEX;
	ViewCast(view).ConvScreenPtToPagePt(*x,*y,page_num);
	EEX;
}

TRN_API TRN_PDFViewConvPagePtToScreenPt( const TRN_PDFView view,  double* x, double* y, int page_num)
{
	BEX;
	ViewCast(view).ConvPagePtToScreenPt(*x,*y,page_num);
	EEX;
}

TRN_API TRN_PDFViewSnapToNearestInDoc(TRN_PDFView view, double* screen_x, double* screen_y)
{
	BEX;
	PDF::Point in(*screen_x, *screen_y);
	PDF::Point out = ViewCast(view).SnapToNearestInDoc(in);
	*screen_x = out.x;
	*screen_y = out.y;
	EEX;
}

TRN_API TRN_PDFViewSetSnappingMode(TRN_PDFView view, unsigned int mode_flags)
{
	BEX;
	ViewCast(view).SetSnappingMode(mode_flags);
	EEX;
}

TRN_API TRN_PDFViewGetDeviceTransform( const TRN_PDFView view,  int page_num, TRN_Matrix2D* result)
{
	BEX;
    Common::Matrix2D mtx = ViewCast(view).GetDeviceTransform(page_num);
	*result=RetM2D(mtx);
	EEX;
}

TRN_API TRN_PDFViewSetErrorReportProc(TRN_PDFView view,  TRN_RasterizerErrorReportProc error_proc, void* data)
{
	BEX;
	ViewCast(view).SetErrorReportProc(error_proc,data);
	EEX;
}


TRN_API TRN_PDFViewEnableUndoRedo(TRN_PDFView view)
{
    BEX;
    ViewCast(view).EnableUndoRedo();
    EEX;
}

TRN_API TRN_PDFViewUndo(TRN_PDFView view, TRN_UString* result)
{
    BEX;
    RetUStr(ViewCast(view).Undo());
    EEX;
}

TRN_API TRN_PDFViewRedo(TRN_PDFView view, TRN_UString* result)
{
    BEX;
    RetUStr(ViewCast(view).Redo());
    EEX;
}

TRN_API TRN_PDFViewTakeSnapshot(TRN_PDFView view, const TRN_UString meta_info)
{
    BEX;
    UString temp(UStrCast(meta_info));
    ViewCast(view).TakeSnapshot(temp);
    EEX;
}

TRN_API TRN_PDFViewGetNextUndoInfo(TRN_PDFView view, TRN_UString* result)
{
    BEX;
    RetUStr(ViewCast(view).GetNextUndoInfo());
    EEX;
}

TRN_API TRN_PDFViewGetNextRedoInfo(TRN_PDFView view, TRN_UString* result)
{
    BEX;
    RetUStr(ViewCast(view).GetNextRedoInfo());
    EEX;
}

TRN_API TRN_PDFViewRevertAllChanges(TRN_PDFView view)
{
    BEX;
    ViewCast(view).RevertAllChanges();
    EEX;
}

TRN_API TRN_PDFViewGetExternalAnnotManager(TRN_PDFView view, const TRN_UString author, TRN_ExternalAnnotManager* externalAnnotManager)
{
    BEX;
    UString temp(UStrCast(author));
    *externalAnnotManager = (TRN_ExternalAnnotManager)(ViewCast(view).GetExternalAnnotManager(temp)->AddReference());
    EEX;
}

TRN_API TRN_PDFViewSetCurrentPageProc(TRN_PDFView view,  TRN_PDFViewCurrentPageProc curr_pagenum_proc, void* data)
{
	BEX;
	ViewCast(view).SetCurrentPageProc(curr_pagenum_proc,data);
	EEX;
}

TRN_API TRN_PDFViewSetCurrentZoomProc(TRN_PDFView view,  TRN_PDFViewCurrentZoomProc curr_zoom_proc, void* data)
{
	BEX;
	ViewCast(view).SetCurrentZoomProc(curr_zoom_proc,data);
	EEX;
}

TRN_API TRN_PDFViewSetJavaScriptEventCallBack(TRN_PDFView view, TRN_JavaScriptEventProc js_proc, void* data)
{
	BEX;
	ViewCast(view).SetJavaScriptEventProc(js_proc,data);
	EEX;
}

TRN_API TRN_PDFViewGetCanvasWidth( const TRN_PDFView view, double* result)
{
	BEX;
	*result=ViewCast(view).GetCanvasWidth();
	EEX;
}

TRN_API TRN_PDFViewGetCanvasHeight( const TRN_PDFView view, double* result)
{
	BEX;
	*result=ViewCast(view).GetCanvasHeight();
	EEX;
}

TRN_API TRN_PDFViewGetHScrollPos( const TRN_PDFView view, double* result)
{
	BEX;
	*result=ViewCast(view).GetHScrollPos();
	EEX;
}

TRN_API TRN_PDFViewGetVScrollPos( const TRN_PDFView view, double* result)
{
	BEX;
	*result=ViewCast(view).GetVScrollPos();
	EEX;
}

TRN_API TRN_PDFViewOnScroll(TRN_PDFView view,  int pix_dx, int pix_dy)
{
	BEX;
	ViewCast(view).OnScroll(pix_dx,pix_dy);
	EEX;
}

TRN_API TRN_PDFViewSetHScrollPos(TRN_PDFView view,  double pos)
{
	BEX;
	ViewCast(view).SetHScrollPos(pos);
	EEX;
}

TRN_API TRN_PDFViewSetVScrollPos(TRN_PDFView view,  double pos)
{
	BEX;
	ViewCast(view).SetVScrollPos(pos);
	EEX;
}

TRN_API TRN_PDFViewOnSize(TRN_PDFView view,  int width, int height)
{
	BEX;
	ViewCast(view).OnSize(width,height, true);
	EEX;
}

TRN_API TRN_PDFViewIsFinishedRendering( const TRN_PDFView view, TRN_Bool visible_region_only, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).IsFinishedRendering(TBToB(visible_region_only)));
	EEX;
}

TRN_API TRN_PDFViewCancelRendering(TRN_PDFView view)
{
	BEX;
	ViewCast(view).CancelRendering();
	EEX;
}

TRN_API TRN_PDFViewCancelFindText(TRN_PDFView view)
{
	BEX;
	ViewCast(view).CancelFindText();
	EEX;
}

TRN_API TRN_PDFViewUpdate(TRN_PDFView view, TRN_Bool all)
{
	BEX;
	ViewCast(view).Update(TBToB(all));
	EEX;
}

TRN_API TRN_PDFViewUpdate2(TRN_PDFView view, const TRN_Rect* rect)
{
	BEX;
	ViewCast(view).Update(RectCast(rect));
	EEX;
}

TRN_API TRN_PDFViewUpdate3(TRN_PDFView view, TRN_Annot annot, int num_page)
{
	BEX;
	ViewCast(view).Update(Annot((Obj*)annot), num_page);
	EEX;
}


TRN_API TRN_PDFViewUpdateField(TRN_PDFView view, const TRN_Field* field)
{
	BEX;
	ViewCast(view).Update(FieldCast(field));
	EEX;
}


TRN_API TRN_PDFViewUpdatePageLayout(TRN_PDFView view)
{
	BEX;
	ViewCast(view).UpdatePageLayout();
	EEX;
}

TRN_API TRN_PDFViewUpdateBuffer(TRN_PDFView view)
{
	BEX;
	ViewCast(view).UpdateBuffer();
	EEX;
}

TRN_API TRN_PDFViewGetBuffer( const TRN_PDFView view, const char** result)
{
	BEX;
	*result=ViewCast(view).GetBuffer();
	EEX;
}

TRN_API TRN_PDFViewGetBufferWidth( const TRN_PDFView view, int* result)
{
	BEX;
	*result=ViewCast(view).GetBufferWidth();
	EEX;
}

TRN_API TRN_PDFViewGetBufferHeight( const TRN_PDFView view, int* result)
{
	BEX;
	*result=ViewCast(view).GetBufferHeight();
	EEX;
}

TRN_API TRN_PDFViewGetBufferStride( const TRN_PDFView view, int* result)
{
	BEX;
	*result=ViewCast(view).GetBufferStride();
	EEX;
}

TRN_API TRN_PDFViewHideAnnotation(TRN_PDFView view, TRN_Annot annot)
{
    BEX;
    ViewCast(view).HideAnnotation((Obj*)annot);
    EEX;
}

TRN_API TRN_PDFViewShowAnnotation(TRN_PDFView view, TRN_Annot annot)
{
    BEX;
    ViewCast(view).ShowAnnotation((Obj*)annot);
    EEX;
}

TRN_API TRN_PDFViewSetDrawAnnotations(TRN_PDFView view,  TRN_Bool render_annots)
{
	BEX;
	ViewCast(view).SetDrawAnnotations(TBToB(render_annots));
	EEX;
}

TRN_API TRN_PDFViewSetHighlightFields(TRN_PDFView view,  TRN_Bool highlight)
{
	BEX;
	ViewCast(view).SetHighlightFields(TBToB(highlight));
	EEX;
}

TRN_API TRN_PDFViewSetRequiredFieldBorderColor(TRN_PDFView view, const TRN_ColorPt* col)
{
	BEX;
	ViewCast(view).SetRequiredFieldBorderColor(CPCast(col));
	EEX;
}

TRN_API TRN_PDFViewSetGamma(TRN_PDFView view, double gamma)
{
	BEX;
	ViewCast(view).SetGamma(gamma);
	EEX;
}

TRN_API TRN_PDFViewSetOverprint(TRN_PDFView view, enum TRN_PDFRasterizerOverprintPreviewMode op)
{
	BEX;
	ViewCast(view).SetOverprint(trn::PDF::PDFRasterizer::OverprintPreviewMode(op));
	EEX;
}

TRN_API TRN_PDFViewSetAntiAliasing(TRN_PDFView view,  TRN_Bool enable_aa)
{
	BEX;
	ViewCast(view).SetAntiAliasing(TBToB(enable_aa));
	EEX;
}

TRN_API TRN_PDFViewSetPathHinting(TRN_PDFView view, TRN_Bool enable_hinting)
{
	BEX;
	ViewCast(view).SetPathHinting(TBToB(enable_hinting));
	EEX;
}

TRN_API TRN_PDFViewSetThinLineAdjustment(TRN_PDFView view, TRN_Bool grid_fit, TRN_Bool stroke_adjust)
{
	BEX;
	ViewCast(view).SetThinLineAdjustment(TBToB(grid_fit), TBToB(stroke_adjust));
	EEX;
}

TRN_API TRN_PDFViewSetImageSmoothing(TRN_PDFView view,  TRN_Bool smoothing_enabled)
{
	BEX;
	ViewCast(view).SetImageSmoothing(TBToB(smoothing_enabled));
	EEX;
}

TRN_API TRN_PDFViewSetCaching(TRN_PDFView view,  TRN_Bool enabled)
{
	BEX;
	ViewCast(view).SetCaching(TBToB(enabled));
	EEX;
}

TRN_API TRN_PDFViewSetRightToLeftLanguage(TRN_PDFView view,  TRN_Bool enabled)
{
	BEX;
	ViewCast(view).SetRightToLeftLanguage(TBToB(enabled));
	EEX;
}

TRN_API TRN_PDFViewSetRasterizerType (TRN_PDFView view,  enum TRN_PDFRasterizerType type)
{
	BEX;
	ViewCast(view).SetRasterizerType((PDFRasterizer::Type)type);
	EEX;
}

TRN_API TRN_PDFViewSetColorPostProcessColors (TRN_PDFView view, unsigned int white_color, unsigned int black_color)
{
    BEX;
    ViewCast(view).SetColorPostProcessColors(BGRA8::FromHex((UInt32)white_color), BGRA8::FromHex((UInt32)black_color));
    EEX;
}

TRN_API TRN_PDFViewSetColorPostProcessMapFile (TRN_PDFView view, TRN_Filter no_own_stream)
{
    BEX;
    AutoPtr<Filters::Filter> temp((Filters::Filter*)no_own_stream);
    ViewCast(view).SetColorPostProcessMapFile(temp);
    EEX;
}

TRN_API TRN_PDFViewSetRenderBeginProc(TRN_PDFView view,  TRN_PDFViewRenderBeginEventProc proc, void* data)
{
	BEX;
	ViewCast(view).SetRenderBeginProc(proc,data);
	EEX;
}

TRN_API TRN_PDFViewSetRenderFinishProc(TRN_PDFView view,  TRN_PDFViewRenderFinishEventProc proc, void* data)
{
	BEX;
	ViewCast(view).SetRenderFinishProc((PDFView::RenderFinishEventProc)proc,data);
	EEX;
}

TRN_API TRN_PDFViewSetToolMode(TRN_PDFView view,  enum TRN_PDFViewToolMode mode)
{
	BEX;
	ViewCast(view).SetToolMode((PDFView::ToolMode)mode);
	EEX;
}

TRN_API TRN_PDFViewGetToolMode( const TRN_PDFView view, enum TRN_PDFViewToolMode* result)
{
	BEX;
	*result=(enum TRN_PDFViewToolMode)ViewCast(view).GetToolMode();
	EEX;
}



TRN_API TRN_PDFViewSelectionCreate(TRN_PDFViewSelection* result)
{
	BEX;
	*result = (TRN_PDFViewSelection) new PDFView::Selection();
	EEX;
}

TRN_API TRN_PDFViewSelectionDestroy(TRN_PDFViewSelection sel)
{
	BEX;
	delete (PDFView::Selection*) sel;
	EEX;
}
    
TRN_API TRN_PDFViewSelectionAssign(TRN_PDFViewSelection left, TRN_PDFViewSelection right)
{
	BEX;
	VSCast(left)=VSCast(right);
	EEX;
}

TRN_API TRN_PDFViewSelectionGetPageNum(const TRN_PDFViewSelection s, int* result)
{
	BEX;
	*result=VSCast(s).GetPageNum();
	EEX;
}

//TRN_API TRN_PDFViewSelectionGetRects(const TRN_PDFViewSelection s, const double** rects, int* result)
//{
//	BEX;
//	*result=VSCast(s).GetRects(*rects);
//	EEX;
//}

TRN_API TRN_PDFViewSelectionGetQuads(const TRN_PDFViewSelection s, const double** quads, int* result)
{
	BEX;
	*result=VSCast(s).GetQuads(*quads);
	EEX;
}

TRN_API TRN_PDFViewSelectionGetAsUnicode(const TRN_PDFViewSelection s, TRN_UString result)
{
	BEX;
	UString str(UStrCast(result));
	VSCast(s).GetAsUnicode(str);
	EEX;
}

TRN_API TRN_PDFViewSelectionGetAsHtml(const TRN_PDFViewSelection s, const char** result)
{
	BEX;
	*result=VSCast(s).GetAsHtml();
	EEX;
}

TRN_API TRN_PDFViewSetTextSelectionMode(TRN_PDFView view, enum TRN_PDFViewTextSelectionMode tm)
{
	BEX;
	ViewCast(view).SetTextSelectionMode((PDFView::TextSelectionMode)tm);
	EEX;
}

TRN_API TRN_PDFViewSelect(TRN_PDFView view,  double x1, double y1, double x2, double y2, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).Select(x1,y1,x2,y2));
	EEX;
}

TRN_API TRN_PDFViewSelectWithSnapping(TRN_PDFView view, double x1, double y1, double x2, double y2, 
	TRN_Bool snap_to_start, TRN_Bool snap_to_end, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCast(view).SelectWithSnapping(x1, y1, x2, y2, TBToB(snap_to_start), TBToB(snap_to_end)));
	EEX;
}

TRN_API TRN_PDFViewSelectWithSmartSnapping(TRN_PDFView view, double x1, double y1, double x2, double y2, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCast(view).SelectWithSmartSnapping(x1, y1, x2, y2));
	EEX;
}

TRN_API TRN_PDFViewSelectByStruct(TRN_PDFView view, double x1, double y1, int page1, double x2, double y2, int page2, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).Select(x1,y1,page1,x2,y2,page2));
	EEX;
}

TRN_API TRN_PDFViewSelectByStructWithSnapping(TRN_PDFView view, double x1, double y1, int page1, double x2, double y2, 
	int page2, TRN_Bool snap_to_start, TRN_Bool snap_to_end, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCast(view).SelectWithSnapping(x1, y1, page1, x2, y2, page2, 
		TBToB(snap_to_start), TBToB(snap_to_end)));
	EEX;
}

TRN_API TRN_PDFViewSelectByStructWithSmartSnapping(TRN_PDFView view, double x1, double y1, int page1, 
	double x2, double y2, int page2, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCast(view).SelectWithSmartSnapping(x1, y1, page1, x2, y2, page2));
	EEX;
}

TRN_API TRN_PDFViewSelectByHighlights(TRN_PDFView view,  TRN_Highlights h, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).Select(HLCast(h)));
	EEX;
}

TRN_API TRN_PDFViewSelectBySelection(TRN_PDFView view, TRN_PDFViewSelection s, TRN_Bool * result)
{
	BEX;
	*result=BToTB(ViewCast(view).Select(VSCast(s)));
	EEX;
}
TRN_API TRN_PDFViewSelectAll(TRN_PDFView view)
{
	BEX;
	ViewCast(view).SelectAll();
	EEX;
}

TRN_API TRN_PDFViewHasSelection(const TRN_PDFView view, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).HasSelection());
	EEX;
}

TRN_API TRN_PDFViewClearSelection(TRN_PDFView view)
{
	BEX;
	ViewCast(view).ClearSelection();
	EEX;
}

TRN_API TRN_PDFViewGetSelection( const TRN_PDFView view, int pagenum, TRN_PDFViewSelection* result)
{
	BEX;
	PDFView::Selection* s = new PDFView::Selection(ViewCast(view).GetSelection(pagenum)); 
	*result = (TRN_PDFViewSelection) s;
	EEX;
}


TRN_API TRN_PDFViewGetSelectionBeginPage( const  TRN_PDFView view, int * result )
{
	BEX;
	*result = ViewCast(view).GetSelectionBeginPage();
	EEX;
}

TRN_API TRN_PDFViewGetSelectionEndPage( const  TRN_PDFView view, int * result )
{
	BEX;
	*result = ViewCast(view).GetSelectionEndPage();
	EEX;
}

TRN_API TRN_PDFViewHasSelectionOnPage( const  TRN_PDFView view, int pagenum, TRN_Bool * result )
{
	BEX;
	*result = ViewCast(view).HasSelectionOnPage(pagenum);
	EEX;
}

TRN_API TRN_PDFViewPrepareWords(const TRN_PDFView view, int page_num)
{
	BEX;
	ViewCast(view).PrepareWords(page_num);
	EEX;
}

TRN_API TRN_PDFViewWereWordsPrepared(const TRN_PDFView view, int page_num, TRN_Bool* result)
{
	BEX;
	*result = ViewCast(view).WereWordsPrepared(page_num);
	EEX;
}

TRN_API TRN_PDFViewIsThereTextInRect(const TRN_PDFView view, double x1, double y1, double x2, double y2, TRN_Bool * result)
{
	BEX;
	*result = ViewCast(view).IsThereTextInRect(x1, y1, x2, y2);
	EEX;
}

TRN_API TRN_PDFViewPrepareAnnotsForMouse(const TRN_PDFView view, int page_num, double distance_threshold, double minimum_line_weight)
{
	BEX;
	ViewCast(view).PrepareAnnotsForMouse(page_num, distance_threshold, minimum_line_weight);
	EEX;
}

TRN_API TRN_PDFViewWereAnnotsForMousePrepared(const TRN_PDFView view, int page_num, TRN_Bool * result)
{
	BEX;
	*result = ViewCast(view).WereAnnotsForMousePrepared(page_num);
	EEX;
}

TRN_API TRN_PDFViewGetAnnotTypeUnder(const TRN_PDFView view, double x, double y, enum TRN_AnnotType * result)
{
	BEX;
	*result = (TRN_AnnotType)ViewCast(view).GetAnnotTypeUnder(x, y);
	EEX;
}

TRN_API TRN_PDFViewSetPageBorderVisibility(TRN_PDFView view, TRN_Bool border_visible)
{
	BEX;
	ViewCast(view).SetPageBorderVisibility(TBToB(border_visible));
	EEX;
}

TRN_API TRN_PDFViewSetPageTransparencyGrid(TRN_PDFView view, TRN_Bool trans_grid_visible)
{
	BEX;
	ViewCast(view).SetPageTransparencyGrid(TBToB(trans_grid_visible));
	EEX;
}

TRN_API TRN_PDFViewSetDefaultPageColor(TRN_PDFView view, TRN_UInt8 r, TRN_UInt8 g, TRN_UInt8 b)
{
	BEX;
	ViewCast(view).SetDefaultPageColor(r,g,b,255);
	EEX;
}

TRN_API TRN_PDFViewSetBackgroundColor(TRN_PDFView view, TRN_UInt8 r, TRN_UInt8 g, TRN_UInt8 b, TRN_UInt8 a)
{
	BEX;
	ViewCast(view).SetBackgroundColor(r,g,b,a);
	EEX;
}

TRN_API TRN_PDFViewSetHorizontalAlign(TRN_PDFView view, int align)
{
	BEX;
	ViewCast(view).SetHorizontalAlign(align);
	EEX;
}




TRN_API TRN_PDFViewSetVerticalAlign(TRN_PDFView view, int align)
{
	BEX;
	ViewCast(view).SetVerticalAlign(align);
	EEX;
}

TRN_API TRN_PDFViewSetPageSpacing(TRN_PDFView view, int horiz_col_space, int vert_col_space, int horiz_pad, int vert_pad)
{
	BEX;
	ViewCast(view).SetPageSpacing(horiz_col_space, vert_col_space, horiz_pad, vert_pad);
	EEX;
}

TRN_API TRN_PDFViewSetViewerCache(TRN_SDFDoc document, TRN_Size max_cache_size, TRN_Bool on_disk)
{
	BEX;
	PDFView::SetViewerCache((SDFDoc*)document, max_cache_size, on_disk != 0);
	EEX;
}

TRN_API TRN_PDFViewSetOCGContext(TRN_PDFView view, TRN_OCGContext ctx) 
{
	BEX;
	ViewCast(view).SetOCGContext((OCG::Context*)ctx);
	EEX;
}

TRN_API TRN_PDFViewGetOCGContext(TRN_PDFView view, TRN_OCGContext* result) 
{
	BEX;
	*result = (TRN_OCGContext) ViewCast(view).GetOCGContext();
	EEX;
}

TRN_API TRN_PDFViewUpdateOCGContext(TRN_PDFView view)
{
	BEX;
	ViewCast(view).UpdateOCGContext();
	EEX;
}

TRN_API TRN_PDFViewGetRotation(TRN_PDFView view, enum TRN_PageRotate* result) 
{
	BEX;
	*result=(enum TRN_PageRotate)ViewCast(view).GetRotation();
	EEX;
}

TRN_API TRN_SetDevicePixelDensity(const TRN_PDFView view, double dpi, double scale_factor)
{
	BEX;
	ViewCast(view).SetDevicePixelDensity(dpi, scale_factor);
	EEX;
}

TRN_API TRN_PDFViewGetScreenRectForAnnot(const TRN_PDFView view, TRN_Annot annot, int page_num, TRN_Rect* result)
{
	BEX;
	RectCast(result) = ViewCast(view).GetScreenRectForAnnot(Annot((Obj*)(annot)), page_num);
	EEX;
}

TRN_API TRN_PDFViewGetAnnotationAt(const TRN_PDFView view, int x, int y, double distanceThreshold, double minimumLineWeight, TRN_Annot* result)
{
    BEX;
    *result = (TRN_Annot)(ViewCast(view).GetAnnotationAt(x, y, distanceThreshold, minimumLineWeight));
    EEX;
}


TRN_API TRN_PDFViewGetAnnotationListAt(const TRN_PDFView view, int x1, int y1, int x2, int y2, TRN_Vector* result)
{
    BEX;
    
	FlexVector<SDF::Obj*> tmp(ViewCast(view).GetAnnotationListAt(x1, y1, x2, y2));
	PolyVectorBase<TRN_Annot>* tmp2 = new PolyVectorBase<TRN_Annot>();

    *result = (TRN_Vector) tmp2;
    try
    {
        for (FlexVector<trn::SDF::Obj*>::iterator it = tmp.begin(); it != tmp.end(); ++it)
        {
            TRN_Annot result;
            TRN_AnnotCopy( (TRN_Annot)(*it), &result );
            tmp2->m_vector.push_back(result);
        }
    }
    catch (...)
    {
        TRN_VectorDestroy(*result);
        throw;
    }

    EEX;
}

TRN_API TRN_PDFViewDoAction(const TRN_PDFView view, TRN_ActionParameter action_param)
{
	BEX;
	ViewCast(view).DoAction(*(ActionParameter*)action_param);
	EEX;
}


TRN_API TRN_PDFViewGetAnnotationsOnPage(TRN_PDFView view, int page_num, TRN_Vector* result)
{
    BEX;
    FlexVector<SDF::Obj*> tmp(ViewCast(view).GetAnnotationsOnPage(page_num));
    
    PolyVectorBase<TRN_Annot>* tmp2 = new PolyVectorBase<TRN_Annot>();
    *result = (TRN_Vector) tmp2;
    try
    {
        for (FlexVector<trn::SDF::Obj*>::iterator it = tmp.begin(); it != tmp.end(); ++it)
        {
            TRN_Annot result;
            TRN_AnnotCopy( (TRN_Annot)(*it), &result );
            tmp2->m_vector.push_back(result);
        }
    }
    catch (...)
    {
        TRN_VectorDestroy(*result);
        throw;
    }
    EEX;
}

#if defined(__iOS__)



TRN_API TRN_PDFViewSetWrapperAnnotBitmapProc(TRN_PDFView view, TRN_PDFViewAnnotBitmapProc annotBitmapProc, void* callingObject, TRN_PDFViewAnnotVectorMode mode)
{
	BEX;
	ViewCast(view).SetWrapperAnnotBitmapProc(annotBitmapProc, callingObject, (trn::PDF::PDFView::AnnotVectorMode)mode);
	EEX;
}

TRN_API TRN_PDFViewOpenURLAsync(TRN_PDFView view, const char* url, TRN_UString cache_file, const char* password, TRN_Obj options)
{
	BEX;
    License::Instance().VerifyPermission(License::e_perm_pdfview);
	UString temp(UStrCast(cache_file));
    HTTPRequestOptions ops(ObjCast(options));
	ViewCast(view).OpenURLAsync(url,password,temp,&ops);
	EEX;
}

TRN_API TRN_PDFViewSetPartDownloadedProc(TRN_PDFView view, TRN_PDFViewPartDownloadedProc proc, void* data)
{
	BEX;
	PDFView::PartDownloadedProc p = (PDFView::PartDownloadedProc)proc;
	ViewCast(view).SetPartDownloadedProc(p, data);
	EEX;
}

TRN_API TRN_PDFViewDownloaderUpdatePage(TRN_PDFView view, TRN_PDFDoc doc, TRN_UInt32 page_num, TRN_UInt32 obj_num, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCast(view).DownloaderUpdatePage(PDocCast(doc), page_num, obj_num));
	EEX;
}

TRN_API TRN_PDFViewDownloaderUpdateOutline(TRN_PDFView view, TRN_PDFDoc doc)
{
	BEX;
	ViewCast(view).DownloaderUpdateOutline(PDocCast(doc));
	EEX;

}

TRN_API TRN_PDFViewDownloaderUpdateThumb(TRN_PDFView view, TRN_PDFDoc doc, TRN_UInt32 page_num, TRN_UInt32 obj_num)
{
	BEX;
	ViewCast(view).DownloaderUpdateThumb(PDocCast(doc), page_num, obj_num);
	EEX;
}

TRN_API TRN_PDFViewDownloaderFinishedDownload(TRN_PDFView view, TRN_PDFDoc doc)
{
	BEX;
	ViewCast(view).DownloaderFinishedDownload(PDocCast(doc));
	EEX;
}

TRN_API TRN_PDFViewDownloaderIsCorrectDoc(TRN_PDFView view, TRN_PDFDoc doc, TRN_Bool* result)
{
	BEX;
	*result = BToTB(ViewCast(view).DownloaderIsCorrectDoc(PDocCast(doc)));
	EEX;	
}

TRN_API TRN_PDFViewDownloaderInitialized(TRN_PDFView view, TRN_PDFDoc doc)
{
	BEX;
	ViewCast(view).DownloaderInitialized(PDocCast(doc));
	EEX;	
}

TRN_API TRN_PDFViewGetCurCanvasId( const TRN_PDFView view,  int* result)
{
	BEX;
	*result=ViewCast(view).GetCurCanvasId();
	EEX;
}

TRN_API TRN_PDFViewGetCellSideLength(const TRN_PDFView view, int* result)
{
	BEX;
	*result = ViewCast(view).GetCellSideLength();
	EEX;
}

TRN_API TRN_PDFViewGetPageVGap(const TRN_PDFView view, double* result)
{
	BEX;
	*result = ViewCast(view).GetPageVGap();
	EEX;
}

TRN_API TRN_PDFViewGetPageHGap(const TRN_PDFView view, double* result)
{
	BEX;
	*result = ViewCast(view).GetPageHGap();
	EEX;
}

TRN_API TRN_PDFViewDoProgressiveRender(const TRN_PDFView view)
{
	BEX;
	ViewCast(view).DoProgressiveRender();
	EEX;
}

TRN_API TRN_PDFViewPushViewingStates(const TRN_PDFView view)
{
	BEX;
	ViewCast(view).PushViewingStates();
	EEX;
}

TRN_API TRN_PDFViewSetFreezeTile(TRN_PDFView view,  TRN_Bool freeze)
{
	BEX;
	ViewCast(view).SetFreezeTile(TBToB(freeze));
	EEX;
}

TRN_API TRN_PDFViewPopViewingStates(TRN_PDFView view,  TRN_Bool restore)
{
	BEX;
	ViewCast(view).PopViewingStates(TBToB(restore));
	EEX;
}

TRN_API TRN_PDFViewRequestRender(TRN_PDFView view)
{
	BEX;
	ViewCast(view).RequestRender();
	EEX;
}


TRN_API TRN_PDFViewSetWrapperDeluxeCreateTileProc(TRN_PDFView view, TRN_PDFViewDeluxeCreateTileProc wrapperDeluxeCreateTileProc, void* callingObject)
{
	BEX;
	ViewCast(view).SetWrapperDeluxeCreateTileProc(wrapperDeluxeCreateTileProc, callingObject);
	EEX;
}

TRN_API TRN_PDFViewSetWrapperCreateTileProc(TRN_PDFView view, TRN_PDFViewCreateTileProc wrapperCreateTileProc, void* callingObject)
{
	BEX;
	ViewCast(view).SetWrapperCreateTileProc(wrapperCreateTileProc, callingObject);
	EEX;
}

TRN_API TRN_PDFViewSetWrapperRemoveTileProc(TRN_PDFView view, TRN_PDFViewRemoveTileProc wrapperRemoveTileProc, void* callingObject)
{
	BEX;
	ViewCast(view).SetWrapperRemoveTileProc(wrapperRemoveTileProc, callingObject);
	EEX;
}

TRN_API TRN_PDFViewGetPageRects(const TRN_PDFView view, const TRN_Rect* rect, double* page_rects_info, int* page_rects_size)
{
	BEX;
	std::vector<double> rects_info;
	ViewCast(view).GetPageRects(RectCast(rect), rects_info);
	if(page_rects_info)
	{
		memcpy(page_rects_info, &rects_info.front(), rects_info.size()*sizeof(double));
	}
	*page_rects_size = rects_info.size();
	EEX;
}

TRN_API TRN_PDFViewGetPageRectsOnCanvas(const TRN_PDFView view, double* page_rects_info, int canvasNumber, int* page_rects_size)
{
	BEX;
	std::vector<double> rects_info;
	ViewCast(view).GetPageRectsOnCanvas	(rects_info, canvasNumber);
	if(page_rects_info)
	{
		memcpy(page_rects_info, &rects_info.front(), rects_info.size()*sizeof(double));
	}
	*page_rects_size = rects_info.size();
	EEX;
}

TRN_API TRN_PDFViewGetRefZoomForPage(const TRN_PDFView view, enum TRN_PDFViewPageViewMode viewMode, int pageNum, double* result)
{

	BEX;
    *result = ViewCast(view).GetRefZoomForPage((PDFView::PageViewMode)viewMode, pageNum);
    EEX;
}

TRN_API TRN_PDFViewPurgeMemory(const TRN_PDFView view)
{
	BEX;
	ViewCast(view).PurgeMemory();
	EEX;
}

TRN_API TRN_PDFViewSetMemInfo(const TRN_PDFView view, double total_mem, double mem_cap)
{
	BEX;
	ViewCast(view).SetMemInfo(total_mem, mem_cap);
	EEX;
}

#endif // defined(__iOS__)

TRN_API TRN_PDFViewSetupThumbnails(TRN_PDFView view, TRN_Bool use_embedded, TRN_Bool generate_at_runtime, TRN_Bool use_disk_cache, int thumb_max_side_length, size_t max_abs_cache_size, double max_perc_cache_size)
{
	BEX;
	ViewCast(view).SetupThumbnails(TBToB(use_embedded), TBToB(generate_at_runtime), TBToB(use_disk_cache), thumb_max_side_length, max_abs_cache_size, max_perc_cache_size);
	EEX;
}

TRN_API TRN_PDFViewGetThumbAsync( TRN_PDFView view, int page_num, TRN_PDFViewThumbAsyncHandler proc, void* custom_data )
{
	BEX;
	PDFView::ThumbAsyncHandler p = (PDFView::ThumbAsyncHandler)proc;
	ViewCast(view).GetThumbAsync(page_num, p, custom_data);
	EEX;
}

TRN_API TRN_PDFViewGetThumbInCacheSize(TRN_PDFView view, int page_num, size_t* result)
{
	BEX;
	*result=ViewCast(view).GetThumbInCacheSize(page_num);
	EEX;
}

TRN_API TRN_PDFViewGetThumbInCache(TRN_PDFView view, int page_num, TRN_UChar* buf, TRN_UInt32* width_result, TRN_UInt32* height_result, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ViewCast(view).GetThumbInCache(page_num, buf, *width_result, *height_result));
	EEX;
}

TRN_API TRN_PDFViewCancelAllThumbRequests( TRN_PDFView view)
{
	BEX;
	ViewCast(view).CancelAllThumbRequests();	
	EEX;
}

TRN_API TRN_PDFViewClearThumbCache(TRN_PDFView view)
{
	BEX;
	ViewCast(view).ClearThumbCache();
	EEX;
}


TRN_API TRN_PDFViewSetRequestRenderinWorkerThreadProc(TRN_PDFView view, TRN_PDFViewRequestRenderInWorkerThreadProc proc, void * custom_data)
{
	BEX;
	ViewCast(view).SetRequestRenderInWorkerThreadProc(proc, custom_data);
	EEX;
}

TRN_API TRN_PDFViewSetFindTextHandler(TRN_PDFView view, TRN_PDFViewFindTextAsyncHandler proc, void * custom_data)
{
	BEX;
	PDFView::FindTextAsyncHandler p = (PDFView::FindTextAsyncHandler)proc;
	ViewCast(view).SetFindTextProc(p, custom_data);
	EEX;
}



TRN_API TRN_PDFViewFindTextAsync(TRN_PDFView view, const TRN_UString search_str, TRN_Bool match_case, TRN_Bool match_whole_word,
								 TRN_Bool search_up, TRN_Bool reg_exp)
{
	BEX;
	UString temp(UStrCast(search_str));	
	ViewCast(view).FindTextAsync(temp, TBToB(match_case), TBToB(match_whole_word), TBToB(search_up), TBToB(reg_exp));
	EEX;
}

TRN_API TRN_PDFViewSetColorPostProcessMode( TRN_PDFView view, enum TRN_PDFRasterizerColorPostProcessMode mode )
{
	BEX;
	ViewCast(view).SetColorPostProcess((PDFRasterizer::ColorPostProcessMode)mode);
	EEX;
}

TRN_API TRN_PDFViewGetColorPostProcessMode( const TRN_PDFView view, enum TRN_PDFRasterizerColorPostProcessMode* result)
{
	BEX;
	*result=(enum TRN_PDFRasterizerColorPostProcessMode)ViewCast(view).GetColorPostProcess();
	EEX;
}
