#include <C/PDF/TRN_Page.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/Field.h>
#include <PDF/Page.hpp>
#include <SDF/Obj.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_PageCreate (TRN_Obj page_dict, TRN_Page* result)
{
	BEX;
	*result=(TRN_Page)page_dict;
	EEX;
}

TRN_API TRN_PageCopy (const TRN_Page p, TRN_Page* result)
{
	BEX;
	*result=(TRN_Page)IMPL_TRN_PageCopy(PageCast(p));
	EEX;
}

TRN_API TRN_PageIsValid(const TRN_Page page, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_PageIsValid(PageCast(page)));
	EEX;
}

TRN_API TRN_PageGetIndex(const TRN_Page page, int* result)
{
	BEX;
	*result=IMPL_TRN_PageGetIndex(PageCast(page));
	EEX;
}

TRN_API TRN_PageGetTriggerAction(TRN_Page page, TRN_PageEventType trigger, TRN_Obj* result)
{
	BEX;
	Page pg(PageCast(page));
	*result = (TRN_Obj)pg.GetTriggerAction(ActionTriggerEvent(trigger));
	EEX;
}

TRN_API TRN_PageGetBox(const TRN_Page page, enum TRN_PageBox type, TRN_Rect* result)
{
	BEX;
	Rect r(IMPL_TRN_PageGetBox(PageCast(page),(Page::Box)type));
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

TRN_API TRN_PageSetBox(TRN_Page page, enum TRN_PageBox type, const TRN_Rect* box)
{
	BEX;
	IMPL_TRN_PageSetBox(PageCast(page),(Page::Box)type,RectCast(box));
	EEX;
}

TRN_API TRN_PageGetCropBox(const TRN_Page page, TRN_Rect* result)
{
	BEX;
	Rect r(IMPL_TRN_PageGetCropBox(PageCast(page)));
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

TRN_API TRN_PageSetCropBox(TRN_Page page, const TRN_Rect* box)
{
	BEX;
	IMPL_TRN_PageSetCropBox(PageCast(page),RectCast(box));
	EEX;
}

TRN_API TRN_PageGetMediaBox(const TRN_Page page, TRN_Rect* result)
{
	BEX;
	Rect r(IMPL_TRN_PageGetMediaBox(PageCast(page)));
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

TRN_API TRN_PageSetMediaBox(TRN_Page page, const TRN_Rect* box)
{
	BEX;
	IMPL_TRN_PageSetMediaBox(PageCast(page),RectCast(box));
	EEX;
}

TRN_API TRN_PageGetVisibleContentBox(const TRN_Page page, TRN_Rect* result)
{
	BEX;
	Rect r(IMPL_TRN_PageGetBoundingBox(PageCast(page)));
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

TRN_API TRN_PageGetRotation(const TRN_Page page,enum TRN_PageRotate* result)
{
	BEX;
	*result=(TRN_PageRotate)IMPL_TRN_PageGetRotation(PageCast(page));
	EEX;
}

TRN_API TRN_PageSetRotation(TRN_Page page, enum TRN_PageRotate angle)
{
	BEX;
	IMPL_TRN_PageSetRotation(PageCast(page),(Page::Rotate)angle);
	EEX;
}

TRN_API TRN_PageAddRotations(enum TRN_PageRotate r0, enum TRN_PageRotate r1, enum TRN_PageRotate* result)
{
	BEX;
	*result = (TRN_PageRotate)PDF::AddPageRotations((Page::Rotate)r0, (Page::Rotate)r1);
	EEX;
}

TRN_API TRN_PageSubtractRotations(enum TRN_PageRotate r0, enum TRN_PageRotate r1, enum TRN_PageRotate* result)
{
	BEX;
	*result = (TRN_PageRotate)PDF::SubtractPageRotations((Page::Rotate)r0, (Page::Rotate)r1);
	EEX;
}

TRN_API TRN_PageRotationToDegree(enum TRN_PageRotate r, int* result)
{
	BEX;
	*result = PDF::RotationToDegree((Page::Rotate)r);
	EEX;
}

TRN_API TRN_PageDegreeToRotation(int r, enum TRN_PageRotate* result)
{
	BEX;
	*result = (TRN_PageRotate)PDF::DegreeToRotation(r);
	EEX;
}

TRN_API TRN_PageGetPageWidth(TRN_Page page, enum TRN_PageBox box_type, double* result)
{
	BEX;
	*result=IMPL_TRN_PageGetPageWidth(PageCast(page),(Page::Box)box_type);
	EEX;
}

TRN_API TRN_PageGetPageHeight(TRN_Page page,enum TRN_PageBox box_type, double* result)
{
	BEX;
	*result=IMPL_TRN_PageGetPageHeight(PageCast(page),(Page::Box)box_type);
	EEX;
}

TRN_API TRN_PageGetDefaultMatrix(const TRN_Page page, TRN_Bool flip_y, 
	enum TRN_PageBox box_type, enum TRN_PageRotate angle, TRN_Matrix2D* result)
{
	BEX;
    Common::Matrix2D mtx = IMPL_TRN_PageGetDefaultMatrix(PageCast(page),
		TBToB(flip_y),(Page::Box)box_type,(Page::Rotate)angle);
	*result=RetM2D(mtx);
	EEX;
}

TRN_API TRN_PageGetPageInfo(const TRN_Page page, TRN_Bool flip_y, enum TRN_PageBox box_type, 
							enum TRN_PageRotate angle, double* out_width, double* out_height, 
							TRN_Matrix2D* out_default_matrix, enum TRN_PageRotate* out_rotation)
{
	BEX;
	Common::Matrix2D mtx;
	Page::Rotate rotation;
	IMPL_TRN_PageGetPageInfo(PageCast(page), TBToB(flip_y),(Page::Box)box_type,(Page::Rotate)angle, *out_width, *out_height, mtx, rotation);
	*out_default_matrix=RetM2D(mtx);
	*out_rotation = (TRN_PageRotate)rotation;
	EEX;
};



TRN_API TRN_PageGetAnnots(const TRN_Page page, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PageGetAnnots(PageCast(page));
	EEX;
}

TRN_API TRN_PageGetNumAnnots(const TRN_Page page, TRN_UInt32* result)
{
	BEX;
	*result=(TRN_UInt32)IMPL_TRN_PageGetNumAnnots(PageCast(page));
	EEX;
}

TRN_API TRN_PageGetAnnot(const TRN_Page page, TRN_UInt32 index, TRN_Annot* result)
{
	BEX;
	*result=(TRN_Annot)IMPL_TRN_PageGetAnnot(PageCast(page),(UInt32)index);
	EEX;
}

TRN_API TRN_PageAnnotInsert(TRN_Page page,TRN_UInt32 pos, TRN_Annot annot)
{
	BEX;
	IMPL_TRN_PageAnnotInsert(PageCast(page),(UInt32)pos,AnnCast(annot));
	EEX;
}

TRN_API TRN_PageAnnotPushBack(TRN_Page page,TRN_Annot annot)
{
	BEX;
	IMPL_TRN_PageAnnotPushBack(PageCast(page),AnnCast(annot));
	EEX;
}

TRN_API TRN_PageAnnotPushFront(TRN_Page page,TRN_Annot annot)
{
	BEX;
	IMPL_TRN_PageAnnotPushFront(PageCast(page),AnnCast(annot));
	EEX;
}

TRN_API TRN_PageAnnotRemove(TRN_Page page,TRN_Annot annot)
{
	BEX;
	IMPL_TRN_PageAnnotRemove(PageCast(page),AnnCast(annot));
	EEX;
}

TRN_API TRN_PageAnnotRemoveByIndex(TRN_Page page,TRN_UInt32 index)
{
	BEX;
	IMPL_TRN_PageAnnotRemove(PageCast(page),(UInt32)index);
	EEX;
}

TRN_API TRN_PageScale(TRN_Page page,double scale)
{
	BEX;
	IMPL_TRN_PageScale(PageCast(page),scale);
	EEX;
}

TRN_API TRN_PageFlattenField(TRN_Page page, TRN_Field* field_to_flatten)
{
	BEX;
	IMPL_TRN_PageFlattenField(PageCast(page), FieldCast(field_to_flatten));
	EEX;
}

TRN_API TRN_PageHasTransition(const TRN_Page page, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_PageHasTransition(PageCast(page)));
	EEX;
}

TRN_API TRN_PageGetUserUnitSize(const TRN_Page page, double* result)
{
	BEX;
	*result=IMPL_TRN_PageGetUserUnitSize(PageCast(page));
	EEX;
}

TRN_API TRN_PageSetUserUnitSize(TRN_Page page,double unit_size)
{
	BEX;
	IMPL_TRN_PageSetUserUnitSize(PageCast(page),unit_size);
	EEX;
}

TRN_API TRN_PageGetResourceDict (const TRN_Page page, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PageGetResourceDict(PageCast(page));
	EEX;
}

TRN_API TRN_PageGetContents (const TRN_Page page, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PageGetContents(PageCast(page));
	EEX;
}
		
TRN_API TRN_PageGetThumb(const TRN_Page page, TRN_Obj* result)
{
	BEX;
	Page pg(PageCast(page));
	*result=(TRN_Obj)pg.GetThumb();
	EEX;
}

TRN_API TRN_PageGetSDFObj (const TRN_Page page, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)page;
	EEX;
}

TRN_API TRN_PageFindInheritedAttribute (const TRN_Page page, const char* attrib, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_PageFindInheritedAttribute(PageCast(page),attrib);
	EEX;
}
