#include <C/PDF/TRN_PageLabel.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/PageLabel.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;

TRN_API TRN_PageLabelCreate(TRN_SDFDoc doc, enum TRN_PageLabelStyle style, const TRN_UString prefix, int start_at, TRN_PageLabel* result)
{
	BEX;
	UString temp(UStrCast(prefix));
    PageLabel pageLabel = PageLabel::Create(DocCast(doc),(PageLabel::Style)style,temp,start_at);
	*result=RetPLbl(pageLabel);
	EEX;
}

TRN_API TRN_PageLabelCreateFromObj(TRN_Obj l, int first_page, int last_page, TRN_PageLabel* result)
{
	BEX;
    PageLabel pageLabel = PageLabel(ObjCast(l),first_page,last_page);
	*result=RetPLbl(pageLabel);
	EEX;
}

TRN_API TRN_PageLabelAssign (TRN_PageLabel* left, const TRN_PageLabel* right)
{
	BEX;
	PLblCast(left)=PLblCast(right);
	EEX;
}

TRN_API TRN_PageLabelCompare(TRN_PageLabel* l, const TRN_PageLabel* d, TRN_Bool* result)
{
	BEX;
	*result=BToTB((PLblCast(l)==PLblCast(d)));
	EEX;
}

TRN_API TRN_PageLabelIsValid(const TRN_PageLabel* l, TRN_Bool* result)
{
	BEX;
	*result=BToTB(PLblCast(l).IsValid());
	EEX;
}

TRN_API TRN_PageLabelGetLabelTitle(TRN_PageLabel* l, int page_num, TRN_UString* result)
{
	BEX;
	RetUStr(PLblCast(l).GetLabelTitle(page_num));
	EEX;
}

TRN_API TRN_PageLabelSetStyle(TRN_PageLabel* l, enum TRN_PageLabelStyle style)
{
	BEX;
	PLblCast(l).SetStyle((PageLabel::Style)style);
	EEX;
}

TRN_API TRN_PageLabelGetStyle(const TRN_PageLabel* l, enum TRN_PageLabelStyle* result)
{
	BEX;
	*result=(enum TRN_PageLabelStyle)PLblCast(l).GetStyle();
	EEX;
}

TRN_API TRN_PageLabelGetPrefix(const TRN_PageLabel* l, TRN_UString* result)
{
	BEX;
	RetUStr(PLblCast(l).GetPrefix());
	EEX;
}

TRN_API TRN_PageLabelSetPrefix(TRN_PageLabel* l, const TRN_UString prefix)
{
	BEX;
	UString temp(UStrCast(prefix));
	PLblCast(l).SetPrefix(temp);
	EEX;
}

TRN_API TRN_PageLabelGetStart(const TRN_PageLabel* l, int* result)
{
	BEX;
	*result=PLblCast(l).GetStart();
	EEX;
}

TRN_API TRN_PageLabelSetStart(TRN_PageLabel* l, int start_at)
{
	BEX;
	PLblCast(l).SetStart(start_at);
	EEX;
}

TRN_API TRN_PageLabelGetFirstPageNum(TRN_PageLabel* l, int* result)
{
	BEX;
	*result=PLblCast(l).GetFirstPageNum();
	EEX;
}

TRN_API TRN_PageLabelGetLastPageNum(TRN_PageLabel* l, int* result)
{
	BEX;
	*result=PLblCast(l).GetLastPageNum();
	EEX;
}

TRN_API TRN_PageLabelGetSDFObj (const TRN_PageLabel* l, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)PLblCast(l).GetSDFObj();
	EEX;
}
