#include <C/PDF/TRN_PatternColor.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/PatternColor.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;

TRN_API TRN_PatternColorCreate(TRN_Obj pattern, TRN_PatternColor* result)
{
	BEX;
	*result=(TRN_PatternColor)new PatternColor(ObjCast(pattern));
	EEX;
}

TRN_API TRN_PatternColorDestroy(TRN_PatternColor pattern)
{
	BEX;
	delete (PatternColor*)pattern;
	EEX;
}

TRN_API TRN_PatternColorAssign(TRN_PatternColor left, TRN_PatternColor right)
{
	BEX;
	PCCast(left)=PCCast(right);
	EEX;
}

TRN_API TRN_PatternColorGetTypeFromObj(TRN_Obj pattern,enum TRN_PatternColorType* result)
{
	BEX;
	*result=(enum TRN_PatternColorType)PatternColor::GetType(ObjCast(pattern));
	EEX;
}

TRN_API TRN_PatternColorGetType(const TRN_PatternColor pc,enum TRN_PatternColorType* result)
{
	BEX;
	*result=(enum TRN_PatternColorType)PCCast(pc).GetType();
	EEX;
}

TRN_API TRN_PatternColorGetSDFObj (TRN_PatternColor pc,TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)PCCast(pc).GetSDFObj();
	EEX;
}

TRN_API TRN_PatternColorGetMatrix(TRN_PatternColor pc, TRN_Matrix2D* result)
{
	BEX;
    Common::Matrix2D mtx = PCCast(pc).GetMatrix();
	*result=RetM2D(mtx);
	EEX;
}

TRN_API TRN_PatternColorGetShading(TRN_PatternColor pc, TRN_Shading* result)
{
	BEX;
	*result=(TRN_Shading)new Shading(PCCast(pc).GetShading());
	EEX;
}

TRN_API TRN_PatternColorGetTilingType(const TRN_PatternColor pc,enum TRN_PatternColorTilingType* result)
{
	BEX;
	*result=(enum TRN_PatternColorTilingType)PCCast(pc).GetTilingType();
	EEX;
}

TRN_API TRN_PatternColorGetBBox(TRN_PatternColor pc, TRN_Rect* result)
{
	BEX;
	Rect r(PCCast(pc).GetBBox());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

TRN_API TRN_PatternColorGetXStep(TRN_PatternColor pc, double* result)
{
	BEX;
	*result=PCCast(pc).GetXStep();
	EEX;
}

TRN_API TRN_PatternColorGetYStep(TRN_PatternColor pc, double* result)
{
	BEX;
	*result=PCCast(pc).GetYStep();
	EEX;
}
