#include <C/PDF/TRN_Rect.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/Rect.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_RectInit (double x1, double y1, double x2, double y2, TRN_Rect* result)
{
	BEX;
	memset(result, 0, sizeof(TRN_Rect));
	((Rect*)result)->Set(x1,y1,x2,y2);
	EEX;
}

TRN_API TRN_RectAssign (TRN_Rect* left, const TRN_Rect* right)
{
	BEX;
	((Rect*)left)->operator=(*((Rect*)right));
	EEX;
}

TRN_API TRN_RectAttach(TRN_Rect* rect, TRN_Obj obj)
{
	BEX;
	((Rect*)rect)->Attach((SDF::Obj*)obj);
	EEX;
}

TRN_API TRN_RectUpdate(TRN_Rect* rect, TRN_Obj obj, TRN_Bool* result)
{
	BEX;
	*result=BToTB(((Rect*)rect)->Update((SDF::Obj*)obj));
	EEX;
}

TRN_API TRN_RectGet(const TRN_Rect* rect, double* out_x1, double* out_y1, double* out_x2, double* out_y2)
{
	BEX;
	((Rect*)rect)->Get(*out_x1,*out_y1,*out_x2,*out_y2);
	EEX;
}

TRN_API TRN_RectSet(TRN_Rect* rect, double x1, double y1, double x2, double y2)
{
	BEX;
	((Rect*)rect)->Set(x1,y1,x2,y2);
	EEX;
}

TRN_API TRN_RectWidth(const TRN_Rect* rect,double* result)
{
	BEX;
	*result=((const Rect*)rect)->Width();
	EEX;
}

TRN_API TRN_RectHeight(const TRN_Rect* rect,double* result)
{
	BEX;
	*result=((const Rect*)rect)->Height();
	EEX;
}

TRN_API TRN_RectContains(const TRN_Rect* rect, double x, double y, TRN_Bool* result)
{
	BEX;
	*result=BToTB(((const Rect*)rect)->Contains(x,y));
	EEX;
}

TRN_API TRN_RectIntersectRect(TRN_Rect* rect, const TRN_Rect* rect1, 
	const TRN_Rect* rect2, TRN_Bool* result)
{
	BEX;
	*result=BToTB(((Rect*)rect)->IntersectRect(*((Rect*)rect1),*((Rect*)rect2)));
	EEX;
}

TRN_API TRN_RectNormalize(TRN_Rect* rect)
{
	BEX;
	((Rect*)rect)->Normalize();
	EEX;
}

TRN_API_T(double) TRN_RectGetX1(const TRN_Rect* rect) {
	return ((Rect*)rect)->x1;
}

TRN_API_T(double) TRN_RectGetY1(const TRN_Rect* rect) {
	return ((Rect*)rect)->y1;
}

TRN_API_T(double) TRN_RectGetX2(const TRN_Rect* rect) {
	return ((Rect*)rect)->x2;
}
TRN_API_T(double) TRN_RectGetY2(const TRN_Rect* rect) {
	return ((Rect*)rect)->y2;
}

TRN_API_T(void) TRN_RectSetX1(TRN_Rect* rect, double x1) {
	((Rect*)rect)->x1 = x1;
}

TRN_API_T(void) TRN_RectSetY1(TRN_Rect* rect, double y1) {
	((Rect*)rect)->y1 = y1;
}

TRN_API_T(void) TRN_RectSetX2(TRN_Rect* rect, double x2) {
	((Rect*)rect)->x2 = x2;
}

TRN_API_T(void) TRN_RectSetY2(TRN_Rect* rect, double y2) {
	((Rect*)rect)->y2 = y2;
}

TRN_API TRN_RectInflate1(TRN_Rect* rect, double amount) {
	BEX;
	((Rect*)rect)->Inflate(amount);
	EEX;
}

TRN_API TRN_RectInflate2(TRN_Rect* rect, double x, double y) {
	BEX;
	((Rect*)rect)->Inflate(x, y);
	EEX;
}
