#include <C/PDF/TRN_Redactor.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <vector>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/Redactor.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_Redactor_RedactionCreate(int page_num, 
									 const TRN_Rect* bbox, 
									 TRN_Bool negative, 
									 const TRN_UString text,
									 TRN_Redaction* result)
{
	BEX;
	UString temp(UStrCast(text));
	*result=(TRN_Redaction)new Redactor::Redaction(page_num, *((Rect*)bbox), TBToB(negative), temp);
	EEX;
}

TRN_API TRN_Redactor_RedactionDestroy(TRN_Redaction redaction)
{
	BEX;
	delete(Redactor::Redaction*)redaction;
	EEX;
}

TRN_API TRN_Redactor_RedactionCopy(TRN_Redaction other, TRN_Redaction* result)
{
	BEX;
	*result=(TRN_Redaction)new Redactor::Redaction(*(Redactor::Redaction*)other);
	EEX;
}

TRN_API TRN_RedactorRedact(TRN_PDFDoc doc, const TRN_Redaction* red_arr, int buf_size, TRN_RedactionAppearance appearance,
							TRN_Bool ext_neg_mode, TRN_Bool page_coord_sys)
{
	BEX;
	PDFDoc* d=(PDFDoc*)&doc;
	std::vector<Redactor::Redaction> in_vec;
	int i;
	for(i=0; i<buf_size; i++)
	{
		in_vec.push_back(*((Redactor::Redaction*)red_arr[i]));
	}

	Redactor::Appearance* app = (Redactor::Appearance*)appearance;	
	Redactor::Redact(*d, in_vec, *app, TBToB(ext_neg_mode), TBToB(page_coord_sys));
	EEX;
}

TRN_API TRN_RedactionAppearanceCreate(TRN_Bool redaction_overlay,
									  const TRN_ColorPt* positive_overlay_color,
									  const TRN_ColorPt* negative_overlay_color,
									  TRN_Bool border,
									  TRN_Bool use_overlay_text,
									  TRN_Font font,
									  double min_font_size,
									  double max_font_size,
									  const TRN_ColorPt* text_color,
									  int horiz_text_alignment,
									  int vert_text_alignment,
									  TRN_Bool show_redacted_content_regions,
									  const TRN_ColorPt* redacted_content_color,
									  TRN_RedactionAppearance* result)
{
	BEX;
	Redactor::Appearance* app = new Redactor::Appearance();
	app->_border = TBToB(border);
	app->_font = FontCast(font);
	app->_min_font_size = min_font_size;
	app->_max_font_size = max_font_size;
	app->_negative_overlay_color = CPCast(negative_overlay_color);
	app->_positive_overlay_color = CPCast(positive_overlay_color);
	app->_redaction_overlay = TBToB(redaction_overlay);
	app->_horiz_text_alignment = horiz_text_alignment;
	app->_vert_text_alignment = vert_text_alignment;	
	app->_text_color = CPCast(text_color);
	app->_use_overlay_text = TBToB(use_overlay_text);
	app->_show_redacted_content_regions = TBToB(show_redacted_content_regions);
	app->_redacted_content_color = CPCast(redacted_content_color);
	*result=(TRN_RedactionAppearance)app;
	EEX;
}

TRN_API TRN_RedactionAppearanceDestroy(TRN_RedactionAppearance app)
{
	BEX;
	delete(Redactor::Appearance*)app;
	EEX;
}
