#include <C/PDF/TRN_TextExtractor.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/Annot.h>
#include <PDF/TextExtractor.h>
#include <PDF/Text/WordBuilder.hpp>
#include <SDF/License.hpp>
#include <Common/XMLUtils.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_TextExtractorCreate(TRN_TextExtractor* result)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_textext);

	*result=(TRN_TextExtractor)new TextExtractor();
	EEX;
}

TRN_API TRN_TextExtractorDestroy(TRN_TextExtractor te)
{
	BEX;
	delete (TextExtractor*)te;
	EEX;
}

TRN_API TRN_TextExtractorBegin(TRN_TextExtractor te, 
			TRN_Page page, const TRN_Rect* clip_ptr, int flags)
{
	BEX;
	Page temp(PageCast(page));
	TECast(te).Begin(temp,(const Rect*)clip_ptr, flags);
	EEX;
}

TRN_API TRN_TextExtractorGetWordCount(TRN_TextExtractor te, 
									  int* result)
{
	BEX;
	*result=TECast(te).GetWordCount();
	EEX;
}

TRN_API TRN_TextExtractorSetRightToLeftLanguage(TRN_TextExtractor te, TRN_Bool rtl)
{
	BEX;
	TECast(te).SetRightToLeftLanguage(rtl != 0);
	EEX;
}

TRN_API TRN_TextExtractorGetRightToLeftLanguage(TRN_TextExtractor te, TRN_Bool* result)
{
	BEX;
	*result = TECast(te).GetRightToLeftLanguage();
	EEX;
}

TRN_API TRN_TextExtractorGetAsText(TRN_TextExtractor te, 
				TRN_Bool dehyphen, TRN_UString result)
{
	BEX;
	UString temp(UStrCast(result));
	TECast(te).GetAsText(temp, TBToB(dehyphen));
	EEX;
}

TRN_API TRN_TextExtractorGetTextUnderAnnot(TRN_TextExtractor te, const TRN_Annot annot, TRN_UString result)
{
	BEX;
    UString temp(UStrCast(result));
	TECast(te).GetTextUnderAnnot(temp, Annot((Obj*)annot));
	EEX;
    
}

TRN_API TRN_TextExtractorGetAsXML(TRN_TextExtractor te, 
			int xml_output_flags, TRN_UString result)
{
	BEX;
	UString temp(UStrCast(result));
	TECast(te).GetAsXML(temp, (UInt32)xml_output_flags);
	EEX;
}

//Style
TRN_API TRN_TextExtractorStyleGetFont(TRN_TextExtractorStyle* tes, 
									  TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)TESCast(tes).GetFont();
	EEX;
}

TRN_API TRN_TextExtractorStyleGetFontName(TRN_TextExtractorStyle* tes, 
										  TRN_UString* result)
{
	BEX;
	RetUStr(TESCast(tes).GetFontName());
	EEX;
}

TRN_API TRN_TextExtractorStyleGetFontSize(TRN_TextExtractorStyle* tes, 
										  double* result)
{
	BEX;
	*result=TESCast(tes).GetFontSize();
	EEX;
}

TRN_API TRN_TextExtractorStyleGetWeight(TRN_TextExtractorStyle* tes, 
										int* result)
{
	BEX;
	*result=TESCast(tes).GetWeight();
	EEX;
}

TRN_API TRN_TextExtractorStyleIsItalic(TRN_TextExtractorStyle* tes, 
									   TRN_Bool* result)
{
	BEX;
	*result=BToTB(TESCast(tes).IsItalic());
	EEX;
}

TRN_API TRN_TextExtractorStyleIsSerif(TRN_TextExtractorStyle* tes, 
									  TRN_Bool* result)
{
	BEX;
	*result=BToTB(TESCast(tes).IsSerif());
	EEX;
}

TRN_API TRN_TextExtractorStyleGetColor(TRN_TextExtractorStyle* tes, 
									   TRN_UInt8 rgb[3])
{
	BEX;
	TESCast(tes).GetColor(rgb);
	EEX;
}

TRN_API TRN_TextExtractorStyleCompare (const TRN_TextExtractorStyle* tes, 
				const TRN_TextExtractorStyle* s, TRN_Bool* result)
{
	BEX;
	*result=BToTB(TESCast(tes)==TESCast(s));
	EEX;
}

TRN_API TRN_TextExtractorStyleCreate(TRN_TextExtractorStyle* result)
{
	BEX;
	TESCast(result)=TextExtractor::Style();
	EEX;
}

TRN_API TRN_TextExtractorStyleCopy(TRN_TextExtractorStyle* s, 
							TRN_TextExtractorStyle* result)
{
	BEX;
	TESCast(result)=TextExtractor::Style(TESCast(s));
	EEX;
}



//Word
TRN_API TRN_TextExtractorWordGetNumGlyphs(TRN_TextExtractorWord* tew, 
										  int* result)
{
	BEX;
	*result=TEWCast(tew).GetNumGlyphs();
	EEX;
}

TRN_API TRN_TextExtractorWordGetBBox(TRN_TextExtractorWord* tew, 
									 double out_bbox[4])
{
	BEX;
	TEWCast(tew).GetBBox(out_bbox);
	EEX;
}

TRN_API TRN_TextExtractorWordGetQuad(TRN_TextExtractorWord* tew, 
									 double out_quad[8])
{
	BEX;
	TEWCast(tew).GetQuad(out_quad);
	EEX;
}

TRN_API TRN_TextExtractorWordGetGlyphQuad(TRN_TextExtractorWord* tew, 
							int glyph_idx, double out_quad[8])
{
	BEX;
	TEWCast(tew).GetGlyphQuad(glyph_idx,out_quad);
	EEX;
}

TRN_API TRN_TextExtractorWordGetCharStyle(TRN_TextExtractorWord* tew, 
						int char_idx, TRN_TextExtractorStyle* result)
{
	BEX;
	TextExtractor::Style s(TEWCast(tew).GetCharStyle(char_idx));
	memcpy(result, &s, sizeof(s));
	EEX;
}  

TRN_API TRN_TextExtractorWordGetStyle(TRN_TextExtractorWord* tew, 
								TRN_TextExtractorStyle* result)
{
	BEX;
	TextExtractor::Style s(TEWCast(tew).GetStyle());
	memcpy(result, &s, sizeof(s));
	EEX;
}

TRN_API TRN_TextExtractorWordGetStringLen(TRN_TextExtractorWord* tew, 
										  int* result)
{
	BEX;
	*result=TEWCast(tew).GetStringLen();
	EEX;
}

TRN_API TRN_TextExtractorWordGetString(TRN_TextExtractorWord* tew, 
									   const TRN_Unicode** result)
{
	BEX;
	*result=TEWCast(tew).GetString();
	EEX;
}

TRN_API TRN_TextExtractorWordGetNextWord(TRN_TextExtractorWord* tew, 
										 TRN_TextExtractorWord* result)
{
	BEX;
	TextExtractor::Word w(TEWCast(tew).GetNextWord());
	memcpy(result, &w, sizeof(w));
	EEX;
}

TRN_API TRN_TextExtractorWordGetCurrentNum(TRN_TextExtractorWord* tew, 
										   int* result)
{
	BEX;
	*result=TEWCast(tew).GetCurrentNum();
	EEX;
}

TRN_API TRN_TextExtractorWordCompare (const TRN_TextExtractorWord* tew, 
				const TRN_TextExtractorWord* word, TRN_Bool* result)
{
	BEX;
	*result=BToTB(TEWCast(tew)==TEWCast(word));
	EEX;
}

TRN_API TRN_TextExtractorWordCreate(TRN_TextExtractorWord* result)
{
	BEX;
	TextExtractor::Word w;
	memcpy(result, &w, sizeof(w));
	EEX;
}

TRN_API TRN_TextExtractorWordIsValid(TRN_TextExtractorWord* tew, 
									 TRN_Bool* result)
{
	BEX;
	*result=BToTB(TEWCast(tew)!=TextExtractor::Word());
	EEX;
}


TRN_API TRN_TextExtractorLineGetNumWords(TRN_TextExtractorLine* line, 
										 int* result)
{
	BEX;
	*result=TELCast(line).GetNumWords();
	EEX;
}

TRN_API TRN_TextExtractorLineIsSimpleLine(TRN_TextExtractorLine* line, 
										  TRN_Bool* result)
{
	BEX;
	*result=BToTB(TELCast(line).IsSimpleLine());
	EEX;
}

TRN_API TRN_TextExtractorLineGetBBox(TRN_TextExtractorLine* line, 
									 const double** result)
{
	BEX;
	*result=TELCast(line).GetBBox();
	EEX;
}

TRN_API TRN_TextExtractorLineGetQuad(TRN_TextExtractorLine* line, 
									 double out_quad[8])
{
	BEX;
	TELCast(line).GetQuad(out_quad);
	EEX;
}

TRN_API TRN_TextExtractorLineGetFirstWord(TRN_TextExtractorLine* line, 
								TRN_TextExtractorWord* result)
{
	BEX;
	TextExtractor::Word w(TELCast(line).GetFirstWord());
	memcpy(result, &w, sizeof(w));
	EEX;
}

TRN_API TRN_TextExtractorLineGetWord(TRN_TextExtractorLine* line, 
					int word_idx, TRN_TextExtractorWord* result)
{
	BEX;
	TextExtractor::Word w(TELCast(line).GetWord(word_idx));
	memcpy(result, &w, sizeof(w));
	EEX;
}

TRN_API TRN_TextExtractorLineGetNextLine(TRN_TextExtractorLine* line,
							TRN_TextExtractorLine* result)
{
	BEX;
	TextExtractor::Line l(TELCast(line).GetNextLine());
	memcpy(result, &l, sizeof(l));
	EEX;
}

TRN_API TRN_TextExtractorLineGetCurrentNum(TRN_TextExtractorLine* line, 
										   int* result)

{
	BEX;
	*result=TELCast(line).GetCurrentNum();
	EEX;
}

TRN_API TRN_TextExtractorLineGetStyle(TRN_TextExtractorLine* line,
							TRN_TextExtractorStyle* result)
{
	BEX;
	TextExtractor::Style s(TELCast(line).GetStyle());
	memcpy(result, &s, sizeof(s));
	EEX;
}

TRN_API TRN_TextExtractorLineGetParagraphID(TRN_TextExtractorLine* line,
											int* result)
{
	BEX;
	*result=TELCast(line).GetParagraphID();
	EEX;
}

TRN_API TRN_TextExtractorLineGetFlowID(TRN_TextExtractorLine* line,
									   int* result)
{
	BEX;
	*result=TELCast(line).GetFlowID();
	EEX;
}

TRN_API TRN_TextExtractorLineEndsWithHyphen(TRN_TextExtractorLine* line,
											TRN_Bool* result)
{
	BEX;
	*result=BToTB(TELCast(line).EndsWithHyphen());
	EEX;
}

TRN_API TRN_TextExtractorLineCompare (const TRN_TextExtractorLine* line, 
				const TRN_TextExtractorLine* line2, TRN_Bool* result)
{
	BEX;
	*result=BToTB(TELCast(line)==TELCast(line2));
	EEX;
}

TRN_API TRN_TextExtractorLineCreate(TRN_TextExtractorLine* result)
{
	BEX;
	TextExtractor::Line l;
	memcpy(result, &l, sizeof(l));
	EEX;
}

TRN_API TRN_TextExtractorLineIsValid(TRN_TextExtractorLine* line,
								   TRN_Bool* result)
{
	BEX;
	*result=BToTB(TELCast(line)!=TextExtractor::Line());
	EEX;
}

TRN_API TRN_TextExtractorGetNumLines(TRN_TextExtractor te, 
									 int* result)
{
	BEX;
	*result=TECast(te).GetNumLines();
	EEX;
}

TRN_API TRN_TextExtractorGetFirstLine(TRN_TextExtractor te, 
							TRN_TextExtractorLine* result)
{
	BEX;
	TextExtractor::Line l(TECast(te).GetFirstLine());
	memcpy(result, &l, sizeof(l));
	EEX;
}

TRN_API TRN_TextExtractorGetAsTextWithOffsets(TRN_TextExtractor te, TRN_UString uni_result, int* offset_arr, TRN_UInt32* offset_size)
{
	BEX;
	UString temp_str(UStrCast(uni_result));
	{
		std::vector<Unicode> temp_str_vec;
		std::vector<int> temp_offsets;
		TECast(te).GetAsText(temp_str_vec, &temp_offsets);
		temp_str.Assign(&temp_str_vec[0], static_cast<int>(temp_str_vec.size()));
		size_t memory_size = sizeof(int)*temp_offsets.size();
		if (offset_arr != NULL)
			memcpy(offset_arr, &temp_offsets[0], memory_size);
		if (offset_size != NULL)
			*offset_size = static_cast<UInt32>(temp_offsets.size());
	}
	// ensure we don't have characters which would be invalid in XML
	std::string str = Common::ConvertUStringToUTF8(temp_str);
	temp_str.Assign2(str.c_str(), static_cast<int>(str.size()), UString::e_utf8);
	EEX;
}

TRN_API TRN_TextExtractorCmptSemanticInfo(TRN_TextExtractor te, TRN_Matrix2D* mtx, double* sen_info_result, TRN_UInt32* sen_info_size)
{
	BEX;
    std::vector<double> temp_struct;
	TECast(te).GetWordBuilder()->CmptSemanticInfo(temp_struct, M2DCast(mtx));
	size_t memory_size=sizeof(double)*temp_struct.size();

    if (sen_info_result != NULL)
	    memcpy(sen_info_result,&temp_struct[0], memory_size);
    if (sen_info_size != NULL)
	    *sen_info_size=static_cast<UInt32>(temp_struct.size());
	
    EEX;
}

TRN_API TRN_TextExtractorGetQuads(TRN_TextExtractor te, const TRN_Matrix2D* mtx, double* quads_result, TRN_UInt32* quads_size)
{
    BEX;
    std::vector<double>& temp_quads = TECast(te).GetWordBuilder()->_quads;
    size_t memory_size = sizeof(double)*temp_quads.size();
    
    if (quads_result != NULL)
        memcpy(quads_result, &temp_quads[0], memory_size);
    if (quads_size != NULL)
        *quads_size = static_cast<UInt32>(temp_quads.size());

    if (quads_result != NULL) {
        // apply the matrix to the quads
        Common::Matrix2D matrix = M2DCast(mtx);
        size_t sz = temp_quads.size() / 2;
        double *p = quads_result;
        for (size_t i = 0; i < sz; ++i) {
            matrix.Mult(*p, *(p + 1));
            p += 2;
        }
    }
	EEX;
}
//#endif

