#include <C/PDF/TRN_TextSearch.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <Common/UString.h>
#include <PDF/Highlights.hpp>
#include <PDF/PDFDoc.h>
#include <PDF/TextSearch.hpp>
#include <SDF/License.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;

TRN_API TRN_TextSearchCreate(TRN_TextSearch* result)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_textext);
	*result=(TRN_TextSearch)new TextSearch();
	EEX;
}


TRN_API TRN_TextSearchDestroy(TRN_TextSearch ts)
{
	BEX;
	delete (TextSearch*)ts;
	EEX;
}


TRN_API TRN_TextSearchBegin(TRN_TextSearch ts, TRN_PDFDoc doc, TRN_UString pattern, TRN_UInt32 mode, int start_page, int end_page, TRN_Bool* result)
{
	BEX;
	UString temp(UStrCast(pattern));
	*result=BToTB( TSCast(ts).Begin( (SDFDoc*)(doc), temp, mode, start_page, end_page ) );
	EEX;
}


TRN_API TRN_TextSearchRun(TRN_TextSearch ts, int* page_num, TRN_UString out_str, TRN_UString ambient_str, TRN_Highlights hlts, enum TRN_TextSearchResultCode* result)
{
	BEX;
	UString t1(UStrCast(out_str));
	UString t2(UStrCast(ambient_str));
	*result = (enum TRN_TextSearchResultCode)TSCast(ts).Run(*page_num, t1, t2, *((Highlights*)hlts));
	EEX;
}

TRN_API TRN_TextSearchSetPattern(TRN_TextSearch ts, TRN_UString pattern, TRN_Bool* result)
{
	BEX;
	UString temp(UStrCast(pattern));
	*result = BToTB(TSCast(ts).SetPattern(temp));
	EEX;
}

TRN_API TRN_TextSearchGetMode(TRN_TextSearch ts, TRN_UInt32* result)
{
	BEX;
	*result = (TRN_UInt32)(TSCast(ts).GetMode());
	EEX;
}


TRN_API TRN_TextSearchSetMode(TRN_TextSearch ts, TRN_UInt32 mode)
{
	BEX;
	TSCast(ts).SetMode((TextSearch::Mode)(mode));
	EEX;
}

TRN_API TRN_TextSearchSetRightToLeftLanguage(TRN_TextSearch ts, TRN_Bool flag)
{
	BEX;
	TSCast(ts).SetRight2LeftLanguage(TBToB(flag));
	EEX;
}

TRN_API TRN_TextSearchGetCurrentPage(TRN_TextSearch ts, int* result)
{
	BEX;
	*result = TSCast(ts).GetCurrentPage();
	EEX;
}
