#include <C/Common/TRN_Iterator.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_IteratorHasNext(TRN_Iterator itr,TRN_Bool* result)
{
	BEX;
	*result=BToTB(((PolyIterator*)itr)->HasNext());
	EEX;
}

TRN_API TRN_IteratorCurrent(TRN_Iterator itr, TRN_ItrData* result)
{
	BEX;
	*result=(TRN_ItrData)(((PolyIterator*)itr)->GetCurrent());
	EEX;
}

TRN_API TRN_IteratorNext(TRN_Iterator itr)
{
	BEX;
	((PolyIterator*)itr)->Next();
	EEX;
}

TRN_API TRN_IteratorAssign(TRN_Iterator right, TRN_Iterator* result)
{
	BEX;
	if(*result)
	{
		if(right)
			((PolyIterator*)*result)->Assign(*((PolyIterator*)right));
		else
		{
			TRN_IteratorDestroy(*result);
			*result=0;
		}

	}
	else if(right){
			*result=((PolyIterator*)right)->Clone();
	}
    EEX;
}

TRN_API TRN_IteratorDestroy(TRN_Iterator itr)
{
	BEX;
	delete ((PolyIterator*)itr);
	EEX;
}

TRN_API TRN_DictIteratorHasNext(TRN_DictIterator itr,TRN_Bool* result)
{
	BEX;
	*result=BToTB(((DictPolyIterator*)itr)->HasNext());
	EEX;
}

TRN_API TRN_DictIteratorKey(TRN_DictIterator itr, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)((DictPolyIterator*)itr)->GetKey();
	EEX;
}

TRN_API TRN_DictIteratorValue(TRN_DictIterator itr, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)((DictPolyIterator*)itr)->GetValue();
	EEX;
}

TRN_API TRN_DictIteratorNext(TRN_DictIterator itr)
{
	BEX;
    	((DictPolyIterator*)itr)->Next();
	EEX;
}

TRN_API TRN_DictIteratorDestroy(TRN_DictIterator itr)
{
	BEX;
	delete ((DictPolyIterator*)itr);
	EEX;
}



TRN_API TRN_DictIteratorAssign(TRN_DictIterator right, TRN_DictIterator* result)
{
	BEX;
	if(*result)
	{
		if(right)
			((DictPolyIterator*)*result)->Assign(*((DictPolyIterator*)right));
		else
		{
			TRN_DictIteratorDestroy(*result);
			*result=0;
		}

	}
	else if(right){
			*result=((DictPolyIterator*)right)->Clone();
	}
    EEX;
}
