#include <C/Filters/TRN_FilterWriter.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <Filters/Filter.h>
#include <Filters/FilterWriter.h>
#include <Common/UString.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace Filters;

TRN_API TRN_FilterWriterCreate(TRN_Filter filter, TRN_FilterWriter* result)
{
	BEX;
	if (filter) {
		*result=(TRN_FilterWriter)new FilterWriter(*((Filter*)filter));
	}
	else {
		*result=(TRN_FilterWriter)new FilterWriter();
	}
	EEX;
}

TRN_API TRN_FilterWriterDestroy (TRN_FilterWriter writer)
{
	BEX;
	delete (FilterWriter*)writer;
	EEX;
}

TRN_API TRN_FilterWriterAttachFilter(TRN_FilterWriter writer, TRN_Filter filter)
{
	BEX;
	((FilterWriter*)writer)->AttachFilter(*((Filter*)filter));
	EEX;
}

TRN_API TRN_FilterWriterGetAttachedFilter(TRN_FilterWriter writer, TRN_Filter* result)
{
	BEX;
	*result=(TRN_Filter)((FilterWriter*)writer)->GetAttachedFilter();
	EEX;
}

TRN_API TRN_FilterWriterSeek(TRN_FilterWriter writer, TRN_Ptrdiff offset, enum TRN_FilterReferencePos origin)
{
	BEX;
	((FilterWriter*)writer)->Seek(offset,(Filter::ReferencePos) origin);
	EEX;
}

TRN_API TRN_FilterWriterTell (TRN_FilterWriter writer, TRN_Ptrdiff* result)
{
	BEX;
	UInt64 result_inner = ((FilterWriter*)writer)->Tell();
	*result = (TRN_Size)Filter::VerifyMemSized(result_inner);
	EEX;
}

TRN_API TRN_FilterWriterCount (TRN_FilterWriter writer, TRN_Size* result)
{
	BEX;
	UInt64 result_inner = ((FilterWriter*)writer)->Count();
	*result = (TRN_Size)Filter::VerifyMemSized(result_inner);
	EEX;
}

TRN_API TRN_FilterWriterFlush (TRN_FilterWriter writer)
{
	BEX;
	((FilterWriter*)writer)->Flush();
	EEX;
}

TRN_API TRN_FilterWriterFlushAll (TRN_FilterWriter writer)
{
	BEX;
	((FilterWriter*)writer)->FlushAll();
	EEX;
}

TRN_API TRN_FilterWriterWriteUChar(TRN_FilterWriter writer, TRN_UChar ch)
{
	BEX;
	((FilterWriter*)writer)->WriteUChar(ch);
	EEX;
}

TRN_API TRN_FilterWriterWriteInt16(TRN_FilterWriter writer, TRN_Int16 num)
{
	BEX;
	((FilterWriter*)writer)->WriteInt((trn::Int16) num);
	EEX;
}

TRN_API TRN_FilterWriterWriteUInt16(TRN_FilterWriter writer, TRN_UInt16 num)
{
	BEX;
	((FilterWriter*)writer)->WriteInt((trn::UInt16) num);
	EEX;
}

TRN_API TRN_FilterWriterWriteInt32(TRN_FilterWriter writer, TRN_Int32 num)
{
	BEX;
	((FilterWriter*)writer)->WriteInt((trn::Int32) num);
	EEX;
}

TRN_API TRN_FilterWriterWriteUInt32(TRN_FilterWriter writer, TRN_UInt32 num)
{
	BEX;
	((FilterWriter*)writer)->WriteInt((trn::UInt32) num);
	EEX;
}

TRN_API TRN_FilterWriterWriteInt64(TRN_FilterWriter writer, TRN_Int64 num)
{
	BEX;
	((FilterWriter*)writer)->WriteInt((trn::Int64) num);
	EEX;
}

TRN_API TRN_FilterWriterWriteUInt64(TRN_FilterWriter writer, TRN_UInt64 num)
{
	BEX;
	((FilterWriter*)writer)->WriteInt((trn::UInt64) num);
	EEX;
}

TRN_API TRN_FilterWriterWriteString(TRN_FilterWriter writer, const char* str)
{
	BEX;
	((FilterWriter*)writer)->WriteString(str);	
	EEX;
}

TRN_API TRN_FilterWriterWriteFilter(TRN_FilterWriter writer, TRN_FilterReader reader)
{
	BEX;
	((FilterWriter*)writer)->WriteFilter(*((FilterReader*)reader));	
	EEX;
}

TRN_API TRN_FilterWriterWriteLine(TRN_FilterWriter writer, const char* line, char eol)
{
	BEX;
	((FilterWriter*)writer)->WriteLine(line,eol);
	EEX;
}

TRN_API TRN_FilterWriterWriteBuffer(TRN_FilterWriter writer, const char* buf, TRN_Size buf_size, TRN_Size* result)
{
	BEX;
	*result=((FilterWriter*)writer)->WriteBuffer(buf,buf_size);
	EEX;
}
