#include <C/PDF/OCG/TRN_OCG.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/OCG/Group.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;
using namespace OCG;

TRN_API TRN_OCGCreateFromObj(TRN_Obj dict, TRN_OCG* result)
{
	*result = (TRN_OCG)dict;
	return 0;
}

TRN_API TRN_OCGCreate(TRN_PDFDoc pdfdoc, const TRN_UString name, TRN_OCG* result) 
{
	BEX;
	PDFDoc* doc=(PDFDoc*)&pdfdoc;
	UString temp(name);
	Group grp = Group::Create(*doc, temp);
	*result = (TRN_OCG)grp.GetSDFObj();
	EEX;
}

TRN_API TRN_OCGCopy(const TRN_OCG c, TRN_OCG* result)
{
	*result = c;
	return 0;
}

TRN_API TRN_OCGGetSDFObj (const TRN_OCG c, TRN_Obj* result)
{
	*result = (TRN_Obj) c;
	return 0;
}

TRN_API TRN_OCGIsValid(const TRN_OCG c, TRN_Bool* result)
{
	BEX;
	Group ocg((Obj*)c);
	*result = BToTB(ocg.IsValidOCG());
	EEX;
}

TRN_API TRN_OCGGetName(const TRN_OCG c, TRN_UString* result)
{
	BEX;
	Group g((Obj*)c);
	RetUStr(g.GetName());
	EEX;
}

TRN_API TRN_OCGSetName(const TRN_OCG c, TRN_UString value)
{
	BEX;
	Group g((Obj*)c);
	UString temp(value);
	g.SetName(temp);
	EEX;
}

TRN_API TRN_OCGGetIntent(const TRN_OCG c, TRN_Obj* result)
{
	BEX;
	Group g((Obj*)c);
	*result = (TRN_Obj) g.GetIntent();
	EEX;
}

TRN_API TRN_OCGSetIntent(const TRN_OCG c, TRN_Obj value)
{
	BEX;
	Group g((Obj*)c);
	g.SetIntent((Obj*)value);
	EEX;
}

TRN_API TRN_OCGHasUsage(const TRN_OCG c, TRN_Bool* result)
{
	BEX;
	Group g((Obj*)c);
	*result = BToTB(g.HasUsage());
	EEX;
}

TRN_API TRN_OCGGetUsage(const TRN_OCG c, const char* key, TRN_Obj* result)
{
	BEX;
	Group g((Obj*)c);
	*result = (TRN_Obj) g.GetUsage(key);
	EEX;
}

//TRN_API TRN_OCGRemoveInitialState(const TRN_OCG c)
//{
//	BEX;
//	Group g((Obj*)c);
//	g.RemoveInitialState();
//	EEX;
//}

TRN_API TRN_OCGGetCurrentState(const TRN_OCG c, const TRN_OCGContext ctx, TRN_Bool* result)
{
	BEX;
	Group g((Obj*)c);
	*result = BToTB(g.GetCurrentState(*((Context*)ctx)));
	EEX;
}

TRN_API TRN_OCGSetCurrentState(const TRN_OCG c, const TRN_OCGContext ctx, TRN_Bool state)
{
	BEX;
	Group g((Obj*)c);
	g.SetCurrentState(*((Context*)ctx), TBToB(state));
	EEX;
}

TRN_API TRN_OCGGetInitialState(const TRN_OCG c, const TRN_OCGConfig cfg, TRN_Bool* result)
{
	BEX;
	Group g((Obj*)c);
	Config config((Obj*)cfg);
	*result = BToTB(g.GetInitialState(config));
	EEX;
}

TRN_API TRN_OCGSetInitialState(const TRN_OCG c, const TRN_OCGConfig cfg, TRN_Bool state)
{
	BEX;
	Group g((Obj*)c);
	Config config((Obj*)cfg);
	g.SetInitialState(config, TBToB(state));
	EEX;
}

TRN_API TRN_OCGIsLocked(const TRN_OCG c, const TRN_OCGConfig cfg, TRN_Bool* result)
{
	BEX;
	Group g((Obj*)c);
	Config config((Obj*)cfg);
	*result = BToTB(g.IsLocked(config));
	EEX;
}

TRN_API TRN_OCGSetLocked(const TRN_OCG c, const TRN_OCGConfig cfg, TRN_Bool state)
{
	BEX;
	Group g((Obj*)c);
	Config config((Obj*)cfg);
	g.SetLocked(config, TBToB(state));
	EEX;
}
