#include <C/PDF/TRN_ActionParameter.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <PDF/ActionUtil.hpp>
#include <PDF/Annots.h>
#include <C/Common/TRN_Exception.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace PDF;
using namespace SDF;

TRN_API TRN_ActionParameterCreate(const TRN_Action action, TRN_ActionParameter *result)
{
	BEX;
	Action act((SDF::Obj*)action);
	*result = (TRN_ActionParameter)new ActionParameter(act);
	EEX;
}

TRN_API TRN_ActionParameterCreateWithField(const TRN_Action action, const TRN_Field* field, TRN_ActionParameter *result)
{
	BEX;
	Action act((SDF::Obj*)action);
	*result = (TRN_ActionParameter)new ActionParameter(act,FieldCast(field));
	EEX;
}

TRN_API TRN_ActionParameterCreateWithAnnot(const TRN_Action action, const TRN_Annot annot, TRN_ActionParameter *result)
{
	BEX;
	Action act((SDF::Obj*)action);
	Annot a((Obj*)annot);
	*result = (TRN_ActionParameter)new ActionParameter(act,a);
	EEX;
}

TRN_API TRN_ActionParameterCreateWithPage(const TRN_Action action, const TRN_Page page, TRN_ActionParameter * result)
{
	BEX;
	Action act((SDF::Obj*)action);
	*result = (TRN_ActionParameter)new ActionParameter(act, Page(PageCast(page)));
	EEX;
}

TRN_API TRN_ActionParameterDestroy(TRN_ActionParameter ap)
{
	BEX;
	delete (ActionParameter*)ap;
	EEX;
}

TRN_API TRN_ActionParameterGetAction(TRN_ActionParameter ap, TRN_Action* result)
{
	BEX;
	*result = (TRN_Action)((*(ActionParameter*)ap).GetAction().GetSDFObj());
	EEX;
}
TRN_API TRN_ActionParameterAssign(TRN_ActionParameter left, TRN_ActionParameter right)
{
	BEX;
	*((ActionParameter*)left) = *((ActionParameter*)right);
	EEX;
}

