#include <C/PDF/TRN_GState.h>
#include <C/PDF/TRN_ColorSpace.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <C/Common/TRN_Exception.hpp>
#include <Common/Matrix2D.h>
#include <PDF/GState.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_GStateGetTransform(TRN_GState gs, TRN_Matrix2D* result)
{
	BEX;
	*result=RetM2D(GSCast(gs)->GetTransform());
	EEX;
}

TRN_API TRN_GStateGetStrokeColorSpace(TRN_GState gs, TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(GSCast(gs)->GetStrokeColorSpace());
	EEX;
}

TRN_API TRN_GStateGetFillColorSpace(TRN_GState gs, TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(GSCast(gs)->GetFillColorSpace());
	EEX;
}

TRN_API TRN_GStateGetStrokeColor(TRN_GState gs, TRN_ColorPt* result)
{
	BEX;
	ColorPt c(GSCast(gs)->GetStrokeColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);	
	EEX;
}

TRN_API TRN_GStateGetStrokePattern(TRN_GState gs, TRN_PatternColor* result)
{
	BEX;
	*result=(TRN_PatternColor)new PatternColor(GSCast(gs)->GetStrokePattern());
	EEX;
}

TRN_API TRN_GStateGetFillColor(TRN_GState gs, TRN_ColorPt* result)
{
	BEX;
	ColorPt c(GSCast(gs)->GetFillColor());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);	
	EEX;
}

TRN_API TRN_GStateGetFillPattern(TRN_GState gs, TRN_PatternColor* result)
{
	BEX;
	*result=(TRN_PatternColor)new PatternColor(GSCast(gs)->GetFillPattern());
	EEX;
}

TRN_API TRN_GStateGetFlatness(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetFlatness();
	EEX;
}

TRN_API TRN_GStateGetLineCap(const TRN_GState gs, enum TRN_GStateLineCap* result)
{
	BEX;
	*result=(enum TRN_GStateLineCap)(GSCast(gs)->GetLineCap());
	EEX;
}

TRN_API TRN_GStateGetLineJoin(const TRN_GState gs,enum TRN_GStateLineJoin* result)
{
	BEX;
	*result=(enum TRN_GStateLineJoin)(GSCast(gs)->GetLineJoin());
	EEX;
}

TRN_API TRN_GStateGetLineWidth(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetLineWidth();
	EEX;
}

TRN_API TRN_GStateGetMiterLimit(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetMiterLimit();
	EEX;
}

TRN_API TRN_GStateGetDashes(const TRN_GState gs, double* out_buf_dashes, int* buf_out_dashes_sz)
{
	BEX;
	std::vector<double> get_dashes;
	GSCast(gs)->GetDashes(get_dashes);
	if (out_buf_dashes)
	{
		memcpy(out_buf_dashes, &(get_dashes[0]), get_dashes.size()*sizeof(double));
	}
	*buf_out_dashes_sz=int(get_dashes.size());
	EEX;
}

TRN_API TRN_GStateGetPhase(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetPhase();
	EEX;
}

TRN_API TRN_GStateGetCharSpacing(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetCharSpacing();
	EEX;
}

TRN_API TRN_GStateGetWordSpacing(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetWordSpacing();
	EEX;
}

TRN_API TRN_GStateGetHorizontalScale(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetHorizontalScale();
	EEX;
}

TRN_API TRN_GStateGetLeading(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetLeading();
	EEX;
}

TRN_API TRN_GStateGetFont(const TRN_GState gs, TRN_Font* result)
{
	BEX;
	*result=(TRN_Font)new Font(GSCast(gs)->GetFont());
	EEX;
}

TRN_API TRN_GStateGetFontSize(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetFontSize();
	EEX;
}

TRN_API TRN_GStateGetTextRenderMode(const TRN_GState gs, enum TRN_GStateTextRenderingMode* result)
{
	BEX;
	*result=(enum TRN_GStateTextRenderingMode)GSCast(gs)->GetTextRenderMode();
	EEX;
}

TRN_API TRN_GStateGetTextRise(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetTextRise();
	EEX;
}

TRN_API TRN_GStateIsTextKnockout(const TRN_GState gs, TRN_Bool* result)
{
	BEX;
	*result=BToTB(GSCast(gs)->IsTextKnockout());
	EEX;
}

TRN_API TRN_GStateGetRenderingIntent(const TRN_GState gs, enum TRN_GStateRenderingIntent* result)
{
	BEX;
	*result=(enum TRN_GStateRenderingIntent)(GSCast(gs)->GetRenderingIntent());
	EEX;
}

TRN_API TRN_GStateGetRenderingIntentType(const char* name, enum TRN_GStateRenderingIntent* result)
{
	BEX;
	*result=(enum TRN_GStateRenderingIntent)(GState::GetRenderingIntentType(name));
	EEX;
}

TRN_API TRN_GStateGetBlendMode(TRN_GState gs,enum TRN_GStateBlendMode* result)
{
	BEX;
	*result=(enum TRN_GStateBlendMode)GSCast(gs)->GetBlendMode();
	EEX;
}

TRN_API TRN_GStateGetFillOpacity(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetFillOpacity();
	EEX;
}

TRN_API TRN_GStateGetStrokeOpacity(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetStrokeOpacity();
	EEX;
}

TRN_API TRN_GStateGetAISFlag(const TRN_GState gs, TRN_Bool* result)
{
	BEX;
	*result=BToTB(GSCast(gs)->GetAISFlag());
	EEX;
}

TRN_API TRN_GStateGetSoftMask(TRN_GState gs, TRN_Obj* result)
{
	BEX;
	const ISoftMask* sm=GSCast(gs)->GetSoftMask();
	*result=(TRN_Obj)(sm?sm->GetSDFObj():0);
	EEX;
}

TRN_API TRN_GStateGetSoftMaskTransform(TRN_GState gs, TRN_Matrix2D* result)
{
	BEX;
	*result=RetM2D(GSCast(gs)->GetSoftMaskTransform());
	EEX;
}

TRN_API TRN_GStateGetStrokeOverprint(const TRN_GState gs, TRN_Bool* result)
{
	BEX;
	*result=BToTB(GSCast(gs)->GetStrokeOverprint());
	EEX;
}

TRN_API TRN_GStateGetFillOverprint(const TRN_GState gs, TRN_Bool* result)
{
	BEX;
	*result=BToTB(GSCast(gs)->GetFillOverprint());
	EEX;
}

TRN_API TRN_GStateGetOverprintMode(const TRN_GState gs, int* result)
{
	BEX;
	*result=GSCast(gs)->GetOverprintMode();
	EEX;
}

TRN_API TRN_GStateGetAutoStrokeAdjust(const TRN_GState gs, TRN_Bool* result)
{
	BEX;
	*result=BToTB(GSCast(gs)->GetAutoStrokeAdjust());
	EEX;
}

TRN_API TRN_GStateGetSmoothnessTolerance(const TRN_GState gs, double* result)
{
	BEX;
	*result=GSCast(gs)->GetSmoothnessTolerance();
	EEX;
}

TRN_API TRN_GStateGetTransferFunct(TRN_GState gs, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)GSCast(gs)->GetTransferFunct();
	EEX;
}

TRN_API TRN_GStateGetBlackGenFunct(TRN_GState gs, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)GSCast(gs)->GetBlackGenFunct();
	EEX;
}

TRN_API TRN_GStateGetUCRFunct(TRN_GState gs, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)GSCast(gs)->GetUCRFunct();
	EEX;
}

TRN_API TRN_GStateGetHalftone(TRN_GState gs, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)GSCast(gs)->GetHalftone();
	EEX;
}

TRN_API TRN_GStateSetTransformMatrix(TRN_GState gs,const TRN_Matrix2D* mtx)
{
	BEX;
	GSCast(gs)->SetTransform(M2DCast(mtx));
	EEX;
}

TRN_API TRN_GStateSetTransform(TRN_GState gs,double a, double b, double c, double d, double h, double v)
{
	BEX;
	GSCast(gs)->SetTransform(a,b,c,d,h,v);
	EEX;
}

TRN_API TRN_GStateConcatMatrix(TRN_GState gs,const TRN_Matrix2D* mtx)
{
	BEX;
	GSCast(gs)->Concat(M2DCast(mtx));
	EEX;
}

TRN_API TRN_GStateConcat(TRN_GState gs,double a, double b, double c, double d, double h, double v)
{
	BEX;
	GSCast(gs)->Concat(a,b,c,d,h,v);
	EEX;
}

TRN_API TRN_GStateSetStrokeColorSpace(TRN_GState gs,TRN_ColorSpace cs)
{
	BEX;
	GSCast(gs)->SetStrokeColorSpace(CSCast(cs));
	EEX;
}

TRN_API TRN_GStateSetFillColorSpace(TRN_GState gs,TRN_ColorSpace cs)
{
	BEX;
	GSCast(gs)->SetFillColorSpace(CSCast(cs));
	EEX;
}

TRN_API TRN_GStateSetStrokeColorWithColorPt(TRN_GState gs,const TRN_ColorPt* c)
{
	BEX;
	GSCast(gs)->SetStrokeColor(CPCast(c));
	EEX;
}

TRN_API TRN_GStateSetStrokeColorWithPattern(TRN_GState gs,TRN_PatternColor pattern)
{
	BEX;
	GSCast(gs)->SetStrokeColor(PCCast(pattern));
	EEX;
}

TRN_API TRN_GStateSetStrokeColor(TRN_GState gs,TRN_PatternColor pattern, const TRN_ColorPt* c)
{
	BEX;
	GSCast(gs)->SetStrokeColor(PCCast(pattern),CPCast(c));
	EEX;
}

TRN_API TRN_GStateSetFillColorWithColorPt(TRN_GState gs,const TRN_ColorPt* c)
{
	BEX;
	GSCast(gs)->SetFillColor(CPCast(c));
	EEX;
}

TRN_API TRN_GStateSetFillColorWithPattern(TRN_GState gs,TRN_PatternColor pattern)
{
	BEX;
	GSCast(gs)->SetFillColor(PCCast(pattern));
	EEX;
}

TRN_API TRN_GStateSetFillColor(TRN_GState gs,TRN_PatternColor pattern, const TRN_ColorPt* c)
{
	BEX;
	GSCast(gs)->SetFillColor(PCCast(pattern),CPCast(c));
	EEX;
}

TRN_API TRN_GStateSetFlatness(TRN_GState gs,double flatness)
{
	BEX;
	GSCast(gs)->SetFlatness(flatness);
	EEX;
}

TRN_API TRN_GStateSetLineCap(TRN_GState gs,enum TRN_GStateLineCap cap)
{
	BEX;
	GSCast(gs)->SetLineCap((GState::LineCap)cap);
	EEX;
}

TRN_API TRN_GStateSetLineJoin(TRN_GState gs,enum TRN_GStateLineJoin join)
{
	BEX;
	GSCast(gs)->SetLineJoin((GState::LineJoin)join);
	EEX;
}

TRN_API TRN_GStateSetLineWidth(TRN_GState gs,double width)
{
	BEX;
	GSCast(gs)->SetLineWidth(width);
	EEX;
}

TRN_API TRN_GStateSetMiterLimit(TRN_GState gs,double miter_limit)
{
	BEX;
	GSCast(gs)->SetMiterLimit(miter_limit);
	EEX;
}

TRN_API TRN_GStateSetDashPattern(TRN_GState gs,const double* buf_dash_array, int buf_dash_array_sz, double phase)
{
	BEX;
	std::vector<double> pass_in;
	pass_in.resize(buf_dash_array_sz);
	memcpy(&(pass_in[0]),buf_dash_array,buf_dash_array_sz*sizeof(double));
	GSCast(gs)->SetDashPattern(pass_in,phase);
	EEX;
}

TRN_API TRN_GStateSetCharSpacing(TRN_GState gs,double char_spacing)
{
	BEX;
	GSCast(gs)->SetCharSpacing(char_spacing);
	EEX;
}

TRN_API TRN_GStateSetWordSpacing(TRN_GState gs,double word_spacing)
{
	BEX;
	GSCast(gs)->SetWordSpacing(word_spacing);
	EEX;
}

TRN_API TRN_GStateSetHorizontalScale(TRN_GState gs,double hscale)
{
	BEX;
	GSCast(gs)->SetHorizontalScale(hscale);
	EEX;
}

TRN_API TRN_GStateSetLeading(TRN_GState gs,double leading)
{
	BEX;
	GSCast(gs)->SetLeading(leading);
	EEX;
}

TRN_API TRN_GStateSetFont(TRN_GState gs,TRN_Font font, double font_sz)
{
	BEX;
	GSCast(gs)->SetFont(FontCast(font),font_sz);
	EEX;
}

TRN_API TRN_GStateSetTextRenderMode(TRN_GState gs,enum TRN_GStateTextRenderingMode rmode)
{
	BEX;
	GSCast(gs)->SetTextRenderMode((GState::TextRenderingMode)rmode);
	EEX;
}

TRN_API TRN_GStateSetTextRise(TRN_GState gs,double rise)
{
	BEX;
	GSCast(gs)->SetTextRise(rise);
	EEX;
}

TRN_API TRN_GStateSetTextKnockout(TRN_GState gs,TRN_Bool knockout)
{
	BEX;
	GSCast(gs)->SetTextKnockout(TBToB(knockout));
	EEX;
}

TRN_API TRN_GStateSetRenderingIntent(TRN_GState gs,enum TRN_GStateRenderingIntent intent)
{
	BEX;
	GSCast(gs)->SetRenderingIntent((GState::RenderingIntent)intent);
	EEX;
}

TRN_API TRN_GStateSetBlendMode(TRN_GState gs,enum TRN_GStateBlendMode BM)
{
	BEX;
	GSCast(gs)->SetBlendMode((GState::BlendMode)BM);
	EEX;
}

TRN_API TRN_GStateSetFillOpacity(TRN_GState gs,double ca)
{
	BEX;
	GSCast(gs)->SetFillOpacity(ca);
	EEX;
}

TRN_API TRN_GStateSetStrokeOpacity(TRN_GState gs,double CA)
{
	BEX;
	GSCast(gs)->SetStrokeOpacity(CA);
	EEX;
}

TRN_API TRN_GStateSetAISFlag(TRN_GState gs,TRN_Bool AIS)
{
	BEX;
	GSCast(gs)->SetAISFlag(TBToB(AIS));
	EEX;
}

TRN_API TRN_GStateSetSoftMask(TRN_GState gs,TRN_Obj SM)
{
	BEX;
	SoftMask sm((Obj*)SM);
	GSCast(gs)->SetSoftMask(&sm);
	EEX;
}

TRN_API TRN_GStateSetStrokeOverprint(TRN_GState gs,TRN_Bool OP)
{
	BEX;
	GSCast(gs)->SetStrokeOverprint(TBToB(OP));
	EEX;
}

TRN_API TRN_GStateSetFillOverprint(TRN_GState gs,TRN_Bool op)
{
	BEX;
	GSCast(gs)->SetFillOverprint(TBToB(op));
	EEX;
}

TRN_API TRN_GStateSetOverprintMode(TRN_GState gs,int OPM)
{
	BEX;
	GSCast(gs)->SetOverprintMode(OPM);
	EEX;
}

TRN_API TRN_GStateSetAutoStrokeAdjust(TRN_GState gs,TRN_Bool SA)
{
	BEX;
	GSCast(gs)->SetAutoStrokeAdjust(TBToB(SA));
	EEX;
}

TRN_API TRN_GStateSetSmoothnessTolerance(TRN_GState gs,double SM)
{
	BEX;
	GSCast(gs)->SetSmoothnessTolerance(SM);
	EEX;
}

TRN_API TRN_GStateSetBlackGenFunct(TRN_GState gs,TRN_Obj BG)
{
	BEX;
	GSCast(gs)->SetBlackGenFunct((Obj*)BG);
	EEX;
}

TRN_API TRN_GStateSetUCRFunct(TRN_GState gs,TRN_Obj UCR)
{
	BEX;
	GSCast(gs)->SetUCRFunct((Obj*)UCR);
	EEX;
}

TRN_API TRN_GStateSetTransferFunct(TRN_GState gs,TRN_Obj TR)
{
	BEX;
	GSCast(gs)->SetTransferFunct((Obj*)TR);
	EEX;
}

TRN_API TRN_GStateSetHalftone(TRN_GState gs,TRN_Obj HT)
{
	BEX;
	GSCast(gs)->SetHalftone((Obj*)HT);
	EEX;
}
