#include <C/PDF/TRN_HTML2PDF.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <Convert/Html2Pdf/Html2Pdf.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

#define WebPageSettingCast(WebPageSetting) (*((HTML2PDF::WebPageSettings*)WebPageSetting))
#define TOCSettingCast(TOCSetting) (*((HTML2PDF::TOCSettings*)TOCSetting))
#define Html2PdfCast(html2pdf) (*((HTML2PDF*)html2pdf))
#define ProxyCast(proxy) (*((HTML2PDF::Proxy*)proxy))

TRN_API TRN_HTML2PDF_WebPageSettingsCreate( TRN_HTML2PDF_WebPageSettings* result )
{
	BEX;
	*result=(TRN_HTML2PDF_WebPageSettings)new HTML2PDF::WebPageSettings();
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsDestroy( TRN_HTML2PDF_WebPageSettings settings )
{
	BEX;
	delete((HTML2PDF::WebPageSettings*)settings);
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetPrintBackground(TRN_HTML2PDF_WebPageSettings settings, TRN_Bool background)
{
	BEX;
	WebPageSettingCast(settings).SetPrintBackground(TBToB(background));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetLoadImages( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool load )
{
	BEX;
	WebPageSettingCast(settings).SetLoadImages(TBToB(load));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetAllowJavaScript( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool enable )
{
	BEX;
	WebPageSettingCast(settings).SetAllowJavaScript(TBToB(enable));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetSmartShrinking( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool enable )
{
	BEX;
	WebPageSettingCast(settings).SetSmartShrinking(TBToB(enable));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetMinimumFontSize( TRN_HTML2PDF_WebPageSettings settings, int size )
{
	BEX;
	WebPageSettingCast(settings).SetMinimumFontSize(size);
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetDefaultEncoding( TRN_HTML2PDF_WebPageSettings settings, const TRN_UString encoding )
{
	BEX;
	UString temp(UStrCast(encoding));
	WebPageSettingCast(settings).SetDefaultEncoding(temp);
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetUserStyleSheet( TRN_HTML2PDF_WebPageSettings settings, const TRN_UString url )
{
	BEX;
	UString temp(UStrCast(url));
	WebPageSettingCast(settings).SetUserStyleSheet(temp);
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetAllowPlugins( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool enable )
{
	BEX;
	WebPageSettingCast(settings).SetAllowPlugins(TBToB(enable));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetPrintMediaType( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool print )
{
	BEX;
	WebPageSettingCast(settings).SetPrintMediaType(TBToB(print));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetIncludeInOutline( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool include )
{
	BEX;
	WebPageSettingCast(settings).SetIncludeInOutline(TBToB(include));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetUsername( TRN_HTML2PDF_WebPageSettings settings, const TRN_UString username )
{
	BEX;
	UString temp(UStrCast(username));
	WebPageSettingCast(settings).SetUsername(temp);
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetPassword( TRN_HTML2PDF_WebPageSettings settings, TRN_UString password )
{
	BEX;
	UString temp(UStrCast(password));
	WebPageSettingCast(settings).SetPassword(temp);
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetJavaScriptDelay( TRN_HTML2PDF_WebPageSettings settings, int msec )
{
	BEX;
	WebPageSettingCast(settings).SetJavaScriptDelay(msec);
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetZoom( TRN_HTML2PDF_WebPageSettings settings, double zoom )
{
	BEX;
	WebPageSettingCast(settings).SetZoom(zoom);
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetBlockLocalFileAccess( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool block )
{
	BEX;
	WebPageSettingCast(settings).SetBlockLocalFileAccess(TBToB(block));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetStopSlowScripts( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool stop )
{
	BEX;
	WebPageSettingCast(settings).SetStopSlowScripts(TBToB(stop));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetDebugJavaScriptOutput( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool forward )
{
	BEX;
	WebPageSettingCast(settings).SetDebugJavaScriptOutput(TBToB(forward));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetLoadErrorHandling( TRN_HTML2PDF_WebPageSettings settings, enum TRN_HTML2PDF_WebPageSettings_ErrorHandling val )
{
	BEX;
	WebPageSettingCast(settings).SetLoadErrorHandling((HTML2PDF::WebPageSettings::ErrorHandling)val);
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetExternalLinks( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool convert )
{
	BEX;
	WebPageSettingCast(settings).SetExternalLinks(TBToB(convert));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetInternalLinks( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool convert )
{
	BEX;
	WebPageSettingCast(settings).SetInternalLinks(TBToB(convert));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetProduceForms( TRN_HTML2PDF_WebPageSettings settings, TRN_Bool forms )
{
	BEX;
	WebPageSettingCast(settings).SetProduceForms(TBToB(forms));
	EEX;
}

TRN_API TRN_HTML2PDF_WebPageSettingsSetProxy( TRN_HTML2PDF_WebPageSettings settings, const TRN_HTML2PDF_Proxy proxy )
{
	BEX;
	HTML2PDF::Proxy* temp=(HTML2PDF::Proxy*)proxy;
	WebPageSettingCast(settings).SetProxy(*temp);
	EEX;
}

TRN_API TRN_HTML2PDFStaticConvert( TRN_PDFDoc doc, const TRN_UString url, TRN_Bool* result )
{
	BEX;
	PDFDoc* d=(PDFDoc*)&doc;
	UString* u=(UString*)&url;
	*result=BToTB(HTML2PDF::Convert(*d, *u));
	EEX;
}

TRN_API TRN_HTML2PDFStaticConvert2( TRN_PDFDoc doc, const TRN_UString url, TRN_HTML2PDF_WebPageSettings settings, TRN_Bool* result )
{
	BEX;
	PDFDoc* d=(PDFDoc*)&doc;
	UString* u=(UString*)&url;
	HTML2PDF::WebPageSettings* s=(HTML2PDF::WebPageSettings*)settings;
	*result=BToTB(HTML2PDF::Convert(*d, *u, *s));
	EEX;
}

TRN_API TRN_HTML2PDFInsertFromUrl( TRN_HTML2PDF html2pdf, const TRN_UString url )
{
	BEX;
	UString* u=(UString*)&url;
	Html2PdfCast(html2pdf).InsertFromURL(*u);
	EEX;
}

TRN_API TRN_HTML2PDFInsertFromUrl2( TRN_HTML2PDF html2pdf, const TRN_UString url, TRN_HTML2PDF_WebPageSettings settings )
{
	BEX;
	UString* u=(UString*)&url;
	HTML2PDF::WebPageSettings* s=(HTML2PDF::WebPageSettings*)settings;
	Html2PdfCast(html2pdf).InsertFromURL(*u, *s);
	EEX;
}

TRN_API TRN_HTML2PDFInsertFromHtmlString( TRN_HTML2PDF html2pdf, const TRN_UString html )
{
	BEX;
	UString* h=(UString*)&html;
	Html2PdfCast(html2pdf).InsertFromHtmlString(*h);
	EEX;
}

TRN_API TRN_HTML2PDFInsertFromHtmlString2( TRN_HTML2PDF html2pdf, const TRN_UString html, TRN_HTML2PDF_WebPageSettings settings )
{
	BEX;
	UString* h=(UString*)&html;
	HTML2PDF::WebPageSettings* s=(HTML2PDF::WebPageSettings*)settings;
	Html2PdfCast(html2pdf).InsertFromHtmlString(*h, *s);
	EEX;
}

TRN_API TRN_HTML2PDFInsertTOC( TRN_HTML2PDF html2pdf )
{
	BEX;
	Html2PdfCast(html2pdf).InsertTOC();
	EEX;
}

TRN_API TRN_HTML2PDFInsertTOC2( TRN_HTML2PDF html2pdf, const TRN_HTML2PDF_TOCSettings settings )
{
	BEX;
	HTML2PDF::TOCSettings* s=(HTML2PDF::TOCSettings*)settings;
	Html2PdfCast(html2pdf).InsertTOC(*s);
	EEX;
}

TRN_API TRN_HTML2PDFConvert( TRN_HTML2PDF html2pdf, TRN_PDFDoc doc, TRN_Bool* result )
{
	BEX;
	PDFDoc* d=(PDFDoc*)&doc;
	*result=BToTB(Html2PdfCast(html2pdf).Convert(*d));
	EEX;
}

TRN_API TRN_HTML2PDFGetHttpErrorCode( TRN_HTML2PDF html2pdf, int* result )
{
	BEX;
	*result=Html2PdfCast(html2pdf).GetHTTPErrorCode();
	EEX;
}

TRN_API TRN_HTML2PDFSetPaperSize(TRN_HTML2PDF html2pdf, enum TRN_PrinterModePaperSize size)
{
	BEX;
	Html2PdfCast(html2pdf).SetPaperSize((Page::PaperSize)size);
	EEX;
}

TRN_API TRN_HTML2PDFSetPaperSize2( TRN_HTML2PDF html2pdf, const TRN_UString width, const TRN_UString height )
{
	BEX;
	UString* w=(UString*)&width;
	UString* h=(UString*)&height;
	Html2PdfCast(html2pdf).SetPaperSize(*w, *h);
	EEX;
}

TRN_API TRN_HTML2PDFSetLandscape( TRN_HTML2PDF html2pdf, TRN_Bool enable )
{
	BEX;
	Html2PdfCast(html2pdf).SetLandscape(TBToB(enable));
	EEX;
}

TRN_API TRN_HTML2PDFSetDPI( TRN_HTML2PDF html2pdf, int dpi )
{
	BEX;
	Html2PdfCast(html2pdf).SetDPI(dpi);
	EEX;
}

TRN_API TRN_HTML2PDFSetOutline( TRN_HTML2PDF html2pdf, TRN_Bool enable, int depth /*= 4*/ )
{
	BEX;
	Html2PdfCast(html2pdf).SetOutline(TBToB(enable), depth);
	EEX;
}

TRN_API TRN_HTML2PDFDumpOutline( TRN_HTML2PDF html2pdf, const TRN_UString xml_file )
{
	BEX;
	UString* temp=(UString*)&xml_file;
	Html2PdfCast(html2pdf).DumpOutline(*temp);
	EEX;
}

TRN_API TRN_HTML2PDFSetPDFCompression( TRN_HTML2PDF html2pdf, TRN_Bool enable )
{
	BEX;
	Html2PdfCast(html2pdf).SetPDFCompression(TBToB(enable));
	EEX;
}

TRN_API TRN_HTML2PDFSetMargins( TRN_HTML2PDF html2pdf, const TRN_UString top, const TRN_UString bottom, const TRN_UString left, const TRN_UString right )
{
	BEX;
	UString* t=(UString*)&top;
	UString* b=(UString*)&bottom;
	UString* l=(UString*)&left;
	UString* r=(UString*)&right;
	Html2PdfCast(html2pdf).SetMargins(*t, *b, *l, *r);
	EEX;
}

TRN_API TRN_HTML2PDFSetImageDPI( TRN_HTML2PDF html2pdf, int dpi )
{
	BEX;
	Html2PdfCast(html2pdf).SetImageDPI(dpi);
	EEX;
}

TRN_API TRN_HTML2PDFSetImageQuality( TRN_HTML2PDF html2pdf, int quality )
{
	BEX;
	Html2PdfCast(html2pdf).SetImageQuality(quality);
	EEX;
}

TRN_API TRN_HTML2PDFSetCookieJar( TRN_HTML2PDF html2pdf, const TRN_UString path )
{
	BEX;
	UString* temp=(UString*)&path;
	Html2PdfCast(html2pdf).SetCookieJar(*temp);
	EEX;
}

TRN_API TRN_HTML2PDFSetModulePath( const TRN_UString path )
{
	BEX;
	UString* temp=(UString*)&path;
	HTML2PDF::SetModulePath(*temp);
	EEX;
}

TRN_API TRN_HTML2PDFSetQuiet( TRN_HTML2PDF html2pdf, TRN_Bool quiet )
{
	BEX;
	Html2PdfCast(html2pdf).SetQuiet(TBToB(quiet));
	EEX;
}

TRN_API TRN_HTML2PDFGetLog( TRN_HTML2PDF html2pdf, TRN_UString* result )
{
	BEX;
	RetUStr(Html2PdfCast(html2pdf).GetLog());
	EEX;
}

TRN_API TRN_HTML2PDFCreate( TRN_HTML2PDF* result )
{
	BEX;
	*result = (TRN_HTML2PDF)new HTML2PDF();
	EEX;
}

TRN_API TRN_HTML2PDFDestroy( TRN_HTML2PDF html2pdf )
{
	BEX;
	delete((HTML2PDF*)html2pdf);
	EEX;
}

TRN_API TRN_HTML2PDF_ProxyCreate( TRN_HTML2PDF_Proxy* result )
{
	BEX;
	*result=(TRN_HTML2PDF_Proxy)new HTML2PDF::Proxy();
	EEX;
}

TRN_API TRN_HTML2PDF_ProxyDestroy( TRN_HTML2PDF_Proxy proxy )
{
	BEX;
	delete((HTML2PDF::Proxy*)proxy);
	EEX;
}

TRN_API TRN_HTML2PDF_ProxySetType( TRN_HTML2PDF_Proxy proxy, enum TRN_HTML2PDF_Proxy_Type type )
{
	BEX;
	ProxyCast(proxy).SetType((HTML2PDF::Proxy::Type)type);
	EEX;
}

TRN_API TRN_HTML2PDF_ProxySetPort( TRN_HTML2PDF_Proxy proxy, int port )
{
	BEX;
	ProxyCast(proxy).SetPort(port);
	EEX;
}

TRN_API TRN_HTML2PDF_ProxySetHost( TRN_HTML2PDF_Proxy proxy, const TRN_UString host )
{
	BEX;
	UString* temp=(UString*)&host;
	ProxyCast(proxy).SetHost(*temp);
	EEX;
}

TRN_API TRN_HTML2PDF_ProxySetUsername( TRN_HTML2PDF_Proxy proxy, const TRN_UString username )
{
	BEX;
	UString* temp=(UString*)&username;
	ProxyCast(proxy).SetUsername(*temp);
	EEX;
}

TRN_API TRN_HTML2PDF_ProxySetPassword( TRN_HTML2PDF_Proxy proxy, const TRN_UString password )
{
	BEX;
	UString* temp=(UString*)&password;
	ProxyCast(proxy).SetPassword(*temp);
	EEX;
}

TRN_API TRN_HTML2PDF_TOCSettingsCreate( TRN_HTML2PDF_TOCSettings* result )
{
	BEX;
	*result=(TRN_HTML2PDF_TOCSettings)new HTML2PDF::TOCSettings();
	EEX;
}

TRN_API TRN_HTML2PDF_TOCSettingsDestroy( TRN_HTML2PDF_TOCSettings settings )
{
	BEX;
	delete((HTML2PDF::TOCSettings*)settings);
	EEX;
}

TRN_API TRN_HTML2PDF_TOCSettingsSetDottedLines( TRN_HTML2PDF_TOCSettings settings, TRN_Bool enable )
{
	BEX;
	TOCSettingCast(settings).SetDottedLines(TBToB(enable));
	EEX;
}

TRN_API TRN_HTML2PDF_TOCSettingsSetLinks( TRN_HTML2PDF_TOCSettings settings, TRN_Bool enable )
{
	BEX;
	TOCSettingCast(settings).SetLinks(TBToB(enable));
	EEX;
}

TRN_API TRN_HTML2PDF_TOCSettingsSetCaptionText( TRN_HTML2PDF_TOCSettings settings, TRN_UString caption )
{
	BEX;
	UString* temp=(UString*)&caption;
	TOCSettingCast(settings).SetCaptionText(*temp);
	EEX;
}

TRN_API TRN_HTML2PDF_TOCSettingsSetLevelIndentation( TRN_HTML2PDF_TOCSettings settings, int indentation )
{
	BEX;
	TOCSettingCast(settings).SetLevelIndentation(indentation);
	EEX;
}

TRN_API TRN_HTML2PDF_TOCSettingsSetTextSizeShrink( TRN_HTML2PDF_TOCSettings settings, double shrink )
{
	BEX;
	TOCSettingCast(settings).SetTextSizeShrink(shrink);
	EEX;
}

TRN_API TRN_HTML2PDF_TOCSettingsSetXsl( TRN_HTML2PDF_TOCSettings settings, const TRN_UString style_sheet )
{
	BEX;
	UString* temp=(UString*)&style_sheet;
	TOCSettingCast(settings).SetXsl(*temp);
	EEX;
}
