#include <C/PDF/TRN_PDFDraw.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <Filters/FilterWriter.h>
#include <PDF/PDFDraw.h>
#include <SDF/License.hpp>
#include <Common/UStringImpl.hpp>
#include <C/Common/TRN_PolymorphicVector.hpp>
#include <C/Common/TRN_Vector.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_PDFDrawCreate(double dpi, TRN_PDFDraw* result)
{
	BEX
	License::Instance().VerifyPermission(License::e_perm_rast);

	*result=(TRN_PDFDraw)new PDFDraw(false, dpi);
	EEX
}

TRN_API TRN_PDFDrawDestroy(TRN_PDFDraw d)
{
	BEX
	delete (PDFDraw*) d;
	EEX
}

TRN_API TRN_PDFDrawSetRasterizerType (TRN_PDFDraw d, enum TRN_PDFRasterizerType type)
{
	BEX
	DrawCast(d).SetRasterizerType((PDFRasterizer::Type)type);
	EEX
}

TRN_API TRN_PDFDrawSetDPI(TRN_PDFDraw d, double dpi)
{
	BEX
	DrawCast(d).SetDPI(dpi);
	EEX
}

TRN_API TRN_PDFDrawSetImageSize(TRN_PDFDraw d, int width, int height, TRN_Bool preserve_aspect_ratio)
{
	BEX
	DrawCast(d).SetImageSize(width,height,TBToB(preserve_aspect_ratio));
	EEX
}

TRN_API TRN_PDFDrawSetPageBox(TRN_PDFDraw d, enum TRN_PageBox region)
{
	BEX
	DrawCast(d).SetPageBox((Page::Box)region);
	EEX
}

TRN_API TRN_PDFDrawSetClipRect(TRN_PDFDraw d, const TRN_Rect* rect)
{
	BEX
	DrawCast(d).SetClipRect(RectCast(rect));
	EEX
}

TRN_API TRN_PDFDrawSetFlipYAxis(TRN_PDFDraw d, TRN_Bool flip_y)
{
	BEX
	DrawCast(d).SetFlipYAxis(TBToB(flip_y));
	EEX
}

TRN_API TRN_PDFDrawSetRotate(TRN_PDFDraw d, enum TRN_PageRotate r)
{
	BEX
	DrawCast(d).SetRotate((Page::Rotate)r);
	EEX
}

TRN_API TRN_PDFDrawSetDrawAnnotations(TRN_PDFDraw d, TRN_Bool render_annots)
{
	BEX
	DrawCast(d).SetDrawAnnotations(TBToB(render_annots));
	EEX
}

TRN_API TRN_PDFDrawSetHighlightFields(TRN_PDFDraw d, TRN_Bool highlight)
{
	BEX
		DrawCast(d).SetHighlightFields(TBToB(highlight));
	EEX
}

TRN_API TRN_PDFDrawSetGamma(TRN_PDFDraw d, double gamma)
{
	BEX
	DrawCast(d).SetGamma(gamma);
	EEX
}

TRN_API TRN_PDFDrawSetOCGContext(TRN_PDFDraw d, TRN_OCGContext ctx)
{
	BEX
	DrawCast(d).SetOCGContext((OCG::Context*)ctx);
	EEX
}

TRN_API TRN_PDFDrawSetPrintMode(TRN_PDFDraw d, TRN_Bool is_printing)
{
	BEX
	DrawCast(d).SetPrintMode(TBToB(is_printing));
	EEX
}

TRN_API TRN_PDFDrawSetPageTransparent(TRN_PDFDraw d, TRN_Bool is_transp)
{
	BEX
	DrawCast(d).SetPageTransparent(TBToB(is_transp));
	EEX
}

TRN_API TRN_PDFDrawSetDefaultPageColor(TRN_PDFDraw d, TRN_UInt8 r, TRN_UInt8 g, TRN_UInt8 b)
{
	BEX
	DrawCast(d).SetDefaultPageColor(r, g, b);
	EEX
}

TRN_API TRN_PDFDrawSetOverprint(TRN_PDFDraw d, TRN_PDFRasterizerOverprintPreviewMode op)
{
	BEX
	DrawCast(d).SetOverprint(PDFRasterizer::OverprintPreviewMode(op));
	EEX
}

TRN_API TRN_PDFDrawSetAntiAliasing(TRN_PDFDraw d, TRN_Bool enable_aa)
{
	BEX
	DrawCast(d).SetAntiAliasing(TBToB(enable_aa));
	EEX
}

TRN_API TRN_PDFDrawSetPathHinting(TRN_PDFDraw d, TRN_Bool enable_hinting)
{
	BEX
		DrawCast(d).SetPathHinting(TBToB(enable_hinting));
	EEX
}

TRN_API TRN_PDFDrawSetThinLineAdjustment(TRN_PDFDraw d, TRN_Bool grid_fit, TRN_Bool stroke_adjust)
{
	BEX
	DrawCast(d).SetThinLineAdjustment(TBToB(grid_fit), TBToB(stroke_adjust));
	EEX
}

TRN_API TRN_PDFDrawSetImageSmoothing(TRN_PDFDraw d, TRN_Bool smoothing_enabled, TRN_Bool hq_image_resampling)
{
	BEX
	DrawCast(d).SetImageSmoothing(TBToB(smoothing_enabled), TBToB(hq_image_resampling) );
	EEX
}

TRN_API TRN_PDFDrawSetCaching(TRN_PDFDraw d, TRN_Bool enabled)
{
	BEX
	DrawCast(d).SetCaching(TBToB(enabled));
	EEX
}

TRN_API TRN_PDFDrawExport(TRN_PDFDraw d, TRN_Page page, const TRN_UString filename, const char* format, TRN_Obj encoder_params)
{
	BEX
	UString temp(UStrCast(filename));
	DrawCast(d).Export(Page(PageCast(page)),temp,format,ObjCast(encoder_params));
	EEX
}

TRN_API TRN_PDFDrawExportStream(TRN_PDFDraw d, TRN_Page page, TRN_Filter stream, const char* format, TRN_Obj encoder_params)
{
	BEX
	Filters::Filter* flt = (Filters::Filter*)stream;
	BASE_ASSERT(flt, "");
	Filters::FilterWriter wri(*flt);
	DrawCast(d).Export(Page(PageCast(page)),wri, format, ObjCast(encoder_params));
	EEX
}

#if defined(WIN32) && !defined(__WINRT__)
TRN_API TRN_PDFDrawDrawInRect(TRN_PDFDraw d, TRN_Page page, void* hdc, const TRN_Rect* rect)
{
	BEX
	Page temp(PageCast(page));
	DrawCast(d).DrawInRect(temp,hdc,RectCast(rect));
	EEX
}
#endif

TRN_API TRN_PDFDrawGetBitmap(TRN_PDFDraw d, TRN_Page page, 
		int* out_width, int* out_height, int* out_stride, double* out_dpi, 
		enum TRN_PDFDrawPixelFormat pix_fmt, TRN_Bool demult, const TRN_UChar** result)
{
	BEX
	*result=DrawCast(d).GetBitmap(Page(PageCast(page)),*out_width,
		*out_height,*out_stride,*out_dpi,(PDFDraw::PixelFormat)pix_fmt, TBToB(demult));
	EEX
}

TRN_API TRN_PDFDrawSetErrorReportProc(TRN_PDFDraw d, 
		TRN_RasterizerErrorReportProc error_proc, void* data)
{
	BEX
	DrawCast(d).SetErrorReportProc((PDFRasterizer::ErrorReportProc)error_proc,data);
	EEX
}

TRN_API TRN_PDFDrawSetColorPostProcessMode( TRN_PDFDraw d, enum TRN_PDFRasterizerColorPostProcessMode mode )
{
	BEX
	DrawCast(d).SetColorPostProcessing((PDFRasterizer::ColorPostProcessMode)mode);
	EEX
}

TRN_API TRN_PDFDrawGetSeparationBitmaps(TRN_PDFDraw d, TRN_Page page, TRN_Vector* result)
{
	BEX
	FlexVector<tOwnedBuffer> output_buffers;
	FlexVector<OutputSeparation> separations = DrawCast(d).GetSeparationBitmaps(Page(PageCast(page)), output_buffers);

	PolyVectorSeparation* final_separations = new PolyVectorSeparation();
	*result = (TRN_Vector)final_separations;
	final_separations->m_vector.resize(separations.size());
	try
	{

		for (UInt32 i = 0; i < separations.size(); ++i)
		{
			OutputSeparation& current = separations[i];

			TRN_Separation new_separation;
			UInt32 separation_buffer_size = current.m_bitmap.W()*current.m_bitmap.H();

			new_separation.data = new UChar[separation_buffer_size];
			AutoPtr<Filters::Filter> channel_filter(current.GetFilterForChannel());
			Filters::FilterReader reader(*channel_filter);
			reader.Read(new_separation.data, separation_buffer_size);

			new_separation.data_size = separation_buffer_size;

			new_separation.c = current.m_separation_color.c();
			new_separation.m = current.m_separation_color.m();
			new_separation.y = current.m_separation_color.y();
			new_separation.k = current.m_separation_color.k();
			new_separation.separation_name = (TRN_UString)IMPL_TRN_UStringCreateFromCharString(current.m_separation_name.c_str(), signed32(current.m_separation_name.size()), UString::e_utf8);
			final_separations->m_vector[i] = new_separation;
		}
	}
	catch (...)
	{
		TRN_VectorDestroy(*result);
		throw;
	}
	EEX;
}
