#include <C/PDF/TRN_PDFNet.h>
#include <C/Common/TRN_Types.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <string>

#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>
#include <PDF/PDFNet.h>
#include <PDF/PDFDoc.h>
#include <SDF/SecurityManager.h>

#ifdef TRN_ENABLE_CRASHDUMPS
    #include <Common/CrashReporter/CrashReporterBase.hpp>
#endif // TRN_ENABLE_CRASHDUMPS

using namespace std;
using namespace PDF;

extern map<string,Common::Exception> exception_map;

// Defined in TRN_SecurityHandler.cpp
SecurityHandler* DelegateStdSecurityHandlerCreate(const char* name, int key_len, int enc_code);

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_PDFNetInitialize(const char* license_key)
{
	BEX;
    PDFNet::Initialize(license_key);

    // TODO: Expose to the wrappers? (at least for Xodo Desktop...)
#ifdef TRN_ENABLE_CRASHDUMPS
    PDFNet::StartCrashReporter(PDFNet::GetPersistentCachePath());
#endif // TRN_ENABLE_CRASHDUMPS
	EEX;
}

TRN_API TRN_PDFNetEnableJavaScript(TRN_Bool enable)
{
	BEX;
	PDFNet::EnableJavaScript(TBToB(enable));
	EEX;
}

TRN_API TRN_PDFNetIsJavaScriptEnabled(TRN_Bool* result)
{
	BEX;
	*result = BToTB(PDFNet::IsJavaScriptEnabled());
	EEX;
}

TRN_API TRN_PDFNetTerminate()
{
	BEX;
	PDFNet::Terminate();
	exception_map.clear();
	EEX;
}

TRN_API TRN_PDFNetSetResourcesPath(TRN_UString path, TRN_Bool* result)
{
	BEX;
	UString temp(UStrCast(path));
	*result=BToTB(PDFNet::SetResourcesPath(temp));
	EEX;
}

TRN_API TRN_PDFNetGetResourcesPath(TRN_UString* result)
{
	BEX;
	RetUStr(PDFNet::GetResourcesPath());
	EEX;
}

TRN_API TRN_PDFNetAddResourceSearchPath(TRN_UString path)
{
	BEX;
	UString temp(UStrCast(path));
	PDFNet::AddResourceSearchPath(temp);
	EEX;
}

TRN_API TRN_PDFNetSetResourceData(TRN_UChar* buf, size_t buf_size)
{
	BEX;
	PDFNet::SetResourceData(buf,buf_size);
	EEX;
}

TRN_API TRN_PDFNetSetColorManagement(enum TRN_PDFNetCMSType t)
{
	BEX;
	PDFNet::SetColorManagement((PDFNet::CMSType)t);
	EEX;
}

TRN_API TRN_PDFNetSetDefaultDeviceCMYKProfile(const TRN_UString icc_filename)
{
	BEX;
	UString temp(UStrCast(icc_filename));
	PDFNet::SetDefaultDeviceCMYKProfile(temp);
	EEX;
}

TRN_API TRN_PDFNetSetDefaultDeviceRGBProfile(const TRN_UString icc_filename)
{
	BEX;
	UString temp(UStrCast(icc_filename));
	PDFNet::SetDefaultDeviceRGBProfile(temp);
	EEX;
}

TRN_API TRN_PDFNetSetDefaultDiskCachingEnabled( TRN_Bool use_disk )
{
	BEX;
	PDFNet::SetDefaultDiskCachingEnabled( use_disk != 0 );
	EEX;
}

TRN_API TRN_PDFNetSetViewerCache(TRN_Size max_cache_size, TRN_Bool on_disk)
{
	BEX;
	PDFNet::SetViewerCache(max_cache_size, on_disk != 0);
	EEX;
}

TRN_API TRN_PDFNetAddFontSubstFromName(const char* fontname, const TRN_UString fontpath, TRN_Bool* result)
{
	BEX;
	UString temp(UStrCast(fontpath));
	*result=BToTB(PDFNet::AddFontSubst(fontname,temp));
	EEX;
}

TRN_API TRN_PDFNetAddFontSubst(enum TRN_PDFNetCharacterOrdering ordering, const TRN_UString fontpath, TRN_Bool* result)
{
	BEX;
	UString temp(UStrCast(fontpath));
	*result=BToTB(PDFNet::AddFontSubst((PDFNet::CharacterOrdering)ordering,temp));
	EEX;
}

TRN_API TRN_PDFNetSetTempPath(const TRN_UString temp_path)
{
	BEX;
	UString temp(UStrCast(temp_path));
	PDFNet::SetTempPath(temp);
	EEX;
}

TRN_API TRN_PDFNetSetPersistentCachePath(const TRN_UString persistent_path)
{
	BEX;
	UString temp(UStrCast(persistent_path));
	PDFNet::SetPersistentCachePath(temp);
	EEX;
}

TRN_API TRN_PDFNetGetVersion(double* result)
{
	BEX;
	*result=PDFNet::GetVersion();
	EEX;
}

TRN_API TRN_PDFNetRegisterSecurityHandler(const char* handler_name, const char* gui_name, TRN_PDFNetCreateSecurityHandler factory_method)
{
	BEX;

	SDF::SecurityManager& sec_mgr = SDF::SecurityManagerSingleton::Instance();
	sec_mgr.RegisterSecurityHandler(handler_name, 
		SDF::SecurityDescriptor(gui_name, (SDF::CreateSecurityHandler) factory_method));

	EEX;
}

TRN_API TRN_PDFNetGetSecHdlrInfoIterator(TRN_Iterator* result)
{
	BEX;
	SDF::SecurityManager& sm = SDF::SecurityManagerSingleton::Instance();
	*result=(TRN_Iterator)new PolySecHdlrInfoIter(sm.Begin(), sm.End());
	EEX;
}

TRN_API TRN_PDFNetSetNumberWriteProc(char* (*write_proc) (double num, char *in_buf, int in_buf_size))
{
	BEX;
	PDFNet::SetNumberWriteProc(write_proc);
	EEX;
}

TRN_API TRN_PDFNetSetNumberReadProc(TRN_Bool (*read_proc) (const TRN_UChar *buf, double *output))
{
	BEX;
	PDFNet::SetNumberReadProc(read_proc);
	EEX;
}

TRN_API TRN_PDFNetSetDefaultFlateCompressionLevel(int level)
{
	BEX;
	trn::PDFNet::SetDefaultFlateCompressionLevel(level);
	EEX;
}

TRN_API TRN_PDFNetSetLogLevel(enum TRN_PDFNetLogLevel level)
{
	BEX;
	trn::PDFNet::SetLogLevel(static_cast<trn::PDFNet::LogLevel>(level));
	EEX;
}

TRN_API TRN_PDFNetGetSystemFontList(TRN_UString* result)
{
	BEX;
	std::string str = trn::PDFNet::GetSystemFontListJSON();
	UString temp(str.c_str(), str.size(), UString::e_utf8);
	RetUStr(temp);
	EEX;
}
