#include <C/PDF/TRN_PageSet.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <PDF/PageSet.h>
#include <SDF/License.hpp>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
	
using namespace PDF;

//CONSTRUCTORS
TRN_API TRN_PageSetCreate(TRN_PageSet* result)
{
	BEX;
	*result = (TRN_PageSet)new PageSet();
	EEX;
}

TRN_API TRN_PageSetCreateSinglePage(int one_page, TRN_PageSet* result)
{
	BEX;
	*result = (TRN_PageSet)new PageSet(one_page);
	EEX;
}

TRN_API TRN_PageSetCreateRange(int range_start, int range_end, TRN_PageSet* result)
{
	BEX;
	*result = (TRN_PageSet)new PageSet(range_start, range_end);
	EEX;
}

TRN_API TRN_PageSetCreateFilteredRange(int range_start, int range_end, TRN_PageSetFilter filter, TRN_PageSet* result)
{
	BEX;
	*result = (TRN_PageSet)new PageSet(range_start, range_end, (PageSet::Filter)filter);
	EEX;
}

//DESTRUCTOR
TRN_API TRN_PageSetDestroy(TRN_PageSet page_set)
{
	BEX;
	delete((PageSet*)page_set);
	EEX;
}

//ADD AND REMOVE PAGES METHODS
TRN_API TRN_PageSetAddPage(TRN_PageSet page_set, int one_page)
{
	BEX;
	((PageSet*)page_set)->AddPage(one_page);
	EEX;
}

TRN_API TRN_PageSetAddRange(TRN_PageSet page_set, int range_start, int range_end, TRN_PageSetFilter filter)
{
	BEX;
	((PageSet*)page_set)->AddRange(range_start, range_end, (PageSet::Filter)filter);
	EEX;
}
