#include <C/PDF/TRN_Shading.h>
#include <C/PDF/TRN_ColorSpace.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <cstring>
#include <C/Common/TRN_Exception.hpp>
#include <PDF/ColorSpace.h>
#include <PDF/Shading.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace PDF;

TRN_API TRN_ShadingCreate(TRN_Obj shading_dict, TRN_Shading* result)
{
	BEX;
	*result=(TRN_Shading)new Shading((Obj*)shading_dict);
	EEX;
}

TRN_API TRN_ShadingAssign(TRN_Shading left, TRN_Shading right)
{
	BEX;
	ShadeCast(left)=ShadeCast(right);
	EEX;
}

TRN_API TRN_ShadingDestroy(TRN_Shading s)
{
	BEX;
	delete (Shading*)s;
	EEX;
}

TRN_API TRN_ShadingGetTypeFromObj(TRN_Obj shading_dict, enum TRN_ShadingType* result)
{
	BEX;
	*result=(enum TRN_ShadingType)Shading::GetType((Obj*)shading_dict);
	EEX;
}

TRN_API TRN_ShadingGetType(const TRN_Shading s,enum TRN_ShadingType* result)
{
	BEX;
	*result=(enum TRN_ShadingType)ShadeCast(s).GetType();
	EEX;
}

TRN_API TRN_ShadingGetSDFObj (TRN_Shading s, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ShadeCast(s).GetSDFObj();
	EEX;
}

TRN_API TRN_ShadingGetBaseColorSpace(TRN_Shading s, TRN_ColorSpace* result)
{
	BEX;
	*result=(TRN_ColorSpace)new ColorSpace(ShadeCast(s).GetBaseColorSpace());
	EEX;
}

TRN_API TRN_ShadingHasBBox(TRN_Shading s, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ShadeCast(s).HasBBox());
	EEX;
}

TRN_API TRN_ShadingGetBBox(TRN_Shading s, TRN_Rect* result)
{
	BEX;
	Rect r(ShadeCast(s).GetBBox());
	assert(sizeof(TRN_Rect)>=sizeof(r));
	memcpy(result, &r, sizeof(r));
	EEX;
}

TRN_API TRN_ShadingHasBackground(TRN_Shading s, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ShadeCast(s).HasBackground());
	EEX;
}

TRN_API TRN_ShadingGetBackground(TRN_Shading s, TRN_ColorPt* result)
{
	BEX;
	ColorPt c(ShadeCast(s).GetBackground());
	assert(sizeof(TRN_ColorPt)>=sizeof(c));
	TRN_ColorPtAssign(result, (const TRN_ColorPt*)&c);
	EEX;
}

TRN_API TRN_ShadingGetAntialias(TRN_Shading s, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ShadeCast(s).GetAntialias());
	EEX;
}

TRN_API TRN_ShadingGetParamStart(TRN_Shading s, double* result)
{
	BEX;
	*result=ShadeCast(s).GetParamStart();
	EEX;
}

TRN_API TRN_ShadingGetParamEnd(TRN_Shading s, double* result)
{
	BEX;
	*result=ShadeCast(s).GetParamEnd();
	EEX;
}

TRN_API TRN_ShadingIsExtendStart(TRN_Shading s, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ShadeCast(s).IsExtendStart());
	EEX;
}

TRN_API TRN_ShadingIsExtendEnd(TRN_Shading s, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ShadeCast(s).IsExtendEnd());
	EEX;
}

TRN_API TRN_ShadingGetColor(TRN_Shading s, double t, TRN_ColorPt* out_color)
{
	BEX;
	ShadeCast(s).GetColor(t,CPCast(out_color));
	EEX;
}

TRN_API TRN_ShadingGetCoords(TRN_Shading s, double* out_x0, double* out_y0, double* out_x1, double* out_y1)
{
	BEX;
	ShadeCast(s).GetCoords(*out_x0,*out_y0,*out_x1,*out_y1);
	EEX;
}

TRN_API TRN_ShadingGetCoordsRadial(TRN_Shading s, double* out_x0, double* out_y0, double* out_r0,
							 double* out_x1, double* out_y1, double* out_r1)
{
	BEX;
	ShadeCast(s).GetCoords(*out_x0,*out_y0,*out_r0,*out_x1,*out_y1,*out_r1);
	EEX;
}

TRN_API TRN_ShadingGetDomain(TRN_Shading s, double* out_xmin, double* out_xmax, double* out_ymin, double* out_ymax)
{
	BEX;
	ShadeCast(s).GetDomain(*out_xmin,*out_xmax,*out_ymin,*out_ymax);
	EEX;
}

TRN_API TRN_ShadingGetMatrix(TRN_Shading s, TRN_Matrix2D* result)
{
	BEX;
    Common::Matrix2D mtx = ShadeCast(s).GetMatrix();
	*result=RetM2D(mtx);
	EEX;
}

TRN_API TRN_ShadingGetColorForFunction(TRN_Shading s, double t1, double t2, TRN_ColorPt* out_color)
{
	BEX;
	ShadeCast(s).GetColor(t1,t2,CPCast(out_color));
	EEX;
}
