//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#include <C/PDF/TRN_VerificationResult.h>
#include <C/PDF/TRN_DigestAlgorithm.h>
#include <C/PDF/TRN_DisallowedChange.h>
#include <C/Common/TRN_API_Config.hpp>
#include <C/Common/TRN_Exception.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <PDF/iVerificationResult.hpp>
#include <C/Common/TRN_PolymorphicVector.hpp>
#include <C/Common/TRN_Vector.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

TRN_API TRN_VerificationResultGetDigitalSignatureField(TRN_VerificationResult self, TRN_DigitalSignatureField* result)
{
	BEX;
	result->mp_field_dict_obj = (TRN_Obj)((trn::PDF::iVerificationResult*)self )->GetDigitalSignatureField().GetSDFObj();
	EEX;
}


TRN_API TRN_VerificationResultGetVerificationStatus(TRN_VerificationResult self, TRN_Bool* result)
{
	BEX;
	*result = ((trn::PDF::iVerificationResult*)self )->GetVerificationStatus();
	EEX;
}


TRN_API TRN_VerificationResultGetDocumentStatus(TRN_VerificationResult self, enum TRN_VerificationResult_DocumentStatus* result)
{
	BEX;
	*(trn::UInt32*)result = ((trn::PDF::iVerificationResult*)self )->GetDocumentStatus();
	EEX;
}


TRN_API TRN_VerificationResultGetDigestStatus(TRN_VerificationResult self, enum TRN_VerificationResult_DigestStatus* result)
{
	BEX;
	*(trn::UInt32*)result = ((trn::PDF::iVerificationResult*)self )->GetDigestStatus();
	EEX;
}


TRN_API TRN_VerificationResultGetTrustStatus(TRN_VerificationResult self, enum TRN_VerificationResult_TrustStatus* result)
{
	BEX;
	*(trn::UInt32*)result = ((trn::PDF::iVerificationResult*)self )->GetTrustStatus();
	EEX;
}


TRN_API TRN_VerificationResultGetPermissionsStatus(TRN_VerificationResult self, enum TRN_VerificationResult_ModificationPermissionsStatus* result)
{
	BEX;
	*(trn::UInt32*)result = ((trn::PDF::iVerificationResult*)self )->GetPermissionsStatus();
	EEX;
}

TRN_API TRN_VerificationResultGetTrustVerificationResult(TRN_VerificationResult self, TRN_TrustVerificationResult* result)
{
	BEX;
	*result = (TRN_TrustVerificationResult) ((trn::PDF::iVerificationResult*)self)->GetTrustVerificationResult().release();
	EEX;
}

TRN_API TRN_VerificationResultHasTrustVerificationResult(TRN_VerificationResult self, TRN_Bool* result)
{
	BEX;
	*result = ((trn::PDF::iVerificationResult*)self)->HasTrustVerificationResult();
	EEX;
}

TRN_API TRN_VerificationResultGetDisallowedChanges(TRN_VerificationResult self, TRN_Vector* result)
{
	// Broadly similar to TRN_PDFDrawGetSeparationBitmaps and TRN_DigitalSignatureFieldGetLockedFields implementations
	BEX;
	std::vector<RefCountedPtr<PDF::iDisallowedChange> > tmp(((trn::PDF::iVerificationResult*)self)->GetDisallowedChanges());

	PolyVectorBase<TRN_DisallowedChange>* tmp2 = new PolyVectorBase<TRN_DisallowedChange>();
	*result = (TRN_Vector)tmp2;
	try
	{
		for (std::vector<RefCountedPtr<PDF::iDisallowedChange> >::iterator it = tmp.begin(); it != tmp.end(); ++it)
		{
			RefCountedPtr<PDF::iDisallowedChange> tmp3(*it);
			tmp2->m_vector.push_back((TRN_DisallowedChange) tmp3.release());
		}
	}
	catch (...)
	{
		TRN_VectorDestroy(*result);
		throw;
	}
	EEX;
}

TRN_API TRN_VerificationResultGetSignersDigestAlgorithm(TRN_VerificationResult self, enum TRN_DigestAlgorithm_Type* result)
{
	BEX;
	*(trn::UInt32*)result = ((trn::PDF::iVerificationResult*)self )->GetSignersDigestAlgorithm();
	EEX;
}


TRN_API TRN_VerificationResultDestroy(TRN_VerificationResult self)
{
	BEX;
	if(self)
	{
		((trn::PDF::iVerificationResult*)(self))->RemoveReference();
	}
	EEX;
}
