#include <C/SDF/TRN_NameTree.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>
#include <SDF/NameTree.hpp>
#include <SDF/SDFDoc.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;


TRN_API TRN_NameTreeCreate(TRN_SDFDoc doc, const char* name, 
						   TRN_NameTree* result)
{
	BEX;
	*result=(TRN_NameTree)IMPL_TRN_NameTreeCreate((*((SDFDoc*)doc)),name);
	EEX;
}

TRN_API TRN_NameTreeFind(TRN_SDFDoc doc, const char* name, 
			 TRN_NameTree* result)
{
	BEX;
	*result=(TRN_NameTree)IMPL_TRN_NameTreeFind((*((SDFDoc*)doc)),name);
	EEX;
}



TRN_API TRN_NameTreeCreateFromObj (TRN_Obj name_tree, TRN_NameTree* result)
{
	BEX;
	*result=(TRN_NameTree)IMPL_TRN_NameTreeCreateFromObj((Obj*)name_tree);
	EEX;
}

TRN_API TRN_NameTreeCopy (TRN_NameTree const d, TRN_NameTree* result)
{
	BEX;
	*result=(TRN_NameTree)IMPL_TRN_NameTreeCopy((const Obj*)d);
	EEX;
}
	
TRN_API TRN_NameTreeIsValid(TRN_NameTree tree, TRN_Bool* result)
{
	BEX;
	*result=BToTB(IMPL_TRN_NameTreeIsValid((Obj*)tree));
	EEX;
}

TRN_API TRN_NameTreeGetIterator(TRN_NameTree tree, TRN_String key, TRN_DictIterator* result)
{
	BEX;
	NameTreeIterator temp = IMPL_TRN_NameTreeFindKey((Obj*)tree, (const UChar*)key.str, (unsigned int)key.len);
	*result=(TRN_DictIterator)(new PolyNameTreeIterator(temp));
	EEX;
}

TRN_API TRN_NameTreeGetValue(TRN_NameTree tree, TRN_String key, TRN_Obj* result)
{
	BEX;
	NameTreeIterator end, temp=IMPL_TRN_NameTreeFindKey((Obj*)tree,(const UChar*)key.str,key.len);
	if(temp!=end)
	{
		*result=(TRN_Obj)(temp->Value());
	}
	else
		*result=0;
	EEX;
}

TRN_API TRN_NameTreeGetIteratorBegin(TRN_NameTree tree, TRN_DictIterator* result)
{
	BEX;
	NameTreeIterator temp=IMPL_TRN_NameTreeGetIterator((Obj*)tree);
	*result=(TRN_DictIterator)(new PolyNameTreeIterator(temp));
	EEX;
}

TRN_API TRN_NameTreePut (TRN_NameTree tree, TRN_String key, TRN_Obj value)
{
	BEX;
	IMPL_TRN_NameTreePut((Obj*)tree, (const UChar*)key.str, (unsigned int)key.len, (Obj*)value);
	EEX;
}

TRN_API TRN_NameTreeEraseKey (TRN_NameTree tree,TRN_String key)
{
	BEX;
	IMPL_TRN_NameTreeEraseKey((Obj*)tree, (const UChar*)key.str, (int)key.len);
	EEX;
}

TRN_API TRN_NameTreeErase (TRN_NameTree tree, TRN_DictIterator pos)
{
	BEX;
	IMPL_TRN_NameTreeErase((Obj*)tree,((PolyNameTreeIterator*)pos)->itr);
	EEX;
}

TRN_API TRN_NameTreeGetSDFObj (TRN_NameTree const tree,TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)IMPL_TRN_NameTreeGetSDFObj((Obj*)tree);
	EEX;
}
