#include <C/SDF/TRN_Obj.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <C/Common/TRN_PolymorphicIterator.hpp>
#include <C/Common/TRN_PolymorphicVector.hpp>
#include <Common/Matrix2D.h>
#include <SDF/ObjHierarchy.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace Filters;
	

TRN_API TRN_ObjGetType (const TRN_Obj o, enum TRN_ObjType* result)
{
	BEX;
	*result=(enum TRN_ObjType)ObjCast(o)->GetType();
	EEX;
}

TRN_API TRN_ObjGetDoc (const TRN_Obj o, TRN_SDFDoc* result)
{
	BEX;
	*result=(TRN_SDFDoc)ObjCast(o)->GetDoc();
	EEX;
}

TRN_API TRN_ObjWrite(TRN_Obj o, TRN_FilterWriter stream)
{
	BEX;
	ObjCast(o)->Write(FWCast(stream));
	EEX;
}

TRN_API TRN_ObjIsEqual(TRN_Obj o, TRN_Obj to, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsEqual(ObjCast(to)));
	EEX;
}

TRN_API TRN_ObjIsBool (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsBool());
	EEX;
}

TRN_API TRN_ObjGetBool (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->GetBool());
	EEX;
}

TRN_API TRN_ObjSetBool (TRN_Obj o, TRN_Bool b)
{
	BEX;
	ObjCast(o)->SetBool(TBToB(b));
	EEX;
}

TRN_API TRN_ObjIsNumber (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsNumber());
	EEX;
}

TRN_API TRN_ObjGetNumber (const TRN_Obj o, double* result)
{
	BEX;
	*result=ObjCast(o)->GetNumber();
	EEX;
}

TRN_API TRN_ObjSetNumber (TRN_Obj o, double n)
{
	BEX;
	ObjCast(o)->SetNumber(n);
	EEX;
}

TRN_API TRN_ObjIsNull (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsNull());
	EEX;
}

TRN_API TRN_ObjIsString (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsString());
	EEX;
}

TRN_API TRN_ObjGetRawBuffer(const TRN_Obj o, TRN_Vector* result)
{
	BEX;
	*result = (TRN_Vector) ( new PolyVectorByte(ObjCast(o)->GetRawBuffer()) );
	EEX;
}

TRN_API TRN_ObjGetBuffer (const TRN_Obj o, const TRN_UChar** result)
{
	BEX;
	*result=ObjCast(o)->GetBuffer();
	EEX;
}

TRN_API TRN_ObjGetAsPDFText(const TRN_Obj o, TRN_UString* result)
{
	BEX;
	UString temp(*result);
	ObjCast(o)->GetAsPDFText(temp);
	*result=(TRN_UString)temp.mp_impl;
	EEX;
}

TRN_API TRN_ObjSetString(TRN_Obj o, TRN_String value)
{
	BEX;
	ObjCast(o)->SetString((const UChar*)value.str, (unsigned int)value.len);
	EEX;
}

TRN_API TRN_ObjSetUString(TRN_Obj o, const TRN_UString value)
{
	BEX;
	UString temp((void*)value);
	ObjCast(o)->SetString(temp);
	EEX;
}

TRN_API TRN_ObjIsName (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsName());
	EEX;
}

TRN_API TRN_ObjGetName (const TRN_Obj o, const char** result)
{
	BEX;
	*result=ObjCast(o)->GetName();
	EEX;
}

TRN_API TRN_ObjSetName (TRN_Obj o, const char* name)
{
	BEX;
	ObjCast(o)->SetName(name);
	EEX;
}

TRN_API TRN_ObjIsIndirect (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsIndirect());
	EEX;
}

TRN_API TRN_ObjGetObjNum (const TRN_Obj o, TRN_UInt32* result)
{
	BEX;
	*result=ObjCast(o)->GetObjNum();
	EEX;
}

TRN_API TRN_ObjGetGenNum (const TRN_Obj o, TRN_UInt16* result)
{
	BEX;
	*result=ObjCast(o)->GetGenNum();
	EEX;
}

TRN_API TRN_ObjGetOffset (const TRN_Obj o, TRN_Size* result)
{
	BEX;
	*result=ObjCast(o)->GetOffset();
	EEX;
}

TRN_API TRN_ObjIsFree (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsFree());
	EEX;
}

TRN_API TRN_ObjSetMark (TRN_Obj o, TRN_Bool mark)
{
	BEX;
	ObjCast(o)->SetMark(TBToB(mark));
	EEX;
}

TRN_API TRN_ObjIsMarked (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsMarked());
	EEX;
}

TRN_API TRN_ObjIsLoaded (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsLoaded());
	EEX;
}

TRN_API TRN_ObjIsContainer (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsContainer());
	EEX;
}

TRN_API TRN_ObjSize (const TRN_Obj o, TRN_Size* result)
{
	BEX;
	*result=ObjCast(o)->Size();
	EEX;
}

TRN_API TRN_ObjGetDictIterator (TRN_Obj o, TRN_DictIterator* result)
{
	BEX;
	DictIterator itr=ObjCast(o)->DictBegin(), end=ObjCast(o)->DictEnd();
	*result=(TRN_DictIterator)new PolyObjDictIterator(itr,end);
	EEX;
}

TRN_API TRN_ObjIsDict (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsDict());
	EEX;
}

TRN_API TRN_ObjFind (TRN_Obj o, const char* key, TRN_DictIterator* result)
{
	BEX;
	DictIterator itr=ObjCast(o)->Find(key), end=ObjCast(o)->DictEnd();
	*result=(TRN_DictIterator)new PolyObjDictIterator(itr,end);
	EEX;
}

TRN_API TRN_ObjFindObj (TRN_Obj o, const char* key, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->FindObj(key);
	EEX;
}

TRN_API TRN_ObjGet (TRN_Obj o, const char* key, TRN_DictIterator* result)
{
	BEX;
	DictIterator itr=ObjCast(o)->Get(key), end=ObjCast(o)->DictEnd();
	*result=(TRN_DictIterator)new PolyObjDictIterator(itr,end);
	EEX;
}

TRN_API TRN_ObjPutName (TRN_Obj o, const char* key, const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PutName(key, name);
	EEX;
}

TRN_API TRN_ObjPutArray (TRN_Obj o, const char* key, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PutArray(key);
	EEX;
}

TRN_API TRN_ObjPutBool (TRN_Obj o, const char* key, TRN_Bool value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PutBool(key, TBToB(value));
	EEX;
}

TRN_API TRN_ObjPutDict (TRN_Obj o, const char* key, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PutDict(key);
	EEX;
}

TRN_API TRN_ObjPutNumber (TRN_Obj o, const char* key, double value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PutNumber(key,value);
	EEX;
}

TRN_API TRN_ObjPutString (TRN_Obj o, const char* key, const char* value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj) ObjCast(o)->PutString(key, value);
	EEX;
}

TRN_API TRN_ObjPutStringWithSize (TRN_Obj o, const char* key, const char* value, int size, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PutString(key, value, size);
	EEX;
}

TRN_API TRN_ObjPutText(TRN_Obj o, const char* key, const TRN_UString t, TRN_Obj* result)
{
	BEX;
	UString temp(UStrCast(t));
	*result=(TRN_Obj)ObjCast(o)->PutText(key, temp);
	EEX;
}

TRN_API TRN_ObjPutNull (TRN_Obj o, const char* key)
{
	BEX;
	ObjCast(o)->PutNull(key);
	EEX;
}

TRN_API TRN_ObjPut(TRN_Obj o, const char* key, TRN_Obj indirect_obj, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->Put2(key, ObjCast(indirect_obj));
	EEX;
}

TRN_API TRN_ObjPutRect(TRN_Obj o, const char* key, double x1, double y1, double x2, double y2, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PutRect(key, x1, y1, x2, y2);
	EEX;
}

TRN_API TRN_ObjPutMatrix(TRN_Obj o, const char* key, const TRN_Matrix2D* mtx, TRN_Obj* result)
{
	BEX;	
	*result=(TRN_Obj)ObjCast(o)->PutMatrix(key, Common::Matrix2D(mtx->m_a, mtx->m_b, mtx->m_c, mtx->m_d, mtx->m_h, mtx->m_v));
	EEX;
}

TRN_API TRN_ObjEraseFromKey (TRN_Obj o, const char* key)
{
	BEX;
	ObjCast(o)->Erase(key);
	EEX;
}

TRN_API TRN_ObjErase (TRN_Obj o, TRN_DictIterator pos)
{
	BEX;
	PolyObjDictIterator* t=dynamic_cast<PolyObjDictIterator*>((DictPolyIterator*)pos);
	BASE_ASSERT(t!=0,"Iterator Type is Incorrect");
	ObjCast(o)->Erase(t->m_itr);
	EEX;
}

TRN_API TRN_ObjRename (TRN_Obj o, const char* old_key, const char* new_key, TRN_Bool* result)
{
	BEX;
	std::string t1(old_key),t2(new_key);
	*result=BToTB(ObjCast(o)->Rename(t1,t2)!=ObjCast(o)->DictEnd());
	EEX;
}

TRN_API TRN_ObjIsArray (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsArray());
	EEX;
}

TRN_API TRN_ObjGetAt (const TRN_Obj o, TRN_Size index, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->GetAt(index);
	EEX;
}

TRN_API TRN_ObjInsertName (TRN_Obj o, TRN_Size pos, const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->InsertName(pos,name);
	EEX;
}

TRN_API TRN_ObjInsertArray (TRN_Obj o, TRN_Size pos, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->InsertArray(pos);
	EEX;
}

TRN_API TRN_ObjInsertBool (TRN_Obj o, TRN_Size pos, TRN_Bool value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->InsertBool(pos,TBToB(value));
	EEX;
}

TRN_API TRN_ObjInsertDict (TRN_Obj o, TRN_Size pos, TRN_Obj* result)
{
	BEX;	
	*result=(TRN_Obj)ObjCast(o)->InsertDict(pos);
	EEX;
}

TRN_API TRN_ObjInsertNumber (TRN_Obj o, TRN_Size pos, double value, TRN_Obj* result)
{
	BEX;	
	*result=(TRN_Obj)ObjCast(o)->InsertNumber(pos,value);
	EEX;
}

TRN_API TRN_ObjInsertString (TRN_Obj o, TRN_Size pos, const char* value, TRN_Obj* result)
{
	BEX;	
	*result=(TRN_Obj)ObjCast(o)->InsertString(pos,value);
	EEX;
}

TRN_API TRN_ObjInsertStringWithSize (TRN_Obj o, TRN_Size pos, const char* value, int size, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->InsertString(pos,value, size);
	EEX;
}

TRN_API TRN_ObjInsertText (TRN_Obj o, TRN_Size pos, const TRN_UString t, TRN_Obj* result)
{
	BEX;
	UString temp(UStrCast(t));
	*result=(TRN_Obj)ObjCast(o)->InsertText(pos,temp);
	EEX;
}

TRN_API TRN_ObjInsertNull(TRN_Obj o, TRN_Size pos, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->InsertNull(pos);
	EEX;
}

TRN_API TRN_ObjInsert(TRN_Obj o, TRN_Size pos, TRN_Obj indirect_obj, TRN_Obj* result)
{
	BEX;
	Obj* obj=((Obj*)indirect_obj)->Clone();
	ObjCast(o)->Insert(pos,obj);
	*result=(TRN_Obj)obj;
	EEX;
}

TRN_API TRN_ObjInsertRect(TRN_Obj o, TRN_Size pos, double x1, double y1, double x2, double y2, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->InsertRect(pos, x1, y1, x2, y2);
	EEX;
}

TRN_API TRN_ObjInsertMatrix(TRN_Obj o, TRN_Size pos, const TRN_Matrix2D* mtx, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->InsertMatrix(pos,Common::Matrix2D(mtx->m_a, mtx->m_b, mtx->m_c, mtx->m_d, mtx->m_h, mtx->m_v));
	EEX;
}

TRN_API TRN_ObjPushBackName (TRN_Obj o, const char* name, TRN_Obj* result)
{
	BEX;	
	*result=(TRN_Obj)ObjCast(o)->PushBackName(name);
	EEX;
}

TRN_API TRN_ObjPushBackArray (TRN_Obj o, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PushBackArray();
	EEX;
}

TRN_API TRN_ObjPushBackBool (TRN_Obj o, TRN_Bool value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PushBackBool(TBToB(value));
	EEX;
}

TRN_API TRN_ObjPushBackDict (TRN_Obj o, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PushBackDict();
	EEX;
}

TRN_API TRN_ObjPushBackNumber (TRN_Obj o, double value, TRN_Obj* result)
{
	BEX;	
	*result=(TRN_Obj)ObjCast(o)->PushBackNumber(value);
	EEX;
}

TRN_API TRN_ObjPushBackString (TRN_Obj o, const char* value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PushBackString(value);
	EEX;
}

TRN_API TRN_ObjPushBackStringWithSize (TRN_Obj o, const char* value, int size, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PushBackString(value, size);
	EEX;
}

TRN_API TRN_ObjPushBackText (TRN_Obj o, const TRN_UString t, TRN_Obj* result)
{
	BEX;
	UString temp(UStrCast(t));
	*result=(TRN_Obj)ObjCast(o)->PushBackText(temp);
	EEX;
}

TRN_API TRN_ObjPushBackNull (TRN_Obj o, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PushBackNull();
	EEX;
}

TRN_API TRN_ObjPushBack(TRN_Obj o, TRN_Obj indirect_obj, TRN_Obj* result)
{
	BEX;
	Obj* obj=((Obj*)indirect_obj)->Clone();
	ObjCast(o)->PushBack(obj);
	*result=(TRN_Obj)obj;
	EEX;
}

TRN_API TRN_ObjPushBackRect(TRN_Obj o, double x1, double y1, double x2, double y2, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PushBackRect(x1, y1, x2, y2);
	EEX;
}

TRN_API TRN_ObjPushBackMatrix(TRN_Obj o, const TRN_Matrix2D* mtx, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)ObjCast(o)->PushBackMatrix(Common::Matrix2D(mtx->m_a, mtx->m_b, mtx->m_c, mtx->m_d, mtx->m_h, mtx->m_v));
	EEX;
}
TRN_API TRN_ObjEraseAt (TRN_Obj o, TRN_Size pos)
{
	BEX;
	ObjCast(o)->EraseAt(pos);
	EEX;
}

TRN_API TRN_ObjIsStream (const TRN_Obj o, TRN_Bool* result)
{
	BEX;
	*result=BToTB(ObjCast(o)->IsStream());
	EEX;
}

TRN_API TRN_ObjGetRawStreamLength (const TRN_Obj o, TRN_Size* result)
{
	BEX;
	*result=ObjCast(o)->GetRawStreamLength();
	EEX;
}

TRN_API TRN_ObjSetStreamData( TRN_Obj obj, const char* data, const size_t data_size )
{
	BEX;
	ObjCast(obj)->SetStreamData( data, data_size, AutoPtr<Filter>() );
	EEX;
}

TRN_API TRN_ObjSetStreamDataWithFilter( TRN_Obj obj, const char* data, const size_t data_size, TRN_Filter filter_chain )
{
	BEX;
	ObjCast(obj)->SetStreamData( data, data_size, AutoPtr<Filter>((Filter*)filter_chain) );
	EEX;
}

TRN_API TRN_ObjGetRawStream (const TRN_Obj o, TRN_Bool decrypt, TRN_Filter* result)
{
	BEX;
	*result=(TRN_Filter)(ObjCast(o)->GetRawStream(TBToB(decrypt))).release();
	EEX;
}

TRN_API TRN_ObjGetDecodedStream (const TRN_Obj o, TRN_Filter* result)
{
	BEX;
	*result=(TRN_Filter)ObjCast(o)->GetDecodedStream().release();
	EEX;
}
