#include <C/SDF/TRN_ObjSet.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <C/Common/TRN_Exception.hpp>
#include <SDF/ObjSet.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;

TRN_API TRN_ObjSetCreate(TRN_ObjSet* result)
{
	BEX;
	*result=(TRN_ObjSet)new ObjSet();
	EEX;
}

TRN_API TRN_ObjSetDestroy(TRN_ObjSet set)
{
	BEX;
	delete (ObjSet*)set;
	EEX;
}
	
TRN_API TRN_ObjSetCreateName(TRN_ObjSet set, const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)OSetCast(set).CreateName(name);
	EEX;
}





TRN_API TRN_ObjSetCreateArray(TRN_ObjSet set, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)OSetCast(set).CreateArray();
	EEX;
}


TRN_API TRN_ObjSetCreateBool(TRN_ObjSet set, TRN_Bool value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)OSetCast(set).CreateBool(TBToB(value));
	EEX;
}

TRN_API TRN_ObjSetCreateDict(TRN_ObjSet set, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)OSetCast(set).CreateDict();
	EEX;
}


TRN_API TRN_ObjSetCreateNull(TRN_ObjSet set, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)OSetCast(set).CreateNull();
	EEX;
}


TRN_API TRN_ObjSetCreateNumber(TRN_ObjSet set, double value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)OSetCast(set).CreateNumber(value);
	EEX;
}


TRN_API TRN_ObjSetCreateString(TRN_ObjSet set, TRN_UString value, TRN_Obj* result)
{
	BEX;
	UString temp(value);
	*result=(TRN_Obj)OSetCast(set).CreateString(temp);
	EEX;
}

TRN_API TRN_ObjSetCreateFromJson(TRN_ObjSet set, TRN_UString json, TRN_Obj* result)
{
	BEX;
	UString temp(json);
	TRN_Obj created = (TRN_Obj)OSetCast(set).CreateFromJson(temp);
	BASE_ASSERT(created != 0, "Unable to parse json string");
	*result=created;
	EEX;
}
