#include <C/SDF/TRN_SDFDoc.h>
#include <C/Common/TRN_Types.h>
#include <C/Common/TRN_API_Config.hpp>

// All C++ standard library and internal header files must be inside these guards.
// C standard header files (e.g. stdio.h) are recommended to be outside of these guards.
#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility push(hidden)
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

#include <list>
#include <C/Common/TRN_Exception.hpp>
#include <Filters/Filter.h>
#include <SDF/License.hpp>
#include <SDF/SDFDoc.h>

#if TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES
#pragma GCC visibility pop
#endif // TRN_ENABLE_HIDE_SYMBOL_DIRECTIVES

using namespace SDF;
using namespace Filters;
using namespace std;

#define DCast(doc) ((SDFDoc*)doc)


TRN_API TRN_SDFDocCreate(TRN_SDFDoc* result)
{
	BEX;
	RefCountedPtr<SDFDoc> ret(new SDFDoc());
	*result=(TRN_SDFDoc)ret.release();
	EEX;
}


TRN_API TRN_SDFDocCreateFromFileUString(const TRN_UString filepath, TRN_SDFDoc* result)
{
	BEX;
	UString temp(UStrCast(filepath));
	RefCountedPtr<SDFDoc> ret(new SDFDoc(temp, false, false));
	*result=(TRN_SDFDoc)ret.release();
	EEX;
}

TRN_API TRN_SDFDocCreateFromFileString(const char* filepath, TRN_SDFDoc* result)
{
	BEX;
	RefCountedPtr<SDFDoc> ret(new SDFDoc(filepath, false, false));
	*result=(TRN_SDFDoc)ret.release();
	EEX;
}

TRN_API TRN_SDFDocCreateFromFilter(TRN_Filter stream,TRN_SDFDoc* result)
{
	BEX;
	AutoPtr<Filter> ptr((Filter*)stream);
	RefCountedPtr<SDFDoc> ret(new SDFDoc(ptr, false, false));
	*result=(TRN_SDFDoc)ret.release();
	EEX;
}

TRN_API TRN_SDFDocCreateFromMemoryBuffer(const char* buf, TRN_Size buf_size,TRN_SDFDoc* result)
{
	BEX;
	RefCountedPtr<SDFDoc> ret(new SDFDoc(buf, buf_size, false, false));
	*result=(TRN_SDFDoc)ret.release();
	EEX;
}

TRN_API TRN_SDFDocCreateShallowCopy(TRN_SDFDoc source, TRN_SDFDoc* result)
{
	BEX;
	SDF::SDFDoc* core_sdfdoc = (SDF::SDFDoc*)source;
	if(core_sdfdoc)
	{
		core_sdfdoc->AddReference();
	}
	*result = source;
	EEX;
}

TRN_API TRN_SDFDocDestroy(TRN_SDFDoc doc)
{
	BEX;
	if(SDF::SDFDoc* core_sdfdoc = DCast(doc))
	{
		core_sdfdoc->RemoveReference();
	}
	EEX;
}

TRN_API TRN_SDFDocReleaseFileHandles(TRN_SDFDoc doc)
{
	BEX;
	if(SDF::SDFDoc* core_sdfdoc = DCast(doc))
	{
		core_sdfdoc->ReleaseFileHandles();
	}
	EEX;
}

TRN_API TRN_SDFDocIsEncrypted(TRN_SDFDoc doc, TRN_Bool* result)
{
	BEX;
	*result=BToTB(DCast(doc)->IsEncrypted());
	EEX;
}

TRN_API TRN_SDFDocInitSecurityHandler(TRN_SDFDoc doc, void* custom_data, TRN_Bool* result)
{
	BEX;
	*result=BToTB(DCast(doc)->InitSecurityHandler(custom_data));
	EEX;
}

TRN_API TRN_SDFDocInitStdSecurityHandler(TRN_SDFDoc doc, const char* password, int password_sz, TRN_Bool* result)
{
	BEX;
	*result=BToTB(DCast(doc)->InitStdSecurityHandler(password,password_sz));
	EEX;
}

TRN_API TRN_SDFDocInitStdSecurityHandlerUString(TRN_SDFDoc doc, TRN_UString password, TRN_Bool* result)
{
    BEX;
    trn::UString tempUStr(UStrCast(password));
    *result = BToTB(DCast(doc)->InitStdSecurityHandler(tempUStr));
    EEX;
}

TRN_API TRN_SDFDocInitStdSecurityHandlerBuffer(TRN_SDFDoc doc, const TRN_UInt8* password_buf, size_t password_buf_size, TRN_Bool* result)
{
    BEX;
    std::vector<trn::UInt8> tempBuf(password_buf, password_buf + password_buf_size);
    *result = BToTB(DCast(doc)->InitStdSecurityHandler(tempBuf));
    EEX;
}

TRN_API TRN_SDFDocIsModified(TRN_SDFDoc const doc,
			TRN_Bool* result)
{
	BEX;
	*result=BToTB(DCast(doc)->IsModified());
	EEX;
}

TRN_API TRN_SDFDocHasRepairedXRef(TRN_SDFDoc const doc,
	TRN_Bool* result)
{
	BEX;
	*result = BToTB(DCast(doc)->HasRepairedXRef());
	EEX;
}

TRN_API TRN_SDFDocIsFullSaveRequired(TRN_SDFDoc const doc,
		TRN_Bool* result)
{
	BEX;
	*result=BToTB(DCast(doc)->IsFullSaveRequired());
	EEX;
}

TRN_API TRN_SDFDocGetTrailer(TRN_SDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->GetTrailer());
	EEX;
}

TRN_API TRN_SDFDocGetObj(TRN_SDFDoc const doc,
		const TRN_UInt32 obj_num, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->GetObj((UInt32)obj_num));
	EEX;
}

TRN_API TRN_SDFDocImportObj(TRN_SDFDoc doc, TRN_Obj obj,
		TRN_Bool deep_copy, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->ImportObj((Obj*)obj,TBToB(deep_copy)));
	EEX;
}

TRN_API TRN_SDFDocImportObjs(TRN_SDFDoc doc, TRN_Obj* obj_list, int length, TRN_Obj* result)
{
	BEX;
	std::list<Obj*> temp;
	for(int q=0; q<length;++q)
		temp.push_back((Obj*)obj_list[q]);

	std::list<Obj*> res=DCast(doc)->ImportObjs(temp);
	std::list<Obj*>::iterator itr,end=res.end();
	int i=0;
	for(itr=res.begin(); itr!=end; ++itr, ++i)
		result[i]=(TRN_Obj)(*itr);
    EEX;
}

TRN_API TRN_SDFDocImportObjsWithExcludeList(TRN_SDFDoc doc, TRN_Obj* obj_list, int length, TRN_Obj* exclude_list, int exclude_length, TRN_Obj* result)
{
    BEX;
    std::list<Obj*> tempObjList;
    for(int q=0; q<length;++q)
        tempObjList.push_back((Obj*)obj_list[q]);
    
    std::list<Obj*> tempExcludeList;
    for(int q=0; q<exclude_length;++q)
        tempExcludeList.push_back((Obj*)exclude_list[q]);
    
    std::list<Obj*> res=DCast(doc)->ImportObjs(tempObjList, &tempExcludeList);
    std::list<Obj*>::iterator itr,end=res.end();
    int i=0;
    for(itr=res.begin(); itr!=end; ++itr, ++i)
        result[i]=(TRN_Obj)(*itr);
    EEX;
}

TRN_API TRN_SDFDocXRefSize(TRN_SDFDoc const doc, TRN_UInt32* result)
{
	BEX;
	*result= (DCast(doc)->XRefSize());
	EEX;
}

TRN_API TRN_SDFDocClearMarks(TRN_SDFDoc doc)
{
	BEX;
	DCast(doc)->ClearMarks();
	EEX;
}

TRN_API TRN_SDFDocSave(TRN_SDFDoc doc, const TRN_UString path, TRN_UInt32 flags,
		TRN_ProgressMonitor progress, const char* header)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_write);
#ifndef TRN_HAS_FILESYSTEM
	License::Instance().IfDemoApplyStamp(DCast(doc));
#endif
	UString temp(UStrCast(path));
	DCast(doc)->Save(temp,(UInt32)flags,0,header);
	EEX;
}

TRN_API TRN_SDFDocSaveMemory(TRN_SDFDoc doc, const char** out_buf, TRN_Size* out_buf_size,
		TRN_UInt32 flags, TRN_ProgressMonitor progress, const char* header)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_write);
#ifndef TRN_HAS_FILESYSTEM
	License::Instance().IfDemoApplyStamp(DCast(doc));
#endif
	DCast(doc)->Save(*out_buf,*out_buf_size,(UInt32)flags,0,header);
	EEX;
}

TRN_API TRN_SDFDocGetHeader(const TRN_SDFDoc doc, const char** result)
{
	BEX;
	*result=DCast(doc)->GetHeader();
	EEX;
}

TRN_API TRN_SDFDocGetSecurityHandler(TRN_SDFDoc doc, TRN_SecurityHandler* result)
{
	BEX;
	*result=(TRN_SecurityHandler)DCast(doc)->GetSecurityHandler();
	EEX;
}

TRN_API TRN_SDFDocSetSecurityHandler(TRN_SDFDoc doc, TRN_SecurityHandler handler)
{
	BEX;
	AutoPtr<SecurityHandler> temp((SecurityHandler*)handler);
	DCast(doc)->SetSecurityHandler(temp);
	EEX;
}

TRN_API TRN_SDFDocRemoveSecurity(TRN_SDFDoc doc)
{
	BEX;
	DCast(doc)->RemoveSecurity();
	EEX;
}

TRN_API TRN_SDFDocSwap(TRN_SDFDoc doc, TRN_UInt32 obj_num1, TRN_UInt32 obj_num2)
{
	BEX;
	DCast(doc)->Swap(obj_num1,obj_num2);
	EEX;
}

TRN_API TRN_SDFDocIsLinearized(TRN_SDFDoc const doc, TRN_Bool* result)
{
	BEX;
	*result=BToTB(DCast(doc)->IsLinearized());
	EEX;
}

TRN_API TRN_SDFDocGetLinearizationDict(TRN_SDFDoc const doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->GetLinearizationDict());
	EEX;
}

TRN_API TRN_SDFDocGetHintStream(TRN_SDFDoc const doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->GetHintStream());
	EEX;
}

TRN_API TRN_SDFDocEnableDiskCaching(TRN_SDFDoc doc, TRN_Bool use_cache_flag )
{
	BEX;
	DCast(doc)->EnableDiskCaching(use_cache_flag!=0);
	EEX;
}


TRN_API TRN_SDFDocLock(TRN_SDFDoc doc)
{
	BEX;
	DCast(doc)->Lock();
	EEX;
}

TRN_API TRN_SDFDocUnlock(TRN_SDFDoc doc)
{
	BEX;
	DCast(doc)->Unlock();
	EEX;
}


TRN_API TRN_SDFDocTryLock( TRN_SDFDoc doc, TRN_Bool* result )
{
	BEX;
	*result=BToTB(DCast(doc)->TryLock(0));
	EEX;
}

TRN_API TRN_SDFDocLockRead(TRN_SDFDoc doc)
{
	BEX;
	DCast(doc)->LockRead();
	EEX;
}

TRN_API TRN_SDFDocUnlockRead(TRN_SDFDoc doc)
{
	BEX;
	DCast(doc)->UnlockRead();
	EEX;
}


TRN_API TRN_SDFDocTryLockRead( TRN_SDFDoc doc, TRN_Bool* result )
{
	BEX;
	*result=BToTB(DCast(doc)->TryLockRead(0));
	EEX;
}


TRN_API TRN_SDFDocTimedLock( TRN_SDFDoc doc, int milliseconds, TRN_Bool* result )
{
	BEX;
	*result=BToTB(DCast(doc)->TryLock(milliseconds));
	EEX;
}

TRN_API TRN_SDFDocTimedLockRead( TRN_SDFDoc doc, int milliseconds, TRN_Bool* result )
{
	BEX;
	*result=BToTB(DCast(doc)->TryLockRead(milliseconds));
	EEX;
}

TRN_API TRN_SDFDocGetFileName(TRN_SDFDoc doc, TRN_UString* result)
{
	BEX;
	RetUStr(DCast(doc)->GetFileName());
	EEX;
}


TRN_API TRN_SDFDocCreateIndirectName(TRN_SDFDoc doc, const char* name, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->CreateIndirectName(name));
	EEX;
}

TRN_API TRN_SDFDocCreateIndirectArray(TRN_SDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->CreateIndirectArray());
	EEX;
}

TRN_API TRN_SDFDocCreateIndirectBool(TRN_SDFDoc doc, TRN_Bool value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->CreateIndirectBool(TBToB(value)));
	EEX;
}

TRN_API TRN_SDFDocCreateIndirectDict(TRN_SDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->CreateIndirectDict());
	EEX;
}

TRN_API TRN_SDFDocCreateIndirectNull(TRN_SDFDoc doc, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->CreateIndirectNull());
	EEX;
}

TRN_API TRN_SDFDocCreateIndirectNumber(TRN_SDFDoc doc, double value, TRN_Obj* result)
{
	BEX;
	*result=(TRN_Obj)(DCast(doc)->CreateIndirectNumber(value));
	EEX;
}

TRN_API TRN_SDFDocCreateIndirectString(TRN_SDFDoc doc, const TRN_UChar* buf_value, TRN_UInt32 buf_size, TRN_Obj* result)
{
	BEX;
	*result =(TRN_Obj)(DCast(doc)->CreateIndirectString(buf_value, buf_size));
	EEX;
}

TRN_API TRN_SDFDocCreateIndirectStringFromUString(TRN_SDFDoc doc, const TRN_UString str, TRN_Obj* result)
{
	BEX;
	UString temp(UStrCast(str));
	*result=(TRN_Obj)(DCast(doc)->CreateIndirectString(temp));
	EEX;
}

TRN_API TRN_SDFDocCreateIndirectStreamFromFilter(TRN_SDFDoc doc, TRN_FilterReader data, TRN_Filter filter_chain, TRN_Obj* result)
{
	BEX;
	AutoPtr<Filter> ptr((Filter*)filter_chain);
	*result=(TRN_Obj)(DCast(doc)->CreateIndirectStream(*((FilterReader*)data),ptr));
	EEX;
}

TRN_API TRN_SDFDocCreateIndirectStream(TRN_SDFDoc doc, const char* data, const TRN_Size data_size, TRN_Filter filter_chain, TRN_Obj* result)
{

	BEX;
	AutoPtr<Filter> ptr((Filter*)filter_chain);
	*result=(TRN_Obj)(DCast(doc)->CreateIndirectStream(data,data_size,ptr));
	EEX;
}

TRN_API TRN_SDFDocSaveStream(TRN_SDFDoc doc, TRN_Filter stream, TRN_UInt32 flags, const char* header)
{
	BEX;
	License::Instance().VerifyPermission(License::e_perm_write);
#ifndef TRN_HAS_FILESYSTEM
	License::Instance().IfDemoApplyStamp(DCast(doc));
#endif
	Filters::Filter* flt = (Filters::Filter*)stream;
	DCast(doc)->Save(*flt, flags, 0 ,header);
	EEX;
}

