//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "PTTool.h"

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

/**
 * Creates a free text annotation.
 */
@interface PTFreeTextCreate: PTTool <UITextViewDelegate>

/**
 * The `UITextView` instance used for interactive text entry.
 */
@property (nonatomic, strong, nullable) UITextView *textView;

/**
 * Commits the free text annotation to the document.
 */
- (void)commitAnnotation;

/**
 * Creates a free text annotation from the tool's current state.
 *
 * @return a new `PTFreeText` instance for the current document
 */
- (PTFreeText *)createFreeText;

/**
 * Sets the rect of the free text annotation.
 *
 * @param freeText the free text annotation
 */
- (void)setRectForFreeText:(PTFreeText *)freeText;

/**
 * Applies the text color, border appearance, and other properties to the free text
 * annotation before it is committed.
 *
 * @param freeText the free text annotation
 */
- (void)setPropertiesForFreeText:(PTFreeText *)freeText;

/**
 Sets the rect for a `PTFreeText` annotation.
 
 @param freeText The `PTFreeText` object.
 @param rect The `PTPDFRect` representation of the associated `UITextView`.
 @param pdfViewCtrl The `PTPDFViewCtrl` object.
 @param isRTL A `BOOL` indicating whether the text is in a right-to-left language.
 */
+(void)setRectForFreeText:(PTFreeText*)freeText withRect:(PTPDFRect*)rect pdfViewCtrl:(PTPDFViewCtrl*)pdfViewCtrl isRTL:(BOOL)isRTL;

@end

NS_ASSUME_NONNULL_END
