//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsConfig.h"
#import "PTFreeTextAnnotationOptions.h"

@implementation PTFreeTextAnnotationOptions

- (instancetype)initWithCanCreate:(BOOL)canCreate canEdit:(BOOL)canEdit
{
    self = [super initWithCanCreate:canCreate canEdit:canEdit];
    if (self) {
    }
    return self;
}

#pragma mark - Equality

- (BOOL)isEqualToAnnotationOptions:(PTAnnotationOptions *)annotationOptions
{
    if (![super isEqualToAnnotationOptions:annotationOptions]) {
        return NO;
    }
    
    if (![annotationOptions isKindOfClass:[PTFreeTextAnnotationOptions class]]) {
        return NO;
    }
    
    PTFreeTextAnnotationOptions *freeTextAnnotationOptions = (PTFreeTextAnnotationOptions *)annotationOptions;
    return ([self.defaultFontName isEqualToString:freeTextAnnotationOptions.defaultFontName]);
}

#pragma mark - <NSObject>

- (NSUInteger)hash
{
    return [super hash];
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"<%@: %p; canCreate: %@; canEdit: %@; defaultFontName: %@>",
            [self class], self,
            PT_NSStringFromBOOL(self.canCreate), PT_NSStringFromBOOL(self.canEdit),
            self.defaultFontName];
}

#pragma mark - <NSCopying>

- (id)copyWithZone:(NSZone *)zone
{
    PTFreeTextAnnotationOptions *options = [super copyWithZone:zone];
    
    options.defaultFontName = self.defaultFontName;
    
    return options;
}

#pragma mark - <NSCoding>

- (instancetype)initWithCoder:(NSCoder *)coder
{
    self = [super initWithCoder:coder];
    if (self) {
        NSString* defaultFontName = @"Helvetica";
        if ([coder containsValueForKey:PT_KEY(self, defaultFontName)]) {
            defaultFontName = [coder decodeObjectForKey:PT_KEY(self, defaultFontName)];
        }
        
        _defaultFontName = [defaultFontName copy];
    }
    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];
    
    [coder encodeObject:self.defaultFontName forKey:PT_KEY(self, defaultFontName)];
}


@end
