//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsConfig.h"
#import "PTSignatureAnnotationOptions.h"

@implementation PTSignatureAnnotationOptions

- (instancetype)initWithCanCreate:(BOOL)canCreate canEdit:(BOOL)canEdit
{
    self = [super initWithCanCreate:canCreate canEdit:canEdit];
    if (self) {
        _canEditAppearance = NO;
    }
    return self;
}

#pragma mark - Equality

- (BOOL)isEqualToAnnotationOptions:(PTAnnotationOptions *)annotationOptions
{
    if (![super isEqualToAnnotationOptions:annotationOptions]) {
        return NO;
    }
    
    if (![annotationOptions isKindOfClass:[PTSignatureAnnotationOptions class]]) {
        return NO;
    }
    
    PTSignatureAnnotationOptions *signatureAnnotationOptions = (PTSignatureAnnotationOptions *)annotationOptions;
    return (self.canEditAppearance == signatureAnnotationOptions.canEditAppearance);
}

#pragma mark - <NSObject>

- (NSUInteger)hash
{
    return ([super hash]) ^ ((NSUInteger)self.canEditAppearance);
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"<%@: %p; canCreate: %@; canEdit: %@; canEditAppearance: %@>",
            [self class], self,
            PT_NSStringFromBOOL(self.canCreate), PT_NSStringFromBOOL(self.canEdit),
            PT_NSStringFromBOOL(self.canEditAppearance)];
}

#pragma mark - <NSCopying>

- (id)copyWithZone:(NSZone *)zone
{
    PTSignatureAnnotationOptions *options = [super copyWithZone:zone];
    
    options.canEditAppearance = self.canEditAppearance;
    
    return options;
}

#pragma mark - <NSCoding>

- (instancetype)initWithCoder:(NSCoder *)coder
{
    self = [super initWithCoder:coder];
    if (self) {
        BOOL canEditAppearance = YES;
        if ([coder containsValueForKey:PT_KEY(self, canEditAppearance)]) {
            canEditAppearance = [coder decodeBoolForKey:PT_KEY(self, canEditAppearance)];
        }
        
        _canEditAppearance = canEditAppearance;
    }
    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];
    
    [coder encodeBool:self.canEditAppearance forKey:PT_KEY(self, canEditAppearance)];
}

@end
