//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsConfig.h"
#import "PTColorPickerViewController.h"

#import "PTStandardColorsViewController.h"

@interface PTColorPickerViewController () <PTStandardColorsViewControllerDelegate>

@property (nonatomic, strong) PTStandardColorsViewController *standardColorsViewController;

@end

@implementation PTColorPickerViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.dataSource = self;
    
    self.standardColorsViewController = [[PTStandardColorsViewController alloc] initWithCollectionViewLayout:[[UICollectionViewFlowLayout alloc] init]];
    self.standardColorsViewController.colorPickerDelegate = self;
    self.standardColorsViewController.selectedColor = self.color;
    
    // Set initial view controller.
    [self setViewControllers:@[self.standardColorsViewController] direction:UIPageViewControllerNavigationDirectionForward animated:NO completion:nil];
    self.extendedLayoutIncludesOpaqueBars = YES;
}

- (void)preferredContentSizeDidChangeForChildContentContainer:(id<UIContentContainer>)container
{
    [super preferredContentSizeDidChangeForChildContentContainer:container];
    
    CGSize maxSize = CGSizeZero;
    for (UIViewController *viewController in self.viewControllers) {
        CGSize preferredContentSize = viewController.preferredContentSize;
        if (preferredContentSize.width > maxSize.width || preferredContentSize.height > maxSize.height) {
            maxSize = preferredContentSize;
        }
    }
    if (!CGSizeEqualToSize(maxSize, CGSizeZero)) {
        self.preferredContentSize = maxSize;
    }
}

#pragma mark - <UIPageViewControllerDataSource>

- (nullable UIViewController *)pageViewController:(nonnull UIPageViewController *)pageViewController viewControllerAfterViewController:(nonnull UIViewController *)viewController {
    return nil;
}

- (nullable UIViewController *)pageViewController:(nonnull UIPageViewController *)pageViewController viewControllerBeforeViewController:(nonnull UIViewController *)viewController {
    return nil;
}

#pragma mark - <StandardColorsViewControllerDelegate>

- (void)standardColorsViewController:(PTStandardColorsViewController *)standardColorsViewController didSelectColor:(UIColor *)color
{
    self.color = color;
    
    if ([self.colorPickerDelegate respondsToSelector:@selector(colorPickerController:didSelectColor:)]) {
        [self.colorPickerDelegate colorPickerController:self didSelectColor:color];
    }
}

- (void)setColor:(UIColor *)color
{
    _color = color;
    
    self.standardColorsViewController.selectedColor = color;
}

@end

