//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsConfig.h"
#import "PTPopoverTableViewController.h"

@interface PTPopoverTableViewController ()

@property (nonatomic, assign) BOOL needsInitialReload;

@end

@implementation PTPopoverTableViewController

// Default init calls this method.
- (instancetype)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    return [self initWithStyle:UITableViewStylePlain];
}

- (instancetype)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithNibName:nil bundle:nil];
    if (self) {
        _tableView = [[UITableView alloc] initWithFrame:CGRectZero style:style];
        _tableView.dataSource = self;
        _tableView.delegate = self;
        
        _tableView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        
        _needsInitialReload = YES;
        _clearsSelectionOnViewWillAppear = YES;
    }
    return self;
}

- (void)loadView
{
    self.view = [[UIView alloc] initWithFrame:CGRectZero];
    self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    
    [self.view addSubview:self.tableView];
}

- (void)viewDidLayoutSubviews
{
    [super viewDidLayoutSubviews];
    
    [self tableViewUpdatePreferredContentSize];
}

- (void)tableViewUpdatePreferredContentSize
{
    CGSize contentSize = self.tableView.contentSize;
    UIEdgeInsets contentInset = self.tableView.contentInset;
    // NOTE: DON'T add the adjustedContentInset, since that will be added by the navigationController.
    
    CGSize preferredContentSize = CGSizeMake(contentSize.width + contentInset.left + contentInset.right,
                                             contentSize.height + contentInset.top + contentInset.bottom);
    
    if (!CGSizeEqualToSize(preferredContentSize, self.preferredContentSize)) {
        self.preferredContentSize = preferredContentSize;
    }
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    if (self.needsInitialReload) {
        [self.tableView reloadData];
        
        self.needsInitialReload = NO;
    } else if (self.clearsSelectionOnViewWillAppear) {
        [self clearSelectionOnViewWillAppear:animated];
    }
}

- (void)clearSelectionOnViewWillAppear:(BOOL)animated
{
    // Deselect the selected row on viewWillAppear.
    NSIndexPath *selectedIndexPath = [self.tableView indexPathForSelectedRow];
    if (selectedIndexPath != nil) {
        // Check if there is a transition coordinator.
        id<UIViewControllerTransitionCoordinator> coordinator = self.transitionCoordinator;
        if (coordinator != nil) {
            // Deselect the selected row in an alongside-animation for the transition.
            [coordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext> context) {
                [self.tableView deselectRowAtIndexPath:selectedIndexPath animated:YES];
            } completion:^(id<UIViewControllerTransitionCoordinatorContext> context) {
                if (context.cancelled) {
                    // Reselect the row for a cancelled transition.
                    [self.tableView selectRowAtIndexPath:selectedIndexPath animated:NO scrollPosition:UITableViewScrollPositionNone];
                }
            }];
        } else {
            // Directly deselect the selected row.
            [self.tableView deselectRowAtIndexPath:selectedIndexPath animated:animated];
        }
    }
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    [self.tableView flashScrollIndicators];
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
    [super setEditing:editing animated:animated];
    
    [self.tableView setEditing:editing animated:animated];
}

#pragma mark - Property accessor overrides

- (void)setTableView:(UITableView *)tableView
{
    // Null-resettable.
    if (!tableView) {
        // Create a default table view.
        tableView = [[UITableView alloc] initWithFrame:self.view.frame style:UITableViewStylePlain];
        tableView.dataSource = self;
        tableView.delegate = self;
        
        tableView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    }
    // Remove old table view from view.
    [_tableView removeFromSuperview];

    _tableView = tableView;
    
    // Add new table view to view.
    [self.view addSubview:tableView];
}

#pragma mark - <UITableViewDataSource>

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return [tableView dequeueReusableCellWithIdentifier:@"" forIndexPath:indexPath];
}

@end
