//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsConfig.h"
#import "PTAnnotStyleColorTableViewCell.h"

#import "PTColorIndicatorView.h"

#include <tgmath.h>

static CGFloat const PT_colorIndicatorViewSize = 30.0; // pts

@interface PTAnnotStyleColorTableViewCell ()

@property (nonatomic, strong) PTColorIndicatorView *colorIndicatorView;

@property (nonatomic) BOOL didSetupConstraints;

@end

@implementation PTAnnotStyleColorTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        _label = [[UILabel alloc] init];
        _label.translatesAutoresizingMaskIntoConstraints = NO;
        
        [self.contentView addSubview:_label];
        
        _colorIndicatorView = [[PTColorIndicatorView alloc] initWithFrame:CGRectZero];
        _colorIndicatorView.translatesAutoresizingMaskIntoConstraints = NO;
        
        [self.contentView addSubview:_colorIndicatorView];
        
        self.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        
        // Schedule constraints.
        [self setNeedsUpdateConstraints];
    }
    return self;
}

#pragma mark - Constraints

- (void)updateConstraints
{
    if (!self.didSetupConstraints) {
        self.label.translatesAutoresizingMaskIntoConstraints = NO;
        self.colorIndicatorView.translatesAutoresizingMaskIntoConstraints = NO;

        UILayoutGuide *layoutMarginsGuide = self.contentView.layoutMarginsGuide;
        
        [NSLayoutConstraint activateConstraints:@[
            [self.label.leadingAnchor constraintEqualToAnchor:layoutMarginsGuide.leadingAnchor],
            [self.label.centerYAnchor constraintEqualToAnchor:layoutMarginsGuide.centerYAnchor],
            
            [self.colorIndicatorView.leadingAnchor constraintGreaterThanOrEqualToAnchor:self.label.trailingAnchor],
            [self.colorIndicatorView.trailingAnchor constraintEqualToAnchor:layoutMarginsGuide.trailingAnchor],
            [self.colorIndicatorView.centerYAnchor constraintEqualToAnchor:layoutMarginsGuide.centerYAnchor],
            [self.colorIndicatorView.widthAnchor constraintEqualToAnchor:self.colorIndicatorView.heightAnchor],
            [self.colorIndicatorView.heightAnchor constraintEqualToConstant:PT_colorIndicatorViewSize],
            
            [self.colorIndicatorView.topAnchor constraintGreaterThanOrEqualToAnchor:self.contentView.topAnchor],
            [self.colorIndicatorView.bottomAnchor constraintLessThanOrEqualToAnchor:self.contentView.bottomAnchor],
        ]];
        
        // Constraints are set up.
        self.didSetupConstraints = YES;
    }
    
    // Call super implementation as final step.
    [super updateConstraints];
}

#pragma mark - Layout

- (CGSize)systemLayoutSizeFittingSize:(CGSize)targetSize withHorizontalFittingPriority:(UILayoutPriority)horizontalFittingPriority verticalFittingPriority:(UILayoutPriority)verticalFittingPriority
{
    CGSize systemLayoutSize = [super systemLayoutSizeFittingSize:targetSize withHorizontalFittingPriority:horizontalFittingPriority verticalFittingPriority:verticalFittingPriority];
    // Enforce a minimum cell (content) height of 44 pts.
    systemLayoutSize.height = fmax(44.0, systemLayoutSize.height);
    return systemLayoutSize;
}

- (void)configureWithItem:(PTAnnotStyleColorTableViewItem *)item
{
    self.label.text = item.title;
        
    self.colorIndicatorView.color = item.color;
}

@end
