//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsConfig.h"
#import "PTAnnotStyleSliderTableViewCell.h"

@interface PTAnnotStyleSliderTableViewCell ()

@property (nonatomic, strong) UIStackView *stackView;

@property (nonatomic) BOOL didSetupConstraints;

@end

@implementation PTAnnotStyleSliderTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        // Set up stack view container.
        _stackView = [[UIStackView alloc] init];
        _stackView.axis = UILayoutConstraintAxisHorizontal;
        _stackView.alignment = UIStackViewAlignmentFill;
        _stackView.distribution = UIStackViewDistributionFill;
        _stackView.spacing = 10.0;
        
        _stackView.preservesSuperviewLayoutMargins = YES;
        _stackView.layoutMarginsRelativeArrangement = YES;
        _stackView.translatesAutoresizingMaskIntoConstraints = NO; // Must be done *before* adding to content view.
        [self.contentView addSubview:_stackView];
        
        _label = [[UILabel alloc] init];
        _label.textAlignment = NSTextAlignmentNatural;
        [_stackView addArrangedSubview:_label];
        
        _slider = [[UISlider alloc] init];
        [_stackView addArrangedSubview:_slider];
        
        _indicator = [[UILabel alloc] init];
        // Trailing text alignment.
        if ([UIView userInterfaceLayoutDirectionForSemanticContentAttribute:self.semanticContentAttribute] == UIUserInterfaceLayoutDirectionRightToLeft) {
            _indicator.textAlignment = NSTextAlignmentLeft;
        } else {
            _indicator.textAlignment = NSTextAlignmentRight;
        }
        [_stackView addArrangedSubview:_indicator];
        
        // Schedule constraints update.
        [self setNeedsUpdateConstraints];
        
        // Respond to slider events.
        [_slider addTarget:self action:@selector(sliderTouchDown) forControlEvents:UIControlEventTouchDown];
        [_slider addTarget:self action:@selector(sliderValueChanged) forControlEvents:UIControlEventValueChanged];
        [_slider addTarget:self action:@selector(sliderTouchUp) forControlEvents:(UIControlEventTouchUpInside | UIControlEventTouchUpOutside)];
        
        // Cell is not selectable.
        self.selectionStyle = UITableViewCellSelectionStyleNone;
    }
    return self;
}

- (void)updateConstraints
{
    if (!self.didSetupConstraints) {
        // Perform setup of constraints.
        self.stackView.translatesAutoresizingMaskIntoConstraints = NO;
        self.label.translatesAutoresizingMaskIntoConstraints = NO;
        self.slider.translatesAutoresizingMaskIntoConstraints = NO;
        self.indicator.translatesAutoresizingMaskIntoConstraints = NO;
        
        [NSLayoutConstraint activateConstraints:
         @[
           [self.stackView.leadingAnchor constraintEqualToAnchor:self.contentView.leadingAnchor],
           [self.stackView.widthAnchor constraintEqualToAnchor:self.contentView.widthAnchor],
           [self.stackView.topAnchor constraintEqualToAnchor:self.contentView.topAnchor],
           [self.stackView.heightAnchor constraintEqualToAnchor:self.contentView.heightAnchor],
           
           [self.label.widthAnchor constraintEqualToConstant:100.0],
           
           [self.indicator.widthAnchor constraintEqualToConstant:60.0],
           ]];

        // Constraints are set up.
        self.didSetupConstraints = YES;
    }

    [super updateConstraints];
}

- (void)configureWithItem:(PTAnnotStyleSliderTableViewItem *)item
{
    self.label.text = item.title;
    self.slider.minimumValue = item.minimumValue;
    self.slider.maximumValue = item.maximumValue;
    self.slider.value = item.value;
    
    self.indicator.text = item.indicatorText;
}

#pragma mark - UISlider action methods

- (void)sliderTouchDown
{
    if ([self.delegate respondsToSelector:@selector(styleSliderTableViewCellSliderBeganSliding:)]) {
        [self.delegate styleSliderTableViewCellSliderBeganSliding:self];
    }
}

- (void)sliderValueChanged
{
    if ([self.delegate respondsToSelector:@selector(styleSliderTableViewCell:sliderValueDidChange:)]) {
        [self.delegate styleSliderTableViewCell:self sliderValueDidChange:self.slider.value];
    }
}

- (void)sliderTouchUp
{
    if ([self.delegate respondsToSelector:@selector(styleSliderTableViewCellSliderEndedSliding:)]) {
        [self.delegate styleSliderTableViewCellSliderEndedSliding:self];
    }
}

@end
