//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "PTAnnot+PTAdditions.h"
#import "PTColorDefaults.h"
#import "PTToolsUtil.h"

@implementation PTAnnot (PTAdditions)

#pragma mark - Annot properties

- (NSString *)IRTAsNSString
{
    PTObj *annotSDFObj = [self GetSDFObj];
    if (annotSDFObj != nil) {
        PTObj *irt = [annotSDFObj FindObj:@"IRT"];
        if (irt != nil) {
            if ([irt IsString]) {
                return [irt GetAsPDFText];
            }
            if ([irt IsDict]) {
                PTObj *nm = [irt FindObj:@"NM"];
                if (nm != nil && [nm IsString]) {
                    return [nm GetAsPDFText];
                }
            }
        }
    }
    return nil;
}

- (BOOL)hasReplyTypeGroup
{
    PTObj *annotSDFObj = [self GetSDFObj];
    if (annotSDFObj != nil) {
        PTObj *irt = [annotSDFObj FindObj:@"IRT"];
        PTObj *rt = [annotSDFObj FindObj:@"RT"];
        if (irt != nil && rt != nil && [rt IsName]) {
            NSString *rtVal = [rt GetName];
            return [rtVal isEqualToString:@"Group"];
        }
    }
    return NO;
}

- (BOOL)isInGroup
{
    return self.annotationsInGroup.count > 1;
}

- (UIColor *)colorPrimary
{
    if( [self extendedAnnotType] == PTExtendedAnnotTypePencilDrawing )
    {
        UIColor *pencilDrawingIconColor = [UIColor blackColor];
        if (@available(iOS 13.0, *)) {
            pencilDrawingIconColor = [UIColor colorNamed:@"UIFGColor" inBundle:[PTToolsUtil toolsBundle] compatibleWithTraitCollection:nil];
        }
        return pencilDrawingIconColor;
    }
    UIColor *color = color = [PTColorDefaults uiColorFromColorPt:[self GetColorAsRGB] compNum:3];;
    
    PTAnnotType annotType = [self GetType];
    if (annotType == e_ptFreeText) {
        PTFreeText *freeText = [[PTFreeText alloc] initWithAnn:self];
        color =  [PTColorDefaults uiColorFromColorPt:[freeText GetTextColor] compNum:[freeText GetTextColorCompNum]];
    }
    else if ([self IsMarkup]) {
        PTMarkup *markup = [[PTMarkup alloc] initWithAnn:self];
        if ([markup GetInteriorColorCompNum] == 3) {
            UIColor *fillColor = [PTColorDefaults uiColorFromColorPt:[markup GetInteriorColor] compNum:3];
            if (CGColorGetAlpha(fillColor.CGColor) != 0.0) {
                color = fillColor;
            }
        }
    }
    
    return color;
}

#pragma mark - Group

- (NSArray<PTAnnot *> *)annotationsInGroup
{
    NSMutableArray *annotations = [NSMutableArray array];
    if (![self IsValid]) {
        return nil;
    }
    PTPage *page = [self GetPage];
    if (![page IsValid]) {
        // Page entry is not always valid
        [annotations addObject:self];
        return [annotations copy];
    }

    NSString *mainAnnotID;
    if ([self hasReplyTypeGroup]) {
        mainAnnotID = [self IRTAsNSString];
    }else if ([self GetUniqueID] != nil){
        mainAnnotID = self.uniqueID;
    }
    
    if (!mainAnnotID) {
        [annotations addObject:self];
        return [annotations copy];
    }

    int annotationCount = [page GetNumAnnots];
    PTAnnot *mainAnnot;
    for (int a = 0; a < annotationCount; a++) {
        PTAnnot *annot = [page GetAnnot:a];
        if (!annot || ![annot IsValid]) { continue; }
        NSString *annotID = [[annot GetUniqueID] GetAsPDFText];
        NSString *irt = annot.IRTAsNSString;
        if ([annotID isEqualToString:mainAnnotID]) {
            mainAnnot = annot;
        }else if (annot.hasReplyTypeGroup && irt != nil && [irt isEqualToString:mainAnnotID]){
            [annotations addObject:annot];
        }
    }
    if (mainAnnot != nil && [mainAnnot IsValid]) {
        [annotations insertObject:mainAnnot atIndex:0];
    }
    if (annotations.count == 0) {
        [annotations addObject:self];
    }
    return [annotations copy];
}

#pragma mark - uniqueID

- (NSString *)uniqueID
{
    PTObj *obj = [self GetUniqueID];
    if ([obj IsValid] && [obj IsString]) {
        return [obj GetAsPDFText];
    }
    return nil;
}

- (void)setUniqueID:(NSString *)uniqueID
{
    [self SetUniqueID:uniqueID id_buf_sz:0];
}

@end

PT_DEFINE_CATEGORY_SYMBOL(PTAnnot, PTAdditions)
