//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsConfig.h"
#import "UIViewController+PTAdditions.h"

@implementation UIViewController (PTAdditions)

- (BOOL)pt_isInPopover
{
    // Check if the view controller has been presented at all.
    if (!self.presentingViewController) {
        // View controller has not been presented.
        return NO;
    }
    
    // Find the outmost (parent) view controller that was presented.
    UIViewController *outmostViewController = self;
    while (outmostViewController.parentViewController) {
        outmostViewController = outmostViewController.parentViewController;
    }
    
    // Check for a popover presentation controller first since it will only be created if the
    // modalPresentationStyle property is UIModalPresentationPopover.
    UIPopoverPresentationController *popoverPresentationController = outmostViewController.popoverPresentationController;
    if (!popoverPresentationController) {
        // Presented view controller does not have a popover presentation controller.
        return NO;
    }
    
    UIPresentationController *presentationController = outmostViewController.presentationController;
    if (presentationController != popoverPresentationController) {
        return NO;
    }
    
    return (popoverPresentationController.arrowDirection < UIPopoverArrowDirectionUnknown);
}

- (NSLayoutYAxisAnchor *)pt_safeTopAnchor
{
    if (@available(iOS 11, *)) {
        return self.view.safeAreaLayoutGuide.topAnchor;
    } else {
        return self.topLayoutGuide.bottomAnchor;
    }
}

@end

PT_DEFINE_CATEGORY_SYMBOL(UIViewController, PTAdditions)
