//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsConfig.h"
#import "PTToolImages.h"

#import "PTToolsUtil.h"

@interface PTToolImages ()

@end

@implementation PTToolImages

+ (nullable NSString *)imageNameForAnnotationType:(PTExtendedAnnotType)annotationType
{
    switch (annotationType) {
        case PTExtendedAnnotTypeText:
            return @"ic_annotation_sticky_note_black_24dp";
        case PTExtendedAnnotTypeLine:
            return @"ic_annotation_line_black_24dp";
        case PTExtendedAnnotTypeSquare:
            return @"ic_annotation_square_black_24dp";
        case PTExtendedAnnotTypeCircle:
            return @"ic_annotation_circle_black_24dp";
        case PTExtendedAnnotTypeUnderline:
            return @"ic_annotation_underline_black_24dp";
        case PTExtendedAnnotTypeStrikeOut:
            return @"ic_annotation_strikeout_black_24dp";
        case PTExtendedAnnotTypeInk:
        case PTExtendedAnnotTypePencilDrawing:
            return @"ic_annotation_freehand_black_24dp";
        case PTExtendedAnnotTypeFileAttachment:
            return @"ic_attach_file_black_24dp";
        case PTExtendedAnnotTypeHighlight:
            return @"ic_annotation_highlight_black_24dp";
        case PTExtendedAnnotTypeFreeText:
            return @"ic_annotation_freetext_black_24dp";
        case PTExtendedAnnotTypeImageStamp:
            return @"ic_annotation_image_black_24dp";
        case PTExtendedAnnotTypeArrow:
            return @"ic_annotation_arrow_black_24dp";
        case PTExtendedAnnotTypeSquiggly:
            return @"ic_annotation_squiggly_black_24dp";
        case PTExtendedAnnotTypePolyline:
            return @"ic_annotation_polyline_black_24dp";
        case PTExtendedAnnotTypePolygon:
            return @"ic_annotation_polygon_black_24dp";
        case PTExtendedAnnotTypeCloudy:
            return @"ic_annotation_cloud_black_24dp";
        case PTExtendedAnnotTypeSignature:
            return @"ic_annotation_signature_black_24dp";
        case PTExtendedAnnotTypeStamp:
            return @"ic_annotation_stamp_black_24dp";
        case PTExtendedAnnotTypeCaret:
        case PTExtendedAnnotTypeRedact:
            return @"ic_edit_black_24dp";
        case PTExtendedAnnotTypeRuler:
            return @"ic_annotation_distance_black_24dp";
        case PTExtendedAnnotTypePerimeter:
            return @"ic_annotation_perimeter_black_24dp";
        case PTExtendedAnnotTypeArea:
            return @"ic_annotation_area_black_24dp";
        case PTExtendedAnnotTypeFreehandHighlight:
            return @"ic_annotation_free_highlight_black_24dp";
        case PTExtendedAnnotTypeCallout:
            return @"ic_annotation_callout_black_24dp";
            
        // Annotation types without images:
        case PTExtendedAnnotTypeLink:
        case PTExtendedAnnotTypePopup:
        case PTExtendedAnnotTypeSound:
        case PTExtendedAnnotTypeMovie:
        case PTExtendedAnnotTypeWidget:
        case PTExtendedAnnotTypeScreen:
        case PTExtendedAnnotTypePrinterMark:
        case PTExtendedAnnotTypeTrapNet:
        case PTExtendedAnnotTypeWatermark:
        case PTExtendedAnnotType3D:
        case PTExtendedAnnotTypeProjection:
        case PTExtendedAnnotTypeRichMedia:
        case PTExtendedAnnotTypeUnknown:
            return nil;
    }
    
    // NOTE: There is no default case in the switch above to ensure that each annotation type is
    // handled and newly added types are not missed.
    
    return nil;
}

+ (UIImage *)imageForAnnotationType:(PTExtendedAnnotType)annotationType
{
    // Get the name of the image for the annotation type.
    NSString *imageName = [self imageNameForAnnotationType:annotationType];
    if (!imageName) {
        // Image name not found.
        return nil;
    }
    
    return [PTToolsUtil toolImageNamed:imageName];
}

@end
