//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsDefines.h"

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

PT_LOCAL const CGFloat PTDocumentTabBarDefaultMinimumTabWidth;

@class PTDocumentTabBar;

@protocol PTDocumentTabBarDataSource <NSObject>
@required

- (NSUInteger)documentTabBarNumberOfItems:(PTDocumentTabBar *)tabBar;

- (NSString *)documentTabBar:(PTDocumentTabBar *)tabBar titleForItemAtIndex:(NSUInteger)index;

@optional

- (void)documentTabBar:(PTDocumentTabBar *)tabBar removeItemAtIndex:(NSUInteger)index;

- (BOOL)documentTabBar:(PTDocumentTabBar *)tabBar canMoveItemAtIndex:(NSUInteger)index;

- (void)documentTabBar:(PTDocumentTabBar *)tabBar moveItemAtIndex:(NSUInteger)sourceIndex toIndex:(NSUInteger)destinationIndex;

@end

@protocol PTDocumentTabBarDelegate <NSObject>
@optional

- (BOOL)documentTabBar:(PTDocumentTabBar *)tabBar shouldSelectItemAtIndex:(NSUInteger)index;

- (void)documentTabBar:(PTDocumentTabBar *)tabBar didSelectItemAtIndex:(NSUInteger)index;

- (void)documentTabBar:(PTDocumentTabBar *)tabBar willRemoveItemAtIndex:(NSUInteger)index;

- (void)documentTabBar:(PTDocumentTabBar *)tabBar didRemoveItemAtIndex:(NSUInteger)index;

- (void)documentTabBar:(PTDocumentTabBar *)tabBar closeButtonTappedForItemWithIndex:(NSUInteger)index;

@end

@interface PTDocumentTabBar : UIView <UICollectionViewDelegateFlowLayout, UICollectionViewDataSource>

@property (nonatomic, assign) NSUInteger selectedIndex;

@property (nonatomic, strong, nullable) UIView *backgroundView;

@property (nonatomic, strong) UICollectionView *collectionView;

// Default value: PTDocumentTabBarDefaultMinimumTabWidth
@property (nonatomic, assign) CGFloat minimumTabWith;

@property (nonatomic, getter=isInteractivelyMoving) BOOL interactivelyMoving;

@property (nonatomic, weak, nullable) id<PTDocumentTabBarDataSource> dataSource;

@property (nonatomic, weak, nullable) id<PTDocumentTabBarDelegate> delegate;

- (void)setSelectedIndex:(NSUInteger)selectedIndex animated:(BOOL)animated;

- (void)removeTabAtIndex:(NSUInteger)index animated:(BOOL)animated;

- (void)reloadItems;

@end

NS_ASSUME_NONNULL_END













