//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "PTDocumentViewController.h"

#import "PTForwardingNavigationItem.h"

NS_ASSUME_NONNULL_BEGIN

@protocol PTDocumentViewControllerToolbarDelegate <NSObject>
@required

/**
 * Informs the delegate that the document view controller will show one of its toolbars.
 */
- (void)documentViewController:(PTDocumentViewController *)documentViewController willShowToolbar:(UIToolbar *)toolbar;

/**
 * Informs the delegate that the document view controller showed one of its toolbars.
 */
- (void)documentViewController:(PTDocumentViewController *)documentViewController didShowToolbar:(UIToolbar *)toolbar;

/**
 * Informs the delegate that the document view controller will hide one of its toolbars.
 */
- (void)documentViewController:(PTDocumentViewController *)documentViewController willHideToolbar:(UIToolbar *)toolbar;

/**
 * Informs the delegate that the document view controller hid one of its toolbars.
 */
- (void)documentViewController:(PTDocumentViewController *)documentViewController didHideToolbar:(UIToolbar *)toolbar;

- (BOOL)documentViewControllerShouldHideNavigationBar:(PTDocumentViewController *)documentViewController;

/**
 * Informs the delegate that the document view controller hid the navigation bar.
 */
- (void)documentViewControllerDidHideNavigationBar:(PTDocumentViewController *)documentViewController animated:(BOOL)animated;

/**
 * Informs the delegate that the document view controller showed the navigation bar.
 */
- (void)documentViewControllerDidShowNavigationBar:(PTDocumentViewController *)documentViewController animated:(BOOL)animated;

- (BOOL)documentViewController:(PTDocumentViewController *)documentViewController shouldOpenExportedFileAttachmentAtURL:(NSURL *)exportedURL;

- (BOOL)documentViewController:(PTDocumentViewController *)documentViewController shouldOpenFileURL:(NSURL *)fileURL;

@end

@interface PTDocumentViewController ()

@property (nonatomic, strong, nullable) NSURL *documentUrl;

@property (nonatomic, strong, nullable) NSURL *localDocumentURL;

@property (nonatomic, readwrite, strong) PTForwardingNavigationItem *navigationItem;

@property (nonatomic, weak, nullable) id<PTDocumentViewControllerToolbarDelegate> toolbarDelegate;

@end

NS_ASSUME_NONNULL_END
