//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "ToolsConfig.h"
#import "PTForwardingNavigationItem.h"

@implementation PTForwardingNavigationItem

- (void)setForwardingTargetItem:(UINavigationItem *)forwardingTargetItem
{
    [self PT_setForwardingTargetItem:forwardingTargetItem animated:NO];
}

- (void)setForwardingTargetItem:(UINavigationItem *)forwardingTargetItem animated:(BOOL)animated
{
    [self willChangeValueForKey:NSStringFromSelector(@selector(forwardingTargetItem))];
    
    [self PT_setForwardingTargetItem:forwardingTargetItem animated:animated];
    
    [self didChangeValueForKey:NSStringFromSelector(@selector(forwardingTargetItem))];
}

#pragma mark Private API

- (void)PT_setForwardingTargetItem:(UINavigationItem *)forwardingTargetItem animated:(BOOL)animated
{
    _forwardingTargetItem = forwardingTargetItem;
    
    if (forwardingTargetItem) {
        forwardingTargetItem.title = self.title;
        
        forwardingTargetItem.prompt = self.prompt;
        
        [forwardingTargetItem setHidesBackButton:self.hidesBackButton animated:animated];
        forwardingTargetItem.leftItemsSupplementBackButton = self.leftItemsSupplementBackButton;
        
        forwardingTargetItem.titleView = self.titleView;
        
        [forwardingTargetItem setRightBarButtonItems:self.rightBarButtonItems animated:animated];
        [forwardingTargetItem setRightBarButtonItem:self.rightBarButtonItem animated:animated];
        
        if (@available(iOS 11.0, *)) {
            forwardingTargetItem.largeTitleDisplayMode = self.largeTitleDisplayMode;
            
            forwardingTargetItem.searchController = self.searchController;
            forwardingTargetItem.hidesSearchBarWhenScrolling = self.hidesSearchBarWhenScrolling;
        }
    }
}

#pragma mark - Property accessor forwarding

- (void)setTitle:(NSString *)title
{
    [super setTitle:title];
    [self.forwardingTargetItem setTitle:title];
}

- (void)setLargeTitleDisplayMode:(UINavigationItemLargeTitleDisplayMode)largeTitleDisplayMode
{
    [super setLargeTitleDisplayMode:largeTitleDisplayMode];
    [self.forwardingTargetItem setLargeTitleDisplayMode:largeTitleDisplayMode];
}

- (void)setPrompt:(NSString *)prompt
{
    [super setPrompt:prompt];
    [self.forwardingTargetItem setPrompt:prompt];
}

- (void)setBackBarButtonItem:(UIBarButtonItem *)backBarButtonItem
{
    if (self.forwardingTargetItem) {
        @throw [NSException exceptionWithName:NSInternalInconsistencyException
                                       reason:@"Cannot set a backBarButtonItem with a parent item"
                                     userInfo:nil];
    }
    [super setBackBarButtonItem:backBarButtonItem];
}

- (void)setHidesBackButton:(BOOL)hidesBackButton animated:(BOOL)animated
{
    [super setHidesBackButton:hidesBackButton animated:animated];
    [self.forwardingTargetItem setHidesBackButton:hidesBackButton animated:animated];
}

- (void)setLeftItemsSupplementBackButton:(BOOL)leftItemsSupplementBackButton
{
    [super setLeftItemsSupplementBackButton:leftItemsSupplementBackButton];
    [self.forwardingTargetItem setLeftItemsSupplementBackButton:leftItemsSupplementBackButton];
}

- (void)setTitleView:(UIView *)titleView
{
    [super setTitleView:titleView];
    [self.forwardingTargetItem setTitleView:titleView];
}

- (void)setLeftBarButtonItems:(NSArray<UIBarButtonItem *> *)leftBarButtonItems animated:(BOOL)animated
{
    if (self.forwardingTargetItem) {
        @throw [NSException exceptionWithName:NSInternalInconsistencyException
                                       reason:@"Cannot set leftBarButtonItems with a parent item"
                                     userInfo:nil];
    }
    [super setLeftBarButtonItems:leftBarButtonItems animated:animated];
}

- (void)setLeftBarButtonItem:(UIBarButtonItem *)leftBarButtonItem animated:(BOOL)animated
{
    if (self.forwardingTargetItem) {
        @throw [NSException exceptionWithName:NSInternalInconsistencyException
                                       reason:@"Cannot set a leftBarButtonItem with a parent item"
                                     userInfo:nil];
    }
    [super setLeftBarButtonItem:leftBarButtonItem animated:animated];
}

- (void)setRightBarButtonItems:(NSArray<UIBarButtonItem *> *)rightBarButtonItems animated:(BOOL)animated
{
    [super setRightBarButtonItems:rightBarButtonItems animated:animated];
    [self.forwardingTargetItem setRightBarButtonItems:rightBarButtonItems animated:animated];
}

- (void)setRightBarButtonItem:(UIBarButtonItem *)item animated:(BOOL)animated
{
    [super setRightBarButtonItem:item animated:animated];
    [self.forwardingTargetItem setRightBarButtonItem:item animated:animated];
}

- (void)setSearchController:(UISearchController *)searchController
{
    [super setSearchController:searchController];
    [self.forwardingTargetItem setSearchController:searchController];
}

- (void)setHidesSearchBarWhenScrolling:(BOOL)hidesSearchBarWhenScrolling
{
    [super setHidesSearchBarWhenScrolling:hidesSearchBarWhenScrolling];
    [self.forwardingTargetItem setHidesSearchBarWhenScrolling:hidesSearchBarWhenScrolling];
}

@end
