//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "AppDelegate.h"
#import "DocumentBrowserViewController.h"

@interface AppDelegate ()
@property (nonatomic, weak) DocumentBrowserViewController* rootViewController;
@end

@implementation AppDelegate


- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    return YES;
}


-(void)buildMenuWithBuilder:(id<UIMenuBuilder>)builder
{
    [super buildMenuWithBuilder:builder];
    if (builder.system != [UIMenuSystem mainSystem]) {
        return;
    }

    [builder removeMenuForIdentifier:UIMenuFormat];
    [builder insertChildMenu:[self fileMenuItems] atStartOfMenuForIdentifier:UIMenuFile];
    #if TARGET_OS_MACCATALYST
    PTDocumentViewController *documentViewController;
    if ([[UIApplication sharedApplication].windows.firstObject.rootViewController isKindOfClass:[DocumentBrowserViewController class]]) {
        DocumentBrowserViewController *docBrowserVC = (DocumentBrowserViewController*)[UIApplication sharedApplication].windows.firstObject.rootViewController;
        documentViewController = docBrowserVC.documentNavigationController.documentViewController;
    }else if ([[UIApplication sharedApplication].windows.firstObject.rootViewController isKindOfClass:[DocumentNavigationController class]]) {
        DocumentNavigationController *docNavigationVC = (DocumentNavigationController*)[UIApplication sharedApplication].windows.firstObject.rootViewController;
        documentViewController = docNavigationVC.documentViewController;
    }
    if (documentViewController == nil) {
        return;
    }
    UIMenu *editMenu = documentViewController.editMenu;
    /** The View Modes menu contains actions for setting the view mode on the document
     *  e.g. Continuous scroll, single page mode, facing page mode.
     */
    UIMenu *viewModesMenu = documentViewController.viewModesMenu;

    // The Additional View menu contains actions for toggling reader mode or displaying the annotation toolbar
    UIMenu *additionalViewMenu = documentViewController.additionalViewMenu;

    /** The Navigation Lists menu contains actions for displaying navigation lists in the side panel:
     *  Table of Contents (document outline), Annotations list, User Bookmarks list, or the thumbnails browser.
     */
    UIMenu *navigationListsMenu = documentViewController.navigationListsMenu;

    // The Navigate Doc menu contains actions for navigating the document
    UIMenu *navigateDocMenu = documentViewController.navigateDocMenu;

    // The Annotate menu contains actions for adding annotations to the document
    UIMenu *annotateMenu = documentViewController.annotateMenu;
    [builder replaceMenuForIdentifier:UIMenuStandardEdit withMenu:editMenu];
    [builder insertChildMenu:viewModesMenu atStartOfMenuForIdentifier:UIMenuView];
    [builder insertChildMenu:navigationListsMenu atStartOfMenuForIdentifier:UIMenuView];
    [builder insertChildMenu:additionalViewMenu atStartOfMenuForIdentifier:UIMenuView];
    [builder insertSiblingMenu:annotateMenu afterMenuForIdentifier:UIMenuView];
    [builder insertSiblingMenu:navigateDocMenu afterMenuForIdentifier:UIMenuView];
    #endif
}

-(UIMenu*)fileMenuItems
{
    UIKeyCommand* open = [UIKeyCommand commandWithTitle:@"Open..." image:Nil action:@selector(open:) input:@"O" modifierFlags:UIKeyModifierCommand propertyList:Nil];

    UIKeyCommand* save = [UIKeyCommand commandWithTitle:@"Save..." image:Nil action:@selector(save:) input:@"S" modifierFlags:UIKeyModifierCommand propertyList:Nil];

    return [UIMenu menuWithTitle:@"Open..." image:Nil identifier:@"open" options:UIMenuOptionsDisplayInline children:@[open, save]];
}

// OPEN

-(void)open:(id)anyObject
{
    [self.rootViewController open:anyObject];
}

// SAVE

-(void)save:(id)anyObject
{
    [self.rootViewController save:anyObject];
}


- (BOOL)application:(UIApplication *)app openURL:(NSURL *)inputURL options:(NSDictionary<UIApplicationOpenURLOptionsKey, id> *)options {
    // Ensure the URL is a file URL
    if (!inputURL.isFileURL) {
        return NO;
    }

    // Reveal / import the document at the URL
    DocumentBrowserViewController *documentBrowserViewController = (DocumentBrowserViewController *)self.window.rootViewController;
    [documentBrowserViewController revealDocumentAtURL:inputURL importIfNeeded:YES completion:^(NSURL * _Nullable revealedDocumentURL, NSError * _Nullable error) {
        if (error) {
            // Handle the error appropriately
            NSLog(@"Failed to reveal the document at URL %@ with error: '%@'", inputURL, error);
            return;
        }
        
        // Present the Document View Controller for the revealed URL
        [documentBrowserViewController presentDocumentAtURL:revealedDocumentURL];
    }];
    return YES;
}

- (UISceneConfiguration *)application:(UIApplication *)application configurationForConnectingSceneSession:(UISceneSession *)connectingSceneSession options:(UISceneConnectionOptions *)options
{
    return [[UISceneConfiguration alloc] initWithName:@"Default Configuration" sessionRole:connectingSceneSession.role];
}

- (BOOL)application:(UIApplication *)application shouldSaveSecureApplicationState:(NSCoder *)coder
{
    return YES;
}

- (BOOL)application:(UIApplication *)application shouldRestoreSecureApplicationState:(NSCoder *)coder
{
    return YES;
}

@end
