//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "DocumentBrowserViewController.h"
#import "PrefetchingDocument.h"
#import <CoreServices/CoreServices.h>

@interface DocumentBrowserViewController () <UIDocumentBrowserViewControllerDelegate, UIDocumentPickerDelegate>

@property (nonatomic, assign) BOOL viewerIsPresenting;
@end

@implementation DocumentBrowserViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.delegate = self;
    self.allowsDocumentCreation = NO;
    self.allowsPickingMultipleItems = NO;
    self.documentNavigationController = [[DocumentNavigationController alloc] init];
    self.documentNavigationController.modalPresentationStyle = UIModalPresentationFullScreen;
    if( IsMacCatalyst == NO )
    {
        self.documentNavigationController.documentViewController.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Open Document" style:UIBarButtonItemStylePlain target:self action:@selector(open:)];
    }
}

#pragma mark UIDocumentBrowserViewControllerDelegate

- (void)documentBrowser:(UIDocumentBrowserViewController *)controller didRequestDocumentCreationWithHandler:(void (^)(NSURL * _Nullable, UIDocumentBrowserImportMode))importHandler {
    NSURL *newDocumentURL = nil;
    
    // Set the URL for the new document here. Optionally, you can present a template chooser before calling the importHandler.
    // Make sure the importHandler is always called, even if the user cancels the creation request.
    if (newDocumentURL != nil) {
        importHandler(newDocumentURL, UIDocumentBrowserImportModeMove);
    } else {
        importHandler(newDocumentURL, UIDocumentBrowserImportModeNone);
    }
}

-(void)documentBrowser:(UIDocumentBrowserViewController *)controller didPickDocumentsAtURLs:(NSArray<NSURL *> *)documentURLs {
    NSURL *sourceURL = documentURLs.firstObject;
    [self presentDocumentAtURL:sourceURL];
}

- (void)documentBrowser:(UIDocumentBrowserViewController *)controller didImportDocumentAtURL:(NSURL *)sourceURL toDestinationURL:(NSURL *)destinationURL {
    // Present the Document View Controller for the new newly created document
    [self presentDocumentAtURL:destinationURL];
}

- (void)documentBrowser:(UIDocumentBrowserViewController *)controller failedToImportDocumentAtURL:(NSURL *)documentURL error:(NSError * _Nullable)error {
    // Make sure to handle the failed import appropriately, e.g., by presenting an error message to the user.
    NSLog(@"Failed to reveal the document at URL %@ with error: '%@'", documentURL, error);
}

- (void)presentDocumentAtURL:(NSURL *)documentURL {
    [self.documentNavigationController openDocumentAtURL:documentURL];
    [self presentViewController:self.documentNavigationController animated:NO completion:nil];
}

-(void)open:(id)anyObject
{
    [self.documentNavigationController open:anyObject];
}

-(void)save:(id)anyObject
{
    [self.documentNavigationController.documentViewController saveDocument:e_ptincremental completionHandler:Nil];
}

@end
