//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2019 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "DocumentNavigationController.h"
#import <CoreServices/CoreServices.h>

@interface DocumentNavigationController () <PTDocumentViewControllerDelegate, UIDocumentPickerDelegate>

@end

@implementation DocumentNavigationController

#if TARGET_OS_MACCATALYST
const BOOL IsMacCatalyst = YES;
#else
const BOOL IsMacCatalyst = NO;
#endif

- (instancetype)init
{
    self = [super init];
    if (self) {
        self.documentViewController = [[PTDocumentViewController alloc] init];
        self.documentViewController.delegate = self;
        [self.documentViewController.pdfViewCtrl SetPagePresentationMode:e_trn_single_continuous];
        // only auto-save on iPadOS
        self.documentViewController.automaticallySavesDocument = !IsMacCatalyst;
        [self setViewControllers:@[self.documentViewController] animated:NO];
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    if( IsMacCatalyst == NO )
    {
        self.documentViewController.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Open Document" style:UIBarButtonItemStylePlain target:self action:@selector(open:)];
    }
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    if (self.documentURL) {
        [self openDocumentAtURL:self.documentURL];
    }
}

- (void)restoreUserActivityState:(NSUserActivity *)activity
{
    self.userActivity = activity;
    NSURL *documentURL = self.userActivity.userInfo[@"documentURL"];
    [self setDocumentURL:documentURL];
}
- (void)openDocumentAtURL:(NSURL *)documentURL
{
    [self.documentViewController openDocumentWithURL:documentURL];
}

-(void)open:(id)anyObject
{
    UIDocumentPickerViewController* picker  = [[UIDocumentPickerViewController alloc] initWithDocumentTypes:@[(__bridge NSString*)kUTTypePDF, @"PDF", @"DOCX", @"JPG", @"DOC", @"PPTX", @"XLSX", @"PNG"] inMode:UIDocumentPickerModeOpen];
    picker.delegate = self;

    picker.modalPresentationStyle = UIModalPresentationPopover;
    picker.popoverPresentationController.barButtonItem = self.documentViewController.navigationItem.leftBarButtonItem;
    [self presentViewController:picker animated:YES completion:nil];
}

-(void)save:(id)anyObject
{
    [self.documentViewController saveDocument:e_ptincremental completionHandler:Nil];
}

#pragma mark - PTDocumentViewControllerDelegate

- (void)documentViewControllerDidOpenDocument:(PTDocumentViewController *)documentViewController
{
    self.documentURL = documentViewController.coordinatedDocument.fileURL;
    #if TARGET_OS_MACCATALYST
    UIWindowScene *windowScene = documentViewController.view.window.windowScene;
    windowScene.titlebar.toolbar = documentViewController.macToolbar;
    NSString *filename = [documentViewController.coordinatedDocument fileURL].lastPathComponent;
    windowScene.title = filename;
    #endif
}

- (void)documentViewController:(PTDocumentViewController *)documentViewController didFailToOpenDocumentWithError:(NSError *)error
{
    NSLog(@"Document failed to open: %@", error);
}

#pragma mark - UIDocumentPickerViewControllerDelegate
-(void)documentPicker:(UIDocumentPickerViewController *)controller didPickDocumentsAtURLs:(NSArray<NSURL *> *)urls
{
    [self.documentViewController openDocumentWithURL:urls.firstObject];
}

@end
