//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "PrefetchingDocument.h"

#import <MobileCoreServices/MobileCoreServices.h>

@interface PrefetchingDocument ()
{
    NSProgress *_progress;
}

@end

@implementation PrefetchingDocument

- (instancetype)initWithFileURL:(NSURL *)url
{
    self = [super initWithFileURL:url];
    if (self) {
        _progress = [NSProgress progressWithTotalUnitCount:1];
        _progress.completedUnitCount = 0;
    }
    return self;
}

- (NSProgress *)progress
{
    return _progress;
}

- (void)openWithCompletionHandler:(void (^)(BOOL))completionHandler
{
    [super openWithCompletionHandler:^(BOOL success) {
        self.progress.completedUnitCount = 1;
        
        if (completionHandler) {
            completionHandler(success);
        }
    }];
}

- (BOOL)readFromURL:(NSURL *)url error:(NSError **)outError
{
    BOOL exists = [NSFileManager.defaultManager fileExistsAtPath:url.path isDirectory:nil];
    if (exists && [url isFileURL]) {
        // Get UTI from file URL.
        NSString *type = nil;

        NSError *error = nil;
        [url getResourceValue:&type forKey:NSURLTypeIdentifierKey error:&error];
        if (error) {
            NSLog(@"Failed to determine UTI for resource at URL \"%@\": %@", url, error);
        }
        
        if (type) {
            // Only PDFs can be opened in-place without conversion.
            self.canOpenInPlace = UTTypeConformsTo((__bridge CFStringRef)type,
                                                   kUTTypePDF);
        }
    }
    return exists;
}

@end
