//---------------------------------------------------------------------------------------
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
// Consult legal.txt regarding legal and license information.
//---------------------------------------------------------------------------------------

#import "SceneDelegate.h"
#import "Tools/Tools.h"

#import "DocumentBrowserViewController.h"
#import "DocumentNavigationController.h"

@interface SceneDelegate ()

@end

@implementation SceneDelegate


- (void)scene:(UIScene *)scene willConnectToSession:(UISceneSession *)session options:(UISceneConnectionOptions *)connectionOptions {

    UIWindowScene *windowScene = (UIWindowScene*)scene;
    UIColor *brandColor = [UIColor colorWithRed:0 green:0.647 blue:0.894 alpha:1.0];
    #if TARGET_OS_MACCATALYST
    brandColor = [UIColor colorNamed:@"brandColor" inBundle:[PTToolsUtil toolsBundle] compatibleWithTraitCollection:nil];
    #endif
    windowScene.windows.firstObject.tintColor = brandColor;

    NSUserActivity *userActivity;
    if (connectionOptions.userActivities.allObjects.firstObject != nil) {
        userActivity = connectionOptions.userActivities.allObjects.firstObject;
    }else if (session.stateRestorationActivity != nil) {
        userActivity = session.stateRestorationActivity;
    }
    if (userActivity != nil) {
        DocumentNavigationController *docNavVC = [[DocumentNavigationController alloc] init];
        [docNavVC restoreUserActivityState:userActivity];
        [self.window setRootViewController:docNavVC];
    }
}


- (void)sceneDidDisconnect:(UIScene *)scene {
    // Called as the scene is being released by the system.
    // This occurs shortly after the scene enters the background, or when its session is discarded.
    // Release any resources associated with this scene that can be re-created the next time the scene connects.
    // The scene may re-connect later, as its session was not neccessarily discarded (see `application:didDiscardSceneSessions` instead).
}


- (void)sceneDidBecomeActive:(UIScene *)scene {
    // Called when the scene has moved from an inactive state to an active state.
    // Use this method to restart any tasks that were paused (or not yet started) when the scene was inactive.
}


- (void)sceneWillResignActive:(UIScene *)scene {
    // Called when the scene will move from an active state to an inactive state.
    // This may occur due to temporary interruptions (ex. an incoming phone call).
}


- (void)sceneWillEnterForeground:(UIScene *)scene {
    // Called as the scene transitions from the background to the foreground.
    // Use this method to undo the changes made on entering the background.
}


- (void)sceneDidEnterBackground:(UIScene *)scene {
    // Called as the scene transitions from the foreground to the background.
    // Use this method to save data, release shared resources, and store enough scene-specific state information
    // to restore the scene back to its current state.
}

- (NSUserActivity *)stateRestorationActivityForScene:(UIScene *)scene
{
    UIWindowScene *windowScene = (UIWindowScene*)scene;
    UIWindow *sceneWindow = windowScene.windows.firstObject;
    NSURL *documentURL;

    // Get the current document URL from each window by checking its root view controller class
    if ([sceneWindow.rootViewController isKindOfClass:[DocumentBrowserViewController class]]) {
        DocumentBrowserViewController *documentBrowserViewController = (DocumentBrowserViewController*)sceneWindow.rootViewController;
        documentURL = documentBrowserViewController.documentNavigationController.documentViewController.coordinatedDocument.fileURL;
    }else if ([sceneWindow.rootViewController isKindOfClass:[DocumentNavigationController class]]) {
        DocumentNavigationController *docNavVC = (DocumentNavigationController*)sceneWindow.rootViewController;
        documentURL = docNavVC.documentViewController.coordinatedDocument.fileURL;
    }

    if (documentURL != nil) {
        NSUserActivity *activity = [[NSUserActivity alloc] initWithActivityType:@"Restore"];
        activity.title = @"Restore";
        NSDictionary *dict = [NSDictionary dictionaryWithObject:documentURL forKey:@"documentURL"];
        activity.userInfo = dict;
        scene.userActivity = activity;
    }
    return scene.userActivity;
}

@end
